#include "PyEMDataWrap.h"
#include "PyEMData.h"
#include "PyEuler.h"
#include "PyVect.h"
#include "PyPickle.h"
#include <boost/python.hpp>
#include <boost/python/to_python_converter.hpp>

namespace python = boost::python;

BOOST_PYTHON_FUNCTION_OVERLOADS(readImages_overloads, py_readImages, 3, 4);
BOOST_PYTHON_FUNCTION_OVERLOADS(readImagesExt_overloads, py_readImagesExt, 4, 5);

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_copy, py_copy, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_readImage, readImage, 1, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_writeImage, writeImage, 1, 6);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_writePNG, writePNG, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_writePGM, writePGM, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_BilateralFilter, BilateralFilter, 2, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_calcHist, py_calcHist, 2, 5);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_setSize, setSize, 1, 3);



void export_EMData()
{
    python::class_<EMData> EMData_class("_EMData", "Don't use this class. Use EMData instead.");
    EMData_class.def(python::init<>());
    
    python::class_<PyEMData, python::bases<EMData> > PyEMData_class("EMData", "EMData() \n  construct an empty EMData instance");
    PyEMData_class.def(python::init<>());


	export_EMData2(EMData_class, PyEMData_class);
	export_EMData3(EMData_class, PyEMData_class);
	export_EMData4(EMData_class, PyEMData_class);

    PyEMData_class.def("getData", &PyEMData::get_num_array, "Numeric.array getData()");
    PyEMData_class.def("setData", &PyEMData::set_num_array, "void setData(Numeric.array array)");

    
    PyEMData_class.def("copyHead", &PyEMData::py_copyHead,
		       python::return_value_policy<python::manage_new_object>(),
		       "EMData copyHead() \n copies descriptive information without any real data");

    PyEMData_class.def("copy", &PyEMData::py_copy,
		       EMData_py_copy("EMData copy(int withfft=0, int withparent=1)\n  returns an exact copy of itself")
		       [python::return_value_policy<python::manage_new_object>()]);

    PyEMData_class.def("clip", (PyEMData* (PyEMData::*)(int, int, int, int)) &PyEMData::py_clip,
		       python::return_value_policy<python::manage_new_object>(),
		       "EMData clip(int x0, int y0, int xsize, int ysize) \nEMData clip(int x0, int y0, int z0, int xsize, int ysize, int zsize) \n  inclusive clip. Pads if larger than data");
    
    PyEMData_class.def("clip", (PyEMData* (PyEMData::*)(int, int, int, int, int, int)) &PyEMData::py_clip,
		       python::return_value_policy<python::manage_new_object>(),
		       "EMData clip(int x0, int y0, int xsize, int ysize) \nEMData clip(int x0, int y0, int z0, int xsize, int ysize, int zsize) \n  inclusive clip. Pads if larger than data");
    
    EMData_class.def("insertClip", &EMData::insertClip, 
		     "insertClip(EMData block, int x0, int y0) \n used for 2D only");
    
    EMData_class.def("readMRCArea", (int (EMData::*)(const char *fsp,int x0,int y0,int w,int h))&EMData::readMRCArea,
		     "int readMRCArea(string fsp, int x0, int y0, int xsize, int ysize) \n  read a rectangular area from a mrc file. works ONLY for real 2d images. Also the area MUST reside within the image. \n\nint readMRCArea(string fsp, int x0, int y0, int z0, int xsize, int ysize, int zsize) \n  read a cube from a 3D mrc file. works ONLY for real 3d images. Also the area MUST reside within the image.");
    
    EMData_class.def("readMRCArea", (int (EMData::*)(const char *fsp,int x0,int y0,int z0,int xl,int yl,int zl))&EMData::readMRCArea,
		     "int readMRCArea(string fsp, int x0, int y0, int xsize, int ysize) \n  read a rectangular area from a mrc file. works ONLY for real 2d images. Also the area MUST reside within the image. \n\nint readMRCArea(string fsp, int x0, int y0, int z0, int xsize, int ysize, int zsize) \n  read a cube from a 3D mrc file. works ONLY for real 3d images. Also the area MUST reside within the image.");
    

    EMData_class.def("readImage", (int (PyEMData::*)(const char*, int, int, ImageType)) &PyEMData::readImage,
		     EMData_readImage(python::args("filename", "n", "nodata", "imagetype"), "int readImage(filename, n = 0, nodata = 0, imagetype = EMData.ANY) \n  Reads a single image from a file. If image type is not specified, the image type will be determined from file type. return 0 if succeeds; return -1 if fails."));
    
    EMData_class.def("writeImage", (int (PyEMData::*)(const char*, int, ImageType, int, int, int)) &PyEMData::writeImage,
		     EMData_writeImage(python::args("filename", "n", "imagetype", "nolock", "swap", "mode"),"int writeImage(filename, n=0, imagetype=EMData.ANY, nolock=0, swap = 0, mode=MODE_float) \n  Writes a single image to a file. If image type is not specified, the image type will be determined from given file type. return 0 if succeeds; return -1 if fails."));

    EMData_class.def("writePNG", (int (PyEMData::*)(const char*, float, float)) &PyEMData::writePNG,
		     EMData_writePNG(python::args("filename", "cmin","cmax"),"int writePNG(filename, float cmin, float cmax) \n  Writes a single image to a file. "));

    EMData_class.def("writePGM", (int (PyEMData::*)(const char*, float, float)) &PyEMData::writePGM,
		     EMData_writePGM(python::args("filename", "cmin","cmax"),"int writePGM(filename, float cmin, float cmax) \n  Writes a single image to a file. "));

    python::def("fileCount", (python::tuple (*)(const char*)) fileCount, "tuple fileCount(string imagefile) \n Returns a tuple with the number of images in a file and the image type.");

    python::def("readImages", py_readImages,
		readImages_overloads("list readImages(string imagefile, int n0, int n1, int nodata=0) \n  Read a set of images. return a list of EMData instances ranging from image n0 to image n1. If nodata is 0, only read image header information. If nodata is 1, read both image data and image header."));

    python::def("readImagesExt", py_readImagesExt,
		readImagesExt_overloads("list readImagesExt(string imagefile, string ext, int n0, int n1, int nodata=0) \n  Read a set of filtered images with an automatically generated filename. return a list of EMData instances ranging from image n0 to image n1. If nodata is 0, only read image header information. If nodata is 1, read both image data and image header."));

    
    EMData_class.def("readLST", &EMData::readLST, "int readLST(string filespec, int n, int nodata=0, string file_ext = None) \n  writes a pointer to an image file to a LST file");
    
    EMData_class.def("writeLST", &EMData::writeLST,
		     "int writeLST(string filespec, int n, string reffile, int refn, string comment)");
    
    EMData_class.def("serialIn", &EMData::serialIn, "serialIn(int file_descriptor)");
    
    EMData_class.def("serialOut", &EMData::serialOut, 
		     "serialOut(int file_descriptor) \n  used ONLY for FILESERVER operation, not safe otherwise");
    
    EMData_class.def("writeDebug", &EMData::writeDebug,
		     "writeDebug(int n) \n  write to a special graphical debug viewer");
    
    EMData_class.def("BilateralFilter", (void (EMData::*)(float, float, int, int)) &EMData::BilateralFilter,
		     EMData_BilateralFilter("void BilateralFilter(float sigma1, float sigma2, int iter=1, int localWidth=1);"));

    EMData_class.def("calcHist", &PyEMData::py_calcHist,
		     EMData_calcHist("calcHist(list hist, int n, float hmin=0, float hmax=0, int add=0)"));

    EMData_class.def("update", &EMData::update, 
		     "update() \n  Tells EMData to update itself in the future.");
    
    EMData_class.def("setPixel", &EMData::setPixel, 
		     "setPixel(float pix) \n  set the pixel value.");
    EMData_class.def("Pixel", &EMData::Pixel, 
		     "float Pixel() \n  returns the pixel value.");

    EMData_class.def("Min", &EMData::Min,
		     "float Min()");
    EMData_class.def("Max", &EMData::Max,
		     "float Max()");
    PyEMData_class.def("MinLoc", &PyEMData::py_MinLoc, 
		       "tuple MinLoc() \n  Returns minimum value locations in a tuple with (min_x_loc, min_y_loc, min_z_loc)");
    PyEMData_class.def("MaxLoc", &PyEMData::py_MaxLoc,
		       "tuple MaxLoc() \n  Returns maxium value locations in a tuple with (max_x_loc, max_y_loc, max_z_loc)");

    PyEMData_class.def("display", &PyEMData::display, 
		       "int display() \n  Display the current image. Returns 0 if success; Returns -1 on error.");    
    EMData_class.def("Mean", &EMData::Mean, 
		     "float Mean()");
    EMData_class.def("Sigma", &EMData::Sigma, 
		     "float Sigma()");
    EMData_class.def("SumSq", &EMData::SumSq, 
		     "float SumSq()");
    EMData_class.def("Mean2", &EMData::Mean2, 
		     "float Mean2()");
    EMData_class.def("Sigma2", &EMData::Sigma2, 
		     "float Sigma2()");
    EMData_class.def("Skewness", &EMData::Skewness, 
		     "float Skewness()");
    EMData_class.def("Kurtosis", &EMData::Kurtosis, 
		     "float Kurtosis()");      
    EMData_class.def("edgeMean", &EMData::edgeMean, 
		     "float edgeMean()");
    EMData_class.def("cirMean", &EMData::cirMean, 
		     "float cirMean()");
    EMData_class.def("setParent", &EMData::setParent, 
		     "setParent(EMData data)");
    PyEMData_class.def("Parent", &PyEMData::py_Parent,
		       python::return_internal_reference<>(),
		       "EMData Parent()");
    EMData_class.def("setName", &EMData::setName,
		     "setName(string name)");
    EMData_class.def("Name", &EMData::Name, 
		     "string Name()");
    EMData_class.def("setPath", &EMData::setPath, 
		     "setPath(string path)");
    EMData_class.def("Path", &EMData::Path, 
		     "string Path()");
    EMData_class.def("setPathN", &EMData::setPathN, 
		     "setPathN(int n)");
    EMData_class.def("PathN", &EMData::PathN, 
		     "int PathN()");

    EMData_class.def("xSize", &EMData::xSize, 
		     "int xSize() \n  Returns x dimension size");
    EMData_class.def("ySize", &EMData::ySize, 
		     "int ySize()  \n  Returns y dimension size");
    EMData_class.def("zSize", &EMData::zSize, 
		     "int zSize() \n  Returns z dimension size");
    EMData_class.def("setSize", &EMData::setSize, 
		     EMData_setSize("int setSize(int x, int y=1, int z=1) \n Sets x, y, z dimension sizes"));
    EMData_class.def("setRI", &EMData::setRI, 
		     "setRI(int i)");
    EMData_class.def("setComplex", &EMData::setComplex, 
		     "setComplex(int i)");
    EMData_class.def("isComplex", &EMData::isComplex, 
		     "int isComplex() \n Returns 1 if the data is in complex number mode; Returns 0 othewise.");
    EMData_class.def("setComplexX", &EMData::setComplexX, 
		     "setComplexX(int i)");
    EMData_class.def("isComplexX", &EMData::isComplexX,
		     "int isComplexX()");
    EMData_class.def("hasCTF", &EMData::hasCTF,
		     "int hasCTF()");
    EMData_class.def("hasCTFF", &EMData::hasCTFF,
		     "int hasCTFF()");
    EMData_class.def("getFlags", &EMData::getFlags, 
		     "int getFlags()");
    EMData_class.def("aliPeak", &EMData::aliPeak, 
		     "float aliPeak()");
    EMData_class.def("setFlipped", &EMData::setFlipped, 
		     "setFlipped(int f)");
    EMData_class.def("isFlipped", &EMData::isFlipped, 
		     "int isFlipped()");
    EMData_class.def("Dx", &EMData::Dx,  
		     "float Dx()");
    EMData_class.def("Dy", &EMData::Dy, 
		     "float Dy()");
    EMData_class.def("Dz", &EMData::Dz, 
		     "float Dz()");
    EMData_class.def("alt", &EMData::alt, 
		     "float alt()");
    EMData_class.def("az", &EMData::az,
		     "float az()");
    EMData_class.def("phi", &EMData::phi,
		     "float phi()");
    EMData_class.def("get_center_x", &EMData::get_center_x,
		     "float get_center_x()");
    EMData_class.def("get_center_y", &EMData::get_center_y,
		     "float get_center_y()");
    EMData_class.def("set_center_x", &EMData::set_center_x,
		     "set_center_x(float cx)");
    EMData_class.def("set_center_y", &EMData::set_center_y,
		     "set_center_y(float cy)");
    PyEMData_class.def("getEuler", &PyEMData::py_getEuler,
		       python::return_internal_reference<>(),
		       "Euler getEuler()");
    EMData_class.def("addAsMask", &EMData::addAsMask, 
		     "void addAsMask(EMData* mask2)");
    EMData_class.def("crossCommonLineSearchPrepare", &EMData::crossCommonLineSearchPrepare,
		     "void crossCommonLineSearchPrepare(ccmlInputParm *inputParm)");
    EMData_class.def("crossCommonLineSearchFinalize", &EMData::crossCommonLineSearchFinalize,
                     "void crossCommonLineSearchFinalize(ccmlInputParm *inputParm)");

    EMData_class.def("crossCommonLineSearchReadRawParticle", &EMData::crossCommonLineSearchReadRawParticle,
    		     "void crossCommonLineSearchReadRawParticle(ccmlParm* inputParm)");
    EMData_class.def("crossCommonLineSearchReleaseParticle", &EMData::crossCommonLineSearchReleaseParticle,
    		     "void crossCommonLineSearchReleaseParticle(ccmlParm* inputParm)");

    EMData_class.def("crossCommonLineSearch", &EMData::crossCommonLineSearch,
                     "ccmlOutputParm crossCommonLineSearch(ccmlInputParm *inputParm)");

}




