#include "PyList.h"
#include "PyEMClasses.h"
#include "PyqEM.h"
#include "EMData.h"
#include "PyEMDataWrap.h"
#include <boost/python.hpp>
#include <boost/python/to_python_converter.hpp>


namespace python = boost::python;


BOOST_PYTHON_MODULE(libpyEM)
{
    vector_to_python<EMData*>();
    vector_from_python<EMData*>();
    
    python::scope().attr("__doc__") = "EMAN classes and routines for image/volume processing in single particle reconstructions.\n\nThis exports:\n  - EMData: the primary class to process EM data.\n  - Euler: used to store the orientation of a slice. \n  - Quater  \n  - Vect \n  - VolFrag \n  - Cylinder \n  - Cylinders \n  - ObjectStat \n  - Helix\n";
    
    export_Euler();
    export_Quater();
    export_Vect();
    export_VolFrag();
    export_Cylinder();
    export_Cylinders();
    export_ObjectStat();
    export_Helix();
    export_util();
    export_enums();
    export_EMData();
    export_XYData();
    export_Quaternion();
    export_ccmlParm();
    
#ifdef GUI
    export_qt();
    export_ImShape();
    export_BPoint();
    export_Image();
    export_ImageMx();
#endif
}
