#include "PyEuler.h"


PyEuler::PyEuler()
    : Euler()
{

}

PyEuler::PyEuler(float a1, float a2, float a3)
    : Euler(a1, a2, a3)
{
}

PyEuler::PyEuler(float a1, float a2, float a3, int type) :
    Euler(a1, a2, a3, type)
{
}

PyEuler::PyEuler(float a1, float a2, float a3, float a0, int type) :
    Euler(a1, a2, a3, a0, type)
{
}

PyEuler::PyEuler(PyList m_list)
{
    float* m_array = new float[9];
    list2array(m_list, m_array);
    setAngle(m_array);
    array2list(m_array, m_list);
    delete [] m_array;
    m_array = 0;

    init();
}

PyEuler::~PyEuler()
{
}

static Euler strip_const(const PyEuler& pe)
{
    return *(const_cast<PyEuler*>(&pe));
}

static PyEuler toPyEuler(Euler e)
{
    return *(static_cast<PyEuler*>(&e));
}

PyEuler operator*(const PyEuler& lhs, const PyEuler& rhs)
{
    Euler e1 = strip_const(lhs);
    Euler e2 = strip_const(rhs);
    Euler r =  e1 * e2;
    return toPyEuler(r);
}

PyEuler operator/(const PyEuler& lhs, const PyEuler& rhs)
{
    Euler e1 = strip_const(lhs);
    Euler e2 = strip_const(rhs);
    Euler r = e1 / e2;
    return toPyEuler(r);
}

int operator!=(const PyEuler& lhs, const PyEuler& rhs)
{
    Euler e1 = strip_const(lhs);
    Euler e2 = strip_const(rhs);
    return (e1 != e2);
}


int operator==(const PyEuler& lhs, const PyEuler& rhs)
{
    return !(lhs != rhs);
}


python::tuple PyEuler::getByType(EulerType type)
{
    float* mvalue = 0;
    
    switch(type) {
    case EMAN:
	return python::make_tuple(alt(), az(), phi());
    case IMAGIC:
	return python::make_tuple(alpha(), beta(), gamma());
    case SPIN:
	return python::make_tuple(Q(), n1(), n2(), n3());
    case QUAT:
	return python::make_tuple(e0(), e1(), e2(), e3());
    case MATRIX:
	mvalue = mat();
	return python::make_tuple(mvalue[0], mvalue[1], mvalue[2],
				  mvalue[3], mvalue[4], mvalue[5], 
				  mvalue[6], mvalue[7], mvalue[8]);
    case SGIROT:
	return python::make_tuple(Qsgi(), n1sgi(), n2sgi(), n3sgi());	
    case SPIDER:
	return python::make_tuple(thetaSpider(), phiSpider(), gammaSpider());
    case MRC:
	return python::make_tuple(thetaMRC(), phiMRC(), omegaMRC());
	
    default:
	fprintf(stderr, "Error: no such Euler type: %d\n", type);
    }
    
    return python::make_tuple();    
}


void PyEuler::setByType(python::tuple value, EulerType type)
{
    float a1 = python::extract<float>(value[0]);
    float a2 = python::extract<float>(value[1]);
    float a3 = python::extract<float>(value[2]);
    float a4 = 0;
    float* mvalue;
    
    switch(type) {
    case EMAN:
    case IMAGIC:
    case SPIDER:
	setAngle(a1, a2, a3, type);
	break;
	
    case SPIN:
    case QUAT:
    case SGIROT:
	a4 = python::extract<float>(value[3]);
	setAngle(a1, a2, a3, a4,type);	
	break;
	
    case MATRIX:
	mvalue = new float[9];
	for (int i = 0; i < 9; i++) {
	    mvalue[i] = python::extract<float>(value[i]);
	}
	setAngle(mvalue);
	delete [] mvalue;
	mvalue = 0;
	break;
    case MRC:
	fromMRC(a1, a2, a3);
	break;
    default:
	fprintf(stderr, "Error: no such Euler type: %d\n", type);
    }    
}
    
