#include "PyEMClasses.h"

#include "Euler.h"
#include "PyEuler.h"
#include "PyPickle.h"
#include <boost/python.hpp>
#include <boost/python/to_python_converter.hpp>


namespace python = boost::python;

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(print_overloads, print, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_printMatrix, printMatrix, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_diff, diff, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_setAngle1, setAngle, 3, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_setAngle2, setAngle, 4, 5);


void export_Euler()
{
    python::class_<Euler> Euler_class("_Euler", "Don't use this class. Use 'Euler' instead.");
    
    python::class_<PyEuler, python::bases<Euler> > PyEuler_class("Euler", "Euler();\nEuler(float a1, float a2, float a3); \nEuler(float a1, float a2, float a3, EulerType type); \nEuler(float e1, float e2, float e3, float e0, EulerType type);");

    PyEuler_class.def(python::init<>());
    PyEuler_class.def(python::init<float, float, float>());
    PyEuler_class.def(python::init<float, float, float, int>());
    PyEuler_class.def(python::init<float, float, float, float, int>());
    PyEuler_class.def(python::init<PyList>());
    
    PyEuler_class.def(python::self * python::self);
    PyEuler_class.def(python::self / python::self);
    //PyEuler_class.def(python::self != python::self);
    PyEuler_class.def(python::self == python::self);
    
    PyEuler_class.def("getByType", &PyEuler::getByType,
		      "tuple getByType(EulerType type) \n  Return Euler values in a tuple given a type.");
    
    PyEuler_class.def("setByType", &PyEuler::setByType,
		      "setByType(tuple value, EulerType type) \n  Set Euler values given a tuple and the type.");

    
    Euler_class.def("init", &Euler::init, "init() \n  initialize an Euler instance.");
    Euler_class.def("rectify", &Euler::rectify, "rectify() \n  rectify an Euler instance.");
    Euler_class.def("increment", &Euler::increment,
		    "increment(float delta_alt, float delta_az, float del_phi) \n  increment an Euler in place");
    Euler_class.def("diff", &Euler::diff,
		    Euler_diff("float diff(Euler angle, bool nophi=true) \n  find angle between two direction vectors"));

    Euler_class.def("inverse", &Euler::inverse, "Euler inverse() \n  takes the inverse of an Euler instance");

    Euler_class.def("EulerMult", &Euler::EulerMult,
		    "Euler EulerMult(Euler bc) \n  returns the value of the given rotation");
    Euler_class.def("getMaxSymEl", &Euler::getMaxSymEl,
		    "int getMaxSymEl() \n  gets the order of the group");
    Euler_class.def("resetSym", &Euler::resetSym,
		    "resetSym() \n  initializes symcounter_ without ing 1st element");
    Euler_class.def("FirstSym", &Euler::FirstSym,
		    "Euler FirstSym() \n  initializes");
    Euler_class.def("NextSym", &Euler::NextSym,
		    "Euler NextSym()");
    Euler_class.def("SymN", &Euler::SymN,
		    "Euler SymN(n) \n  returns the n'th symmetric element");
    Euler_class.def("valid", &Euler::valid,
		    "int valid()");
    Euler_class.def("fromMRC", &Euler::fromMRC,
		    "fromMRC(theta, phi, omega) \n  set from MRC angles ");
    Euler_class.def("thetaMRC", &Euler::thetaMRC,"float thetaMRC()");
    Euler_class.def("phiMRC", &Euler::phiMRC, "float phiMRC()");
    Euler_class.def("omegaMRC", &Euler::omegaMRC, "float omegaMRC()");
    Euler_class.def("setAngleFromMRC", &Euler::setAngleFromMRC,"setAngleFromMRC(theta, phi, omega) \n input is MRC angles, internal angles are defined by EMAN");
    Euler_class.def("convertToMRCAngle", &Euler::convertToMRCAngle, "void convertToMRCAngle()");
    Euler_class.def("alt_MRC", &Euler::alt_MRC,"float alt_MRC()");
    Euler_class.def("az_MRC",  &Euler::az_MRC, "float az_MRC()");
    Euler_class.def("phi_MRC", &Euler::phi_MRC,"float phi_MRC()");
    Euler_class.def("alt", &Euler::alt, "float alt()");
    Euler_class.def("az", &Euler::az, "float az()");
    Euler_class.def("phi", &Euler::phi, "float phi()");
    Euler_class.def("alpha", &Euler::alpha, "float alpha()");
    Euler_class.def("beta", &Euler::beta, "float beta()");
    Euler_class.def("gamma", &Euler::gamma, "float gamma()");
    Euler_class.def("phiSpider", &Euler::phiSpider, "float phiSpider()");
    Euler_class.def("thetaSpider", &Euler::thetaSpider, "float thetaSpider()");
    Euler_class.def("gammaSpider", &Euler::gammaSpider, "float gammaSpider()");
    Euler_class.def("e0", &Euler::e0, "float e0()");
    Euler_class.def("e1", &Euler::e1, "float e1()");
    Euler_class.def("e2", &Euler::e2, "float e2()");
    Euler_class.def("e3", &Euler::e3, "float e3()");
    Euler_class.def("Q", &Euler::Q, "float Q()");
    Euler_class.def("n1", &Euler::n1, "float n1()");
    Euler_class.def("n2", &Euler::n2, "float n2()");
    Euler_class.def("n3", &Euler::n3, "float n3()");
    Euler_class.def("Qsgi", &Euler::Qsgi, "float Qsgi()");
    Euler_class.def("n1sgi", &Euler::n1sgi, "float n1sgi()");
    Euler_class.def("n2sgi", &Euler::n2sgi, "float n2sgi()");
    Euler_class.def("n3sgi", &Euler::n3sgi, "float n3sgi()");
    Euler_class.def("setSym", &Euler::setSym,
		    "setSym(string symmetry) \n  initialize symcounter to zero");
    Euler_class.def("Print", &Euler::print,
		    print_overloads("print(string id=None, FILE out=stdout)"));
    
    Euler_class.def("printMatrix", &Euler::printMatrix,
		    Euler_printMatrix("printMatrix(string id=None, FILE out=stdout)"));
    
    Euler_class.def("fromLeft", (void (Euler::*)(Euler)) &Euler::fromLeft,
		    "fromLeft(Euler leftEuler) \n  rotates a given rotation  on the left \n\n fromLeft(alt, az, phi) \n  rotates a given rotation  on the left");

    Euler_class.def("fromLeft", (void (Euler::*)(float, float, float)) &Euler::fromLeft,
		    "fromLeft(Euler leftEuler) \nfromLeft(alt, az, phi) \n  rotates a given rotation  on the left");
    
    Euler_class.def("setAngle", (void (Euler::*)(float, float, float, int))0,
		    Euler_setAngle1("setAngle(float a1, float a2, float a3, EulerType type = Euler.EMAN) \nsetAngle(float e1, float e2, float e3, float e0, EulerType type)"));
		    
    Euler_class.def("setAngle", (void (Euler::*)(float, float, float, float, int))0,
		    "setAngle(float a1, float a2, float a3, EulerType type = Euler.EMAN) \nsetAngle(float e1, float e2, float e3, float e0, EulerType type)");


    Euler_class.def_pickle(Euler_pickle_suite());



    python::enum_<Euler::EulerType>("EulerType")
	.value("eman", Euler::EMAN)
	.value("imagic", Euler::IMAGIC)
	.value("spin", Euler::SPIN)
	.value("quat", Euler::QUAT)
	.value("matrix", Euler::MATRIX)
	.value("sgirot", Euler::SGIROT)
	.value("spider", Euler::SPIDER)
	.value("mrc", Euler::MRC)
	;
 
 
}


