#include "PyEMClasses.h"
#include "PyVect.h"
#include <boost/python.hpp>
#include <boost/python/to_python_converter.hpp>


namespace python = boost::python;

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(print_overloads, print, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Vect_angle, Vect::angle, 1, 2);


void export_Vect()
{
    python::class_<Vect> Vect_class("_Vect", "Don't use this class. Use 'Vect' instead.");
    python::class_<PyVect, python::bases<Vect> > PyVect_class("Vect", "Vect()\nVect(float xx, float yy, float zz)");
    
    PyVect_class.def(python::init<>());
    PyVect_class.def(python::init<float, float, float>());
    
    Vect_class.def("normalize", &Vect::normalize, "normalize()");
    Vect_class.def("rotate",  (Vect (Vect::*)(Euler&)) &Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");

    Vect_class.def("rotate",  (Vect (Vect::*)(Quater&)) &Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");
    
    Vect_class.def("rotate",  (Vect (Vect::*)(Vect&, float)) &Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");

    Vect_class.def("rotate",  (Vect (Vect::*)(float, float, float))&Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");
    
    Vect_class.def("rotate",  (Vect (Vect::*)(float, float, float, float))&Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");
    
    Vect_class.def("cross", &Vect::cross, "Vect cross(Vect p)");
    Vect_class.def("getQuater", &Vect::getQuater, "Quater getQuater()");
    Vect_class.def("X", &Vect::X, "float X()");
    Vect_class.def("Y", &Vect::Y, "float Y()");
    Vect_class.def("Z", &Vect::Z, "float Z()");
    Vect_class.def("Alt", &Vect::Alt, "float Alt()");
    Vect_class.def("Az", &Vect::Az, "float Az()");
    Vect_class.def("setX", &Vect::setX, "setX(float xx)");
    Vect_class.def("setY", &Vect::setY, "setY(float yy)");
    Vect_class.def("setZ", &Vect::setZ, "setZ(float zz)");
    Vect_class.def("setXYZ", &Vect::setXYZ, "setXYZ(float xx, float yy, float zz)");
    Vect_class.def("inner", &Vect::inner, "float inner(Vect p)");
    Vect_class.def("angle", &Vect::angle,
		   Vect_angle("float angle(Vect p, Vect y=Vect())"));
    
    Vect_class.def("length", &Vect::length, "float length()");
    Vect_class.def("euler", &Vect::euler, "Euler euler()");
    Vect_class.def("eulerTo", (Euler (Vect::*)(Vect&)) &Vect::eulerTo,
		   "Euler eulerTo(Vect p)\nEuler eulerTo(float x, float y, float z)");
    Vect_class.def("eulerTo", (Euler (Vect::*)(float, float, float))&Vect::eulerTo);
    Vect_class.def("isNeighbor", (bool (Vect::*)(Vect&))&Vect::isNeighbor,
		   "bool isNeighbor(Vect p)\n  26 neighbor connection");
    Vect_class.def("Print", &Vect::print, print_overloads("print(string id=None, FILE out=stdout)"));

    
    PyVect_class.def(python::self * float());
    PyVect_class.def(python::self / float());
    PyVect_class.def(python::self + float());
    PyVect_class.def(python::self + python::self);
    PyVect_class.def(python::self - float());
    PyVect_class.def(python::self - python::self);
    PyVect_class.def(python::self == python::self);
}
