#include "ImageMx.h"
#include "EMData.h"
#include "qmessagebox.h"
#include "qapplication.h"
#include <qprinter.h>
#include <stdio.h>

ImageMx::ImageMx(QWidget* parent,const char* name):QScrollView(parent,name) {
data.setAutoDelete(FALSE);
loc.setAutoDelete(TRUE);
orx=ory=0;
invert=0;
mode=0;
autodel=0;
label=1;
scale=1.0;
cont=.2;
brite=0;
imc=NULL;
split=NULL;
viewport()->setBackgroundMode(NoBackground);
labelfont=new QFont("helvetica",8);
setCaption("Image View (middle click for controls)");
}

ImageMx::~ImageMx() {
if (imc) delete imc;
if (split) delete split;
delete labelfont;
}

void ImageMx::drawContentsOffset ( QPainter * qp, int ox, int oy, int cx, int cy, int cw, int ch ) {
BPoint *p;
EMData *d;
float mm,b;
int i,sx,sy,mo,x0,y0,lx,ly,drx,dry;
char *s;
//int w=viewport()->width(),h=viewport()->height();
int w=cw,h=ch;

orx=ox;
ory=oy;

drx=cx-ox;
dry=cy-oy;

ox=cx;
oy=cy;

//printf("draw %d %d %d %d %d %d %d %d\n",ox,oy,cx,cy,cw,ch,drx,dry);

if (data.count()==0) {
	buildMx();
	qp->fillRect ( cx-ox,cy-oy,cw,ch,QBrush(black) );
	return;
}

if (loc.count()!=data.count()) {
	buildMx();
	viewport()->update();
	return;
}

// a drawing area to put the images in
s=(char *)malloc(h*w+80);
memset(s,10,h*w+80);
sprintf(s,"P5\n%d %d\n255\n",w,h);
i=strlen(s);

// This renders each image into the drawing region
for (d=data.first(),p=loc.first(); d; d=data.next(),p=loc.next()) {
	if (p->x()>ox+w || p->y()>oy+h ||
		p->x()+d->xSize()*scale<ox || p->y()+d->ySize()*scale<oy) continue;

	sx=d->xSize()*scale;
	sy=d->ySize()*scale;
//	mm=-d->Min()<d->Max()?d->Max():-d->Min();
//	b=mm*brite*2.0;
//	mm*=(1.0-sqrt(cont));
	lx=p->x()-ox;
	ly=p->y()-oy;
	x0=y0=0;
	if (lx<0) { x0=-lx; sx+=lx; lx=0; }
	if (ly<0) { y0=ly; /*sy+=ly;*/ ly=0; }
	if (lx+sx>w) { sx=w-lx; }
	if (ly+sy>h) { y0-=(h-(ly+sy)); sy=h-ly; }
	mo=lx+ly*w;
	if (d->isComplex()) d->ri2ap();
	b=d->Mean();
	if (brite>0) b-=brite*(d->Mean()-d->Min());
	else b-=brite*(d->Max()-d->Mean());

	float wf=SQR(1.001-cont)*(d->Max()-d->Min())/2.0;

	d->renderAmp8((unsigned char *)s+i+mo,x0/scale,y0/scale,sx,sy,w,
		scale,invert?254:1,invert?1:254,b-wf,b+wf);
}
qpm.loadFromData((uchar *)s,(uint)(i+h*w));

// If enabled, this puts a numerical label on each image
if (label) {
	QPainter qp2(&qpm);
	qp2.setBackgroundMode(OpaqueMode);
	qp2.setBackgroundColor(QColor(128,0,128));
	qp2.setPen(QColor(235,235,235));
	qp2.setFont(*labelfont);
	y0=fontMetrics().descent();
	for (d=data.first(),p=loc.first(),i=0; d; d=data.next(),p=loc.next(),i++) {
		if (p->x()>ox+w || p->y()>oy+h ||
			p->x()+d->xSize()*scale<ox || p->y()+d->ySize()*scale<oy) continue;
	
		lx=p->x()-ox;
		ly=p->y()-oy+d->ySize()*scale;
		s[0]=0;
		if (label&1) sprintf(s,"%0d%s",i,(label&2)?",":"");
		if (label&2) sprintf(s+strlen(s),"%0d",d->NImg());
		qp2.drawText(lx,ly-y0,s);
//		printf("%d. %d %d\n",i,lx,ly);
	}
}


free(s);
	
bitBlt(viewport(),drx,dry,&qpm);

}

void ImageMx::clearData() {
data.clear();
emit imsChanged(&data);
}

void ImageMx::replaceData(int n,EMData *d) {
//printf("Repl image %d\n",n);
if (data.count()>n) data.remove(n);
data.insert(n,d);
loc.clear();
emit imsChanged(&data);
viewport()->update();
}

void ImageMx::addData(EMData *d,int n) {
char s[140];
//printf("Add image\n");

if (!d) return;

setCaption(d->Name());
if (n==-1) data.append(d);
else data.insert(n,d);
loc.clear();

d=data.first();
strncpy(s,d->Name(),100);
s[100]=0;
strcat(s," (middle mouse for controls)");
setCaption(s);

emit imsChanged(&data);
viewport()->update();
}

void ImageMx::makeVisible(int i) {
int x,y;
if (i>=loc.count()||i<0) return;

x=loc.at(i)->x()+data.at(i)->xSize()*scale/2;
y=loc.at(i)->y()+data.at(i)->ySize()*scale/2;
center(x,y);

}
void ImageMx::setBC(float v,float v2) { brite=v; cont=v2; viewport()->update(); }
void ImageMx::setInvert(int i) {  invert=i; viewport()->update(); }
void ImageMx::setScale(float s) {  scale=s>0?s:1.0; buildMx(); viewport()->update(); }
void ImageMx::setLabel(int l) { label=l+(label&2); viewport()->update(); }
void ImageMx::setLabel2(int l) { label=l*2+(label&1); viewport()->update(); }
void ImageMx::setMode(int m) { if (mode!=-1) mode=m; }
void ImageMx::setReadOnly(int ro) { 
	if (ro) mode=-1; 
	else if (mode<0) mode=0; 
	if (imc) imc->setup(brite,cont,invert,label,mode);
}

void ImageMx::notVisible() { if (imc) imc->hide(); }

void ImageMx::viewportMousePressEvent(QMouseEvent *e) {
int i,x,y;

x=e->x()+orx;
y=e->y()+ory;

if (loc.count()==0) return;

for (i=0; i<loc.count(); i++) {
	if (x>loc.at(i)->x() && x<loc.at(i)->x()+data.at(i)->xSize()*scale &&
		y>loc.at(i)->y() && y<loc.at(i)->y()+data.at(i)->ySize()*scale) break;
}
if (i==loc.count()) mim=-1; else mim=i;

if ((e->button()&LeftButton)) {
	if ((e->state()&ShiftButton)) {
		
//		QImageDrag *idr = new QImageDrag(qpm->convertToImage(),this);
//		if (idr) idr->dragCopy();
//		else printf("Drag error\n");
	}
	else {
		// pass event on
		if (mode<=0) {
			if (mim<0) emit(mousePress(e,mim,x,y));
			else emit(mousePress(e,mim,x-loc.at(mim)->x(),y-loc.at(mim)->y()));
		}
		// delete
		else if (mode==1) {
			/*//this is for record the deletion index of image in a stack image file
			FILE * logfile = fopen("remove.log", "a+");
			fprintf (logfile, "%d\n", mim);
    		fclose (logfile);*/
			if (mim>=0) data.remove(mim);
			viewport()->update();
		}
		// rearrange
		else if (mode==2) {
			QApplication::setOverrideCursor( crossCursor );
		}
		// split
		else if (mode==3 && mim>=0) {
			if (!split) { split=new ImageMx(); split->show(); }
			split->addData(data.at(mim)->copy(0,0));
		}
		// probe 
		else if (mode==4 && mim>=0) {
			unsigned char mx[49];
			EMData *d=data.at(mim);
			float mm=-d->Min()<d->Max()?d->Max():-d->Min();
			float b=mm*brite*2.0;
			mm*=(1.0-sqrt(cont));
			memset(mx,0,25);
			d->renderAmp8(mx,(int)pxtodx(x,mim)-3,(int)pytody(y,mim)-3,7,7,7,1.0,invert?254:1,invert?1:254,-mm-b,mm-b);
			emit probe(mx,mim,(int)pxtodx(x,mim),(int)pytody(y,mim),d->valueAtSafe((int)pxtodx(x,mim),(int)pytody(y,mim)),1);
//printf("probe %d,%d  %f\n",(int)pxtodx(x,mim),(int)pytody(y,mim),d->valueAt((int)pxtodx(x,mim),(int)pytody(y,mim)));
/*			int x0,y0;
			float f;
			for (y=0; y<5; y++) {
				for (x=0; x<5; x++) {
					x0=x-loc.at(mim)->x();
					y0=y-loc.at(mim)->y();
					if (x0<0||y0<0||x0>=data[mim].xSize()||y0>=data[mim].ySize()) { mx[x+y*5]=0; continue; }
					f=data[mim].valueAt(x0,y0);
					mx[x+y*5]= }}*/

		}
	}
}
/*if ((e->button()&MidButton)) { 
	invert^=1; 
	if (imc) imc->setup(brite,cont,invert,label); 
	viewport()->update();
}*/
if ((e->button()&MidButton)||(e->button()&LeftButton && e->state()&ControlButton)) { 
	if (!imc) {	
		imc=new ImageMxCon; 
		connect(imc,SIGNAL(setBC(float,float)),SLOT(setBC(float,float)));
		connect(imc,SIGNAL(invToggle(int)),SLOT(setInvert(int)));
		connect(imc,SIGNAL(labToggle(int)),SLOT(setLabel(int)));
		connect(imc,SIGNAL(lab2Toggle(int)),SLOT(setLabel2(int)));
		connect(imc,SIGNAL(saveImage()),SLOT(saveImage()));
		connect(imc,SIGNAL(saveFile()),SLOT(saveFile()));
		connect(imc,SIGNAL(newMode(int)),SLOT(setMode(int)));
		connect(imc,SIGNAL(setScale(float)),SLOT(setScale(float)));
		connect(imc,SIGNAL(print()),SLOT(print()));
		connect(this,SIGNAL(imsChanged(QList<EMData> *)),imc,SLOT(imagesChanged(QList<EMData> *)));
		connect(this,SIGNAL(probe(unsigned char *,int,int,int,float,int)),imc,SLOT(doProbe(unsigned char *,int,int,int,float,int)));
		imc->setup(brite,cont,invert,label,mode);
	}
	imc->show();
	imc->raise();
	emit imsChanged(&data);
}

mdx=e->x();
mdy=e->y();

}

void ImageMx::print() {
QPrinter pr;
pr.setup();
QPainter pa(&pr);

pa.drawPixmap(0,0,qpm);
pa.end();
}

void ImageMx::saveImage() {
QString s1( QFileDialog::getSaveFileName(getenv("PWD"),"*.png") );
if ( s1.isNull() ) return;

viewport()->repaint(TRUE);
qpm.save(s1.data(),"PNG");
}

void ImageMx::saveFile() {
if (data.count()<=0) return;
QString s1( QFileDialog::getSaveFileName(data.at(0)->Path(),"*.hed") );
if ( s1.isNull() ) return;
QString s2=s1;
s2.replace(s2.length()-3,3,"img");

if (!access(s1.data(),F_OK)) {
	int i=QMessageBox::warning( this, "EMAN",
		"File exists!\n\nDo you wish to overwrite the existing\nfile, erasing its current contents ?",
		"Yes", "No", 0,0, 1 );
	if (i) return;
}

unlink(s1.data());
unlink(s2.data());
for (int i=0; i<data.count(); i++) {
	data.at(i)->writeIMAGIC(s1.data(),i);
}


}

void ImageMx::viewportMouseReleaseEvent(QMouseEvent *e) {
int *a,b;
int x,y,i,dmim=-1;

x=e->x()+orx;
y=e->y()+ory;
if (loc.count()==0) return;

if ((e->state()&LeftButton)) {
	if (!(e->state()&ShiftButton)) {
		if (mode<=0) {
			if (mim<0||mim>=loc.count()) emit(mouseRelease(e,mim,x,y));
			else emit(mouseRelease(e,mim,x-loc.at(mim)->x(),y-loc.at(mim)->y()));
		}
		else if (mode==2) {
			QApplication::restoreOverrideCursor();

			for (i=0; i<loc.count(); i++) {
				if (x>loc.at(i)->x() && x<loc.at(i)->x()+data.at(i)->xSize()*scale &&
					y>loc.at(i)->y() && y<loc.at(i)->y()+data.at(i)->ySize()*scale) break;
			}
			if (i==loc.count()||i==mim) return;
			EMData *dt;
			dt=data.take(mim);
			data.insert(i,dt);
			viewport()->update();
		}
		else if (mode==4) {
			for (i=0; i<loc.count(); i++) {
				if (x>loc.at(i)->x() && x<loc.at(i)->x()+data.at(i)->xSize()*scale &&
					y>loc.at(i)->y() && y<loc.at(i)->y()+data.at(i)->ySize()*scale) break;
			}
			if (i==loc.count()||i==mim) return;
			emit probe(NULL,mim,(int)pxtodx(x,mim),(int)pytody(y,mim),0,-1);
		}

	}
	return;
}

mdx=e->x();
mdy=e->y();

}

void ImageMx::viewportMouseMoveEvent(QMouseEvent *e) {
int x,y,sz,nx;
float *d,az,r,a=0,c=0,scale;

x=e->x()+orx;
y=e->y()+ory;
if (loc.count()==0) return;

if ((e->state()&LeftButton) ) {
	if (!(e->state()&ShiftButton)) {
		if (mode<=0) {
			if (mim<0||mim>=loc.count()) emit(mouseMove(e,mim,x,y));
			else emit(mouseMove(e,mim,x-loc.at(mim)->x(),y-loc.at(mim)->y()));
		}
		else if (mode==4 && mim>=0) {
			unsigned char mx[49];
			EMData *d=data.at(mim);
			float mm=-d->Min()<d->Max()?d->Max():-d->Min();
			float b=mm*brite*2.0;
			mm*=(1.0-sqrt(cont));
			memset(mx,0,49);
			d->renderAmp8(mx,(int)pxtodx(x,mim)-3,(int)pytody(y,mim)-3,7,7,7,1.0,invert?254:1,invert?1:254,-mm-b,mm-b);
			emit probe(mx,mim,(int)pxtodx(x,mim),(int)pytody(y,mim),d->valueAtSafe((int)pxtodx(x,mim),(int)pytody(y,mim)),0);
//			d->renderAmp8(mx,x-loc.at(mim)->x()-3,y-loc.at(mim)->y()-3,7,7,7,1.0,invert?254:1,invert?1:254,-mm-b,mm-b);
//			emit probe(mx,mim,x-loc.at(mim)->x(),y-loc.at(mim)->y(),d->valueAt(x-loc.at(mim)->x(),y-loc.at(mim)->y()));
		}
	}
	return;
}
else if (e->state()&RightButton) {
}
else {
}

}

void ImageMx::removeData(EMData *d) {
data.remove(d);
loc.clear();
emit imsChanged(&data);
viewport()->update();
}

void ImageMx::removeData(int n) {
data.remove(n);
loc.clear();
emit imsChanged(&data);
viewport()->update();
}

void ImageMx::buildMx() {
int x=0,y=0,ch=0,w=0,h=0,lw=viewport()->width();
EMData *cur;

//printf("buildmx %d\n",data.count());
loc.clear();
if (data.count()==0) { resizeContents(50,50); return; }

for (cur=data.first(); cur; cur=data.next()) {
	if (x+cur->xSize()*scale>lw && x!=0) {
		x=0;
		y+=ch+2;
		ch=0;
	}
	loc.append(new BPoint(x,y));
	if (cur->ySize()*scale>ch) ch=cur->ySize()*scale;
	if (ch+y>h) h=ch+y;
	if (w<x+cur->xSize()*scale) w=x+cur->xSize()*scale;
	x+=cur->xSize()*scale+2;	
}

resizeContents(w,h);
setContentsPos(orx,ory);
}

void ImageMx::closeEvent ( QCloseEvent * e ) {
if (autodel) delete this;
else e->ignore();
}

void ImageMx::resizeEvent ( QResizeEvent *e ) {
QScrollView::resizeEvent(e);
buildMx();
}
