// alignhelix.C  Copyright 2001 Baylor College of Medicine
// 2/17/01

//N alignhelix
//F This will take a set of filament 'particles' and align them so they are vertical and centered
//T LA
//1
//P <input file>	Images to be processed
//P <output file>	Processed output
//P [trans]	Centers the particle horizontally
//P [rot]	Rotates the particle so it is as vertical as possible
//P [htov]	Rotates horizontal particles 90 degrees
//P [unbend]	Try to unbend the particles so they are truely vertical
//P [clip=<width>]	Clip the width of the centered images to <width> pixels
//P [despot]	Applies a bright-spot filter to the image
//U alignhelix start.hed aligned.hed rot trans
//D WARNING: This program is still experimental
//D This program will take a set of filament 'particles', and align them so they are horizontally
//E centered and vertical in the box. The particles should span, or almost span, the box vertically.
//E the box need not be square, but it should at least be a good size for FFT's.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i, j, n, trans=0,rot=0,htov=0,unbend=0,clip=-1,despot=0;
float dang;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"trans",5)==0) trans=1; 
	else if (strncmp(argv[i],"rot",3)==0) rot=1; 
	else if (strncmp(argv[i],"htov",4)==0) htov=1; 
	else if (strncmp(argv[i],"unbend",6)==0) unbend=1;
	else if (strncmp(argv[i],"despot",6)==0) despot=1;
	else if (strncmp(argv[i],"clip=",5)==0) clip=atoi(&argv[i][5]); 
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

if (htov) error(ERR_EXIT,"Sorry, htov not implemented yet");
if (unbend) error(ERR_EXIT,"Sorry, unbend not implemented yet");


n=fileCount(argv[1]);

EMData mask;
mask.readImage(argv[1],0);
mask.one();
float *d=mask.getData();
int w;
if (clip>0) w=(mask.xSize()-clip)/4;
else w=mask.xSize()/8;
for (i=0; i<w; i++) {
	for (j=0; j<mask.ySize(); j++) {
		d[i+j*mask.xSize()]=0;
		d[(mask.xSize()-i-1)+j*mask.xSize()]=0;
	}
}
mask.doneData();

for (i=0; i<n; i++) {
	EMData orig;
	
	printf("%d. ",i);
	
	orig.readImage(argv[1],i);
	int nx = orig.xSize(),ny = orig.ySize();
	orig.normalize();

	// This will run a filter to remove bright spots outside 80% of the helix radius
	if (despot) {
		float *d=orig.getData();
		int x1,y1,x2,y2;
		for (y1=1; y1<ny-1; y1++) {
			for (x1=1; x1<nx-1; x1++) {
				if (clip>0 && x1>nx/2-clip*.4 && x1<nx/2+clip*.4) continue;
				int ng=0;
				for (y2=y1-2; y2<=y1+2; y2++) {
					for (x2=x1-2; x2<=x1+2; x2++) {
						if (d[x2+y2*nx]==0 || d[x2+y2*nx]>1.0) ng++;
					}
				}
				if (ng==25) {
					for (y2=y1-1; y2<=y1+1; y2++) {
						for (x2=x1-1; x2<=x1+1; x2++) {
							d[x2+y2*nx]=0;
						}
					}		
				}
			}
		}
		orig.doneData();
	}
	orig.mult(&mask);

	// normalize the image based on the left and right edges only
	orig.normalize();
	int y;
	float av=0;
	for (y=0; y<ny; y++) av+=orig.valueAt(0,y)+orig.valueAt(1,y)+orig.valueAt(nx-1,y)+orig.valueAt(nx-2,y); 
	av/=(ny*4);
	orig.addConst(-av);
	
	// horizontal centering
	if (trans) {
		orig.transAlign(NULL,0,1,nx/3);
		orig.setTAlign(orig.Dx(),0);
		printf("dx = %d\t",(int)orig.Dx());
		orig.fastTranslate();
	}
		
	// rotation
	if (rot) {
		EMData *cop = orig.copy();
		EMData *cop2;
		float a, da = atan(1.0/(float)ny),dot,bdot=0,ba=0;
		for (a=-10.0*PI/180.0; a<10.0*PI/180.0; a+=da) {
			cop->setRAlign(a);
			cop->rotateAndTranslate();
			cop2=cop->copy(0,0);
			cop2->hFlip();
			dot=cop2->dot(cop);
			delete cop2;
			if (dot>bdot) { bdot=dot; ba=a; }
		}
		printf("rot = %1.1f",ba*180.0/PI);
		orig.setRAlign(ba);
		orig.rotateAndTranslate();
		delete cop;
	}
	printf("\n");
	if (clip>0) {
		EMData *tmp=orig.clip((nx-clip)/2,0,clip,ny);
		tmp->writeIMAGIC(argv[2],-1);
		delete tmp;
	}
	else orig.writeIMAGIC(argv[2],-1);
}

}
