// classhelix.C  Copyright 2001 Baylor College of Medicine
// 2/17/01

//N classhelix
//F This will take a set of filament 'particles' and generate a class-average
//T LA
//1
//P <input file>	Images to be processed
//P <output file>	Processed output
//P [ref=<n>,<start file>]	Image to reference alignments to (must be twice the height of <input> images)
//P [keep=<sigmult>]	Sigma threshold for keeping particles
//P [hard=<hard thr>]	A fixed threshold for particle inclusion
//P [first=<n>]	Limit the range of 'particles' to use.
//P [last=<n>]	Limit the range of 'particles' to use.
//U classhelix centered.hed avg.hed keep=1
//D Warning: This program is still experimental
//D This generates a class-average of helix particles. This differs from
//E programs like <a href=classalign2.html>classalign2</a> in the allowable
//E translations/roations. In this program, only vertical translation is
//E allowed for the alignment. In addition, the final result is twice the
//E length of a single 'particle'. 

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i, n, kp=0,first=0,last=-1;
float keep=10.0,hard=0;
EMData *ref = NULL;


appinit(argc,argv);

if (argc<3) appusage(argv[0]);

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"keep=",5)==0) keep=atof(&argv[i][5]); 
	else if (strncmp(argv[i],"ref=",4)==0) {
		int refn;
		char fsp[1024];
		sscanf(argv[i]+4,"%d,%s",&refn,fsp);
		ref=new EMData;
		if (ref->readImage(fsp,refn)) { delete ref; ref=NULL; error(ERR_CRIT,"Failed to read ref image"); }
	}
	else if (strncmp(argv[i],"hard=",5)==0) hard=atof(&argv[i][5]); 
	else if (strncmp(argv[i],"first=",6)==0) first=atoi(&argv[i][6]); 
	else if (strncmp(argv[i],"last=",5)==0) last=atoi(&argv[i][5]); 
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

if (keep!=10.0) error(ERR_EXIT,"Sorry, sigma keep not implemented.");

n=fileCount(argv[1]);
if (last==-1) last=n-1;
EMData orig,onemap;
orig.readImage(argv[1],0);
int nx = orig.xSize(),ny = orig.ySize();

onemap.setSize(nx,ny);	// used for normalization
onemap.one();

EMData *avg,norm,sum,*tmp;
norm.setSize(nx,ny*2,1);
sum.setSize(nx,ny*2,1);

norm.zero();
sum.zero();

if (!ref) {
	tmp=orig.clip(0,-ny/2,nx,ny*2);
	sum.add(tmp);
	delete tmp;

	tmp=onemap.clip(0,-ny/2,nx,ny*2);
	norm.add(tmp);
	delete tmp;

	avg=sum.copy();
	avg->mult(&norm,1);
}
else avg=sum.copy();

if (!ref&&first==0) first=1;

for (i=first; i<=last; i++) {
	orig.readImage(argv[1],i);
	int dy,x,y,by=0;
	float bd=1.0e12;
	for (dy=0; dy<ny; dy++) {
		float df=0,no=0; 
		for (y=0; y<ny; y++) {
			for (x=0; x<nx; x++) {
				if (ref) {
					df+=SQR(ref->valueAt(x,y+dy)-orig.valueAt(x,y));
					no++;
				}
				else {
					df+=SQR(avg->valueAt(x,y+dy)-orig.valueAt(x,y))*norm.valueAt(x,y+dy);
					no+=norm.valueAt(x,y+dy);
				}
			}
		}
		df/=no;
		if (df<bd) { bd=df; by=dy; }
	}
	printf("%d. %d (%1.2f)",i,by,bd);

	if (hard&&bd>hard) { printf("*\n"); continue; }

	printf("\n");
	kp++;
	
	tmp=orig.clip(0,-by,nx,ny*2);
	sum.add(tmp);
	delete tmp;

	tmp=onemap.clip(0,-by,nx,ny*2);
	norm.add(tmp);
	delete tmp;

	if (!ref) {
		delete avg;
		avg=sum.copy();
		avg->mult(&norm,1);
	}
}
if (ref) {
	delete avg;
	avg=sum.copy();
	avg->mult(&norm,1);
}

avg->edgeNormalize();
avg->writeIMAGIC(argv[2],-1);

printf("Used %d/%d images\n",kp,last-first+1);
}
