// quarter.C Copyright 1999 Steve Ludtke
// This is a program to take top views of a homotetrameric molecule
// and align them to a reference, then split each into 4 'identical'
// pieces for statistical analysis

#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <math.h>
#include "List.h"

#include "EMData.h"

main(int argc, char** argv)
{
int i,j,mask=-1,n,ny,ms;
EMData *ref,*im,*tmp,*tmpc,*q,*msk;

if (argc<3) { 
	printf("Quarter top views of homotetramers\nUsage: quarter <input images> <top reference> [mask=<rad>]\n");
	exit(0); 
}

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

ref=new EMData;
ref->readImage(argv[2],0);
ref->normalize();
ny=ref->ySize();
if (mask>0) ms=ny/2-mask; else ms=0;
if (mask<=0) mask=ny/2;
msk=new EMData;
msk->setSize(ny,ny,1);
msk->one();
msk->applyMask(mask,4);

n=fileCount(argv[1]);
im=new EMData;

for (i=0; i<n; i++) {
	if (i%10==0) { printf(" %d\r",i); fflush(stdout); }
	im->readImage(argv[1],i);
	im->normalize();
	tmpc=im->RTFAlign(ref,NULL,1,ms);
	tmpc->refineAlign(ref,1);
	tmpc->mult(msk);
	tmp=tmpc->copy(0,0);
	for (j=0; j<4; j++) {
		tmp->setRAlign(j*PI/2,0,0);
		tmp->setTAlign(0,0,0);
		tmp->rotateAndTranslate();
		q=tmp->clip(ms+1,ms,mask,mask);
		q->writeIMAGIC("quarter.hed",-1);
		delete q;
	}
	delete tmpc;
	delete tmp;
}

}

