// tiltseries.C  Copyright 2001 Baylor College of Medicine
// 1/29/01
// This program will take a tilt series of the same particle and
// align the individual images and set Euler angles

//N tiltseries
//F Aligns a set of images from a tilt series and assigns Eulers
//T LT
//1
//P <input file>	Images to be processed
//P <output file>	Processed output
//P <dang>	Angular step between images
//P [indccl]	Writes individual CCL images to ccl.hed
//U tiltseries particle.hed particle.tilt.hed 10
//D This program is still somewhat experimental
//D This proram is used for analysis of tilt-series, ie - tomography.
//E This program will take a tilt series of a single particle, mutually align them,
//E determine the tilt axis, and write the aligned images to a new file with Eulers.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i, indccl=0;
float dang;

appinit(argc,argv);

if (argc<4) appusage(argv[0]);

for (i=4; i<argc; i++) {	
	if (strncmp(argv[i],"indccl",5)==0) { 
		indccl=1;
		unlink("ccl.hed");
		unlink("ccl.img");
	}
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

dang=atof(argv[3]);
if (dang==0||fabs(dang)>70.0) { dang=10.0; error(ERR_WARN,"Illegal angle. Set to 10 degrees.\n"); }

List *data1;

data1=readImages(argv[1],-1,-1);

printf("Aligning particles\n");
for (i=1; i<data1->count(); i++) {
	EMData *ob1,*ob2;
	ob1=(EMData *)data1->objectAt(i-1);
	ob2=(EMData *)data1->objectAt(i);
	
	ob2->transAlign(ob1,0,1,ob1->xSize()/8);
	ob2->fastTranslate();
	ob2->edgeNormalize();
	if (i==1) ob1->edgeNormalize();
}

EMData common,*sum = NULL;

printf("Calculating common lines\n");
int j;
for (i=0; i<data1->count()-1; i++) {
	for (j=i+1; j<data1->count(); j++) {
		EMData *ob1,*ob2;
		ob1=(EMData *)data1->objectAt(i);
		ob2=(EMData *)data1->objectAt(j);

		common.commonLines(ob1,ob2,0,180);
		if (indccl) common.writeIMAGIC("ccl.hed",-1);
		if (!sum) sum=common.copy(0,0);
		else sum->add(&common);	
	}
}
sum->multConst(1.0/(data1->count()));
sum->writeIMAGIC("ccl.hed",-1);

float b=0;
int bi=0;
for (i=0; i<360; i++) {
	if (sum->valueAt(i,i)>b) { b=sum->valueAt(i,i); bi=i; }
}
printf("Common line at %d degrees\n",bi);

for (i=0; i<data1->count(); i++) {
	EMData *ob1;
	ob1=(EMData *)data1->objectAt(i);
	ob1->setRAlign(PI/2.0-(float)bi*PI/180.0);
	ob1->rotateAndTranslate();
	ob1->setRAlign((float)(i-data1->count()/2)*dang*PI/180.0,0,0);
	ob1->writeIMAGIC(argv[2],-1);
}

}

