#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

#define MINMEM	8		// minimum members in a class to keep it
#define WRITEAV			// this will write out all of the images in each class

struct DOT { float d; unsigned short i1,i2; };

inline int sgn(float x) { if (x==0) return 0; return x<0?-1:1; }

int dotcomp(const void *a, const void *b) { return sgn(*((float *)b)- *((float *)a)); }

main(int argc, char *argv[]) {
int i,j,k,l,m,mm,mn,goal,c,cc,n,n2,nx,final;
float adot,a1,a2,mean,sigma,m2,s2;
float **f;
int *u;
char s[120],t[120];
FILE *in,*out;
EMData *d1,*d2,*d3;
List *toav,*ave;
DOT *dot;

if (argc!=4) { printf("classes.c\nUsage: classes <footprints> <output file> <# classes>\n"); exit(0); }

in=fopen(argv[1],"r");
if (!in) { printf("Cannot open footprints\n"); exit(0); }
fread(s,120,1,in);
fread(&n,4,1,in);
fread(&n2,4,1,in);
f=(float **)malloc(n2*sizeof(float *));

for (i=0; i<n2; i++) {
	f[i]=(float *)malloc(n*sizeof(float));
	fread(f[i],n,sizeof(float),in);
}
fclose(in);

out=fopen("classes.out","w");

final=atoi(argv[3]);

u=(int *)malloc(n*sizeof(int));
memset(u,0,n*sizeof(int));

dot=(DOT *)malloc((n-1)*(n-1)/2*sizeof(DOT));

// calculate all possible dot products
c=0;
for (i=0; i<n; i++) {
	for (j=i+1; j<n; j++) {		
		//	dot product
		adot=0;
		for (k=0; k<n2; k++) adot+=f[k][i]*f[k][j];

		if (adot<.2) continue;		// reduce amount of sorting required
		
		dot[c].d=adot;
		dot[c].i1=i;
		dot[c].i2=j;
		c++;
	}
	printf("%d complete\r",i);
	fflush(stdout);
}

printf("%d/%d (%d min required) dots exceed threshold\n",c,(n-1)*(n-1)/2,n-final);
if (c<n-final) printf("WARNING: insufficient number of dots, reduce threshold\n");

qsort(dot,c,sizeof(DOT),dotcomp);

//for (i=0; i<20; i++) printf("%f\t%d\t%d\n",dot[i].d,dot[i].i1,dot[i].i2);

// u is an array which will map images into classes
// any images with the same value in u will be averaged (except 0)
cc=1;
goal=n-final;
for (i=0; i<goal; i++) {
	j=dot[i].i1;
	k=dot[i].i2;

	if (i>c) { printf("Too many dots reqd.\n"); break; }

	if (u[j]==0 && u[k]==0) {			// this is a new class
		u[j]=u[k]=cc;
		cc++;
	}
										// this doesn't reduce the number of classes
	else if (u[j]==u[k] || u[j]<0 || u[k]<0) goal++;		
	else if (u[j]!=0 && u[k]!=0) {		// combine 2 classes
		m=u[j]>u[k]?u[j]:u[k];			// use the smaller of the 2 class numbers
		mm=u[j]>u[k]?u[k]:u[j];
		for (l=0; l<n; l++) if (u[l]==m) u[l]=mm;

// limit size of classes
		for (l=m=0; l<n; l++) if (u[l]==mm) m++;
		if (m>n/final) {
			for (l=m=0; l<n; l++) if (u[l]==mm) u[l]= -u[l];
		}
	}
	else if (u[j]==0) u[j]=u[k];		// add to existing class
	else u[k]=u[j];
}
printf("%d duplicates\n",goal-n+final);

// count unaveraged images
m=0;
for (i=0; i<n; i++) {
	u[i]= abs(u[i]);
	if (!u[i]) m++;
}
printf("%d unclassed images, %d good classes\n", m,final-m);

// get the image size for later use
d1=new EMData;
if (d1->readImage(s,0)) { printf("Error reading image '%s'\n"); exit(1); }
nx=d1->xSize();
delete d1;

// count images in each class
l=0;
toav=new List;
for (i=1; i<cc; i++) {				// loop through all possible class numbers
	k=0;
	for (j=0; j<n; j++) {			// read images in this class
		if (u[j]==i) {
			k++;
			if (k==1) fprintf(out,"Class %d\n",i);
			d1=new EMData;
			if (d1->readImage(s,j)) { printf("Error reading image '%s'\n"); exit(1); }
			d1->unum=j;
			toav->addObject(d1);
			fprintf(out,"\t%d\n",j);
		}
	}

	// align and average class members, drop classes with less than MINMEM members
	if (k>=MINMEM) { 
		l++; 
		printf("Class %d(%d) has %d members\n",l,i,k); 

// iteration #1
		d1=(EMData *)toav->objectAt(0);
		d1=d1->copy();
		d1->setTAlign(0,0,0);
		d1->setRAlign(0,0,0);
		d1->setParent(NULL);
		for (j=1; j<toav->count(); j++) {
			d2=(EMData *)toav->objectAt(j);
			d3=d2->RTFAlign(d1);
			if (d3) {
				d1->add(d3);
				delete d3;
			}
			putchar('.');
			fflush(stdout);
		}
		putchar('\n');
		d1->normalize();
		d1->transAlign(NULL);			// center the average
		d1->rotateAndTranslate();

// iteration #2
		d3=(EMData *)toav->objectAt(0);
		d3=d3->RTFAlign(d1);
		if (!d3) { 
			d3=((EMData *)toav->objectAt(0))->copy(); 
			printf("Alignment error\n"); 
		}
		d3->setParent(NULL);

		mean=sigma=0;
		m=1;
		for (j=1; j<toav->count(); j++) {
			d2=(EMData *)toav->objectAt(j);
			d2=d2->RTFAlign(d1);
			if (d2) {
				a1=d2->dot(d1);
				mean+=a1;
				sigma+=a1*a1;
				m++;
				d3->add(d2);
				delete d2;
			}
			putchar('.');
			fflush(stdout);
		}
		mean/=(float)m;
		sigma=sqrt(sigma/(float)m-mean*mean);
		putchar('\n');
		d3->normalize();
		delete d1;
		d1=d3;
		d1->transAlign(NULL);			// center the average
		d1->rotateAndTranslate();
		printf("Mean=%f Sigma=%f\n",mean,sigma);

// iteration #3
		d3=(EMData *)toav->objectAt(0);
		d3=d3->RTFAlign(d1);
		if (!d3) { 
			d3=((EMData *)toav->objectAt(0))->copy(); 
			printf("Alignment error\n"); 
		}
		d3->setParent(NULL);

		mean=sigma=0;
		m=1;
		for (j=1; j<toav->count(); j++) {
			d2=(EMData *)toav->objectAt(j);
			d2=d2->RTFAlign(d1);
			if (d2) {
				a1=d2->dot(d1);
				mean+=a1;
				sigma+=a1*a1;
				m++;
				d3->add(d2);
				delete d2;
			}
			putchar('.');
			fflush(stdout);
		}
		mean/=(float)m;
		sigma=sqrt(sigma/(float)m-mean*mean);
		putchar('\n');
		d3->normalize();
		delete d1;
		d1=d3;
		d1->transAlign(NULL);			// center the average
		d1->rotateAndTranslate();
		printf("Mean=%f Sigma=%f\n",mean,sigma);

// iteration #4
		d3=(EMData *)toav->objectAt(0);
		d3=d3->RTFAlign(d1);
		if (!d3) { 
			d3=((EMData *)toav->objectAt(0))->copy(); 
			printf("Alignment error\n"); 
		}
		d3->setParent(NULL);

#ifdef WRITEAV
		sprintf(t,"class%03d.hed",l);
		unlink(t);
		d3->writeIMAGIC(t,-1);			
#endif

		m2=mean;
		s2=sigma;
		mean=sigma=0;
		m=1;
		for (j=1; j<toav->count(); j++) {
			d2=(EMData *)toav->objectAt(j);
			d2=d2->RTFAlign(d1);
			if (d2) {
				a1=d2->dot(d1);
				printf("\t%1.4f %c\n",a1,a1>(m2-s2)?' ':'*');
				if (a1>m2-s2) {
					d3->add(d2);		// only keep images that match well
					mean+=a1;
					sigma+=a1*a1;
					m++;
				}
#ifdef WRITEAV
				if (a1>m2-s2) d2->writeIMAGIC(t,-1);
#endif
				delete d2;
			}
		}
		mean/=(float)m;
		sigma=sqrt(sigma/(float)m-mean*mean);
		printf("Mean=%f Sigma=%f  with %d good\n",mean,sigma,m);
		d3->normalize();
					
//		d1->writeIMAGIC(argv[2],-1);
		d3->writeIMAGIC(argv[2],-1);
		delete d1;
		delete d3;
	}


	// clean up
	for (j=0; j<toav->count(); j++) delete ((EMData *)toav->objectAt(j));
	toav->empty();
}
delete toav;
printf("%d good classes (%d desired)\n",l,final);

fclose(out);
// now take the assignments in u and construct the individual classes

}
