/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/
void Control::init()
{
databig=NULL;	// Overall micrograph which is boxed out to produce data
data=NULL;	// FFT of region to process as a 2D crystal fragment
datar=NULL;	// real space version
dataavg=NULL;	// data representing processed average from datar
dataref=NULL;	// This is a reference image used for more accurate processing

// Image objects corresponding to the EMData objects above
bigview=NULL;
imview=NULL;
imviewr=NULL;
avgview=NULL;
refview=NULL;

clipsize=512;
cliprefsize=48;
mx[0]=1.0;
mx[1]=0;
mx[2]=0;
mx[3]=1.0;
boxcen[0]=boxcen[1]=256;
refboxcen[0]=refboxcen[1]=0;
}

// This is for reading scanned diffraction mode images
void Control::fileOpenDif()
{
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;
//QFile file(s);

QApplication::setOverrideCursor( waitCursor );

if (data) delete data;
data=new EMData;
data->readImage(s.data(),0);
datar=data->doIFT();
data->gimmeFFT();
newData();
//imResized(imview->width(),imview->height());

QApplication::restoreOverrideCursor( );

}

// This reads real-space images (and does an FFT)
void Control::fileOpen()
{
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;
//QFile file(s);

QApplication::setOverrideCursor( waitCursor );
if (!databig) databig=new EMData();
databig->readImage((const char *)s,0);
if (databig->isComplex()) {
    QMessageBox::warning ( NULL, "No FFTs", "Sorry, qindex currently demands the right to perform its own FFTs\nPlease load a real-space image.", 1,0,0 );
    delete databig;
    databig=NULL;
    return;
}
databig->normalize();

/*
if (data) delete data;

// We cannot tolerate odd image sizes
if (datar->xSize()%2!=0 || datar->ySize()%2!=0) {
    EMData *d3=datar;
    datar=d3->clip(0,0,d3->xSize()-(d3->xSize()%2),d3->ySize()-(d3->ySize()%2));
}    
data=datar->doFFT();
datar->gimmeFFT();
*/
newData();

//imResized(imview->width(),imview->height());

QApplication::restoreOverrideCursor( );

}

void Control::fileExit()
{
exit(0);
}

void Control::helpIndex()
{
About *abo = new About();
abo->show();
}

void Control::helpContents()
{
About *abo = new About();
abo->show();
    }

void Control::helpAbout()
{
About *abo = new About();
abo->show();
}

void Control::newData()
{
/*    if (!data) {
	datar=databig->clip(0,0,512,512);
	data=datar->doFFT();
	datar->gimmeFFT();
    }*/
    
    if (!imview) {
	imview=new Image();
	imviewr=new Image();
	connect(imview,SIGNAL(mousePress(QMouseEvent *)),this,SLOT(imageMouseSel(QMouseEvent *)));
	connect(imview,SIGNAL(mouseMove(QMouseEvent *)),this,SLOT(imageMouseSelMove(QMouseEvent *)));
    }
    bigMouseRel(NULL);
 
    imview->setCaption("Fourier space region view");
    imview->resize(data->xSize()<512?data->xSize():512,data->ySize()<512?data->ySize():512);
    imview->setScale(1.0);
    imview->setBC(.35,.5);
    imview->show();
	    
    imviewr->setCaption("Real space region view");
    imviewr->setScale(1.0);
    imviewr->resize(datar->xSize()<512?datar->xSize():512,datar->ySize()<512?datar->ySize():512);
    imviewr->show();    
    
    if (!bigview) {
	bigview=new Image();
	connect(bigview,SIGNAL(resized(int,int)),SLOT(imResized(int,int)));
	connect(bigview,SIGNAL(moved(int,int)),SLOT(imMoved(int,int)));
	connect(bigview,SIGNAL(keyPress(QKeyEvent *)),this,SLOT(bigKeySel(QKeyEvent *)));	
	connect(bigview,SIGNAL(mousePress(QMouseEvent *)),this,SLOT(bigMouseSel(QMouseEvent *)));	
	connect(bigview,SIGNAL(mouseRelease(QMouseEvent *)),this,SLOT(bigMouseRel(QMouseEvent *)));
	connect(bigview,SIGNAL(mouseMove(QMouseEvent *)),this,SLOT(bigMouseSelMove(QMouseEvent *)));
	
	ImShape *bx=new ImShape();
	bx->x0=boxcen[0]-clipsize/2;
	bx->y0=boxcen[1]-clipsize/2;
	bx->x1=clipsize;
	bx->y1=clipsize;
	bx->color=Qt::green;
	bx->type=ImShape::BOX;
	bigview->addShape(bx);
	bigview->setCursor(Qt::CrossCursor);
    }
    
    bigview->setCaption("Full Micrograph View (left-drag to select region)");
    bigview->setData(databig);
    bigview->resize(databig->xSize()<800?databig->xSize():800,databig->ySize()<800?databig->ySize():800);
    bigview->setScale(1.0);
    bigview->show();
    
    
    // Display the image size
    QString s;
    s.sprintf("%0d x %0d",databig->xSize()-2,databig->ySize());	    
    imdimW->setText(s);
    
    newUCP();
}

// New unit cell parameters
void Control::newUCP()
{
if (!data) return;
float angh=atof((const char *)anghW->text())*PI/180.0;
float angk=atof((const char *)anghkW->text())*PI/180.0+angh;
float cellh=fabs(atof((const char *)cellhW->text()));
float cellk=fabs(atof((const char *)cellkW->text()));
float ap=atof((const char *)apixW->text());

if (ap<=0) return;	// invalid parameters, keep the old ones

apix=ap;
mx[0]=cos(angh)/cellh;	// h dx
mx[1]=sin(angh)/cellh;	// h dy
mx[2]=cos(angk)/cellk;	// k dx
mx[3]=sin(angk)/cellk;	// k dy

float d=ap*(mx[0]*mx[3]-mx[2]*mx[1]);
rmx[0]= mx[3]/d;
rmx[2]=-mx[1]/d;
rmx[1]=-mx[2]/d;
rmx[3]= mx[0]/d;

//rmx[0]=cos(angh-PI/2.0)*cellk/ap;	// h dx
//rmx[1]=sin(angh-PI/2.0)*cellk/ap;	// h dy
//rmx[2]=cos(angk-PI/2.0)*cellh/ap;	// k dx
//rmx[3]=sin(angk-PI/2.0)*cellh/ap;	// k dy

regrid();
}

#define CIRSIZE 7
void Control::regrid()
{
    if (!data) return;
    imview->clearShapes();
    imviewr->clearShapes();
   
    // These are how much s changes per pixel
    dsy=1.0/(float)(apix*data->ySize());
    if (data->isComplex()) dsx=1.0/(float)(apix*(data->xSize()-2));
    else dsx=1.0/(float)(apix*data->xSize());
    
    int xc=data->xSize()/2;
    int yc=data->ySize()/2;
    QColor color(24,24,192);
    QColor color2(64,192,192);
    QColor color3(64,192,192);
    ImShape *ts;
    int h,k,hs,ks;
    for (hs=-1; hs<2; hs+=2) {
	for (ks=-1; ks<2; ks+=2) {
	    for (h=0; h<100; h++) {
		int got1=0;
		for (k=0; k<100; k++) {
		    int x=xc+round((hs*h*mx[0]+ks*k*mx[2])/dsx);
		    int y=yc+round((hs*h*mx[1]+ks*k*mx[3])/dsy);
		    if (x+CIRSIZE>=data->xSize() || x<0 || y+CIRSIZE>=data->ySize() || y<0) continue;
		    got1=1;
	    
		    ts=new ImShape;
		    ts->type=ImShape::CIRCLE;
		    ts->x0=x-CIRSIZE/2-1;
		    ts->y0=y-CIRSIZE/2;
		    ts->x1=ts->y1=CIRSIZE;
		    if (h==0 && k==0) ts->color=color;
		    else if (h==0) ts->color=color2;
		    else if (k==0) ts->color=color3;
		    else ts->color=color;
		    ts->user=h*hs;
		    ts->user2=k*ks;
		    imview->addShape(ts);
		}
		if (!got1) break;
	    }
	}
    }
    
//    float angh=atof((const char *)anghW->text())*PI/180.0;
//    float angk=atof((const char *)anghkW->text())*PI/180.0+angh;
    float cellh=atof((const char *)cellhW->text());
    float cellk=atof((const char *)cellkW->text());
//    float ap=atof((const char *)apixW->text());
    
    for (h=-100; h<100; h++) {
//	if (fabs(h*cellh)>datar->ySize()) continue;
	ts=new ImShape;
	ts->type=ImShape::LINE;
	ts->x0=h*rmx[0]-100.0*rmx[2];
	ts->y0=h*rmx[1]-100.0*rmx[3];
	ts->x1=h*rmx[0]+100.0*rmx[2];
	ts->y1=h*rmx[1]+100.0*rmx[3];
	ts->color=color;
	imviewr->addShape(ts);
    }
    for (k=-100; k<100; k++) {
//	if (fabs(k*cellk)>datar->ySize()) continue;
	ts=new ImShape;
	ts->type=ImShape::LINE;
	ts->x0=k*rmx[2]-100.0*rmx[0];
	ts->y0=k*rmx[3]-100.0*rmx[1];
	ts->x1=k*rmx[2]+100.0*rmx[0];
	ts->y1=k*rmx[3]+100.0*rmx[1];
	ts->color=color;
	imviewr->addShape(ts);
    }
    imviewr->update();
    
    hilight();
 }


void Control::hilight()
{
int i,h,k;
h=inthW->value();
k=intkW->value();
    
    for (i=0; i<imview->nShapes(); i++) {
	ImShape *sh=imview->shapeAt(i);
	if (sh->user==h && sh->user2==k) {
	    imview->hilight(i); 
	    break; 
	}
    }
    
    float x=(h*mx[0]+k*mx[2]);
    float y=(h*mx[1]+k*mx[3]);
        
    if (x==0 || y==0) {
	peakRW->setText(QString("-"));
	peakAW->setText(QString("-"));
    }
    else {
	peakRW->setText(QString::number(1/hypot(x,y),'f',2));
	peakAW->setText(QString::number(atan2(y,x)*180./PI,'f',2));
    }
    
    if (data->isComplex()) {
	int imx=round((h*mx[0]+k*mx[2])/dsx)*2;
	int imy=round((h*mx[1]+k*mx[3])/dsy);
	if (imx<0) { imx=-imx; imy=data->ySize()/2-imy; }
	else { imy+=data->ySize()/2; }

	float sum=0;
	int X,Y;
	for (Y=imy-3; Y<=imy+3; Y++) {
	    for (X=imx-6; X<=imx+6; X+=2) {
		sum+=SQR(data->valueAtSafe(X,Y));
	    }
	}
	sum*=data->xSize()*data->ySize()/49.0;
	peakIW->setText(QString::number(sum,'g',3));
    }
   imview->update();
}




void Control::imageMouseMove(QMouseEvent *e)
{
float x=(imview->pxtodx(e->x())-data->xSize()/2)*dsx;
float y=(imview->pytody(e->y())-data->ySize()/2)*dsy;
if (x==0 && y==0) {
    imview->setCaption("Cursor: 0, 0");
    return;
}

QString s;
s.sprintf("Cursor: %d, %d -> %1.2f A^-1 %1.2f deg",(int)round(x),(int)round(y),1.0/hypot(x,y),atan2(y,x));

imview->setCaption(s);
}


void Control::imageMouseSel( QMouseEvent *e )
{
int i,j=0;
float x=imview->pxtodx(e->x());
float y=imview->pytody(e->y());

    for (i=0; i<imview->nShapes(); i++) {
	ImShape *sh=imview->shapeAt(i);
	
	if (x>sh->x0 && y>sh->y0 && x<sh->x0+sh->x1 && y<sh->y0+sh->y1) {
	    inthW->setValue(sh->user);
	    intkW->setValue(sh->user2);
	    j=1;
	    break;
	}
    }

    if (!j) { 
	inthW->setValue(0);
	intkW->setValue(0);
    }
    
}

void Control::imageMouseSelMove( QMouseEvent *e )
{
int h=inthW->value();
int k=intkW->value();

float x=(imview->pxtodx(e->x())-data->xSize()/2)*dsx;
float y=(imview->pytody(e->y())-data->ySize()/2)*dsy;

float r=hypot(x,y);
float a=atan2(y,x)*180.0/PI;
float angh=atof((const char *)anghW->text());

if (h==0 && k!=0) {
    cellkW->setText(QString::number((float)k/r));
    anghkW->setText(QString::number(a-angh));
}

if (k==0 && h!=0) {
    float anghk=atof((const char *)anghkW->text());
    cellhW->setText(QString::number((float)h/r));
    anghW->setText(QString::number(a));
    anghkW->setText(QString::number(anghk-(a-angh)));
}

}


void Control::cricleOnOff()
{
    imview->clearShapes();
    imviewr->clearShapes();
    imview->update();
    imviewr->update();
}


void Control::panned(float x, float y)
{
//    printf("thsi function was call\n");
    bigview->setOrigin(x,y);
}
void Control::imResized(int w, int h)
{
 //   printf("thsi function was call\n");
    float xh,yh;
    float ls=imview->lscale;
    if (!ls) ls=1.0;

    if (databig->xSize()>databig->ySize())
	pan->resize(150,150*databig->ySize()/databig->xSize());
    else pan->resize(150*databig->xSize()/databig->ySize(),150);
    pan->setXRange(0,databig->xSize()*ls-w);
    pan->setYRange(0,databig->ySize()*ls-h);
    if (databig->xSize()*ls<w) xh=1.0;
    else xh=w/(databig->xSize()*ls);
    if (databig->ySize()*ls<h) yh=1.0;
    else yh=h/(databig->ySize()*ls);
    //printf("xh %f, yh: %f \n", xh, yh);
    pan->setHandleSize(xh,yh);
    pan->update();
}

void Control::imMoved( int x, int y)
{
    pan->setValue(x,y);
}



void Control::Average_clicked()
{
int h,k,it;
float ap=atof((const char *)apixW->text());
float cellh=atof((const char *)cellhW->text())/ap;
float cellk=atof((const char *)cellkW->text())/ap;
int sz=cellh>cellk?cellh*1.6:cellk*1.6;  
sz*=4;

EMData *avg,*oavg=NULL;
oavg=datar->clip(0,0,sz*2,sz*2);
oavg->transAlign(NULL,0,1,sz/8);
oavg->rotateAndTranslate();
avg=oavg->clip(sz/2,sz/2,sz,sz);
delete oavg;
oavg=NULL;
//unlink("clips.hed");
//unlink("clips.img");

QProgressDialog progress( "Averaging", 0, 5*400, this, 0 ,TRUE );

for (it=0; it<5; it++) {
    for (h=-200; h<200; h++) {
	progress.setProgress(200+h+it*400);
	for (k=-200; k<200; k++) {
	    int x=h*rmx[0]+k*rmx[1];
	    int y=h*rmx[2]+k*rmx[3];
	
	    if (x<0||y<0||x+sz>=datar->xSize()||y+sz>=datar->ySize()) continue;
	    EMData *tmp,*tmp2;
	    tmp=datar->clip(x,y,sz,sz);
//	    if (it==4) tmp->writeIMAGIC("clips.hed",-1);
	    if (it==0) {
//		tmp2=tmp->RTFAlign(avg);
		tmp2=tmp->copy();
		tmp2->transAlign(avg,0,1,-1);
		tmp2->rotateAndTranslate();
		avg->add(tmp2);
		delete tmp2;
	    }
	    else {
//		tmp2=tmp->RTFAlign(oavg);
		tmp2=tmp->copy();
		tmp2->transAlign(avg,0,1,-1);
		tmp2->rotateAndTranslate();
		avg->add(tmp2);
		delete tmp2;
	    }
	    delete tmp;
	}
    }
    printf("Iteration %d\n",it);
    if (oavg) delete oavg;
    oavg=avg->copy(0,0);
}
if (dataavg) delete dataavg;
dataavg=oavg->copy(0,0);
delete oavg;

if (!avgview) {
    avgview=new Image();
    avgview->setScale(1.0);
    avgview->setBC(.35,.5);
    connect(avgview,SIGNAL(mousePress(QMouseEvent *)),this,SLOT(avgMouseSel(QMouseEvent *)));
    connect(avgview,SIGNAL(mouseMove(QMouseEvent *)),this,SLOT(avgMouseSelMove(QMouseEvent *)));
}

dataavg->writeMRC("avg.mrc");
avgview->setData(dataavg);
avgview->show();
delete avg;
}

void Control::Fix_clicked()
{
databig->realFilter(91); // peak filter
bigview->update();
}


/*
void Control::Corr_clicked()
{
if (!dataavg) return;
    
int h,k,it;
float ap=atof((const char *)apixW->text());
float cellh=atof((const char *)cellhW->text())/ap;
float cellk=atof((const char *)cellkW->text())/ap;
int sz=cellh>cellk?cellh*1.4:cellk*1.4;  
sz*=2;

EMData *clip;
EMData *cmp=dataavg->clip(-sz/2,-sz/2,sz*2,sz*2);

for (h=-200; h<200; h++) {
	for (k=-200; k<200; k++) {
	    int x=h*rmx[0]+k*rmx[1];
	    int y=h*rmx[2]+k*rmx[3];
	
	    if (x<0||y<0||x+sz>=datar->xSize()||y+sz>=datar->ySize()) continue;
	    EMData *tmp;
	    clip=datar->clip(x-sz/2,y-sz/2,sz*2,sz*2);
	    tmp=clip->RTFAlign(cmp);
	    float v=tmp->lcmp(dataavg);
	    printf("%f\n",v);
	    delete clip;
	}
    }

}
*/

void Control::Corr_clicked()
{
if (!dataavg) return;
if (dataref) delete dataref;

// This is a clipped out area of the aligned average generated earlier
dataref=dataavg->clip(refboxcen[0],refboxcen[1],cliprefsize,cliprefsize);
//dataref->edgeNormalize();
dataref->normalize();

EMData *bigref=dataref->clip(-1024+cliprefsize/2,-1024+cliprefsize/2,2048,2048);	// reference in a big box
bigref->multConst(1.0/sqrt(bigref->SumSq()));

// Used to normalize
EMData *tmp=dataref->copyHead();
tmp->one();
EMData *bignorm=tmp->clip(-1024+cliprefsize/2,-1024+cliprefsize/2,2048,2048);
delete tmp;

// make a copy of the main image size to use for output
EMData *cp=databig->copyHead();
cp->zero();

for (int x=0; x<databig->xSize(); x+=2048-cliprefsize) {
//    if (x+2048-cliprefsize>=databig.xSize()) x=databig.xSize()-2048+cliprefsize;
    for (int y=0; y<databig->ySize(); y+=2048-cliprefsize) {
//	if (y+2048-cliprefsize>=databig.ySize()) y=databig.ySize()-2048+cliprefsize;
	EMData *clip=databig->clip(x,y,2048,2048);
	EMData *ccf=clip->calcCCF(bigref,1,NULL);
	
	clip->realFilter(18);	// clip -> clip^2
	EMData *ccf2=clip->calcCCF(bignorm,1,NULL);
	ccf2->realFilter(19);	// ccf2 -> sqrt(ccf2)
	ccf->mult(ccf2,1);		// ccf/=ccf2
	
	// clip off the edges of the CCF to get rid of edge effects
	int xsize=2048-cliprefsize;
	int ysize=2048-cliprefsize;
	if (x+2048>databig->xSize()) xsize=databig->xSize()-x-cliprefsize;
	if (y+2048>databig->ySize()) ysize=databig->ySize()-y-cliprefsize;
	if (ysize<=4||xsize<=4) continue;
	
//	printf("%d %d %d\n",cliprefsize/2,xsize,ysize);	
	EMData *ccfclip=ccf->clip(cliprefsize/2,cliprefsize/2,xsize,ysize);
	ccfclip->multConst(2048*2048);
//	printf("%d %d   %d %d\n",ccfclip->xSize(),ccfclip->ySize(),x+cliprefsize/2,y+cliprefsize/2);
	cp->insertClip(ccfclip,x+cliprefsize/2,y+cliprefsize/2);
	
	delete clip;
	delete ccf;
	delete ccf2;
	delete ccfclip;
    }
}
cp->realFilter(105,cliprefsize/2,cliprefsize/2);	// zero edges
//cp->realFilter(91); // peak filter

cp->update();
bigview->setData(cp);
delete databig;
databig=cp;
delete bigref;
delete bignorm;
}
    
void Control::bigMouseRel( QMouseEvent *e )
{
    if (data) delete data;
    if (datar) delete datar;
    datar=databig->clip(boxcen[0]-clipsize/2,boxcen[1]-clipsize/2,clipsize,clipsize);
    datar->edgeNormalize();
    data=datar->doFFT();
    data->ri2ap();
    datar->gimmeFFT();
    imview->setData(data);
    imviewr->setData(datar);
    
//    imview->update();
//    imviewr->update();
    imview->repaint(0);
    imviewr->repaint(0);
    
}

void Control::bigMouseSelMove( QMouseEvent *e )
{
    boxcen[0]=(bigview->pxtodx(e->x()));
    boxcen[1]=(bigview->pytody(e->y()));
    
    bigview->shapeAt(0)->x0=boxcen[0]-clipsize/2;
    bigview->shapeAt(0)->y0=boxcen[1]-clipsize/2;
    bigview->shapeAt(0)->x1=clipsize;
    bigview->shapeAt(0)->y1=clipsize;
    bigview->update();
    
    if (clipsize<=512 && mousedelay.elapsed()>250) {
	bigMouseRel(e);
	mousedelay.start();
    }
}


void Control::bigMouseSel( QMouseEvent *e )
{
    mousedelay.start();
    bigMouseSelMove(e);
}

void Control::newSize()
{
clipsize=regionsize->text().toInt();
bigMouseRel(NULL);
regrid();
}

void Control::newRefSize()
{
cliprefsize=refsize->text().toInt();
}


void Control::avgMouseSel( QMouseEvent *e )
{
avgMouseSelMove(e);
}


void Control::avgMouseSelMove( QMouseEvent *e )
{
    refboxcen[0]=(avgview->pxtodx(e->x()))-cliprefsize;
    refboxcen[1]=(avgview->pytody(e->y()));
    
    if (avgview->nShapes()<1) {
	ImShape *bx=new ImShape();
	bx->x0=0;
	bx->y0=0;
	bx->x1=cliprefsize;
	bx->y1=cliprefsize;
	bx->color=Qt::green;
	bx->type=ImShape::DOTBOX;
	avgview->addShape(bx);
	avgview->setCursor(Qt::CrossCursor);
    }
    
    avgview->shapeAt(0)->x0=refboxcen[0];
    avgview->shapeAt(0)->y0=refboxcen[1];
    avgview->shapeAt(0)->x1=cliprefsize;
    avgview->shapeAt(0)->y1=cliprefsize;
    avgview->update();
    
}

struct BOXLOC { float val; int x; int y; };

int BOXLOCcmp (const void *pa, const void *pb) {
	const BOXLOC * a = (BOXLOC *) pa;
	const BOXLOC * b = (BOXLOC *) pb;
	return (a->val<b->val)?1.0:(a->val>b->val)?-1:0; 
}

inline int maxhypot(float x,float y,float m) {
	if (fabs(x)>m || fabs(y)>m || hypot(x,y)>m) return 0;
	return 1;
}

void Control::box_clicked()
{
float thr=bigview->get128();
BOXLOC *bl=(BOXLOC *)malloc(128*sizeof(BOXLOC));
int i,j,bla=128,bln=0;

for (int y=0; y<databig->ySize(); y++) {
    for (int x=0; x<databig->xSize(); x++) {
		if (databig->valueAt(x,y)>thr) {
			bl[bln].val=databig->valueAt(x,y);
			bl[bln].x=x;
			bl[bln].y=y;
			bln++;
			if (bln==bla) {
				bla*=2;
				bl=(BOXLOC *)realloc(bl,bla*sizeof(BOXLOC));
			}
		}
	}
}

qsort((void *)bl,bln,sizeof(BOXLOC),BOXLOCcmp);

int mb=maxboxes->text().toInt();
float xclude=exclude->text().toFloat();

char s[MAXPATHLEN];
strcpy(s,databig->Path());
strcpy(s+strlen(s)-3,"box");
FILE *out=fopen(s,"w");
if (!out) return;

int saved=0;
for (i=0; (saved<(bln<mb?bln:mb)) && i<bln; i++) {
	for (j=0; j<i; j++) if (maxhypot(bl[i].x-bl[j].x,bl[i].y-bl[j].y,xclude)) { bl[i].x=bl[i].y=-10000.0; break; }
	if (j!=i) continue;
	fprintf(out,"%d %d %d %d -3\n",bl[i].x-cliprefsize/2,bl[i].y-cliprefsize/2,cliprefsize,cliprefsize);
	saved++;
}

fclose(out);

boxdisp->setText(QString::number(saved));
}


void Control::bigKeySel( QKeyEvent * k )
{
	static int imgno=0;
	if (k->key()==Qt::Key_S) {
		char s[MAXPATHLEN],t[MAXPATHLEN];
		strcpy(s,databig->Path());
		s[strlen(s)-4]=0;
		sprintf(t,"%s-%d.mrc",s,imgno);
		imgno++;
		printf("%s saved\n",t);
		datar->writeMRC(t);
	}

}
