#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include "Equation.h"
#include "math.tab.h"

#ifndef M_PI
	#define M_PI	3.14159265358979323846264338328		/* pi */
#endif
#ifndef M_E
	#define M_E	2.71828182845904523536028747135			/* e */
#endif

//#define DEBUG1
//#define DEBUG2

M_OPS *Equation::oplist=NULL;
int Equation::nops=0;
int Equation::nopsa=0;

char *mathparsestr;
Equation *mathcur;
Term *mathret;

int mathlex()
{
return mathcur->parseToken();
}

// Seems to be a glibc bug, these are a workaround
inline int Isalpha(char x) { return ((x>='A'&&x<='Z')||(x>='a'&&x<='z'))?1:0; }
inline int Isdigit(char x) { return (x>='0'&&x<='9')?1:0; }
inline char Tolower(char x) { return (x>='A'&&x<='Z')?x-32:x; }

int mathparse();

double lessthan(double a,double b) { if (a<b) return 1.0; return 0.0; }
double lesseqthan(double a,double b) { if (a<=b) return 1.0; return 0.0; }

void Equation::init()
{
if (oplist) return;

Equation::addop1("sin",sin);
Equation::addop1("cos",cos);
Equation::addop1("tan",tan);
Equation::addop1("asin",asin);
Equation::addop1("acos",acos);
Equation::addop1("atan",atan);
Equation::addop1("sqrt",sqrt);
Equation::addop1("log",log);
Equation::addop1("log10",log10);
Equation::addop1("exp",exp);
Equation::addop1("fabs",fabs);
Equation::addop1("j0",j0);
Equation::addop1("y0",y0);
Equation::addop1("j1",j1);
Equation::addop1("y1",y1);
Equation::addop2("less",lessthan);
Equation::addop2("lesseq",lesseqthan);
}

void Equation::addop1(char *name,double (*fn)(double parm))
{
if (nops==nopsa) { 
	nopsa+=6;
	oplist=(M_OPS *)realloc(oplist,sizeof(M_OPS)*nopsa);
}

strncpy(oplist[nops].name,name,9);
oplist[nops].name[9]=0;
oplist[nops].nparm=1;
oplist[nops].fn1=fn;
nops++;
}

void Equation::addop2(char *name,double (*fn)(double parm,double parm2))
{
if (nops==nopsa) { 
	nopsa+=6;
	oplist=(M_OPS *)realloc(oplist,sizeof(M_OPS)*nopsa);
}

strncpy(oplist[nops].name,name,9);
oplist[nops].name[9]=0;
oplist[nops].nparm=2;
oplist[nops].fn2=fn;
nops++;
}

void Equation::addopn(char *name,double (*fn)(int n,double *parm))
{
if (nops==nopsa) { 
	nopsa+=6;
	oplist=(M_OPS *)realloc(oplist,sizeof(M_OPS)*nopsa);
}

strncpy(oplist[nops].name,name,9);
oplist[nops].name[9]=0;
oplist[nops].nparm=-1;
oplist[nops].fnn=fn;
nops++;
}

Equation::Equation()
{
//initMetaObject();                           // initialize meta object
tree=NULL;
varlist.setAutoDelete(TRUE);
//nterm=0;
//nterma=24;
//tlist=(Term *)malloc(sizeof(Term)*24);
}

Equation::~Equation()
{
if (tree) freeTree(tree);
//free(tlist);
}

int Equation::parseEqn(QString s)
{
char *parse = (char *)malloc(s.length()+1);

eqn=s;
strcpy(parse,(const char *)eqn);

if (tree) freeTree(tree);
//nterm=0;
mathcur=this;
mathparsestr=parse;
mathret=NULL;
#ifdef DEBUG2
printf("Equation %p: Parse '%s'\n",this,(const char *)s);
#endif
mathparse();
tree=mathret;

emit changed();
free(parse);
return 0;
}

double Equation::eval()
{
if (!tree) return 0;

#ifdef DEBUG1
printf("-------\n");
#endif

return evalTerm(tree);
}

// recursively get rid of any dependencies
void Equation::freeTree(Term *term)
{
switch (term->type) {
case E_VAR:
	break;
case E_FN:
	freeTree(term->exp2);
	break;
case E_FN2:
	freeTree(term->exp2);
	freeTree(term->exp3);
	break;
case E_FNN:
	break;
case E_NUMBER:
	break;
case '+':
case '-':
case '/':
case '*':
case '^':
	freeTree(term->exp1);
	freeTree(term->exp2);
	break;
case E_UMINUS:
	freeTree(term->exp1);
	break;
}
free(term);
return;
}

double Equation::evalTerm(Term *term)
{
switch (term->type) {

case E_VAR:
#ifdef DEBUG1
printf("Var -> %f\n",*term->var);
#endif
	return *term->var;

case E_FN:
#ifdef DEBUG1
printf("FN1 \n");
#endif
	return term->fn1(evalTerm(term->exp2));

case E_FN2:
	return term->fn2(evalTerm(term->exp2),evalTerm(term->exp3));

case E_FNN:
	break;

case E_NUMBER:
#ifdef DEBUG1
printf("NUM -> %f\n",term->value);
#endif
	return term->value;

case '+':
#ifdef DEBUG1
printf("+\n");
#endif
	return evalTerm(term->exp1)+evalTerm(term->exp2);

case '-':
#ifdef DEBUG1
printf("-\n");
#endif
	return evalTerm(term->exp1)-evalTerm(term->exp2);

case '/':
#ifdef DEBUG1
printf("/\n");
#endif
	return evalTerm(term->exp1)/evalTerm(term->exp2);

case '*':
#ifdef DEBUG1
printf("*\n");
#endif
	return evalTerm(term->exp1)*evalTerm(term->exp2);

case '^':
#ifdef DEBUG1
printf("^\n");
#endif
	return pow(evalTerm(term->exp1),evalTerm(term->exp2));

case E_UMINUS:
#ifdef DEBUG1
printf("UM\n");
#endif
	return -evalTerm(term->exp1);

default:
	printf("UNKNOWN TERM! type=%d\n",term->type);
}

return 0;
}

//#define DEBUG1

int Equation::parseToken()
{
int i,j;
char *u,uu;
Term *term;

term=mathlval.value=newTerm();

if (!mathparsestr || *mathparsestr==0) return 0;
while(*mathparsestr==' '||*mathparsestr=='\n'||*mathparsestr=='\r') mathparsestr++;


	if ((*mathparsestr>='0' && *mathparsestr<='9') || *mathparsestr=='.') {
		term->type=E_NUMBER;
		sscanf(mathparsestr," %lf%n",&term->value,&i);
		mathparsestr+=i;

#ifdef DEBUG1
printf("NUMBER %f\n",term->value);
#endif

		return E_NUMBER;
	}
	if (Isalpha(*mathparsestr)) {
		u=mathparsestr;
		while (Isalpha(*u) || Isdigit(*u)) {
			u++;
			if (Isalpha(*u)) *u=Tolower(*u);
		}
		j=u-mathparsestr;
		if (*u=='('||*u=='[') {
			for (i=0; i<nops; i++) 
				if (!strncmp(mathparsestr,oplist[i].name,j) && j==strlen(oplist[i].name)) break;
			if (i==nops) {
				term->type=E_ERROR;
				printf("Unk fn %d '%s'\n",j,mathparsestr);
				return E_ERROR;
			}

			mathparsestr=u;

			if (oplist[i].nparm==1) {
				term->type=E_FN;
				term->fn1=oplist[i].fn1;
#ifdef DEBUG1
printf("FN1 %s\n",oplist[i].name);
#endif

				return E_FN;
			}
			if (oplist[i].nparm==2) {
				term->type=E_FN2;
				term->fn2=oplist[i].fn2;
				return E_FN2;
			}
			term->type=E_FNN;
			term->fn=oplist[i].fnn;
			return E_FNN;			
		}
		uu=*u;
		*u=0;
		if (!strcmp(mathparsestr,"e")) {
			term->type=E_NUMBER;
			term->value=M_E;
		}
		else if (!strcmp(mathparsestr,"pi")) {
			term->type=E_NUMBER;
			term->value=M_PI;
		}
		else {
			term->type=E_VAR;
			term->var=varlist[mathparsestr];
			if (!term->var) {
				setVar(mathparsestr,0);
				term->var=varlist[mathparsestr];
				if (!term->var) { printf("Error, unknown variable '%s'\n",mathparsestr); exit(1); }
			}
#ifdef DEBUG1
printf("VARIABLE %s\n",mathparsestr);
#endif
		}
		*u=uu;
		mathparsestr=u;
		return term->type;
	}

#ifdef DEBUG1
printf("OP %c\n",*mathparsestr);
#endif

	term->type=*mathparsestr;
	uu=*mathparsestr;
	mathparsestr++;
	return uu;
}

// note that once a variable has been added, its double pointer never changes
void Equation::setVar(char *name,double val,bool quiet) {
double *v;

if (!name) return;

v=varlist[name];
if (v) *v=val;
else {
	v=new double;
	*v=val;
	varlist.insert(name,v);
}

if (!quiet) emit changed();
}

double Equation::getVar(char *name) {
double *r;

r=varlist[name];
if (!r) { printf("Unk var %s\n",name); return 0; }
return *r;
}

// unset variables are returned as NULL, indicating they are unused in
// the parsed expression
double *Equation::getVarP(char *name) {
double *r;
if (!name) return NULL;
r=varlist[name];
/*if (!r) {
	r=new double;
	*r=0;
	varlist.insert(name,r);
}*/
return r;
}
