#include "InspAxis.h"
#include "Axes2d.h"
#include "qfontdialog.h"
#include "qcolordialog.h"
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <math.h>

#define Inherited inspAxisBase

InspAxis::InspAxis(QWidget* parent,const char* name):Inherited( parent, name ) {
}


InspAxis::~InspAxis() {
}

void InspAxis::refresh() {
    // printf("this axis refes was called \n");
if (!target) return;

int opt=target->getOptions();
int xopt=target->getXOpt();
int yopt=target->getYOpt();
fRect fr=target->getLimits();

switch (axis) {
case 0:
	title->setText(target->getXLabel());
	min->setText(QString::number(fr.x0(),'g',3));
	max->setText(QString::number(fr.x1(),'g',3));
	maj->setText(QString::number(target->getXTicspace(),'g',3));
	
	minor->setCurrentItem(target->getMinorX());
	
	autos->setOn(opt&D2_OPT_AUTOSCALE);
	majtick->setOn(xopt&D2_OPT_TIC);
	majgrid->setOn(xopt&D2_OPT_GRID);
	mingrid->setOn(xopt&D2_OPT_GRIDM);
	
	cbtitle->setChecked(xopt&D2_OPT_LABEL);
	cbaxis->setChecked(xopt&D2_OPT_AXIS);
	cblog->setChecked(xopt&D2_OPT_LOG);

	axiscolor->setPalette(QPalette(target->getAxisColor()));
	break;
case 1:
	title->setText(target->getYLabel());
	min->setText(QString::number(fr.y0(),'g',3));
	max->setText(QString::number(fr.y1(),'g',3));
	maj->setText(QString::number(target->getYTicspace(),'g',3));
	
	minor->setCurrentItem(target->getMinorY());
	
	autos->setOn(opt&D2_OPT_AUTOSCALE);
	majtick->setOn(yopt&D2_OPT_TIC);
	majgrid->setOn(yopt&D2_OPT_GRID);
	mingrid->setOn(yopt&D2_OPT_GRIDM);
	
	cbtitle->setChecked(yopt&D2_OPT_LABEL);
	cbaxis->setChecked(yopt&D2_OPT_AXIS);
	cblog->setChecked(yopt&D2_OPT_LOG);

	axiscolor->setPalette(QPalette(target->getAxisColor()));
	break;
}

}


void InspAxis::newCBTitle(bool b) { 
if (!target) return; 

if (axis) {
	if (b) target->setYFlags(D2_OPT_LABEL); else target->resetYFlags(D2_OPT_LABEL);
}
else {
	if (b) target->setXFlags(D2_OPT_LABEL); else target->resetXFlags(D2_OPT_LABEL);
}
target->update();
}

void InspAxis::newCBLog(bool b) { 
if (!target) return;
// printf ("axis is %d\n",axis); 

if (axis) {
	if (b) target->setYFlags(D2_OPT_LOG); else target->resetYFlags(D2_OPT_LOG);
}
else {
	if (b) target->setXFlags(D2_OPT_LOG); else target->resetXFlags(D2_OPT_LOG);
}
target->update();
}

void InspAxis::newAuto(bool b) { 
if (!target) return; 

if (b) target->setFlags(D2_OPT_AUTOSCALE); else target->resetFlags(D2_OPT_AUTOSCALE);
target->update();
}


void InspAxis::newCBAxis(bool b) { 
if (!target) return; 

if (axis) {
	if (b) target->setYFlags(D2_OPT_AXIS); else target->resetYFlags(D2_OPT_AXIS);
}
else {
	if (b) target->setXFlags(D2_OPT_AXIS); else target->resetXFlags(D2_OPT_AXIS);
}
target->update();
}

void InspAxis::newMajGrid(bool b) { 
if (!target) return; 

if (axis) {
	if (b) target->setYFlags(D2_OPT_GRID); else target->resetYFlags(D2_OPT_GRID);
}
else {
	if (b) target->setXFlags(D2_OPT_GRID); else target->resetXFlags(D2_OPT_GRID);
}
target->update();
}

void InspAxis::newMaj() { 
if (!target) return; 
float f=maj->text().toFloat();

if (axis) target->setYTicspace(f);
else target->setXTicspace(f);

target->update();
}

void InspAxis::newMajTick(bool b) { 
if (!target) return; 

if (axis) {
	if (b) target->setYFlags(D2_OPT_TIC); else target->resetYFlags(D2_OPT_TIC);
}
else {
	if (b) target->setXFlags(D2_OPT_TIC); else target->resetXFlags(D2_OPT_TIC);
}
target->update();
}

void InspAxis::newMin() { 
if (!target) return; 

float v=min->text().toFloat();
fRect r=target->getLimits();

if (axis) {
	int yopt=target->getYOpt();
	if (yopt&D2_OPT_LOG) v=pow(10.0F,v);
	r.sety0(v);
}
else {
	int xopt=target->getXOpt();
	if (xopt&D2_OPT_LOG) v=pow(10.0F,v);
	r.setx0(v);
}
target->setLimits(r);
target->update();
}

void InspAxis::newMax() {
if (!target) return; 

float v=max->text().toFloat();
fRect r=target->getLimits();

if (axis) {
	int yopt=target->getYOpt();
	if (yopt&D2_OPT_LOG) v=pow(10.0F,v);
	r.sety1(v);
}
else {
	int xopt=target->getXOpt();
	if (xopt&D2_OPT_LOG) v=pow(10.0F,v);
	r.setx1(v);
}
target->setLimits(r);
target->update();

}

void InspAxis::newAxisColor() { 
if (!target) return; 

QColor nc;
nc=QColorDialog::getColor ( target->getFrameColor());
axiscolor->setPalette(QPalette(nc));
target->setAxisColor(nc);
target->update();
}

void InspAxis::newTFont() { 
if (!target) return; 

QFont f(QFontDialog::getFont(NULL,target->getLFont()));
target->setLFont(f);
target->update();

}


void InspAxis::newAFont() { 
if (!target) return; 

QFont f(QFontDialog::getFont(NULL,target->getSFont()));
target->setSFont(f);
target->update();
}


void InspAxis::newMinor(int i) { 
if (!target) return; 

if (axis) {
	target->setMinorY(i);
}
else {
	target->setMinorX(i);
}
target->update();
}

void InspAxis::newMinGrid(bool b) { 
if (!target) return; 

if (axis) {
	if (b) target->setYFlags(D2_OPT_GRIDM); else target->resetYFlags(D2_OPT_GRIDM);
}
else {
	if (b) target->setXFlags(D2_OPT_GRIDM); else target->resetXFlags(D2_OPT_GRIDM);
}
target->update();
}

void InspAxis::newTitle() { 
if (!target) return; 

if (axis) {
	target->setYLabel(title->text());
}
else {
	target->setXLabel(title->text());
}
cbtitle->setChecked(1);
target->update();
}
