#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "ValSlider.h"

ValSlider::ValSlider( QWidget *parent, const char *name ) : QWidget( parent, name )
{
    setBackgroundColor( lightGray );            // set background color

	line=new QLineEdit2(parent,NULL);
	line->setMaxLength(12);
	line->setText("0");
	QObject::connect( line, SIGNAL(returnPressed()), this, SLOT(textchange()) );
	QObject::connect( line, SIGNAL(lostFocus2()), this, SLOT(textchange()) );

	slider=new QSlider(0,1000000,500,0,QSlider::Horizontal,parent,NULL);
	slider->setTracking(TRUE);
	slider->setFocusPolicy(QWidget::StrongFocus);
	QObject::connect( slider, SIGNAL(valueChanged(int)), this, SLOT(sliderchange()) );

	min=0;
	max=1.0;
	value=0;
	active=0;
	setTSize(8);
	vname=NULL;
}

ValSlider::~ValSlider() {

if (vname) free(vname);
//delete slider;
//delete line;
}

void ValSlider::resizeEvent( QResizeEvent * )
{
int w = width();
int h = height();
int X = x();
int Y = y();

line->setGeometry(X,Y+(h-20)/2,twidth,20);
line->setMaxLength(tsize);
line->update();

slider->setGeometry(X+twidth+4,Y,w-(twidth+4),h);
slider->update();

slider->setSteps((int)floor(1000000.0/w*4.0),100000);
}

void ValSlider::setTSize(int w) {
char *c;
QFontMetrics *qm;
int i;

tsize=w;
c=(char *)malloc(w+1);
for (i=0; i<w; i++) c[i]='0';
c[w]=0;
qm=new QFontMetrics(font());
twidth=qm->width(c)+8;

//printf("'%s' %d\n",c,twidth);

free(c);
update();
delete qm;
}

void ValSlider::sliderchange() {
char s[16],t[8];

if (active) return;
active++;
value=slider->value()*(max-min)/1000000.0+min;

sprintf(t,"%%1.%1df",tsize-2);
sprintf(s,t,value);
line->setText(s);
line->setMaxLength(tsize);

emit newValue(value);
if (vname) emit newVarVal(vname,value);
active--;
}

void ValSlider::setVarName(char *n) {
if (!n) {
	if (vname) { free(vname); vname=NULL; }
	return;
}

if (!vname) vname=(char *)malloc(strlen(n)+1);
else vname=(char *)realloc(vname,strlen(n)+1);
strcpy(vname,n);
return;
}

void ValSlider::textchange() {
if (active) return;
active++;

if (line->text().left(1)==">") { setMin(line->text().mid(1).toFloat()); line->selectAll(); active--; return; }
if (line->text().left(1)=="<") { setMax(line->text().mid(1).toFloat()); line->selectAll(); active--; return; }
value=line->text().toFloat();
if (value>max) setMax(value);
if (value<min) setMin(value);

slider->setValue((int)floor(1000000*(value-min)/(max-min)+.5));

emit newValue(value);
if (vname) emit newVarVal(vname,value);
line->selectAll();
active--;
}

void ValSlider::setValueQ(float f) {
char s[16],t[8];

active++;
value=f;
sprintf(t,"%%1.%1df",tsize-2);
sprintf(s,t,value);
line->setText(s);
line->setMaxLength(tsize);

slider->setValue((int)floor(1000000*(value-min)/(max-min)+.5));
active--;
}

void ValSlider::setValue(float f) {
char s[16],t[8];

active++;
value=f;
sprintf(t,"%%1.%1df",tsize-2);
sprintf(s,t,value);
line->setText(s);
line->setMaxLength(tsize);

slider->setValue((int)floor(1000000*(value-min)/(max-min)+.5));

emit newValue(f);
if (vname) emit newVarVal(vname,value);
active--;
}

void ValSlider::setMin(float m) {
min=m;
setValueQ(value);
}

void ValSlider::setMax(float m) {
max=m;
setValueQ(value);
}
