// align3d.C - Copyright 06/15/98 Steve Ludtke
// Copyright 1999 Baylor College of Medicine
// This program will rotationally and translationally align 2 3d models.
// It uses a simple grid search rather than r/t CCF's, this makes it
// somewhat slow. Eventually it should probably be rewritten

#include <cstdio>
#include <cmath>
#include "EMData.h"
#include "List.h"
#include "util.h"

inline float asub(float x,float y) {
float r;

r=fabs(x-y);
if (r>PI) r=2.0*PI-r;

return r;
}

int main(int argc, char *argv[]) {
int i,Ref;
EMData *d1,*d2,*d3,*r1,*r2,*r3=NULL;
float alt,az,phi,lc,best,eul[3],eul2[3],fbest,feul[3],step=20.0;
int dot=0,flip=0,refine=0,saveref=0;
FILE *p3d=NULL;

appinit(argc,argv);

if (argc<4) appusage(argv[0]);


for (i=4; i<argc; i++) {	
	if (strncmp(argv[i],"dot",3)==0) dot=1;
	else if (strncmp(argv[i],"p3d",3)==0) p3d=fopen("a3d.p3d","wb");
	else if (strncmp(argv[i],"checkflip",9)==0) flip=1;
	else if (strncmp(argv[i],"refine",6)==0) refine=1;
	else if (strncmp(argv[i],"slow",4)==0) step=9.0;
	else if (strncmp(argv[i],"saveref",7)==0) saveref=1;
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}



printf("Read reference image\n");
d1=new EMData;
d1->readImage(argv[1],-1);
d1->normalize();
printf("Make rotational reference\n");
r1=d1->makeRFP();
r1->setParent(NULL);

printf("Read image to be aligned\n");
d2=new EMData;
d2->readImage(argv[2],-1);
d2->normalize();
d2=d2->copy();
printf("Make rotational reference\n");
r2=d2->makeRFP();
r2->setParent(NULL);
if (flip) {
	r3=r2->copy();
	r3->vFlip();
}
r2=r2->copy();	// must go after the 'if' above

Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_REFFILE,NULL);
LOG(Ref,argv[2],LOG_INFILE,NULL);

/*
r1->writeMRC("x1.mrc");
r2->writeMRC("x2.mrc");
r2->setRAlign(-30.0*PI/180.0,0,0);
r2->rotateAndTranslate();
r2->writeMRC("x3.mrc");
*/

eul[0]=eul[1]=eul[2]=0;
r2->setRAlign(0,0,0);
r2->rotateAndTranslate();
if (dot) best=-r1->dot(r2); else best=r1->lcmp(r2);

// should really put all this stuff into 1 loop...

if (!refine) {
printf("Align to +-%0.1f degrees\n",step);
// locate best alignment to +-30 degrees
for (alt=0; alt<=180.0; alt+=step) {
	for (az=0; az<360.0; az+=step) {
		for (phi=0; phi<360.0; phi+=step) {
			r2->setRAlign(alt*PI/180.0,az*PI/180.0,phi*PI/180.0);
			r2->rotateAndTranslate();
			if (dot) lc=-r1->dot(r2); else lc=r1->lcmp(r2);
			if (p3d) fprintf(p3d,"%f\t%f\t%f\t%f\n",alt,az,phi,lc);
			if (lc<best) { best=lc; eul[0]=phi; eul[1]=alt; eul[2]=az; }
		}
	printf("%f %f        \r",alt,az); fflush(stdout);
	}
}
if (p3d) fclose(p3d);
printf("Best %f %f %f\t%f\n",eul[1],eul[2],eul[0],best);

if (flip) {
	feul[0]=feul[1]=feul[2]=0;
	r3->setRAlign(0,0,0);
	r3->rotateAndTranslate();
	if (dot) best=-r1->dot(r3); else best=r1->lcmp(r3);

// should really put all this stuff into 1 loop...

	printf("Align to +-%0.2f degrees\n",step);
// locate best alignment to +-30 degrees
	for (alt=0; alt<=180.0; alt+=step) {
		for (az=0; az<360.0; az+=step) {
			for (phi=0; phi<360.0; phi+=step) {
				r3->setRAlign(alt*PI/180.0,az*PI/180.0,phi*PI/180.0);
				r3->rotateAndTranslate();
				if (dot) lc=-r1->dot(r3); else lc=r1->lcmp(r3);
				if (p3d) fprintf(p3d,"%f\t%f\t%f\t%f\n",alt,az,phi,lc);
				if (lc<best) { fbest=lc; feul[0]=phi; feul[1]=alt; feul[2]=az; }
			}
		printf("%f %f        \r",alt,az); fflush(stdout);
		}
	}
	printf("Best flipped %f %f %f\t%f\n",feul[1],feul[2],feul[0],fbest);

	// flipped model better than unflipped?
	if (fbest<best) {
		printf("Model flipped\n");
		delete r2;
		r2=r3;
		r3=NULL;
		d2->vFlip();
		eul[0]=feul[0];
		eul[1]=feul[1];
		eul[2]=feul[2];
		best=fbest;
	}
}
}
else { eul[0]=eul[1]=eul[2]=0; }

while (step>.4) {
step/=4.0;

eul2[0]=eul[0];
eul2[1]=eul[1];
eul2[2]=eul[2];
printf("Align to +-%0.2f degrees\n",step);
// locate best alignment to +-6 degrees
for (alt=eul[1]-step*4; alt<eul[1]+step*4.5; alt+=step) {
	for (az=eul[2]-step*4; az<eul[2]+step*4.5; az+=step) {
		for (phi=eul[0]-step*4; phi<eul[0]+step*4.5; phi+=step) {
			r2->setRAlign(alt*PI/180.0,az*PI/180.0,phi*PI/180.0);
			r2->rotateAndTranslate();
			if (dot) lc=-r1->dot(r2); else lc=r1->lcmp(r2);
			if (lc<best) { best=lc; eul2[0]=phi; eul2[1]=alt; eul2[2]=az; }
		}
	printf("%f %f        \r",alt,az); fflush(stdout);
	}
}
printf("Best %f %f %f\t%f\n",eul2[1],eul2[2],eul2[0],best);

}

if (saveref) {
	r1->writeMRC("r1.mrc");
	r2->setRAlign(eul2[1]*PI/180.0,eul2[2]*PI/180.0,eul2[0]*PI/180.0);
	r2->rotateAndTranslate();
	r2->writeMRC("r2.mrc");
}

//d3=d2->copy();
//d3->setParent(d2->getParent());
printf("Checking 180 degree rotations\n");
d2->setTAlign(0,0,0);
d2->setRAlign(eul2[1]*PI/180.0,eul2[2]*PI/180.0,eul2[0]*PI/180.0);
d2->rotateAndTranslate();
d2->transAlign3d(d1,0,1);
d2->rotateAndTranslate();
printf("%f r: %f %f %f\tt:%f %f %f\n",d2->dot(d1),d2->phi()*180.0/PI,d2->alt()*180.0/PI,d2->az()*180.0/PI,d2->Dx(),d2->Dy(),d2->Dz());
eul[0]=eul2[0];
eul[1]=eul2[1];
eul[2]=eul2[2];
best=d2->dot(d1);

d2->setTAlign(0,0,0);
d2->setRAlign(eul[1]*PI/180.0,(eul[2]+180.0)*PI/180.0,eul[0]*PI/180.0);
d2->rotateAndTranslate();
d2->transAlign3d(d1,0,1);
d2->rotateAndTranslate();
printf("%f r: %f %f %f\tt:%f %f %f\n",d2->dot(d1),d2->phi()*180.0/PI,d2->alt()*180.0/PI,d2->az()*180.0/PI,d2->Dx(),d2->Dy(),d2->Dz());
if (d2->dot(d1)>best) {
	eul[2]+=180.0;
	best=d2->dot(d1);
}


d2->setTAlign(0,0,0);
d2->setRAlign((eul[1]+180.0)*PI/180.0,eul[2]*PI/180.0,eul[0]*PI/180.0);
d2->rotateAndTranslate();
d2->transAlign3d(d1,0,1);
d2->rotateAndTranslate();
printf("%f r: %f %f %f\tt:%f %f %f\n",d2->dot(d1),d2->phi()*180.0/PI,d2->alt()*180.0/PI,d2->az()*180.0/PI,d2->Dx(),d2->Dy(),d2->Dz());
if (d2->dot(d1)>best) {
	eul[1]+=180.0;
	best=d2->dot(d1);
}

d2->setTAlign(0,0,0);
d2->setRAlign(eul[1]*PI/180.0,eul[2]*PI/180.0,eul[0]*PI/180.0);
d2->rotateAndTranslate();
d2->transAlign3d(d1,0,1);
d2->rotateAndTranslate();

//printf("0=%f  180=%f\n",d2->dot(d1),d3->dot(d1));
printf("%1.1f,%1.1f,%1.1f\t%1.1f,%1.1f,%1.1f\n",eul[1],eul[2],eul[0],eul2[1],eul2[2],eul2[0]);

d2->writeMRC(argv[3]);
LOG(Ref,argv[3],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);

return 0;
}
