// align3dvec.C - 4/16/00 Steve Ludtke
// Copyright 2000 Baylor College of Medicine
// This program will rotationally and translationally align 2 3d models.

//N align3dvec
//F Rotational and translational alignment of 3d models via vectorization
//T X
//1
//P <fixed input>	The volume to align to
//P <free input>	The model to be aligned
//P <aligned output>	The aligned output
//P [nseg=<n>]	Number of segments to split models into
//P [thr=<threshold>]	Threshold density for segmentation, default is 1
//P [qmap]	This will output a 3D 'quality map' showing the match at all possible Euler angles
//U
//D This program is still highly experimental. You can play with it, but don't
//E expect too much.
//D This program will take one 3d model and optimally align it to another 'reference' model.
//E It uses segment3d to 'vectorize' the 3D models, then aligns the resulting vectors. A
//E sufficient number of vectors (nseg=) should be used to represent the major features
//E of the model.

struct Segment { float x,y,z,s,v,sigx,sigy,sigz; };

float dotseg(int n, Segment *s1, Segment *s2);
float normc(int n, Segment *s);
void normalize(int n, Segment *s);


#include <cstdio>
#include <cmath>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include "Euler.h"

inline float asub(float x,float y) {
float r;

r=fabs(x-y);
if (r>PI) r=2.0*PI-r;

return r;
}

int main(int argc, char *argv[]) {
int i,nseg=12,Ref,x,y,z,ny;
float thr=1.0;
EMData *d1,*d2,*d3;
char s[180];
EMData *qmap=NULL;
float cenr[3] = { 0,0,0 },cen[3] = { 0,0,0 };
Segment *seg,*segr;
float *mx,norm,astep=5.0*PI/180.0;
float alt,az,phi,balt,baz,bphi,bdot,dot;
Euler eul;

appinit(argc,argv);

if (argc<4) appusage(argv[0]);


for (i=4; i<argc; i++) {	
	if (strncmp(argv[i],"nseg=",5)==0) nseg=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"thr=",4)==0) thr=atof(&argv[i][4]);
	else if (strncmp(argv[i],"qmap",4)==0) {
		qmap=new EMData;
		qmap->setSize(72,72,72);
	}
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

seg=(Segment *) malloc(nseg*sizeof(Segment));
segr=(Segment *) malloc(nseg*sizeof(Segment));

printf("Read reference image\n");
d1=new EMData;
d1->readImage(argv[1],-1);
d1->normalize();

printf("Read image to be aligned\n");
d2=new EMData;
d2->readImage(argv[2],-1);
d2->normalize();
d2=d2->copy();

d3=d2->copy();
d3->vFlip();
d3=d3->copy();
ny=d3->ySize();

Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_REFFILE,NULL);
LOG(Ref,argv[2],LOG_INFILE,NULL);

sprintf(s,"segment3d %s seg.mrc %d thr=%1.3f vector=vec.txt",argv[1],nseg,thr);
run(s);
FILE *in = fopen("vec.txt","rb");
for (i=0; i<nseg; i++) {
	fgets(s,179,in);
	sscanf(s," %f %f %f %f %f %f",&segr[i].x,&segr[i].y,&segr[i].z,
		&segr[i].sigx,&segr[i].sigy,&segr[i].sigz);
	cenr[0]+=segr[i].x;
	cenr[1]+=segr[i].y;
	cenr[2]+=segr[i].z;
}
fclose(in);
cenr[0]/=nseg;
cenr[1]/=nseg;
cenr[2]/=nseg;

for (i=0; i<nseg; i++) { segr[i].x-=cenr[0]; segr[i].y-=cenr[1]; segr[i].z-=cenr[2]; }

sprintf(s,"segment3d %s seg.mrc %d thr=%1.3f vector=vec.txt",argv[2],nseg,thr);
run(s);
in = fopen("vec.txt","rb");
for (i=0; i<nseg; i++) {
	fgets(s,179,in);
	sscanf(s," %f %f %f %f %f %f",&seg[i].x,&seg[i].y,&seg[i].z,
		&seg[i].sigx,&seg[i].sigy,&seg[i].sigz);
	cen[0]+=seg[i].x;
	cen[1]+=seg[i].y;
	cen[2]+=seg[i].z;
}
fclose(in);
cen[0]/=nseg;
cen[1]/=nseg;
cen[2]/=nseg;

for (i=0; i<nseg; i++) { 
	seg[i].x-=cen[0]; 
	seg[i].y-=cen[1]; 
	seg[i].z-=cen[2]; 
	seg[i].sigx=seg[i].x;
	seg[i].sigy=seg[i].y;
	seg[i].sigz=seg[i].z;
}

norm=normc(nseg,seg)*normc(nseg,segr);

printf("Reference center:\t%1.3f\t%1.3f\t%1.3f\n",cenr[0],cenr[1],cenr[2]);
printf("          center:\t%1.3f\t%1.3f\t%1.3f\n",cen[0],cen[1],cen[2]);

bdot=0;
for (alt=-PI/2,x=0; alt<PI/2; alt+=2.501*PI/180.0,x++) {
	for (az=0,y=0; az<2.0*PI; az+=5.001*PI/180.0,y++) {
		for (phi=0,z=0; phi<2.0*PI; phi+=5.001*PI/180.0,z++) {
			eul.setAngle(alt,az,phi);
			mx=eul.mat();
			for (i=0; i<nseg; i++) {
//				seg[i].x=seg[i].sigx*mx[0]+seg[i].sigy*mx[3]+seg[i].sigz*mx[6];
//				seg[i].y=seg[i].sigx*mx[1]+seg[i].sigy*mx[4]+seg[i].sigz*mx[7];
//				seg[i].z=seg[i].sigx*mx[2]+seg[i].sigz*mx[5]+seg[i].sigz*mx[8];
				seg[i].x=seg[i].sigx*mx[0]+seg[i].sigy*mx[1]+seg[i].sigz*mx[2];
				seg[i].y=seg[i].sigx*mx[3]+seg[i].sigy*mx[4]+seg[i].sigz*mx[5];
				seg[i].z=seg[i].sigx*mx[6]+seg[i].sigz*mx[7]+seg[i].sigz*mx[8];
			}

			dot=dotseg(nseg,segr,seg)/norm;
			if (dot>1.5) { printf("illegal %f %f %f\t%f\t%f\n",alt,az,phi,dot,bdot); dot=0; }
			if (dot>bdot) { bdot=dot; balt=alt; baz=az; bphi=phi; }

			if (qmap) qmap->setValueAt(x,y,z,dot);
		}
	}
	printf(" %1.1f  \r",alt*180.0/PI); fflush(stdout);
}

if (qmap) {
	qmap->getData();
	qmap->doneData();
	qmap->writeMRC("qmap.mrc");
	LOG(Ref,"qmap.mrc",LOG_OUTFILE,NULL);
}

/*
eul.setAngle(balt,baz,bphi);
mx=eul.mat();
for (i=0; i<nseg; i++) {
	seg[i].x=seg[i].sigx*mx[0]+seg[i].sigy*mx[3]+seg[i].sigz*mx[6];
	seg[i].y=seg[i].sigx*mx[1]+seg[i].sigy*mx[4]+seg[i].sigz*mx[7];
	seg[i].z=seg[i].sigx*mx[2]+seg[i].sigz*mx[5]+seg[i].sigz*mx[8];
}*/


printf("Best angle is %1.1f,%1.1f,%1.1f\t%1.5f\n",balt*180/PI,baz*180/PI,bphi*180/PI,bdot);
//d3->setTAlign(-cen[0]+ny/2,-cen[1]+ny/2,-cen[2]+ny/2);
d3->setRAlign(balt,baz,bphi);
d3->rotateAndTranslate();
d3->normalize();
d3->writeMRC(argv[3]);

d1->setTAlign(-cenr[0]+ny/2,-cenr[1]+ny/2,-cenr[2]+ny/2);
d1->setRAlign(0,0,0);
d1->rotateAndTranslate();
char *nf=(char *)malloc(strlen(argv[1])+5);
strcpy(nf,argv[1]);
strcpy(nf+strlen(argv[1])-4,".cen.mrc");
d1->writeMRC(nf);

LOG(Ref,argv[3],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);
}

// This determines the normalization constant for a set of Segments
float normc(int n, Segment *s) {
float r = 0;

for (int i=0; i<n; i++) r+=hypot3(s[i].x,s[i].y,s[i].z);

return sqrt(r);
}

// This normalizes a set of Segments
void normalize(int n, Segment *s) {
float r = 0;

r=normc(n,s);

for (int i=0; i<n; i++) { s[i].x/=r; s[i].y/=r; s[i].z/=r; }

}

// This will take the 'dot product' of 2 Segment arrays
// The trick here is that the segments aren't necessarily in
// the same order.  Uses s in the segment for a label.
float dotseg(int n, Segment *s1, Segment *s2) {
int i,j,bj;
float b,h,dot=0;

for (i=0; i<n; i++) s2[i].s=0;

for (i=0; i<n; i++) {
	b=1.0e20;
	for (j=0; j<n; j++) {
		if (s2[j].s) continue;
		h=hypot3(s1[i].x-s2[j].x,s1[i].y-s2[j].y,s1[i].z-s2[j].z);
		if (h<b) { b=h; bj=j; }
	}
	s2[bj].s=1.0;
	
	dot+=s1[i].x*s2[bj].x+s1[i].y*s2[bj].y+s1[i].z*s2[bj].z;
}

return dot;
}
