//cenalignacf	1998 Steve Ludtke
// Copyright 1999 Baylor College of Medicine

//N cenalignacf
//F This program will center a set of particles using an autocorrelation function
//T LA
//1
//P <input file>	File containing images to be centered
//P [mask=<rad>]	Circular mask radius in pixels
//P [maxshift=<pix>]	Maximum shift to use in centering
//U cenalignacf start.hed mask=42 maxshift=8
//D This program centers a set of particles individually using an
//E autocorrelation function. Clearly this works well only if the
//E particles have a suitable shape in all orienations.

#include <cstdio>
#include <cmath>
#include "EMData.h"
#include "List.h"
#include "util.h"

int main(int argc, char *argv[]) {
int i,j,k,nx,nm,n,mask=-1,max=0;
List *data,*data2,*bad;
EMData *d,*d2,*d3;
char s[80];
float f,mean;

appinit(argc,argv);

if (argc<2) { printf("cenalignacf\nUsage: cenalign <input file> [mask=<rad>] [maxshift=<rad>]\n"); exit(0); }

for (i=2; i<argc; i++) {
	if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp("maxshift=",argv[i],9)==0) max=atof(&argv[i][9]);
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

d=new EMData;
d->readImage(argv[1],0);
nx=d->xSize();
if (max==0) {
	if (mask==-1) max=nx/4;
	else max=nx/2-mask;
}
if (mask==-1) mask=nx/2;

n=fileCount(argv[1]);

unlink("ali.hed");
unlink("ali.img");

for (i=0; i<n; i++) {
//	if (i%10==0) { printf("\t%d complete\r",i); fflush(stdout); }
	d->readImage(argv[1],i);
	d->edgeNormalize();
	d2=d->clip(-nx/2,-nx/2,nx*2,nx*2);
	d2->applyMask(nx/2,4);
	d2->transAlign(NULL,0,1,max);
	printf("%4d. %4.0f    %4.0f\n",i,d2->Dx(),d2->Dy());
	d2->rotateAndTranslate();
	d3=d2->clip(nx/2,nx/2,nx,nx);
	d3->applyMask(mask,4);
//	d->toCorner();
//	d->applyMask(nx*2/5,1);
	d3->writeIMAGIC("ali.hed",-1);
	delete d2;
	delete d3;
}

return 0;
}
