#include <cstdio>
#include <cmath>
#include "EMData.h"
#include "List.h"
#include "util.h"

int main(int argc, char *argv[]) {
int i,j,k,n,iter=2,r1=0,nosum=0,badf=0,stat=0,saver=0,mask=-1,locffile=-1;
List *data,*dataf;
EMData *d,*df,*d2,*avg,*oavg=NULL,*first=NULL,*firstf=NULL,*mask1;
char s[80];
float f,m,mean,sigma,cut,keep=1.2,norm,norm2;
float eul[3];
FILE *out;

appinit(argc,argv);

if (argc<2) { printf("classalign.c\nUsage: classalign <input file> \
[\"ref1\"] [\"finalref\"] [\"nosum\"] [\"badfile\"] [\"stat\"] [\"saverefs\"] \
[iterations] [mask=<rad>] [keep=<sigmamult>] [locfromfile]\n"); exit(0); }

for (i=2; i<argc; i++) {
	if (strncmp("ref1",argv[i],4)==0) r1=1;
	else if (strncmp("finalref",argv[i],8)==0) r1=2;
	else if (strncmp("nosum",argv[i],5)==0) nosum=1;
	else if (strncmp("badfile",argv[i],7)==0) badf=1;
	else if (strncmp("saverefs",argv[i],8)==0) saver=1;
	else if (strncmp("stat",argv[i],4)==0) stat=1;
	else if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp("keep=",argv[i],5)==0) keep=atof(&argv[i][5]);
	else if (strncmp("locfromfile",argv[i],11)==0) locffile=1;
	else iter=atoi(argv[i])-2;
}
if (r1==1) iter++;
if (iter<1) exit(1);

printf("Classalign: %s\n",argv[1]);

n=fileCount(argv[1]);
if (n<5) { printf("Not enough elements\n"); exit(0); }
data=readImages(argv[1],0,n);		// read all input images
if (data==NULL) error("Cannot read input",ERR_EXIT);
printf("Read %d images.\n",data->count());

mask1=((EMData *)data->objectAt(0))->copyHead();
mask1->one();
mask1->applyMask(mask,4);
//mask1->writeMRC("mask.img");

dataf=new List;
for (i=0; i<n; i++) {
	d=(EMData *)data->objectAt(i);
//	d->applyMask(-1,0);
	d2=d->copy();
	d2->vFlip();
	dataf->addObject(d2);
//	d=d->copy();
}

/*
d=data->objectAt(2);
d2=data->objectAt(3);
d=d->RTFAlign(d2);
d->rotateAndTranslate();

d2->writeIMAGIC("x.hed",-1);
d->writeIMAGIC("x.hed",-1);
*/

// If we're starting with a reference, we don't need the first pass
if (r1) {
	avg=d=(EMData *)data->objectAt(0);
	first=avg->copy();
	first->setParent(NULL);
	data->removeObjectAt(0);
	dataf->removeObjectAt(0);
	if (mask<0) {
		norm=d->xSize()*d->ySize()*PI/4000.0;
		norm2=PI/(4.0*d->xSize()*d->ySize());
	}
	else{
		norm=mask*mask*PI/1000.0;
		norm2=PI/(d->xSize()*d->ySize());
	}
	eul[0]=d->alt();  //phi
	eul[1]=d->az();  //alt
	eul[2]=d->phi(); //az
	printf("Eulers %f,%f,%f\n",eul[0]*180.0/PI,eul[1]*180.0/PI,eul[2]*180.0/PI);
	n--;
}
if (r1!=1) {
	// pass 1, align to the first data set
	d=(EMData *)data->objectAt(0);
	if (mask<0) {
		norm=d->xSize()*d->ySize()*PI/4000.0;
		norm2=PI/(4.0*d->xSize()*d->ySize());
	}
	else{
		norm=mask*mask*PI/1000.0;
		norm2=PI/(d->xSize()*d->ySize());
	}
	avg=d->copy();
	for (i=1; i<n; i++) {
		d=(EMData *)data->objectAt(i);
		df=(EMData *)dataf->objectAt(i);
		d2=d->RTFAlign(avg,df);
		d2->mult(mask1);
		avg->add(d2);
		delete d2;
		putchar('.'); fflush(stdout);
	}
	putchar('\n');
	avg->cmCenter();
	avg->rotateAndTranslate();
}

if (n==1) iter=0;
cut=0;
for (j=0; j<iter; j++) {
// pass 2-?, align to the first average
	mean=sigma=m=0;
	if (oavg) delete oavg;
	oavg=avg;
	oavg->normalize();
//	oavg->writeIMAGIC("out.hed",-1);
	avg=new EMData();
	avg->setSize(oavg->xSize(),oavg->ySize(),1);
	avg->zero();
	for (i=0; i<n; i++) {
		d=(EMData *)data->objectAt(i);
		df=(EMData *)dataf->objectAt(i);
		d2=d->RTFAlign(oavg,df);
		d2->mult(mask1);
//		f=oavg->dot(d2);
		f=(1.0-oavg->lcmp(d2))/norm2;
//	printf("%f %f %f\n",oavg->lcmp(d2),norm2,f);
		if (f>cut && finite(f)) {
			avg->add(d2);
			putchar('.'); fflush(stdout);
			mean+=f;
			sigma+=f*f;
			m++;
		} else {
			putchar('*'); fflush(stdout);	
		}
		delete d2;
	}
	putchar('\n');
	if (!r1) {
		avg->cmCenter();		// if we have an initial reference
		avg->rotateAndTranslate();
									// don't spoil the centering
	}

	avg->normalize();
	avg->mult(mask1);
	
	mean/=(float)m;
	sigma=sqrt(sigma/(float)m-mean*mean);
	cut=mean-sigma*keep*(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0));

	printf("Mean=%f\tSigma=%f\tCut=%f\twt=%f\n",mean/norm,sigma/norm,cut/norm,
		(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0)));

	// if we have an original reference, align the early averages to it
	// just to get the orientation correct
	if (avg->Sigma()==0 || !finite(avg->Sigma())) printf("Bad Sum\n");

	if (j<iter-2 && r1) {
		if (oavg) delete oavg;
		df=avg->copy();
		df->setParent(NULL);
		df->vFlip();
		oavg=avg->RTFAlign(first,df);
//		printf("firstalign %p %5.2f %5.2f %5.2f\n",oavg,oavg->Dx(),oavg->Dy(),oavg->Daz()*180.0/PI);
		if (oavg) { 
			delete avg;
			delete df;
			oavg->setParent(NULL);
			avg=oavg;
			oavg=NULL;
			avg->setRAlign(0,0,0);
			avg->setTAlign(0,0,0);
			avg->update();
		}	
	}
	if (avg->Sigma()==0 || !finite(avg->Sigma())) printf("Bad Sum\n");

}

// last pass, align to the second average
mean=sigma=m=0;
oavg=avg;
oavg->normalize();
//oavg->writeIMAGIC("out.hed",-1);
if (stat) out=fopen("class.stat","ab");
avg=new EMData();
avg->setSize(oavg->xSize(),oavg->ySize(),1);
avg->zero();
for (i=0; i<n; i++) {
	d=(EMData *)data->objectAt(i);
	df=(EMData *)dataf->objectAt(i);
	d2=d->RTFAlign(oavg,df);
	d2->mult(mask1);
//	f=oavg->dot(d2);
	f=(1.0-oavg->lcmp(d2))/norm2;
	if (f>cut) {
		if (nosum) {
			d2->setRAlign(eul[0],eul[1],eul[2]);
			d2->setNImg(1);
			d2->writeIMAGIC("witheuler.hed",-1);
		}
		avg->add(d2);
		mean+=f;
		sigma+=f*f;
		m++;
	}
	else if (badf) d->writeIMAGIC("nogood.hed",-1);
	if (stat) fprintf(out,"%f\n",f/norm);
	printf("%6.1f %c  ",f/norm,f>cut?' ':'*');
	if (i%8==0) putchar('\n');
	delete d2;
//	putchar('.'); fflush(stdout);
}
putchar('\n');
if (!r1) {
	avg->cmCenter();
	avg->rotateAndTranslate();
}
if (stat) fclose(out);

mean/=(float)m;
sigma=sqrt(sigma/(float)m-mean*mean);
printf("Mean=%f\tSigma=%f\n%0.0f/%d good images\n",mean/norm,sigma/norm,m,data->count());

if (locffile!=-1) {
	locffile=atoi(argv[1]+strlen(argv[1])-7);
	if (saver) locffile*=2;
}

if (r1==2) {
	avg->rotAlign(first);
	f=avg->alt();
	f=f-PI*floor(f/PI+.5);
	printf("Final realign %4.2f  (%4.2f %4.2f %4.2f)\n",f*180.0/PI,eul[0]*180.0/PI,eul[1]*180.0/PI,eul[2]*180.0/PI);
	eul[2]-=f;
	if (saver) {
		first->setNImg(-1);
		first->normalize();
		first->writeIMAGIC("classes.hed",locffile);
		if (locffile>=0) locffile++;
	}
}
avg->mult(mask1);
avg->normalize();
avg->setRAlign(eul[0],eul[1],eul[2]);
avg->setNImg((int)m);
avg->writeIMAGIC("classes.hed",locffile);

return 0;
}


