// classalign2 ?/98  Steve Ludtke
// Copyright 1998 Baylor College of Medicine 

//N classalign2
//F Aligns a set of images iteratively to generate a class-average.
//T LA
//1
//P <input file>	Input file, usually a cls*img file
//P <iterations>	Number of iterative loops
//P [ref1]	Indicates that the first image in the file is a reference
//P [ign1]	Indicates that the first image is a dummy which should be ignored
//P [finalref]	Once the average has been generated, a final alignment to the reference is performed
//P [saveali]	Saves the individual aligned images in a separate file
//P [savecen]	Save the translationally (but not rotationally) aligned images
//P [savesteps]	Save the class average after each iterative step to monitor convergence
//P [badfile]	Save all excluded images in 'bad' files.
//P [stat]	Writes individual image statistics to 'class.stat'
//P [saverefs]	Write the reference image as well as the class average to the output file
//P [mask=<rad>]	Circular mask radius for the class average
//P [refmask]	This will use a mask generated from the reference image. Assumes the 3D model has been sensibly masked.
//P [refmaskali]	This will use a mask generated from the reference image during alignment, but will not apply it to the final model.
//P [maxshift=<rad>]	Maximum translation allowed in alignment. If not specified, implied value from mask is used.
//P [keep=<sigmamult>]	The threshold value for keeping images. Standard deviation multiplier	
//P [locfromfile]	Determine the output file image position from the input filename.
//P [median]	When CTF correction is NOT used, this produces better statistics.
//P [nweight]	Individual images are actually averages and NImg should be used in weighting. No effect with CTF correction or median.
//P [ctfc=<res>]	Enables CTF correction, thru resolution <res> in angstroms.
//P [ctfcw=<SF file>]	Enables CTF correction, with Wiener filtration, requires s vs inten struct. factor file.
//P [phaseopt]	In conjunction with ctfcw=, this will perform experimental per-pixel optimal phase-flipped averaging (experimental!)
//P [ctfcw]	Enables CTF correction, with Wiener filtration, SNR is estimated from the data (DOESN'T WORK)
//P [fixac=<0 to 1>]	This allows you to set the %AC value in the image headers to a fixed value. Note that this will NOT compensate defocus for change in %AC.
//P [imask=<rad>]	Inside mask radius.
//P [sigfilt]	Filters the class averages so only intense values are used in alignment
//P [even|odd]	Only use the even or odd images in the final average
//P [sigmaimg]	Generate a standard deviation image
//P [logit=<label>]	This specifies a label to use in the particle.log file. This records particle number classification throughout a refinement.
//P [refine]	This enables 1/2 pixel accuracy in alignment, slows alignment down quite a bit.
//P [usefilt]	This will use a filtered dataset for alignment, but uses the real data for averaging.
//P [slow]	This will do alignment via exhaustive search. It is VERY slow.
//P [phase]	This will use phase error as a similarity criteria
//P [fsc]	This will use weighted FSC as a similarity criteria
//P [frm2d[=<rad>,<samp>]]	This will classify using Fast Rotational Matching 2D. rad is the maximum radius of particles; samp is the number of angular sampling points at each radius circle
//P [classalign2log]	Together with logit, save the particles that were used in the class average to classalign2.log. [A particle# image_file alt,az,Dx,Dy logit]
//U classalign2 cls000.hed 8 ref1 finalref saverefs mask=32 keep=.8 locfromfile ctfc=15
//D This is one of the most important programs in EMAN. It is responsible for taking a set
//E of particles within a single class, that is, they are supposed to be in the same 
//E orientation, and performing a mutual alignement to generate a class-average. This 
//E program is largely responsible for the model independance of EMAN, ie - why EMAN can 
//E converge in so few iterations. This program is also where CTF amplitude correction occurs. 

//D There are quite a few options for this program. It is typically called from classalignall,
//E which has almost identical options. The few options that are present here, but not in 
//E classalignall, are intended for occasional use when checking robustness of the proceedures. 

//D There are 3 manditory options which are specified in all cases. First, is the
//E &lt;input file>. This file contains the images which are to be used to generate a class
//E average. Optionally, the first image in this file may be a reference image with
//E assigned euler angles. The reference will be used only during the first iteration
//E of the alignment process, so the final class average is nearly independent of this
//E reference. It also provides a final rotational orientation for the class average. 

//D The number of iterations (with no prefixed text) and the
//E 'keep=&lt;sigmamult>' values must also be specified. This program functions
//E by: first, aligning all of the images to a common reference, then
//E averaging the aligned images together to generate a new reference. This
//E reference is then used for another round of alignment, etc. The number
//E of iterations specifies how many times this process should repeat.
//E Each time, the images should generate a class average more consitent
//E with the data. In addition, after each loop, each aligned image is
//E compared with the class average from the previous iteration. If the
//E quality of the match is worse than a certain threshold value, the image
//E is discarded. Each iteration this threshold is made more stringent
//E until &lt;sigmamult> is reached in the final iteration. The cutoff value
//E is determined by calculating the standard deviation of all of the match
//E qualities and multiplying by &lt;sigmamult>. The smaller &lt;sigmamult> is,
//E the more consistent the images in the final class average will be, but
//E as more images are excluded, the class average will also become more
//E noisy. Similarly, increasing the number of iterations will increase the
//E consistency of the final class average, but will also typically cause
//E more data to be discarded. There are no 'correct' values to use. There
//E is always a tradeoff between data homogeneity and noise level.  Typical
//E starting values are 8 iterations with &lt;sigmamult>=1.2. &lt;sigmamult> is
//E usually >= 1.0, but may be lower when data homogenity is very poor. More
//E iterations can always be used, but 8 is usually sufficient to converge
//E quite well.

//D When the program runs, it displays some useful information on the
//E screen. First you will see 1 or more rows of '.'s appear. Each '.'
//E represents an image being aligned. If a '*' appears instead of a '.',
//E that means that particular image was below the threshold and discarded.
//E Note that '*' images are only excluded from the average during a
//E particular cycle. They may be included again in later cycles if the
//E class average 'wanders' in their direction.

//D If your input file contains a reference image (which the other images
//E are expected to look at least vaguely similar to), then 'ref1' should be
//E specified. 'finalref' implies ref1, and both should not be specified. Normally the initial
//E reference is used only during the first iteration, however, during
//E several iterations the orientation of the class average in the plane
//E can drift. 'finalref' insures that the final class average is oriented
//E as closely as possible to the orientation of the initial reference. It
//E does not change the appearance of the class average, only its orientation.

//D 'saverefs' is also usually specified with 'ref1'. This causes the
//E reference images to be saved along with the class averages in the output
//E file.  This is used to compare each reference with the corresponding class
//E average. When the refinement cycle begins to converge, the reference
//E images should be nearly identical to the class averages, although the
//E class averages will always be noisier.

//D 'saveali', 'savesteps', 'badfile' and 'stat' are all used primarily when
//E something seems to be going wrong. They cause various output files to be
//E generated, which can then be used to see exactly what's happening. They
//E are rarely used in a normal refinement cycle.

//D The default output image from classalign2 is a class average. This means,
//E each pixel is equal to the average of that pixel's value in each of the
//E aligned images. However, very often the median value will have better
//E properties than the mean value (less affected by outliers). Median
//E images can be generated instead of averages by specifying the 'median'
//E option. Note that this option is incompatible with CTF correction (see
//E 'ctfc' below).

//D Usually, classalign2 will simply append each output class average to the
//E end of 'classes.hed'. However, if the program is being run in parallel,
//E it is usually desirable to preserve the ordering of the images in the
//E output file. 'locfromfile' will cause the output images to be put in
//E specific 'slots' in the output file, overwriting any image already in that
//E location in the file. This is used by default when classalignall is run.

//D 'sigfilt' is used when a refinement is failing to produce consistent
//E results in the early rounds of refinement. This applies a non-linear
//E noise reducing filter to the reference image to try to get better
//E initial alignment. It is generally only used when necessary, and then,
//E only for early refinement rounds. It should not be used when the final
//E few refinement iterations are performed.

//D 'ctfc' and 'ctfcw' are provided to enable CTF correction when
//E generating class averages. For this to work properly all of the images
//E in the input file must be pretreated by the ctfit program, so the
//E phases are flipped and the CTF parameters are stored in the individual
//E image headers. See the documentation for that program 
//E and <a href=../ctfc/ctfc.html> the CTF correction primer</a>
//E for a discussion of how CTF correction
//E is performed. ctfc is specified with a filter resolution for the final
//E image. Otherwise high resolution noise will dominate. eg - 'ctfc=12' will
//E apply a 12A filter to the final image. NOTE: This IS specified in Angstroms,
//E not pixels like most other filter options.
//D 'ctfcw' is the preferred variant, but requires an x-ray scattering curve
//E or a simulated x-ray scattering curve. This curve MUST be used directly
//E with ctfit when the data is pretreated, or the class average will be 
//E improperly filtered. This option applies a true Wiener filter to the
//E data based on the SNR (signal to noise ratio) estimated from the data
//E combined with the scattering curve. eg- 'ctfcw=groel.sm' would ctf-correct
//E the data with an applied wiener filter. groel.sm would be the 2-column
//E data file defining the x-ray scattering curve.

//D 'frm2d' better used in combination with maxshift, which indicates the 
//E center of mass uncertainty. 


#include <cstdio>
#include <cmath>
#include <cctype>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include "XYData.h"

int main(int argc, char *argv[]) {
int i,j,k,n,ny,iter=2,r1=0,savesteps=0,badf=0,stat=0,ign1=0,usephase=0,nweight=0;
int refine=0,saver=0,half=0,sigfilt=0,savecen=0,filtali=0,slow=0,dfilt=0,phaseopt=0;
int mask=-1,imask=-1,locffile=-1,median=0,saveali=0,ctfc=0,precen=0,quiet=0,maxshift=-1,radon=0;
int classalign2log=0;
int frm2d=0,ccc=0,size=-1,maxshift_input=-1;			//--ycong
List *data,*dataf,*drad,*dradf,*d2a,*d2ma,*dataa=NULL;
EMData *d,*da,*df,*d2,*avg,*oavg=NULL,*first=NULL,*firstf=NULL,*mask1,*masknorm=NULL,*tmp,*sigim=NULL;
EMData *reforig=NULL;
char s[1024],*ctfcw=NULL,*logit=NULL;
float f,m,mean,sigma,mean2,sigma2,cut,keep=1.2,norm,filt=0,fixac=-1.0;
float eul[3];
float *ctf;
float dot_frm=0.0,rmaxH=-1.0f;					//--ycong
FILE *out;
XYData SF;
char* classfile = "classes.hed";

appinit(argc,argv);

if (argc<2) { appusage(argv[0]); exit(0); }

for (i=2; i<argc; i++) {
	if (strncmp("ref1",argv[i],4)==0) r1=1;
	else if (strncmp("ign1",argv[i],4)==0) ign1=1;
	else if (strncmp("finalref",argv[i],8)==0) r1=2;
	else if (strncmp("savesteps",argv[i],9)==0) savesteps=1;
	else if (strncmp("badfile",argv[i],7)==0) badf=1;
	else if (strncmp("sigfilt",argv[i],7)==0) sigfilt=1;
	else if (strncmp("sigmaimg",argv[i],8)==0) sigim=new EMData;
	else if (strncmp("saverefs",argv[i],8)==0) saver=1;
	else if (strncmp("saveali",argv[i],7)==0) saveali=1;
	else if (strncmp("savecen",argv[i],7)==0) savecen=1;
	else if (strncmp("precen",argv[i],6)==0) precen=1;
	else if (strncmp("phaseopt",argv[i],8)==0) phaseopt=1;
	else if (strncmp("radon",argv[i],5)==0) radon=1;
	else if (strncmp("dfilt2",argv[i],6)==0) dfilt=2;
	else if (strncmp("dfilt",argv[i],5)==0) dfilt=1;
	else if (strncmp("slow",argv[i],4)==0) slow=1;
	else if (strncmp("stat",argv[i],4)==0) stat=1;
	else if (strncmp("ctfc=",argv[i],5)==0) { ctfc=1; filt=atof(&argv[i][5]); }
	else if (strncmp("ctfcw=",argv[i],6)==0) { ctfc=2; ctfcw=&argv[i][6]; }
	else if (strncmp("ctfcw",argv[i],5)==0) { ctfc=2; }
	else if (strncmp("fixac=",argv[i],6)==0) fixac=atof(&argv[i][6]);
	else if (strncmp("mask=",argv[i],5)==0&&mask!=-2) mask=atoi(&argv[i][5]);
	else if (strncmp("maxshift=",argv[i],9)==0) maxshift=atoi(&argv[i][9]);
	else if (strncmp("refmaskali",argv[i],10)==0) mask=-3;
	else if (strncmp("refmask",argv[i],7)==0) mask=-2;
	else if (strcmp("phase",argv[i])==0) usephase=1;
	else if (strncmp("fsc",argv[i],3)==0) usephase=2;
	else if (strncmp("imask=",argv[i],6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp("keep=",argv[i],5)==0) keep=atof(&argv[i][5]);
	else if (strncmp("locfromfile",argv[i],11)==0) locffile=1;
	else if (strncmp("median",argv[i],6)==0) median=1;
	else if (strncmp("nweight",argv[i],7)==0) nweight=1;
	else if (strncmp("refine",argv[i],6)==0) refine=1;
	else if (strncmp("even",argv[i],4)==0) half=2;
	else if (strncmp("odd",argv[i],3)==0) half=1;
	else if (strncmp("usefilt",argv[i],7)==0) filtali=1;
	else if (strncmp(argv[i],"logit=",6)==0) logit=&argv[i][6];
	else if (strncmp("quiet",argv[i],5)==0) quiet=1;
	else if (isdigit(argv[i][0])) iter=atoi(argv[i])-2;
	else if (strncmp(argv[i],"usefs=",6)==0) ;
	else if (strncmp(argv[i],"classalign2log",14)==0) classalign2log=1 ;
	else if (strncmp("frm2d",argv[i],5)==0) {		//ycong
		frm2d=1;
		if (argv[i][5] == '=') sscanf(argv[i]+6,"%f,%d",&rmaxH,&size);
	}
	else if (strncmp("ccc",argv[i],3)==0) ccc=1;		//ycong
	//else if (strncmp("ccc",argv[i],3)==0) ccc=1;
	else if (strncmp(argv[i],"classfile=",10)==0) classfile=&argv[i][10];
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

if(frm2d) maxshift_input=maxshift;		// ycong for frm2d

if (r1==1) iter++;
if (iter<1) exit(1);
if (ctfc) median=0;
if (mask==-2 && !r1) error(ERR_EXIT,"'refmask' requires 'ref1'. I need a reference to generate the mask from!");

//if (imask>0 && !precen) { printf("The imask option must be used in conjunction with precen\n"); exit(0); }

// This reads the structure factor file used for Wiener filtration, and takes the required log10()
if (ctfc==2) {
	if (ctfcw) {
		if (ctfcw && SF.readFile(ctfcw)) { printf("ERROR: Couldn't read structure factor file!\n"); exit(1); }

		float *d=SF.getData();
		for (i=1; i<SF.n()*2; i+=2) d[i]=(d[i]<=0)?-4.0:log10(d[i]);
		SF.update();
	}
}

printf("Classalign: %s\n",argv[1]);

n=fileCount(argv[1]);
//if (half) n/=2;
if ((n<3 && !r1) || (n<2 && r1) || (n<3 && half)) { printf("Not enough elements\n");  exit(0); }
if (n<10) iter=1;	// only do 2 passes if <10 data sets to average
if (n<3) iter=0;	// only 1 pass if a single image

if (filtali) {
	if (r1) {
		EMData *ed=new EMData;
		ed->readImage(argv[1],0);
		data=readImagesExt(argv[1],".filt",1,n);
		data->insertObjectAt(ed,0);
	}
	else data=readImagesExt(argv[1],".filt",0,n);
	dataa=readImages(argv[1],0,n);
}
else data=readImages(argv[1],0,n);		// read all input images
if (ign1) data->removeObjectAt(0);

if (data==NULL) error("Cannot read input",ERR_EXIT);
printf("Read %d images.\n",data->count());

/*
if (half) {
	n=data->count();
	if (r1) {
		if (half==1) {
			for (i=n-1; i>=1; i-=2) data->removeObjectAt(i);
			if (dataa) { for (i=n-1; i>=1; i-=2) dataa->removeObjectAt(i); }
		}
		else {
			for (i=n-2; i>=1; i-=2) data->removeObjectAt(i);
			if (dataa) {for (i=n-2; i>=1; i-=2) dataa->removeObjectAt(i); }
		}
	}
	else {
		if (half==1) {
			for (i=n-1; i>=0; i-=2) data->removeObjectAt(i);
			if (dataa) { for (i=n-1; i>=0; i-=2) dataa->removeObjectAt(i); }
		}
		else {
			for (i=n-2; i>=0; i-=2) data->removeObjectAt(i);
			if (dataa) { for (i=n-2; i>=0; i-=2) dataa->removeObjectAt(i); }
		}	
	}
	n=data->count();
	printf("Processing %d images\n",n);
}
*/

int Ref=0;
if (!quiet) {
	Ref=LOGbegin(argc,argv);
	LOG(Ref,argv[1],LOG_INFILE,NULL);
}

if (mask==-2 || mask==-3) {
	mask1=((EMData *)data->objectAt(0))->copy(0,0);
	mask1->realFilter(6);	// value != 0 -> 1
	masknorm=mask1->copy(0,0);
	masknorm->addMaskShell(1);
	masknorm->subtract(mask1);
	masknorm->addMaskShell(2);
//mask1->writeIMAGIC("mask.hed",0);
//masknorm->writeIMAGIC("mask.hed",1);
}
else {
	mask1=((EMData *)data->objectAt(0))->copyHead();
	mask1->one();
	mask1->applyMask(mask,4);
}
//mask1->writeMRC("mask.img");

d2a=new List;
d2ma=new List;

dataf=new List;
for (i=0; i<n; i++) {
	d=(EMData *)data->objectAt(i);
	if (fixac>=0 && d->hasCTF() && ctfc) d->getCTF()[3]=fixac;
//	d->normalize();
	d->edgeNormalize(1);	// normalize to circular edgemean
//if (i==0) d->writeIMAGIC("abc.hed",-1);
//	d->applyMask(-1,0);
	d2=d->copy();
	d2->hFlip();
	d2->setParent(NULL);
	dataf->addObject(d2);

	if (dataa) {
		d=(EMData *)dataa->objectAt(i);
		d->edgeNormalize();
	}	
	
//	d=d->copy();
}

if (radon) {
	printf("Precalculate Radon transforms\n");
	drad=new List;
	dradf=new List;
	
	for (i=0; i<n; i++) {
		if (i%10==0) { printf(" %8d        \r",i); fflush(stdout); }
		d=(EMData *)data->objectAt(i);
		drad-> addObject(d->doRadon());
		d=(EMData *)dataf->objectAt(i);
		dradf->addObject(d->doRadon());
	}
}

d=(EMData *)data->objectAt(0);
ny=d->ySize();
if (maxshift<0) {
	if (mask<=0) maxshift=ny/8; else maxshift=ny/2-mask;
}

if (sigim) { sigim->setSize(ny,ny,1); sigim->zero(); }

// If we're starting with a reference, we don't need the first pass
if (r1) {
	avg=d=(EMData *)data->objectAt(0);
	first=avg->copy();
	first->setParent(NULL);
	data->removeObjectAt(0);
	dataf->removeObjectAt(0);
	if (dataa) dataa->removeObjectAt(0);
	if (mask<0) {
		norm=d->xSize()*d->ySize()*PI/4000.0;
	}
	else{
		norm=mask*mask*PI/1000.0;
	}
	eul[0]=d->alt();  
	eul[1]=d->az();  
	eul[2]=d->phi();
	printf("Eulers %f,%f,%f\n",eul[0]*180.0/PI,eul[1]*180.0/PI,eul[2]*180.0/PI);
	n--;
}

// simple shell sort, fast enough
d=(EMData *)data->objectAt(0);
if ((d->hasCTF()||d->hasCTFF()) && ctfc && (!filtali)) {
	for (i=0; i<n-1; i++) {
		d=(EMData *)data->objectAt(i);
		for (j=i+1; j<n; j++) {
			d2=(EMData *)data->objectAt(j);
			if (d->CTFCmp(d2)<0) {
				data->swap(i,j);
				dataf->swap(i,j);
				if (dataa) dataa->swap(i,j);
				d=(EMData *)data->objectAt(i);
			}
		}
	}
}


/********************************************************
*             parameter settings for frm2d 		*
*		       9/27/06  ycong  			*
********************************************************/
float p_max,rmaxL;
float com_with_x=0.0, com_with_y=0.0, com_self_x=0.0, com_self_y=0.0;
float r_size;		// possible r value, to allocate h_haH array 
float MAXR;			// maximum r values for sampling the densities
float tstep=1.0;	// radius (linear) search step size (in pixel), default is 1 pix
float angstep;		// angular sampling points at each radius circle, i.e. 32, 64, 128, 256
float rhomax=-0.1, rhomin=0.0;		// not really in use in the current version
float *Int_g;
//float *com_pre;
//FILE *in_com;	

if(frm2d) {
	if (size<0) {
		int do360=1;
		size = (int) floor(do360*PI*d->ySize()/4);
		size = bestfftsize(size);
	}
	
	if(maxshift_input > 0) {
		p_max=float(maxshift_input);
		if(p_max > maxshift) p_max=maxshift;
	}
	else p_max=maxshift;
	
	if(rmaxH < 0){
		if(mask<=0) rmaxH=d->ySize()/2.0;
		else rmaxH=float(mask);
	}
	else if(ceil(rmaxH) > mask && mask>0) {
		fprintf(stderr,"WARNING: the maximum radius of particles (%6.1f) should be <= the radius of mask (%d), reset to the mask radius \n",rmaxH,mask);	
		rmaxH = float(mask);
	}
	else if(rmaxH > d->ySize()/2.0) {
		fprintf(stderr,"WARNING: the maximum radius of particles (%6.1f) should be <= half of the box size (%d), reset to half of the box size \n",rmaxH,d->ySize()/2);
		rmaxH = d->ySize()/2.0;
	}
	else rmaxH=rmaxH;
		
	rmaxL = rmaxH;			
	r_size=rmaxH+p_max;			
	
	if(mask<=0) MAXR=d->ySize()/2.0+p_max;
	else        MAXR=float(mask)+p_max;
	
	angstep=size;

	//printf("size=%d, p_max/maxshift=%f,%d, rmaxH=%f, r_size=%f, MAXR=%f, y=%d\n",size,p_max,maxshift,rmaxH,r_size,MAXR,d->ySize() );
	//d->writeIMAGIC("d.hed",-1);

	/*if(rmaxH >= rmaxL)
		rhomax=rmaxH-rminL;			// rhomax -> maximum value of rho 
	else
		rhomax=rmaxL-rminH;
	if(p_max < rhomax) rhomax = p_max;
	*/
	
	int max_ext=(floor)(sqrt((float)(d->xSize()*d->xSize()+d->ySize()*d->ySize()))+0.5);
	
	Int_g = new float[max_ext+1];
	if(Int_g == NULL ){
		cout <<"can't allocate more memory, terminating 005. \n";
		exit(1);
	}

 
	float* sum_samp = new float[((int)MAXR+1)*size];
	if(sum_samp == NULL ){
		cout <<"can't allocate more memory, terminating 006. \n";
		exit(1);
	}
	for (int m=0;m<size*(int)(MAXR+1);m++)
		sum_samp[m] = 0.0;
	
	if (!r1) {			// ref1==0, align to the first image in the stack,  ycong 10/29/06
		avg=(EMData *)data->objectAt(0);
	}
	//avg->writeIMAGIC("avg.hed",-1);

	EMData *d_ref = avg->copy(); 
	
	//d_ref->norm_frm2d();		// image will be normalized in unwrap_largerR()
	
	float com_x=0.0f, com_y=0.0f;
	d_ref->COM(1, com_x, com_y); 
	//printf("com_x,com_y=%f,%f\n",com_x,com_y);
	
	EMData* ref_pcs = d_ref->unwrap_largerR(0,(int)MAXR,size,rmaxH);
	delete d_ref;
	
	float *data_ref=ref_pcs->getDataRO();
	for (int r=0;r<=(int)rmaxH;r++)
		for (int k=0;k<size;k++) {
			sum_samp[k+r*size] += data_ref[k+r*size];
			//if(sum_samp[k+r*size] !=0) printf("j=%d, r=%d, k=%d, %f, %f  \n ",j,r,k, sum_samp[k+r*size], data_ref[k+r*size]);
		}
	ref_pcs->doneData();
	delete ref_pcs;
	data_ref = NULL;

	for(int r=0;r<=max_ext;r++) {
		Int_g[r]=0.0f;
		if (r<=(int)MAXR){
			for (int i=0;i<size;i++) {
				Int_g[r] += sum_samp[i+r*size];		
			}
		}
	}
	delete[] sum_samp; 

	// read COM from predetermined result
	/*com_pre = new float[(n+1)*4];
	if(com_pre == NULL ){
		cout <<"can't allocate more memory, terminating 007. \n";
		exit(1);
	}
	
	in_com=fopen("com_predeterm.txt","r");
	if(frm2d && in_com) { 
		for(int nt=0;nt<n;nt++) {
			int num=0, yes=0;
			float xx=0.0,yy=0.0;
	
			fscanf(in_com, "%d, %f, %f, %d \n",&num, &xx,&yy, &yes); 
			com_pre[0+nt*4]=(float)(num);
			com_pre[1+nt*4]=(float)(floor(xx+0.5));
			com_pre[2+nt*4]=(float)(floor(yy+0.5));
			com_pre[3+nt*4]=(float)(yes);
			//printf("n=%d, %f, %f, %f, %f \n ",n, com_pre[0+n*4],com_pre[1+n*4], com_pre[2+n*4],com_pre[3+n*4]);
		}
		fclose(in_com);
	} */
}
//--------------ycong-----------------------

if (!r1) {
	// pass 1, align to the first data set
	d=(EMData *)data->objectAt(0);
	if (mask<0) {
		norm=d->xSize()*d->ySize()*PI/4000.0;
	}
	else{
		norm=mask*mask*PI/1000.0;
	}
	avg=d->copy();
	//-------ycong---------
	float* raw_hhat = 0;
	if (frm2d) {	// 1Dfft for reference image  !r1 == no reference image, align to the first image
		
		EMData *avg_frm=avg->copy();
		avg_frm->COM(1, com_with_x, com_with_y);
		 
		//EMData* withpcs = avg->unwrap(0,MAXR,size,0,0,1);       //~~~ assign value for MAXR and size
		EMData *withpcs = avg_frm->unwrap_largerR(0,int(MAXR),size,rmaxH); 
		EMData *withpcsfft = withpcs->oneDfftPolar(size,rmaxH,MAXR);
		delete avg_frm;
		delete withpcs;
		
		if(raw_hhat != 0) { 
			free(raw_hhat); 
			raw_hhat=0;
		}
		raw_hhat= withpcsfft->makeFRM2D_H_hat(rhomax, rhomin, rmaxH, size, tstep, r_size);
		delete withpcsfft;
	}
	//-------ycong---------
	
	for (i=1; i<n; i++) {
		d=(EMData *)data->objectAt(i);
		df=(EMData *)dataf->objectAt(i);
		if (precen) d2=d->RFAlign(avg,df);
		else if (radon) d2=d->RTFAlignRadon(avg,maxshift,df,NULL,(EMData *)drad->objectAt(i),(EMData *)dradf->objectAt(i));
		else if (slow) d2=d->RTFAlignBest(avg,df,maxshift);
		else if (frm2d) {	//ycong
			float dot_frm=0.0;
			
			Euler e = *d->getEuler();

			EMData *d_frm=d->copy();			// d: images need to be aligned
			/*if(in_com && com_pre[3+i*4] == 0.0) {
				float dxx = com_pre[1+i*4];
				float dyy = com_pre[2+i*4];
				com_self_x = d_frm->xSize()/2 - dxx;
				com_self_y = d_frm->ySize()/2 - dyy;
			}		
			else{*/
				d_frm->COM(3, com_self_x, com_self_y);		// in COM(), the object has been moved to the COM except "3" option
				d_frm->exact_COM(1,Int_g, com_self_x, com_self_y,rmaxL);
			//}
			delete d_frm;
			
			int neg_rt=2;
			d2=d->frm2dAlignFlip(df, dot_frm, avg,0, raw_hhat, rmaxL, MAXR, tstep, angstep, r_size, com_self_x, com_self_y, com_with_x, com_with_y,neg_rt);

			if(refine) d2->refineAlign(avg);
			d->setRAlign(&e);
		}
		else d2=d->RTFAlign(avg,df,1,maxshift);
//		d2->mult(mask1);
		d2->applyMask(mask,6);
		avg->add(d2);
		delete d2;
		putchar('o'); fflush(stdout);
	}
	free(raw_hhat);	//ycong
	raw_hhat=NULL;	//ycong
	putchar('\n');
	if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	if (!precen) {
		avg->cmCenter();
		avg->rotateAndTranslate();
		if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	}
	//if(frm2d) delete withpcsfft;
}

float *c1,*c2;
if (dfilt) {
	c1=(float *)malloc(sizeof(float)*4*ny);
	c2=(float *)malloc(sizeof(float)*4*ny);
}

#ifdef DEBUG
printf("DEBUG 1\n");
#endif

if (n==1) iter=0;
cut=0;
for (j=0; j<iter; j++) {
// pass 2-?, align to the first average
	mean=sigma=m=0;
	if (reforig && reforig!=oavg) delete reforig;
	if (oavg) { delete oavg; oavg=NULL; }
	reforig=avg;
//	oavg->writeIMAGIC("aaa.hed",-1);
	float* raw_hhat = 0;	//ycong
	for (i=0; i<n; i++) {
		d=(EMData *)data->objectAt(i);		// data to align
		df=(EMData *)dataf->objectAt(i);	// flipped version of data

		EMData *ff=d->doFFT();		// for dfilt
		if (dfilt) ff->calcRadDist(ny,0,.5,c1);

		// if there's a CTF, modify the reference by the same CTF
		if(dfilt||(d->hasCTF()||d->hasCTFF()) && ctfc && (!filtali)) {
			// if several images in a row have the same ctf, don't recalculate
			if (dfilt || !oavg || d->CTFCmp(oavg)) {
				if (oavg) delete oavg;
				float *ctfp=d->getCTF();
				float B=ctfp[1];		// temporarily sets B factor to 0, doesn't work for CTFF
				ctfp[1]=0;
				ctf=d->ctfCurve(2);
				d2=reforig->doFFT()->copy();
				if (dfilt) {
					d2->calcRadDist(ny,0,.5,c2);
					for (int jj=0; jj<ny; jj++) {
						if (c1[jj]>0 && c2[jj]>0) c2[jj]=sqrt(c1[jj]/c2[jj]);
						else c2[jj]=0;
					}
					d2->applyRadFn(ny,0,.5,c2);
					if (dfilt==2) {
						float *snr2=d->ctfCurve(5,&SF);		// Wiener filter (5)
						for (int jj=0; jj< CTFOS*d->ySize()/2; jj++) snr2[jj]=snr2[jj]<=0?0.0:1.0/(1.0+sqrt(1.0/snr2[jj]));  
						d2->applyRadFn(CTFOS*d->ySize()/2,0,1.0/CTFOS,snr2,1);
						free(snr2);
					}
				}
				else {
					d2->applyRadFn(CTFOS*d->ySize()/2,0,1.0/CTFOS,ctf,0);
				}
				ctfp[1]=B;
				oavg=d2->doIFT();
				d2->gimmeFFT();
				if (masknorm) oavg->maskNormalize(masknorm);
				else oavg->edgeNormalize();
//oavg->writeMRC("tst1.mrc");
				oavg->mult(mask1);
				oavg->setParent(NULL);
				if (d->hasCTFF() && ctfc) oavg->setCTFF(d->Name());
				else oavg->setCTF(d->getCTF());
				delete d2;
				free(ctf);
//oavg->writeIMAGIC("aaa.hed",-1);
			}
		}
		else {
			if (masknorm) reforig->maskNormalize(masknorm);
			else reforig->edgeNormalize();
			reforig->mult(mask1);
			oavg=reforig;
		}
		
		//----------ycong---------
		if (frm2d && ((!dfilt && i==0) || dfilt) ) {		// get 1Dfft of the ref.(averaged) image
									// if dfilt, the oavg is updated for every raw image
									// so the 1Dfft and hhat need update  7/19/04
			//oavg->writeIMAGIC("oavg.img",-1);
			EMData *oavg_frm = oavg->copy();
			oavg_frm->COM(1, com_with_x, com_with_y); 
			EMData *withpcs = oavg_frm->unwrap_largerR(0,int(MAXR),size,rmaxH);  
			delete oavg_frm;
			EMData *withpcsfft = withpcs->oneDfftPolar(size, rmaxH, MAXR);
			delete withpcs;
			
			if(raw_hhat != 0) { 
				free(raw_hhat); 
				raw_hhat = 0;
			}
			raw_hhat = withpcsfft->makeFRM2D_H_hat(rhomax, rhomin, rmaxH, size, tstep, r_size);
			delete withpcsfft;
		}
		//----------ycong---------

		if (sigfilt) oavg->realFilter(1,oavg->Sigma());
		
		if (precen) d2=d->RFAlign(oavg,df);
		else {
			if (radon) d2=d->RTFAlignRadon(oavg,maxshift,df,NULL,(EMData *)drad->objectAt(i),(EMData *)dradf->objectAt(i));
			else if (slow) d2=d->RTFAlignBest(oavg,df,maxshift);
			else if (frm2d) {		//-------------ycong -------------
				Euler e = *d->getEuler();
				float dot_frm=0.0;
				EMData *d_frm = 0;
								
				d_frm=d->copy();
				/*if(in_com && com_pre[3+i*4] == 0.0) {
					float dxx = com_pre[1+i*4];
					float dyy = com_pre[2+i*4];
					com_self_x = d_frm->xSize()/2 - dxx;
					com_self_y = d_frm->ySize()/2 - dyy;
				}		
				else {*/
					d_frm->COM(3, com_self_x, com_self_y);   // 3: don't move the object to COM; 
					d_frm->exact_COM(1,Int_g, com_self_x, com_self_y,rmaxL);	// object will be moved to COM
				//}
				//oavg->writeIMAGIC("oavg.hed",-1);
				//printf("#%d,  com=%f, %f\n", i+1, 150.0-com_self_x, 150.0-com_self_y);
				//continue;
				delete d_frm;
				
				int neg_rt=2;
				d2=d->frm2dAlignFlip(df, dot_frm, oavg,0, raw_hhat, rmaxL, MAXR, tstep, angstep, r_size, com_self_x, com_self_y, com_with_x, com_with_y,neg_rt);
				if (i==(n-1) || dfilt) {
					free(raw_hhat);
					raw_hhat = NULL;
				}
				if(refine) d2->refineAlign(oavg);		//not in use previously  9/28/06
				//printf("1~~~Corr=%g, self.ang=%f, %f, %f \n\n",dot_frm,d2->alt()*180/PI, d2->Dx(), d2->Dy() );
				d->setRAlign(&e);
			}				//-------------ycong -------------
			else d2=d->RTFAlign(oavg,df,1,maxshift);
			if (refine && (radon || j>iter-4)) d2->refineAlign(oavg);
		}
		if (d->hasCTF() && ctfc) d2->setCTF(d->getCTF());
		if (d->hasCTFF() && ctfc) d2->setCTFF(d->Name());
//		d2->mult(mask1);
//		d2->applyMask(mask,6);
//		f=oavg->dot(d2);
		float scale=1.0,shift=0;
#ifndef ALTCMP
		if (usephase==1||dfilt) f=1000.0*(2.0-d2->pcmp(oavg,NULL));
		else if (usephase==2) f=500.0*(1.0+d2->fscmp(oavg,NULL));
		else f=250.0*(5.0-d2->lcmp(oavg,1,1,&scale,&shift));
#else
		d2->lcmp(oavg,1,1,&scale,&shift);	// sigh, need to do this better, but we need scale & shift
		f=20.*d2->fscmp(oavg);
#endif
		d2->ufloat=f;
		if (usephase) d2->normalizeTo(oavg,1,1);
		else {
			d2->multConst(scale);	// we must normalize each particle to the average before averageCTFCW
			d2->addConst(shift);
		}
//printf("%f %f %f\n",oavg->lcmp(d2),cut,f);
//d2->writeIMAGIC("aaa.hed",-1);
//printf(" %f,%f\n",f,cut);

		d2ma->addObject(d2);
		mean+=f;
		sigma+=f*f;

		putchar('?'); fflush(stdout);


/*		if ((f>cut || keep>5 || (i>n-3 && m<2)) && finite(f)) {
			d2a->addObject(d2);
			putchar('.'); fflush(stdout);
			mean+=f;
			sigma+=f*f;
			m++;
		} else {
			putchar('*'); fflush(stdout);	
			delete d2;
		}*/
	}
	free(raw_hhat);		//ycong
	raw_hhat = NULL;	//ycong
	putchar('\r');
	//if(frm2d) delete withpcsfft;  //~~~for frm2d

// calculate statistics
	mean/=(float)n;
	sigma=sqrt(sigma/(float)n-mean*mean);
	if (!finite(mean)) mean=1.0;
	if (!finite(sigma)) sigma=mean/4.0;
//	if (sigma==0) exit(0);		// down to 1 image, just quit
//	cut=mean-sigma*keep*(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0));
	cut=mean-sigma*keep*(4.0-(float)j*3.0/iter);

	mean2=sigma2=m=0;
// decide which images to keep
	for (i=0; i<n; i++) {
		d2=(EMData *)d2ma->objectAt(i);
//		f=oavg->dot(d2);
//		f=250.0*(5.0-d2->lcmp(oavg,1,1));
		f=d2->ufloat;
		
		if ((f>=cut || keep>5 || (i>n-3 && m<2)) && finite(f)) {
			putchar('.'); fflush(stdout);
			if (dataa) {
				da=((EMData *)dataa->objectAt(i))->copy(0,0);
				if (d2->isFlipped()) da->hFlip();
				da->setRAlign(d2->alt());
				da->setTAlign(d2->Dx(),d2->Dy(),0);
//printf("%f,%f,%f\n",d2->alt(),d2->Dx(),d2->Dy());
				da->rotateAndTranslate();
				if (da->hasCTFF() && ctfc) da->setCTFF(d2->Name());
				else da->setCTF(d2->getCTF());
				delete d2;
				d2=da;
			}
			d2->setTAlign(0,0);
			d2->setRAlign(eul[0],eul[1],eul[2]);
//			d2->mult(mask1);
			d2a->addObject(d2);
			mean2+=f;
			sigma2+=f*f;
			m++;
		} else {
			putchar('*'); fflush(stdout);		
			delete d2;
		}
	}
	putchar('\n');
	d2ma->empty();

	if (m==0) { printf("Not enough elements\n");  exit(0); }
	mean2/=(float)m;
	sigma2=sqrt(sigma2/(float)m-mean2*mean2);
	if (!finite(mean2)) mean2=1.0;
	if (!finite(sigma2)) sigma2=0;

	if (savesteps) oavg->writeIMAGIC("steps.hed",-1);

	avg=new EMData;
	if (median) avg->makeMedian(d2a); 
	else if (ctfc==1) avg->makeAverageCTFC(d2a,filt);
	else if (ctfc==2) {
		if (ctfcw) avg->makeAverageCTFCW(d2a,&SF,NULL,phaseopt);
		else avg->makeAverageCTFCWauto(d2a,"snr0.dat");
	}
	else avg->makeAverage(d2a,NULL,0,nweight);
//	avg->edgeNormalize();
	if (masknorm) avg->maskNormalize(masknorm);
	else {
		avg->multConst(1.0/avg->Sigma());
		avg->addConst(-avg->dCenter());
	}
	
//avg->writeMRC("tst2.mrc");
	avg->mult(mask1);
//	avg->applyMask(mask,6);
	
	for (i=0; i<d2a->count(); i++) { d2=(EMData *)d2a->objectAt(i); delete d2; }
	d2a->empty();
	

//	printf("Mean=%f\tSigma=%f\tCut=%f\twt=%f\n",mean/norm,sigma/norm,cut/norm,
//		(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0)));
//	printf("Mean=%f\tSigma=%f\n",mean2/norm,sigma2/norm);

	printf("Mean=%f\tSigma=%f\tCut=%f\twt=%f\n",mean2/norm,sigma2/norm,cut/norm,
		4.0-3.0*j/(float)iter);

	// if we have an original reference, align the early averages to it
	// just to get the orientation correct
	if (avg->Sigma()==0 || !finite(avg->Sigma())) printf("Bad Sum\n");

//	if (j<iter-2 && r1) {
	if (r1) {
		if (oavg) { delete oavg; reforig=NULL; }
		df=avg->copy();
		df->setParent(NULL);
		df->hFlip();
		if (precen) oavg=d->RFAlign(first,df);
		else if (radon) oavg=avg->RTFAlignRadon(first,maxshift);
		else if (slow) oavg=avg->RTFAlignBest(first,df,maxshift);
		else if (frm2d) {		//-------------- ycong -------------
			EMData *first_frm=first->copy();
			first_frm->COM(1, com_with_x, com_with_y);
			EMData *withpcs = first_frm->unwrap_largerR(0,int(MAXR),size,rmaxH);       //~~~ assign value for MAXR and size
			EMData *withpcsfft = withpcs->oneDfftPolar(size,rmaxH, MAXR);
			delete first_frm;
			delete withpcs;
			
			float dot_frm=0.0;
			float* raw_hhat = 0;
			
			//if(rmaxH >= rmaxL)
			//  rhomax=rmaxH-rminL;				// rhomax -> maximum value of rho 
			//else
			//  rhomax=rmaxL-rminH;
			//if(eha_rho<rhomax) rhomax = eha_rho;		// rho exhaustive search range (eha_rho) ~20 already big enough
			raw_hhat= withpcsfft->makeFRM2D_H_hat(rhomax, rhomin, rmaxH, size, tstep, r_size);
			delete withpcsfft;

			//oavg=avg->copy();
			EMData *avg_frm=avg->copy();
			avg_frm->COM(3, com_self_x, com_self_y);	// in COM function, the object has been moved to the COM
			avg_frm->exact_COM(1,Int_g, com_self_x, com_self_y,rmaxL);
			delete avg_frm;
			
			int neg_rt=2;
			oavg=avg->frm2dAlignFlip(df, dot_frm, first,0, raw_hhat, rmaxL, MAXR, tstep, angstep, r_size, com_self_x, com_self_y, com_with_x, com_with_y,neg_rt);
			free(raw_hhat);
			raw_hhat=NULL;
			if(refine) oavg->refineAlign(first);
			//printf("2~~Corr=%g, self.ang=%f, %f, %f \n\n",dot_frm,oavg->alt()*180.0/PI, oavg->Dx(), oavg->Dy() );
			
		}				//-------------- ycong -------------
		else oavg=avg->RTFAlign(first,df,1,maxshift);
//		printf("firstalign %p %5.2f %5.2f %5.2f\n",oavg,oavg->Dx(),oavg->Dy(),oavg->Daz()*180.0/PI);
		if (oavg) { 
			delete avg;
			delete df;
			oavg->setParent(NULL);
			avg=oavg;
			oavg=NULL;
			avg->setRAlign(0,0,0);
			avg->setTAlign(0,0,0);
			avg->update();
		}
	}
	if (avg->Sigma()==0 || !finite(avg->Sigma())) printf("Bad Sum\n");
	if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	if (!r1 && !precen) {
		avg->cmCenter();		// if we have an initial reference
		avg->rotateAndTranslate();
								// don't spoil the centering
		if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	}
}

if (savesteps && !quiet) LOG(Ref,"steps.hed",LOG_OUTFILE,NULL);
#ifdef DEBUG
printf("DEBUG 2\n");
#endif


// last pass, align to the final previous average
mean=sigma=m=0;
//oavg=avg;
//oavg->normalize();
if (reforig && reforig!=oavg) delete reforig;
if (oavg) { delete oavg; oavg=NULL; }
reforig=avg;
//reforig->normalize();

if (stat) out=fopen("class.stat","ab");
#ifdef DEBUG
printf("DEBUG 2a\n");
#endif

float* raw_hhat = 0;	//ycong
for (i=0; i<n; i++) {
	d=(EMData *)data->objectAt(i);
	df=(EMData *)dataf->objectAt(i);

	EMData *ff=d->doFFT();		// for dfilt
	if (dfilt) ff->calcRadDist(ny,0,.5,c1);

	// if there's a CTF, modify the reference by the same CTF
	if (dfilt||(d->hasCTF()||d->hasCTFF()) && ctfc && (!filtali)) {
		// if several images in a row have the same ctf, don't recalculate
		if (dfilt || !oavg || d->CTFCmp(oavg)) {
			if (oavg) delete oavg;
			float *ctfp=d->getCTF();
			float B=ctfp[1];		// temporarily sets B factor to 0, doesn't work for CTFF
			ctfp[1]=0;
			ctf=d->ctfCurve(2);
			d2=reforig->doFFT()->copy();
			if (dfilt) {
				d2->calcRadDist(ny,0,.5,c2);
				for (int jj=0; jj<ny; jj++) {
					if (c1[jj]>0 && c2[jj]>0) c2[jj]=sqrt(c1[jj]/c2[jj]);
					else c2[jj]=0;
				}
				d2->applyRadFn(ny,0,.5,c2);
				if (dfilt==2) {
					float *snr2=d->ctfCurve(4,&SF);
					d2->applyRadFn(CTFOS*d->ySize()/2,0,1.0/CTFOS,snr2,1);
					free(snr2);
				}
			}
			else {
				d2->applyRadFn(CTFOS*d->ySize()/2,0,1.0/CTFOS,ctf,0);
			}
			ctfp[1]=B;
			oavg=d2->doIFT();
			d2->gimmeFFT();
			if (masknorm) oavg->maskNormalize(masknorm);
			else oavg->edgeNormalize();
			oavg->mult(mask1);
			oavg->setParent(NULL);
			if (d->hasCTFF()) oavg->setCTFF(d->Name());
			else oavg->setCTF(d->getCTF());
			delete d2;
			free(ctf);
		}
	}
	else {
		if (masknorm) reforig->maskNormalize(masknorm);
		else reforig->edgeNormalize();
		reforig->mult(mask1);
		oavg=reforig;
	}
//oavg->writeIMAGIC("aaa.hed",-1);

	//----------ycong----------
	if (frm2d && ( (!dfilt && i==0) || dfilt) ) {		// 1Dfft for reference image
		EMData *oavg_frm=oavg->copy();
		oavg_frm->COM(3, com_with_x, com_with_y);
		oavg_frm->exact_COM(1,Int_g, com_with_x, com_with_y,rmaxH);             
		EMData *withpcs = oavg_frm->unwrap_largerR(0,int(MAXR),size, rmaxH);       //~~~ assign value for MAXR and size
		EMData *withpcsfft = withpcs->oneDfftPolar(size, rmaxH, MAXR);
		delete oavg_frm;
		delete withpcs;
		
		if(raw_hhat != 0) { 
			free(raw_hhat); 
			raw_hhat = 0;
		}
		raw_hhat= withpcsfft->makeFRM2D_H_hat(rhomax, rhomin, rmaxH, size, tstep, r_size);
		delete withpcsfft;
	}
	//----------ycong----------

#ifdef DEBUG
printf("DEBUG 2b\n");
#endif
	if (sigfilt) oavg->realFilter(1,oavg->Sigma());

	if (precen) d2=d->RFAlign(oavg,df);
	else if (slow) d2=d->RTFAlignBest(oavg,df,maxshift);
	else if (frm2d) {	//--------------ycong-------------
		float dot_frm=0.0;
		
		//d2=d->copy();
		EMData *d_frm=d->copy();
		/*if(in_com && com_pre[3+i*4] == 0.0) {
			float dxx = com_pre[1+i*4];
			float dyy = com_pre[2+i*4];
			com_self_x = d_frm->xSize()/2 - dxx;
			com_self_y = d_frm->ySize()/2 - dyy;
		}		
		else{*/
			d_frm->COM(3, com_self_x, com_self_y);       // in COM function, the object has been moved to the COM
			d_frm->exact_COM(1,Int_g, com_self_x, com_self_y,rmaxL);
		//}
		delete d_frm;
		//printf("com=%f, %f\n", com_self_x, com_self_y);
		
		int neg_rt=2;
		d2=d->frm2dAlignFlip(df, dot_frm, oavg, 0, raw_hhat, rmaxL, MAXR, tstep, angstep, r_size, com_self_x, com_self_y, com_with_x, com_with_y,neg_rt);
		if (i==(n-1) || dfilt ){
			free(raw_hhat);
			raw_hhat = NULL;
		}
		if(refine) d2->refineAlign(oavg);
		//printf("33~~Corr=%g, self.ang=%f, %f, %f \n\n",dot_frm,d2->alt()*180.0/PI, d2->Dx(), d2->Dy() );		
		
	} 			//--------------ycong-------------
	else {
		d2=d->RTFAlign(oavg,df,1,maxshift);
		if (refine) d2->refineAlign(oavg);
	}
	if (d->hasCTF() && ctfc) d2->setCTF(d->getCTF());
	if (d->hasCTFF() && ctfc) d2->setCTFF(d->Name());
//	d2->mult(mask1);
//	d2->applyMask(mask,6);
//d2->writeIMAGIC("aaa.hed",-1);
//	f=oavg->dot(d2);
	float scale=1.0,shift=0;
#ifndef ALTCMP
	if (usephase==1 || dfilt) f=1000.0*(2.0-d2->pcmp(oavg,NULL));
	else if (usephase==2) f=500.0*(1.0+d2->fscmp(oavg,NULL));
	else f=250.0*(5.0-d2->lcmp(oavg,1,1,&scale,&shift));
#else
	d2->lcmp(oavg,1,1,&scale,&shift);	// sigh, need to do this better, but we need scale & shift
	f=20.*d2->fscmp(oavg);
#endif

	d2->ufloat=f;
	if (usephase) d2->normalizeTo(oavg,1,1);
	else {
		d2->multConst(scale); // we must normalize each particle to the average before averageCTFCW
		d2->addConst(shift);
	}

	if (sigim) {
		tmp=d2->copy(0);
		tmp->normalizeTo(oavg);
		tmp->subtract(oavg);
		tmp->mult(tmp);
		sigim->add(tmp);
		delete tmp;
	}

	if (!nweight) d2->setNImg(1);
//	d2->setRAlign(eul[0],eul[1],eul[2]);
	d2ma->addObject(d2);
	mean+=f;
	sigma+=f*f;
	putchar('-'); fflush(stdout);

/*	if ((f>cut || keep>5) &&finite(f)) {
		d2->setNImg(1);
		d2->setRAlign(eul[0],eul[1],eul[2]);
		d2a->addObject(d2);
		mean+=f;
		sigma+=f*f;
		m++;
	}
	else {
		sprintf(s,"b%s",argv[1]);
		if (badf) d->writeIMAGIC(s,-1);
		delete d2;
	} 

	if (stat) fprintf(out,"%f\n",f/norm);
	printf("%6.1f %c  ",f/norm,f>cut?' ':'*');
	if (i%8==0) putchar('\n');
	fflush(stdout); */
#ifdef DEBUG
printf("DEBUG 2e\n");
#endif
}
putchar('\n');

if (sigim) {
	sigim->multConst(1.0/(float)n);
	sigim->realFilter(10,0,0,0);	// sqrt
	sigim->setRAlign(eul[0],eul[1],eul[2]);
	sigim->setNImg((int)n);
	sigim->writeIMAGIC("classsig.hed",-1);
}

#ifdef DEBUG
printf("DEBUG 3\n");
#endif

mean/=(float)n;
sigma=sqrt(sigma/(float)n-mean*mean);
if (!finite(mean)) mean=1.0;
if (!finite(sigma)) sigma=mean/4.0;
//if (sigma==0) exit(0);		// down to 1 image, just quit
//cut=mean-sigma*keep*(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0));
cut=mean-sigma*keep*(4.0-(float)j*3.0/iter);

mean2=sigma2=m=0;

// decide which images to keep
for (i=0; i<n; i++) {
	d2=(EMData *)d2ma->objectAt(i);
//	f=oavg->dot(d2);
//	f=250.0*(5.0-d2->lcmp(oavg,1,1));
	f=d2->ufloat;

	if (stat) fprintf(out,"%f\n",f/norm);
	printf("%6.1f %c  ",f/norm,f>=cut?' ':'*');
	if (i%8==0) putchar('\n');
	fflush(stdout);

	if ((f<=cut || keep>5) && logit) {
		char s[120];
		sprintf(s,"X\t%d\t%s\n",d2->PathN(),logit);
		FSsend(FS_TEXT,strlen(s)+1,"particle.log",s);
	}
	
	if ((f>=cut || keep>5) &&finite(f)) {
		if (dataa) {
			da=((EMData *)dataa->objectAt(i))->copy(0,0);
			if (d2->isFlipped()) da->hFlip();
			da->setRAlign(d2->alt());
			da->setTAlign(d2->Dx(),d2->Dy(),0);
//printf("%f,%f,%f\n",d2->alt(),d2->Dx(),d2->Dy());
			da->rotateAndTranslate();
			if (d2->hasCTFF() && ctfc) da->setCTFF(d2->Name());
			else da->setCTF(d2->getCTF());
			delete d2;
			d2=da;
		}
		d2->setTAlign(0,0);
		d2->setRAlign(eul[0],eul[1],eul[2]);
//		d2->mult(mask1);
		d2a->addObject(d2);			
		mean2+=f;
		sigma2+=f*f;
		m++;
	} else {
		sprintf(s,"b%s",argv[1]);
		if (badf) d2->writeIMAGIC(s,-1);
		delete d2;
	}
}
putchar('\n');
d2ma->empty();

#ifdef DEBUG
printf("DEBUG 4\n");
#endif

if (m==0) { printf("Not enough elements\n");  exit(0); }
mean2/=(float)m;
sigma2=sqrt(sigma2/(float)m-mean2*mean2);
if (!finite(mean2)) mean2=1.0;
if (!finite(sigma2)) sigma2=0;


//if (badf && !quiet) LOG(Ref,"nogood.hed",LOG_OUTFILE,NULL);

// now strip out 1/2 of the particles if requested
if (half) {
	int n=d2a->count();
	for (i=n-half; i>=1; i-=2) d2a->removeObjectAt(i);
}

EMData SNR;
SNR.setSize(CTFOS*ny/2,1,1);
if (d2a->count()==0) { 
	if (!quiet) LOG(Ref,NULL,LOG_END,NULL);
	printf("ERROR: No images left!\n"); 
	exit(0); 
}
avg=new EMData;
if (median) avg->makeMedian(d2a); 
else if (ctfc==1) avg->makeAverageCTFC(d2a,filt);
else if (ctfc==2) {
	if (ctfcw) { avg->makeAverageCTFCW(d2a,&SF,SNR.getData(),phaseopt); SNR.doneData(); }
	else avg->makeAverageCTFCWauto(d2a,"snr.dat");
}
else avg->makeAverage(d2a,NULL,0,nweight);
m=d2a->count();

sprintf(s,"a%s",argv[1]);
if (saveali && !quiet) LOG(Ref,s,LOG_OUTFILE,NULL);
if (savecen && !quiet) LOG(Ref,"start.cen.hed",LOG_OUTFILE,NULL);
for (i=0; i<d2a->count(); i++) { 
	d2=(EMData *)d2a->objectAt(i);
	if (saveali) {
		d2->writeIMAGIC(s,-1);
	}
	if (savecen) {
//		printf("%4.1f\t%4.1f\n",d2->Dx(),d2->Dy());
		d2->setRAlign(0,0,0);
		d2->setTAlign(floor(d2->Dx()+.5),floor(d2->Dy()+.5),0);
		d2->rotateAndTranslate();
		d2->writeIMAGIC("start.cen.hed",-1);
	}
	if(classalign2log) {
		sprintf(s,"A\t%i\t%s\t%.3f,%.3f,%.3f,%.3f\t%s\n",i,argv[1],d2->alt(),d2->az(),d2->Dx(),d2->Dy(),logit);
		FSsend(FS_TEXT, strlen(s)+1, "classalign2.log", s);
	}
	delete d2; 
}
d2a->empty();

#ifdef DEBUG
printf("DEBUG 5\n");
#endif

//if (!r1) avg->cmCenter();
if (stat) fclose(out);

//mean/=(float)n;
//sigma=sqrt(sigma/(float)n-mean*mean);
printf("Mean=%f\tSigma=%f\n%0.0f/%d good images\n",mean/norm,sigma/norm,m,data->count());
printf("Mean=%f\tSigma=%f\n",mean2/norm,sigma2/norm);

if (locffile!=-1) {
	locffile=atoi(argv[1]+3);
	if (saver) locffile*=2;
}


// Final alignment to reference
/*
if (r1==2) {
	avg->RTAlign(first);
	f=avg->alt();
//	f=f-PI*floor(f/PI+.5);
	printf("Final realign %4.2f  (%4.2f %4.2f %4.2f)\n",f*180.0/PI,eul[0]*180.0/PI,eul[1]*180.0/PI,eul[2]*180.0/PI);
//	eul[2]-=f; 		// CORRECTED 2/4/01
	eul[2]+=f;
}*/

if (r1==2) {

	EMData *a2=avg->copy();
	float a,dt,b=0,bd=-1.0e30,bb=0,bm=1.0;
	for (a=-4.0*PI/180.0; a<4.1*PI/180.0; a+=.5*PI/180.0) {
		a2->setRAlign(a);
		a2->rotateAndTranslate();
//		dt=a2->dot(first);
#ifndef ALTCMP
		if (usephase==1) dt=200.0*(2.0-a2->pcmp(first,NULL));
		else if (usephase==2) dt=500.0*(1.0+a2->fscmp(first,NULL));
		else dt=250.0*(5.0-a2->lcmp(first,1,1));
#else
		dt=20.*a2->fscmp(first);
#endif

		if (dt>bd) { bd=dt; b=a; a2->normalizeTo(first,1,1,&bm,&bb); }
	}
	printf("Final realign %4.2f  (%4.2f %4.2f %4.2f)\n",b*180.0/PI,eul[0]*180.0/PI,eul[1]*180.0/PI,eul[2]*180.0/PI);
	eul[2]+=b;
//	mask1->setRAlign(-b);
//	mask1->setTAlign(0,0);
//	mask1->rotateAndTranslate();
	avg->multConst(bm);
//	avg->addConst(bb);

/*	EMData *tfa;
	tfa=avg->RTFAlignSlow(first,NULL,first->xSize()/10);
	tfa->refineAlign(first);
	tfa->normalizeTo(first);
	avg=tfa;*/
}
else avg->multConst(1.0/avg->Sigma());

if (saver) {
	first->setNImg(-1);
	first->normalize();
	first->writeImage(classfile,locffile);
	if (locffile>=0) locffile++;
}

if (badf && r1) {
	sprintf(s,"b%s",argv[1]);
	first->writeIMAGIC(s,0);
}

#ifdef DEBUG
printf("DEBUG 6\n");
#endif

//avg->edgeNormalize();
//
if (masknorm) avg->maskNormalize(masknorm,0);
else avg->addConst(-avg->dCenter());
if (mask!=-3) avg->mult(mask1);
//avg->applyMask(mask,6);
avg->setRAlign(eul[0],eul[1],eul[2]);
//avg->setNImg((int)m);
avg->writeImage(classfile,locffile);
//if (ctfc==2 && ctfcw) SNR.writeImage("SNR.hed",locffile);
if (!quiet) LOG(Ref,classfile,LOG_OUTFILE,NULL);
if (savesteps) avg->writeImage("steps.hed",-1);
if (!quiet) LOG(Ref,NULL,LOG_END,NULL);

return 0;
}


