// classalignml 5/1/00  Steve Ludtke
// Copyright 2000 Baylor College of Medicine

//N classalignml
//F Aligns a set of images using maximum likelihood
//T X
//1
//P <input file>	Input file, usually a cls*img file
//P <iterations>	Number of iterative loops
//P [ref1]	Indicates that the first image in the file is a reference
//P [finalref]	Once the average has been generated, a final alignment to the reference is performed
//P [savesteps]	Save the class average after each iterative step to monitor convergence
//P [saverefs]	Write the reference image as well as the class average to the output file
//P [mask=<rad>]	Circular mask radius for the class average
//P [locfromfile]	Determine the output file image position from the input filename.
//P [ctfc=<res>]	Enables CTF correction, thru resolution <res> in angstroms.
//P [ctfcw=<SF file>]	Enables CTF correction, with Wiener filtration, requires s vs inten struct. factor file.
//P [even|odd]	Only use the even or odd images in the input file
//P [maxtran=<dist>]	Maximum translational shift
//P [verbose]	Display details during optimization
//U classalignml cls000.hed 8 ref1 finalref saverefs mask=32 keep=.8 locfromfile ctfc=15
//D Generates a class-average from a set of particle images using a maximum liklihood technique.
//E This program is still highly experimental and should not be used unless you feel like playing
//E with the code.

#include <cstdio>
#include <cmath>
#include <cctype>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include "XYData.h"

EMData *iterate(List *toav,EMData *ref,EMData *radprob,EMData *raddist);

int verbose=0;
int ctfc=0;
int median=0,oe=0;
float filt;
XYData SF;

int main(int argc, char *argv[]) {
int i,j,k,n;
int r1=0,savesteps=0,saver=0,mask=0,locffile=-1,half=0,quiet=0,iter=0;
float maxt=-1;
char *ctfcw=NULL;
List *data;
EMData *sum,*ref,*ref2,*mask1,*mask2,*mask3,*d,*dc;

appinit(argc,argv);

if (argc<2) appusage(argv[0]);

for (i=2; i<argc; i++) {
	if (strncmp("ref1",argv[i],4)==0) r1=1;
	else if (strncmp("verbose",argv[i],7)==0) verbose=1;
	else if (strncmp("finalref",argv[i],8)==0) r1=2;
	else if (strncmp("savesteps",argv[i],9)==0) savesteps=1;
	else if (strncmp("saverefs",argv[i],8)==0) saver=1;
	else if (strncmp("median",argv[i],6)==0) median=1;
	else if (strncmp("ctfc=",argv[i],5)==0) { ctfc=1; filt=atof(&argv[i][5]); }
	else if (strncmp("ctfcw=",argv[i],6)==0) { ctfc=2; ctfcw=&argv[i][6]; }
	else if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp("maxtran=",argv[i],8)==0) maxt=atof(&argv[i][8]);
	else if (strncmp("locfromfile",argv[i],11)==0) locffile=1;
	else if (strncmp("even",argv[i],4)==0) half=2;
	else if (strncmp("odd",argv[i],3)==0) half=1;
	else if (strncmp("oe",argv[i],2)==0) oe=1;
	else if (strncmp("quiet",argv[i],5)==0) quiet=1;
	else if (isdigit(argv[i][0])) iter=atoi(argv[i]);
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}
if (iter<1) exit(1);

// This reads the structure factor file used for Wiener filtration, and takes the required log10()
if (ctfc==2) {
	if (SF.readFile(ctfcw)) { printf("Couldn't read structure factor file!\n"); exit(1); }
	
	float *d=SF.getData();
	for (i=1; i<SF.n()*2; i+=2) d[i]=(d[i]<=0)?-4.0:log10(d[i]);
	SF.update();
}

printf("Classalign: %s\n",argv[1]);

if (locffile!=-1) {
	locffile=atoi(argv[1]+strlen(argv[1])-8);
	if (saver) locffile*=2;
}

n=fileCount(argv[1]);
//if (half) n/=2;
if ((n<5 && !r1) || (n<2 && r1) || (n<3 && half)) { printf("Not enough elements\n"); exit(0); }
data=readImages(argv[1],0,n);		// read all input images
if (data==NULL) error("Cannot read input",ERR_EXIT);
printf("Read %d images.\n",data->count());

if (r1) {
	ref=(EMData *)data->objectAt(0);
	data->removeObjectAt(0);
}
else {
	ref=((EMData *)data->objectAt(0))->copy();
	ref->applyMask(1,1);		//just noise
}
n=data->count();
ref->normalize();

ref2=ref->copy();

//sigma=ref->xSize()*ref->ySize();

if (maxt<0) maxt=ref->ySize()/8;

if (saver) { ref->writeIMAGIC("classes.hed",locffile); if (locffile>=0) locffile++; }

if (half) {
	if (half==1) {
		for (i=n-1; i>=0; i-=2) data->removeObjectAt(i);
	}
	else {
		for (i=n-2; i>=0; i-=2) data->removeObjectAt(i);
	}
}
n=data->count();
printf("Processing %d images\n",n);

for (i=0; i<n; i++) {
	d=(EMData *)data->objectAt(i);
	d->normalize();
	d->ufloat=1.5;
	d->ufloat2=maxt;
	d->ufloat3=sqrt((float)(d->xSize()*d->ySize()));
}

int Ref=0;
if (!quiet) {
	Ref=LOGbegin(argc,argv);
	LOG(Ref,argv[1],LOG_INFILE,NULL);
}

mask1=((EMData *)data->objectAt(0))->copyHead();
mask1->one();
mask1->applyMask(mask,4);

mask2=((EMData *)data->objectAt(0))->copyHead();

// This makes a map of radial distances for speed
mask3=((EMData *)data->objectAt(0))->copyHead();
mask3->realFilter(100);
mask3->toCorner();

for (i=0; i<iter; i++) {
	mask2->one();
	mask2->applyMask(maxt,3);
	mask2->toCorner();

	if (savesteps) ref->writeIMAGIC("step.hed",-1);
	d=iterate(data,ref,mask2,mask3);
	delete ref;
	ref=d;
	ref->mult(mask1);
}

if (!quiet) LOG(Ref,"classes.hed",LOG_OUTFILE,NULL);
if (r1==2) {
	float f;
	ref->rotAlign(ref2);
	f=ref2->alt();
	f=f-PI*floor(f/PI+.5);
	printf("Final realign %4.2f \n",f*180.0/PI);
//	eul[2]-=f;
}
ref->writeIMAGIC("classes.hed",locffile);
if (!quiet) LOG(Ref,NULL,LOG_END,NULL);

return 0;
}

// Iteratively apply maximum liklihood to determine a new reference image
EMData *iterate(List *toav,EMData *ref,EMData *radprob,EMData *radmap) {
EMData *ret=ref->copy();
EMData *avg,*refc=ref->copy();
EMData *d,*dc,*c,*co,*cn,*cr;
List rot,ccf,d2a;
int i,j,n = toav->count();
float angle,da = 1.0/(ref->ySize()/4.0);
float sig=0,sign=0,siga=0,max=-1.0,max2,dr=0,drs=0;
float np=ref->xSize()*ref->ySize();

ret=ref->copyHead();
ret->zero();
d2a.empty();

for (i=0; i<n; i++) {
	if (verbose) { printf("%d ",i); fflush(stdout); }
	else { putchar('.'); fflush(stdout); }
	d=(EMData*)toav->objectAt(i);

// if we're doing CTF correction, this generates a reference that has been appropriately
// 'decorrected'. Hopefully the input images are sorted so images from the same micrograph
// are grouped together
	if (ctfc) {
		if (memcmp(d->getCTF(),refc->getCTF(),11*sizeof(float))) {
			EMData *d2=ref->doFFT()->copy();
			float *ctf=d->ctfCurve(2);
			d2->applyRadFn(CTFOS*d->ySize()/2,0,1.0/CTFOS,ctf,0);
			delete refc;
			refc=d2->doIFT();
			d2->gimmeFFT();
			delete d2;
			refc->normalize();
			refc->setParent(NULL);
			refc->setCTF(d->getCTF());
			free(ctf);
		}
	}

	max2=-1.0;
	rot.empty();
	ccf.empty();
	for (angle=da/2; angle<2.0*PI; angle+=da) {
		dc=d->copy();
		dc->setRAlign(angle);
		dc->rotateAndTranslate();
		dc->normalize();	// interpolation can mess up the normalization

		c=refc->calcCCF(dc);		// cross correlation with reference, each pixel represents relative probability
		c->multConst(np);
		if (c->Max()*2< -c->Min()) { printf("Invert %f << %f\n",c->Max(),-c->Min()); }

		rot.addObject(dc);
		ccf.addObject(c);
		
		delete dc->doFFT();		// note that this is potentially not thread-safe
		dc->gimmeFFT();
		
//		putchar('.'); fflush(stdout);
		
		if (c->Max()>max2) max2=c->Max();
	}
	d->ufloat3=max2;

	sig=sign=dr=0;
	avg=d->copy();
	avg->zero();		// avg is used to prevent floating point overflow
	for (angle=da/2,j=0; angle<2.0*PI; angle+=da,j++) {
		dc=(EMData *)rot.objectAt(j);
		c=(EMData *)ccf.objectAt(j);
		cn=c->copy();

//printf("%f\t%f\t%f\t%f\n",angle,c->valueAt(0,0),c->valueAt(4,0),c->valueAt(8,0));

/*		if (c->Max()>max2) max2=c->Max();
		if (c->Max()/sigma-sigmax/sigma>40.0) {
			printf("  > (sig=%f,%f) restart with max ",sigma,c->Max()/sigma-sigmax/sigma);
			delete c;
			delete cn;
			delete dc;
//			sigmax=(sigmax/sigma+20.0)*sigma;
//			d->ufloat*=4.0;
			d->ufloat3=max2*2.0;
			printf("%f\n",d->ufloat3);
			i--;	//redo this element
			break;
		}*/

//		c->realFilter(11,sigma,sqrt(np)/sigma);	// x->exp(x/sigma-np/sigma)

		c->realFilter(11,d->ufloat,max2/d->ufloat-4.6);	// x->exp(x/sigma-max2/sigma)
		c->mult(radprob);		// This masks out translations we are defining as improbable

		// new value for maxt
		cr=c->copy();
		cr->mult(radmap);
		dr+=cr->Mean();

		// determine the new value for sigma
		cn->realFilter(12,np);		// this maps x -> 2*(np-x)
		cn->mult(c);			// apply weights
		sig+=cn->Mean();
		sign+=c->Mean();
//printf("%g  %g  %g\n",cn->Mean(),c->Mean(),cn->Mean()/c->Mean());

//c->writeIMAGIC("ccf.hed",-1);								// of image with that shift being correct
//if (i==0) 
//	printf("%f\t%g\t%g\t%g\n",angle*180/PI,c->valueAt(0,0),c->valueAt(4,0),c->valueAt(8,0));
		
		co=dc->convolute(c);
		avg->add(co);
//co->writeIMAGIC("av.hed",-1);

		delete dc;
		delete co;
		delete c;
		delete cn;
		delete cr;
	}	
	if (angle<2.0*PI) continue;		// this means we broke out of the loop above
	
	siga+=sig/sign;
	drs+=dr/sign;

	d->ufloat=sig/(sign*np);
	if (oe) d->ufloat/=2.0;
	d->ufloat2=sqrt(dr/(2*sign));
	d->ufloat3=max2;
	if (max2>max) max=max2;

	if (verbose) printf("Sig -> %f  dr -> %f  max -> %f\n",d->ufloat,d->ufloat2,d->ufloat3);

	avg->normalize();
	d2a.addObject(avg);

	// only use this image if there's something significant there
/*	if (max2/sigma-sigmax/sigma>-20.0) {
		avg->normalize();
		ret->add(avg);
	}
	else {
		putchar('*'); fflush(stdout);
		i--;
	}*/

}
delete refc;

if (median) ret->makeMedian(&d2a); 
else if (ctfc==1) ret->makeAverageCTFC(&d2a,filt);
else if (ctfc==2) ret->makeAverageCTFCW(&d2a,&SF);
else ret->makeAverage(&d2a);

for (i=0; i<d2a.count(); i++) delete (EMData *)(d2a.objectAt(i));

//maxt=sqrt(drs/(2.0*n));
//float f=siga/(n*np);
//if (goodf(&f)) sigma=f; else sigma*=2.0;
//if (goodf(&max)) sigmax=max;
//if (sigma<.001) sigma=.001;
printf("   New Sigma = %f  maxt = %f  max=%f\n",siga/(n*np),sqrt(drs/(2.0*n)),max);

ret->normalize();
return ret;
}
