// by Wen Jiang, June 2000
//
//N cosec2iv
//F This program converts helices in COSEC secondary structure elements file format to Open Inventor format 
//T X
//1
//P <input COSEC file>	The input COSEC format file
//P <output IV file>	The output Open Inventor format file
//P	[verbose]	will output some intermediate information
//U	cosec2iv in.cosec out.iv verbose
//D	Visit web page <a href="http://ncmi.bcm.tmc.edu/~wjiang/hhfh">Helixhunter and Foldhunter</a> for 
//E	more information.

#include <iostream.h>
#include "Cylinder.h"

void	main(int argc, char **argv)
{

	if (argc < 3) {
		cerr << argv[0] << ":\tconvert COSEC Secondary Structure Elements file describing the helices into Open Inventor / VRML file\n";
		cerr << "Usage:\t" << argv[0] << " <COSEC file> <iv file> [center=<x,y,z>] [mirror] [sort] [verbose]\n";
		exit(1);
	}

	bool center = false, mirror=false, verbose=false, sort=false, isgood;
	float cx ,cy, cz;
	for (int i = 3; i < argc; i++) {
		if (strncmp(argv[i],"center=",7)==0) {
			center=true;
			sscanf(&argv[i][7],"%f,%f,%f",&cx,&cy,&cz);
		}
		else if (strncmp(argv[i], "verbose", 7) == 0)
			verbose = true;
		else if (strncmp(argv[i], "sort", 4) == 0)
			sort = true;
		else if (strncmp(argv[i], "mirror", 6) == 0)
			mirror = true;
		else {
			cerr << argv[0] << ":\tunknown parameter '" << argv[i] << "'\n";
			exit(2);
		}
	}

	Cylinders cyls;
	isgood=cyls.ReadCosec(argv[1]);

	if(isgood) {
		if(sort)	cyls.SortLength();
		if(mirror) {
			cyls.mirror();
			if(verbose)cout <<argv[0]<<": cylinders are mirror transformed\n";
		}
		if(center) {
			for(int i=0; i<cyls.size(); i++){
				cyls[i]->setCenter(cyls[i]->Xc()-cx,cyls[i]->Yc()-cy,cyls[i]->Zc()-cz);
			}
		}

		if(!strcmp(&argv[2][strlen(argv[1])-4],".wrl")) {
			cyls.WriteVRML(argv[2]);
			if(verbose)
				cout << cyls.size() <<" helices wirtten into VRML file \""<<argv[2]<<"\"\n";
		}
		else {
			cyls.WriteInventor(argv[2]);
			if(verbose){
				cout <<argv[0]<<": "<< cyls.size() <<" helices wirtten into Inventor file \""<<argv[2]<<"\"\n";
				cyls.print();
			}
		}
	}
}
