// fitctf.C 7/5/2001	Steve Ludtke

//N	fitctf
//F	This program estimates CTF parameters in batch. The results should be checked in CTFIT.
//T	LM
//1
//P	<input file>	File containing boxed particles to be evaluated
//P	[sf=<SF File>]	File containing estimated 1D structure factor for this specimen
//P	[apix=<a/pix>]	Angstroms/pixel in the image
//P	[V=<voltage>]	Voltage in KV
//P	[Cs=<spher aber>]	Spherical aberation in mm
//P	[over=<oversampling>]	eg - 2 -> 2x oversampling
//P	[acmax=<max AC>]	eg - .1 is max 10% ac, default is .14
//P	[acfix=<max AC>]	fixed amplitude contrast
//P	[Bhint=<B factor]	A strong hint for the B-factor, strictly optional
//P	[Bfix=<B factor>]	Force the B factor to a specific value
//P	[Bexp]	Use an exponential 'B-factor' rather than a Gaussian
//P	[debug]	Verbose debugging output
//U	fitctf jj1023.hed groel.sm
//D	This program will attempt to determine the CTF parameters of a a set of
//E	particles. It will write the result into a .tnf file, which can then be 
//E	read into ctfit. The results of this program should always be manually
//E	verified in ctfit. This program is just designed to save the user a
//E	little manual effort.
//D	If you have a fairly small model (size in pixels<128) or are very far from focus, 
//E	you might consider using the oversampling option with a value of 2 or 3.
//D	Please note that this program uses an exceptionally inefficient algorithm. However,
//E	there is a lot of embedded 'magic' in this routine. It is designed to be functional,
//E	not fast.

#include "EMData.h"
#include "XYData.h"

float cmp(EMData *data,float *with);
float cmp2(EMData *data,float *with,int firstz);
float firstzero(float *p);

void fitbk(float *parm,float *data,EMData *fft,int sub,int firstz=0);
void fitdfb1(float *parm,float *data,EMData *fft);
void fitdf2(float *parm,float *data,EMData *fft,int firstz);
void cleanbkg(float *cparm,float *sub,EMData *avg,int &firstz);
void fitb1(float *parm,float *data,EMData *fft,int firstz);
void fitdf3(float *parm,float *data,EMData *fft,int firstz);
void fitdfac2(float *parm,float *data,EMData *fft,int firstz);
void fitb2(float *parm,float *data,EMData *fft,int firstz);

int debug=0;
float ds;
float acmax=.14;
XYData SF;	// sigh, too much work to make these nonglobal now
float bhint=0,bfix=0;
int smooth=0,bexp=1;

int main(int argc, char *argv[]) {

appinit(argc,argv);
if (argc<3) appusage(argv[0]);

float apix=2.333,voltage=300,Cs=1.6;
char *sffile=NULL;
int i,over=1;
int is_source_complex = 0; 
		
for (i=2; i<argc; i++) {	
	if (strncmp(argv[i],"sf=",3)==0) sffile=argv[i]+3;
	else if (strncmp(argv[i],"apix=",5)==0) apix=atof(argv[i]+5);
	else if (strncmp(argv[i],"acmax=",6)==0) acmax=atof(argv[i]+6);
	else if (strncmp(argv[i],"acfix=",6)==0) acmax=-atof(argv[i]+6);
	else if (strncmp(argv[i],"over=",5)==0) over=atoi(argv[i]+5);
	else if (strncmp(argv[i],"Bhint=",6)==0) bhint=atof(argv[i]+6);
	else if (strncmp(argv[i],"Bfix=",5)==0) bfix=atof(argv[i]+5);
	else if (strncmp(argv[i],"Bexp",4)==0) bexp=-1;
	else if (strncmp(argv[i],"smooth=",7)==0) smooth=atoi(argv[i]+7);
	else if (tolower(argv[i][0])=='c'&&tolower(argv[i][1])=='s') Cs=atof(argv[i]+3);
	else if (tolower(argv[i][0])=='v'&&argv[i][1]=='=') voltage=atof(argv[i]+2);
	else if (strncmp(argv[i],"debug",5)==0) debug=1;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

// Structure factor file
if (SF.readFile(sffile)) { printf("Couldn't read structure factor file (%s)!\n",sffile); exit(1); }
SF.setN(SF.n()+1);
float *d=SF.getData();

d[SF.n()*2-1]=d[SF.n()*2-3];	// This adds an extra point at the end in case the
d[SF.n()*2-2]=2.0;		// necesary range isn't covered

for (i=1; i<SF.n()*2; i+=2) d[i]=(d[i]<=0)?-4.0:log10(d[i]);
SF.update();
//SF.writeFile("q");

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

EMData *avg,*t;
int n=fileCount(argv[1]);

char* mgraph_id=file_basename(argv[1]);
		
// Calculate 2D power spectrum
printf("Calculate power spectrum\n");
t=new EMData;
if (over>1) {
	EMData u;
	u.readImage(argv[1],0);
	u.applyMask(-1,6);
	u.normalize();
	t=u.clip(-u.xSize()*(over-1)/2,-u.ySize()*(over-1)/2,u.xSize()*over,u.ySize()*over);
}
else {
	t->readImage(argv[1],0);
	if (!t->isComplex()) {
		t->applyMask(-1,6);
		if (smooth) {
			t->edgeNormalize();
			t->applyMask(smooth,3);		// Gaussian mask in real-space
		}
		t->normalize();
	}
}
if (!t->isComplex()) {
//	t->applyMask(-1,6);
//	t->normalize();
	avg=t->doFFT();
	t->gimmeFFT();
	avg->multConst(avg->ySize());
	avg->ri2ap();

	for (i=1; i<n; i++) {
		EMData *d2;
		if (over>1) {
			EMData u;
			u.readImage(argv[1],i);
			u.applyMask(-1,6);
			u.normalize();
			t=u.clip(-u.xSize()*(over-1)/2,-u.ySize()*(over-1)/2,u.xSize()*over,u.ySize()*over);
		}
		else {
			t->readImage(argv[1],i);
			t->applyMask(-1,6);
			if (smooth) {
				t->edgeNormalize();
				t->applyMask(smooth,3);		// Gaussian mask in real-space
			}
			t->normalize();
		}
		d2=t->doFFT();
		t->gimmeFFT();
		d2->multConst(d2->ySize());
		avg->addIncoherent(d2);
		delete d2;
		if (i%10==0) { printf("    %d\r",i); fflush(stdout); }
	}
	printf("\n");

	delete t;
	avg->multConst(1.0/sqrt((float)n));
}
else {
	avg=t;
	avg->ri2ap();
	is_source_complex=1;
}

int points = avg->ySize()/2;

// Now generate the 1D power spectrum to fit
float *pspec=(float *)calloc((points+1),sizeof(float));
ds=1.0/(apix*avg->ySize());
avg->calcRadDist(points,0,1.0,pspec);

float bparm[11] = { -1.0,50.0,0,.1,0,0,0,0,voltage,Cs,apix };

printf("Fit Background:\n");
fitbk(bparm,pspec,avg,0);

// Write some intermediate files and generate a background subtracted spectrum
if (debug) save_data(0,ds,pspec,points,"pspec.txt");
bparm[1]*=bexp;
avg->setCTF(bparm);
float *c=avg->ctfCurve(1,&SF);
bparm[1]*=bexp;
if (debug) save_data(0,ds/CTFOS,c,points*CTFOS,"calc.txt");
printf("%1.4g*exp(-%1.4g*s-(%1.4g*pi/2*s)^2)\n\n",bparm[6],bparm[5],bparm[7]);

float *sub=(float *)calloc((points+1),sizeof(float));
for (i=0; i<points; i++) sub[i]=pspec[i]-c[i*CTFOS];
if (debug) save_data(0,ds,sub,points,"sub.txt");
free(c);

printf("First guess at defocus and B:\n");
fitdfb1(bparm,sub,avg);
printf("defocus = %1.4g microns, B ~ %1.4g A^2, amp = %1.4g\n",-bparm[0],bparm[1],bparm[2]);

// find the first zero (approximately)
bparm[1]*=bexp;
avg->setCTF(bparm);
float *ca = avg->ctfCurve(0,&SF);
bparm[1]*=bexp;
int firstz=-1;
for (i=CTFOS*5; i<points*CTFOS; i++) {
	if (ca[i-1]*ca[i]<=0) { firstz=i/CTFOS; break; }
}
if (debug) save_data(0,ds/CTFOS,ca,points*CTFOS,"amp.txt");
free(ca);
if (firstz<0) error(ERR_EXIT,"No first zero found !?!?");
printf("First zero @ ~%d pixels (1/s ~ %1.4g A)\n\n",firstz,1.0/(firstz*ds));

printf("Refine defocus for background corrections\n");
fitdf2(bparm,sub,avg,firstz);
printf("defocus = %1.4g microns, B ~ %1.4g A^2, amp = %1.4g\n",-bparm[0],bparm[1],bparm[2]);

// note that this modifies sub,avg and firstz
cleanbkg(bparm,sub,avg,firstz);
if (debug) save_data(0,ds,sub,points,"sub2.txt");
printf("First zero @ ~%d pixels (1/s ~ %1.4g A)\n\n",firstz,1.0/(firstz*ds));

printf("First real try at a B-factor:\n");
fitb1(bparm,sub,avg,firstz);
printf("B ~ %1.4g A^2, amp = %1.4g\n\n",bparm[1],bparm[2]);
	
printf("Now we should be able to get the defocus and amp. contr.:\n");

fitdf3(bparm,sub,avg,firstz);

fitdfac2(bparm,sub,avg,firstz);

printf("defocus = %1.4g microns, ac = %d %%, amp = %1.4g\n\n",-bparm[0],(int)round(bparm[3]*100.0),bparm[2]);

printf("A last cleanup on the B-factor:\n");
fitb2(bparm,sub,avg,firstz);
printf("B ~ %1.4g A^2, amp = %1.4g\n\n",bparm[1],bparm[2]);

int j;
for (j=0; j<4; j++) {
	// Ok, now we go back and try for better noise parameters
	// Previously sub contained pspec - background, now it will contain pspec - CTF
	{
		bparm[6]=0;
		bparm[1]*=bexp;
		avg->setCTF(bparm);
		float *c=avg->ctfCurve(8,&SF),av=0;
		bparm[1]*=bexp;
		for (i=0; i<points; i++) if (!c[i*CTFOS]) sub[i]=0; else sub[i]=pspec[i]-c[i*CTFOS]; 
		free(c);
	}
	
	if (debug) {
		char s[80];
		sprintf(s,"bkg%d.txt",j+1);
		save_data(0,ds,sub,points,s);
	}

	printf("Now redetermine background parameters with bkg subtracted curve:\n");
	fitbk(bparm,sub,avg,1,firstz);

	// now make sub back into a background subtracted curve
	bparm[1]*=bexp;
	avg->setCTF(bparm);
	c=avg->ctfCurve(1,&SF);
	bparm[1]*=bexp;
	if (debug) {
		char s[80];
		sprintf(s,"bkgf%d.txt",j+1);
		save_data(0,ds/CTFOS,c,points*CTFOS,s);
	}
	printf("%1.4g*exp(-%1.4g*s-(%1.4g*pi/2*s)^2)\n\n",bparm[6],bparm[5],bparm[7]);
	for (i=0; i<points; i++) sub[i]=pspec[i]-c[i*CTFOS];
	free(c);

	// and clean it up at the zeroes again
	cleanbkg(bparm,sub,avg,firstz);
	printf("First zero at %d (%1.3g 1/A)\n",firstz,1/(firstz*ds));
	if (debug) save_data(0,ds,sub,points,"sub3.txt");

	// Ok, clean up the defocus and %AC with the new background
	fitdfac2(bparm,sub,avg,firstz);
	printf("defocus = %1.4g microns, ac = %d %%, amp = %1.4g\n\n",-bparm[0],(int)round(bparm[3]*100.0),bparm[2]);


	if (debug) {
		char s[80];
		sprintf(s,"bkgg%d.txt",j+1);
		save_data(0,ds,sub,points,s);
	}
	// Fix B for the new background
	fitb2(bparm,sub,avg,firstz);
	printf("B ~ %1.4g A^2, amp = %1.4g\n\n",bparm[1],bparm[2]);
}
// OK! done. now set the parameters in the image for use in the file
if (bfix) {
	bparm[1]=bfix;
	printf("B factor reset to %1.1f\n",bfix);
}
bparm[1]*=bexp;
avg->setCTF(bparm);
c=avg->ctfCurve(8,&SF);
bparm[1]*=bexp;
if (debug) save_data(0,ds/CTFOS,c,points*CTFOS,"calc2.txt");
free(c);

char s[1024];
strcpy(s,argv[1]);
if(!is_source_complex) {
	strcpy(s+strlen(s)-4,".avg.tnf");
}
avg->writeMRC(s);

// silly, but we won't have a MRC header until we've written the file, so we have to do 2 writes...
mrcH *mrch = avg->getMRCHed();
mrch->nlabl=2;
mrch->labels[1][79]=0;
sprintf(mrch->labels[1],"%s",sffile);
avg->writeMRC(s);
LOG(Ref,s,LOG_OUTFILE,NULL);

//also write ctf parms in the source file if it looks like imagic format
if (strcmp(argv[1]+strlen(argv[1])-4,".hed")==0 || 
    strcmp(argv[1]+strlen(argv[1])-4,".img")==0 ||
    strcmp(argv[1]+strlen(argv[1])-4,".hdf")==0 || 
    strcmp(argv[1]+strlen(argv[1])-4,".hd5")==0 ) {
	EMData X;
	for (i=0; i<n; i++) {
		X.readImage(argv[1],i,1);	// heaader only
		X.setCTF(bparm);
		X.writeImage(argv[1],i,1);	// header only
	}
}

printf("\n------------------------------------\n");
bparm[1]*=bexp;
printf("%1.4g,%1.4g,%1.4g,%1.4g,%1.4g,%1.4g,%1.4g,%1.4g,%1.4g,%1.4g,%1.4g\n",
	bparm[0],bparm[1],bparm[2],bparm[3],bparm[4],bparm[5],bparm[6],bparm[7],bparm[8],bparm[9],bparm[10]);
FILE *out=fopen("ctfparm.txt","ab");
if (out) {
	fprintf(out,"%s\t%1.5g,%1.5g,%1.5g,%1.5g,%1.5g,%1.5g,%1.5g,%1.5g,%1.5g,%1.5g,%1.5g,%s\n",
		mgraph_id,bparm[0],bparm[1],bparm[2],bparm[3],bparm[4],bparm[5],bparm[6],bparm[7],bparm[8],bparm[9],bparm[10],sffile);
	fclose(out);
}
printf("\n------------------------------------\n");
printf("And the answer is:\n");
printf("defocus = %1.4g microns\nB = %1.4g A^2\nac = %d %%  (+-10%%)\namp = %1.4g\n",-bparm[0],bparm[1],(int)round(bparm[3]*100.0),bparm[2]);
printf("%1.4g*exp(-%1.4g*s-(%1.4g*pi/2*s)^2)\n",bparm[6],bparm[5],bparm[7]);
LOG(Ref,NULL,LOG_END,NULL);

return 0;
}

// Refine B-factor
void fitb2(float *parm,float *data,EMData *fft,int firstz) {
int points = fft->ySize()/2;
float cparm[11],rng;
int i;
for (i=0; i<11; i++) cparm[i]=parm[i];
cparm[6]=0;
if (bhint>0) parm[1]=bhint;
//if (bfix>0) parm[1]=bfix;
float bestb=parm[1],bestamp;
float best=1.0e20,sqp;
//if (bfix>0) rng=.000001; else rng=.75;
rng=.75;
for (sqp=sqrt(parm[1])-rng; sqp<sqrt(parm[1])+rng; sqp+=.05) {
	cparm[1]=sqp*sqp;
	cparm[2]=1.0;
	cparm[1]*=bexp;
	fft->setCTF(cparm);
	float *c=fft->ctfCurve(8,&SF),av=0;
	cparm[1]*=bexp;
	cparm[2]=0;
	for (i=points/10; i<points/2; i++) {
		if (c[i*CTFOS]) { 
			if (i>firstz) { cparm[2]+=data[i]; av+=c[i*CTFOS];  }
			else { cparm[2]+=data[i]/10.0; av+=c[i*CTFOS]/10.0; }
		}
	}
	cparm[2]/=av;
	if (cparm[2]<0) cparm[2]=0;
	float sig=0;
	for (i=points/10; i<points*2/3; i++) {
		if (!c[i*CTFOS]) continue;
		if (i<firstz) sig+=SQR(data[i]-cparm[2]*c[i*CTFOS])/400.0;
		else sig+=SQR(data[i]-cparm[2]*c[i*CTFOS])*exp(2.0*i/points);
	}
	if (sig<best) { best=sig; bestb=cparm[1]; bestamp=cparm[2]; 
		if (debug) printf("%1.2f\t%1.2f\t%1.4g\n",cparm[1],cparm[2],sig);
	}
	free(c);
}
parm[1]=bestb;
parm[2]=sqrt(bestamp);
return;
}

// Refine defocus and amp contr
void fitdfac2(float *parm,float *data,EMData *fft,int firstz) {
int points = fft->ySize()/2;
float cparm[11];
int i;
for (i=0; i<11; i++) cparm[i]=parm[i];
cparm[6]=0;
float bestdf=parm[0],bestac=parm[3],bestamp;
float best=1.0e20;
for (cparm[3]=acmax<0?-acmax:0; cparm[3]<=fabs(acmax); cparm[3]+=.02) {
	for (cparm[0]=parm[0]-.1; cparm[0]<parm[0]+.1; cparm[0]+=.001) {
		cparm[2]=1.0;
		cparm[1]*=bexp;
		fft->setCTF(cparm);
		float *c=fft->ctfCurve(8,&SF),av=0;
		cparm[1]*=bexp;
		cparm[2]=0;
		for (i=3; i<points/2; i++) if (c[i*CTFOS]) { cparm[2]+=data[i]; av+=c[i*CTFOS]; }
		cparm[2]/=av;
		if (cparm[2]<0) cparm[2]=0;
		float sig=0;
		for (i=3; i<points*2/3; i++) {
			if (!c[i*CTFOS]) continue;
			if (i<firstz) sig+=SQR(data[i]-cparm[2]*c[i*CTFOS])/50.0;
			else sig+=SQR(data[i]-cparm[2]*c[i*CTFOS]);
		}
		if (sig<best) { best=sig; bestdf=cparm[0]; bestamp=cparm[2]; bestac=cparm[3]; 
			if (debug) printf("%1.2f\t%d\t%1.2f\t%1.4g\n",cparm[0],(int)(cparm[3]*100),cparm[2],sig);
		}
		free(c);
	}
}
parm[0]=bestdf;
parm[2]=sqrt(bestamp);
parm[3]=bestac;
return;
}

// refine defocus
void fitdf3(float *parm,float *data,EMData *fft,int firstz) {
int points = fft->ySize()/2;
float cparm[11];
int i;
for (i=0; i<11; i++) cparm[i]=parm[i];
cparm[6]=0;
float bestdf=parm[0],bestamp;
float best=1.0e20;

for (cparm[0]=bestdf-.3; cparm[0]<bestdf+.3; cparm[0]+=.01) {
	cparm[2]=1.0;
	cparm[1]*=bexp;
	fft->setCTF(cparm);
	float *c=fft->ctfCurve(8,&SF),av=0;
	cparm[1]*=bexp;
	cparm[2]=0;
	for (i=3; i<points/3; i++) if (c[i*CTFOS]) { cparm[2]+=data[i]; av+=c[i*CTFOS]; }
	cparm[2]/=av;
	if (cparm[2]<0) cparm[2]=0;
	float sig=0;
	for (i=3; i<points*2/3; i++) {
		if (!c[i*CTFOS]) continue;
		if (i<firstz) sig+=SQR(data[i]-cparm[2]*c[i*CTFOS])/100.0;
		else sig+=SQR(data[i]-cparm[2]*c[i*CTFOS]);
	}
	if (sig<best) { best=sig; bestdf=cparm[0]; bestamp=cparm[2];
		if (debug) printf("%1.2f\t%1.2f\t%1.4g\n",cparm[0],cparm[2],sig);
	}
	free(c);
}
parm[0]=bestdf;
parm[2]=sqrt(bestamp);
return;
}

// global search for B factor
void fitb1(float *parm,float *data,EMData *fft,int firstz) {
int points = fft->ySize()/2;
float cparm[11],b0,b1;
int i;
for (i=0; i<11; i++) cparm[i]=parm[i];
cparm[6]=0;
// Rough guess at B-factor
if (bhint>0) cparm[1]=bhint;
if (bfix>0) cparm[1]=bfix;

float bestb=parm[1],bestamp;
float best=1.0e20;
float sqp;
//if (bfix>0) { b0=sqrt(bfix); b1=b0+.001; }
//else if (bhint>0) { b0=sqrt(bhint)-5.0; b1=b0+10.0; }
if (bhint>0) { b0=sqrt(bhint*.625); b1=sqrt(bhint*1.333); }
else { b0=0; b1=40.0; }
for (sqp=b0; sqp<b1; sqp+=.5) {
	cparm[1]=sqp*sqp;
	cparm[2]=1.0;
	cparm[1]*=bexp;
	fft->setCTF(cparm);
	float *c=fft->ctfCurve(8,&SF),av=0;
	cparm[1]*=bexp;
	cparm[2]=0;
	for (i=points/10; i<points/3; i++) if (c[i*CTFOS]) { cparm[2]+=data[i]; av+=c[i*CTFOS]; }
	cparm[2]/=av;
	if (cparm[2]<0) cparm[2]=0;
	float sig=0;
	for (i=points/10; i<points*2/3; i++) {
		if (!c[i*CTFOS]) continue;
		if (i<firstz) sig+=SQR(data[i]-cparm[2]*c[i*CTFOS])/200.0;
		else sig+=SQR(data[i]-cparm[2]*c[i*CTFOS])*exp(3.0*i/points);
	}
	if (sig<best) { best=sig; bestb=cparm[1]; bestamp=cparm[2];
		if (debug) printf("%1.2f\t%1.2f\t%1.4g\n",cparm[1],cparm[2],sig);
	}
	free(c);
}
parm[1]=bestb;
parm[2]=sqrt(bestamp);
}

// This will do a first guess at defocus and B-factor. data must be background subtracted
void fitdfb1(float *parm,float *data,EMData *fft) {
int points = fft->ySize()/2;
// First guess at defocus, using a couple of trial b-factors
float bestdf,bestamp,bestb;
float tryb[] = { 35.0,100.0,200.0,400.0,800.0 };
float dfmin[] = { 0,-.5,-1.0,-1.5, -4.5 }	;	// if we get too close to focus with a large B, we cannot fit well
float cparm[11];
int i,j;
float best=1.0e20;

for (i=0; i<11; i++) cparm[i]=parm[i];
cparm[6]=0;	// we disable noise for comparison with subtracted curve
cparm[3]=.1;	// use 10% ac for this test
best=1.0e30;
//for (j=0; j<(bfix>0?1:5); j++) {
for (j=0; j<5; j++) {
	cparm[1]=tryb[j];
//	if (bfix>0) cparm[1]=bfix;
	if (bhint>0) cparm[1]=bhint+(j-2.0)*25.0;
	for (cparm[0]=-5.0; cparm[0]<dfmin[j]*sqrt(parm[8]/400.0); cparm[0]+=.05) {
		cparm[2]=1.0;
		cparm[1]*=bexp;
		fft->setCTF(cparm);
		float *c=fft->ctfCurve(8,&SF),av=0;
		cparm[1]*=bexp;
		cparm[2]=0;
		for (i=points/10; i<points/3; i++) if (c[i*CTFOS]) { cparm[2]+=data[i]; av+=c[i*CTFOS]; }
		cparm[2]/=av;
		if (cparm[2]<0) cparm[2]=0;
		float sig=0;
		for (i=points/10; i<points/2; i++) sig+=fabs(data[i]-cparm[2]*c[i*CTFOS]);
		if (sig<best) { best=sig; bestdf=cparm[0]; bestb=cparm[1]; bestamp=cparm[2];
			if (debug) printf("%1.3f\t%1.3f\t%1.3f\t%1.4g\n",cparm[0],cparm[1],cparm[2],sig);
		}
		free(c);
	}
}
parm[0]=bestdf;
parm[1]=bestb;
parm[2]=sqrt(bestamp);
return;
}

//refine defocus using approx location of first zero
void fitdf2(float *parm,float *data,EMData *fft,int firstz) {
int points = fft->ySize()/2,i;
float best=1.0e20,bestdf,bestamp;
float cparm[11];
for (i=0; i<11; i++) cparm[i]=parm[i];
cparm[6]=0;

printf("Search from %1.2f to %1.2f\n",parm[0]-.5,parm[0]>-1.0?0:parm[0]+.5);
// refine a bit so the background cleanup will work
for (cparm[0]=parm[0]-.5; cparm[0]<(parm[0]>-1.0?0:parm[0]+.5); cparm[0]+=.01) {
	cparm[2]=1.0;
	cparm[1]*=bexp;
	fft->setCTF(cparm);
	float *c=fft->ctfCurve(8,&SF),av=0;
	cparm[1]*=bexp;
	cparm[2]=0;
	for (i=firstz/3; i<points/3; i++) if (c[i*CTFOS]) { cparm[2]+=data[i]; av+=c[i*CTFOS]; }
	cparm[2]/=av;
	if (cparm[2]<0) cparm[2]=0;
	float sig=0;
	for (i=3; i<points/2; i++) {
		if (!c[i*CTFOS]) continue;
		if (i<firstz/2) sig+=SQR(data[i]-cparm[2]*c[i*CTFOS])/100.0;
		else sig+=SQR(data[i]-cparm[2]*c[i*CTFOS]);
	}
	if (sig<best) { best=sig; bestdf=cparm[0]; bestamp=cparm[2];
		if (debug) printf("%1.2f\t%1.2f\t%1.4g\n",cparm[0],cparm[2],sig);
	}
	free(c);
}

parm[0]=bestdf;
parm[2]=sqrt(bestamp);
}

#define MAXZ	100
// Takes a curve containing residual background, and using the predicted zero locations,
// does a better job at background subtraction 
void cleanbkg(float *cparm,float *sub,EMData *avg,int &firstz) {
int points = avg->ySize()/2;
int i,j;
float zy[MAXZ];
int zx[MAXZ],nz=0;
float *ca;
cparm[1]*=bexp;
avg->setCTF(cparm);
ca = avg->ctfCurve(0,&SF);
cparm[1]*=bexp;
//save_data(0,ds/CTFOS,ca,points*CTFOS,"amp.txt");
int lzx=-1;
float lzy;
for (i=3; i<points*CTFOS-4; i++) {
	int k=i/CTFOS;
	if (ca[i-1]*ca[i]<=0 && nz<MAXZ) {
		zx[nz]=k;
		zy[nz]=sub[k];
		int fj=k-3;
		if (nz && zx[nz-1]>=fj) fj=zx[nz-1]+1;		//zero positions cannot overlap
		for (j=fj; j<=k+3; j++) {
			if (sub[j]<zy[nz]) { zx[nz]=j; zy[nz]=sub[j]; }
		}
		nz++;
	}
}
firstz=zx[0];

for (j=1; j<nz; j++) {
	if (debug) printf("%d\t%f\n",zx[j],zy[j]);
	for (i=zx[j-1]; i<zx[j]; i++) {
		float r=(i-zx[j-1])/(zx[j]-zx[j-1]);
		sub[i]-=(1.0-r)*zy[j-1]+r*zy[j];
	}
}

return;
}

// If sub is 1 then it tries to fit the actual curve passed in data,
// otherwise it assumes the passed curve has SF & ctf in it too...
void fitbk(float *parm,float *data,EMData *fft,int sub,int firstz) {
// Find a fixed point in the tail
int points = fft->ySize()/2;
int i,i0=points*3/4-4,i1=points*3/4+4,fixi=(i0+i1)/2;
float fixy=0;
for (i=i0; i<i1; i++) fixy+=data[i];
fixy/=i1-i0;
printf("Fixed point at %d,%1.3g\n",fixi,fixy);

if (firstz<10) firstz=10;
// First guess at the background
float k1=0,k2=0,k3=0;
float f,best = 1.0e30,bestk1=0,bestk2,bestk3;
float cparm[11];
for (i=0; i<11; i++) cparm[i]=parm[i];
cparm[4]=cparm[2]=0;
for (k2=0; k2<60.0; k2+=.5) {
	for (k3=0; k3<10.0; k3+=.25) {
//		f=fixi*ds;
//		cparm[6]=fixy*exp(k2*f+SQR(k3*PI/2.0*f));	// scale
		cparm[6]=1.0e10;
		for (i=firstz; i<points*3/4; i++) {
			float s=i*ds;
			float r=data[i]/exp(-(SQR(PI/2*k3*s))-s*k2);
			if (r>0 && r<cparm[6]) cparm[6]=r;
		}
		cparm[5]=k2;								// e^-ks
		cparm[7]=k3;								// e^-(k pi/2 s)^2
		fft->setCTF(cparm);
		if (sub) f=cmp2(fft,data,firstz);
		else f=cmp(fft,data);
		if (f<best) { 
			bestk2=k2; 
			bestk3=k3; 
			best=f; 
		}
			if (debug) printf ("%f\t%f\t%f\t| %1.4g\n",cparm[6],k2,k3,f);
	}
}
parm[5]=bestk2;
parm[7]=bestk3;

printf("Refine background:\n");
// fine tuning on the background
for (k2=(parm[5]<1.6?0:parm[5]-1.6); k2<parm[5]+1.6; k2+=.04) {
	for (k3=(parm[7]<.4?0:parm[7]-.4); k3<parm[7]+.4; k3+=.02) {
//		f=fixi*ds;
//		cparm[6]=fixy*exp(k2*f+SQR(k3*PI/2.0*f));	// scale
		cparm[6]=1.0e10;
		for (i=firstz; i<points*3/4; i++) {
			float s=i*ds;
			float r=data[i]/exp(-(SQR(PI/2*k3*s))-s*k2);
			if (r>0 && r<cparm[6]) cparm[6]=r;
		}
		cparm[5]=k2;								// e^-ks
		cparm[7]=k3;								// e^-(k pi/2 s)^2
		fft->setCTF(cparm);
		if (sub) f=cmp2(fft,data,firstz);
		else f=cmp(fft,data);
		if (f<best) { bestk2=k2; bestk3=k3; best=f; if (debug) printf ("%f\t%f\t%f\t| %1.4g\n",cparm[6],bestk2,bestk3,best); }
	}
}

//f=fixi*ds;
//parm[6]=fixy*exp(bestk2*f+SQR(bestk3*PI/2.0*f));	// scale
parm[6]=1.0e10;
for (i=firstz; i<points*3/4; i++) {
	float s=i*ds;
	float r=data[i]/exp(-(SQR(PI/2*bestk3*s))-s*bestk2);
	if (r>0 && r<parm[6]) parm[6]=r;
}
parm[5]=bestk2;								// e^-ks
parm[7]=bestk3;								// e^-(k pi/2 s)^2
if (debug) printf ("Final\n%f\t%f\t%f\t| %1.4g\n",parm[6],bestk2,bestk3,best); 

return;
}


// measure of fit quality (somewhat qualitative)
// This has a strong penalty for the fit exceeding the raw data
// it also ignores the first and last 10% of the curve
float cmp(EMData *data,float *with) {
float *calc=data->ctfCurve(8,&SF);
float ret=0;
int np=data->ySize()/2;
int i;

for (i=4; i<np-np/4; i++) {
	if (calc[i*CTFOS]>with[i]) ret+=10.0;
}

for (i=np/4; i<np-np/10; i++) {
//	if (calc[i*CTFOS]>with[i]) ret+=(exp(-with[i]+calc[i*CTFOS])-1.0)/with[i];
	if (calc[i*CTFOS]>with[i]) ret+=pow(calc[i*CTFOS]-with[i],(float)2.0)*500.0;
//	if (calc[i*CTFOS]>with[i]) ret+=pow(calc[i*CTFOS]-with[i],(float)2.0)+25.0;
	else ret+=pow(calc[i*CTFOS]-with[i],(float)2.0);
}
free(calc);
return ret;
}

// This one is for the later stages of 'accurate' fitting
float cmp2(EMData *data,float *with,int firstz) {
float *calc=data->ctfCurve(8,&SF);
float ret=0;
int np=data->ySize()/2;
int i;

for (i=firstz/2; i<np-np/10; i++) {
//	if (calc[i*CTFOS]>with[i]) ret+=(exp(-with[i]+calc[i*CTFOS])-1.0)/with[i];
	if (!with[i]) continue;
//	if ((calc[i*CTFOS]>with[i] && i>firstz) || i==firstz) ret+=pow(calc[i*CTFOS]-with[i],2.0)*100.0;
//	else ret+=pow(calc[i*CTFOS]-with[i],2.0);
//	ret+=fabs(calc[i*CTFOS]-with[i])*sin(PI*i/(float)np);
	ret+=fabs(calc[i*CTFOS]-with[i]);
}

free(calc);
return ret;
}

// computes theoretical location of 1st zero
float firstzero(float *p) {
float lambda=12.2639/sqrt(p[8]*1000.0+.97845*p[8]*p[8]);
float zero=sqrt(-(p[0]*10000.0+sqrt(p[0]*p[0]*1.0e8-2.0*lambda*p[9]*1.0e7+
	2.0*lambda*p[9]*1.0e7/PI*asin(p[3])))/(lambda*lambda*p[9]*1.0e7));

	return zero;
}
