// 1/17/97  Steve Ludtke
// Copyright 1997 Baylor College of Medicine
// This version of footprint tries to find something that's both
// rotationally and translationally invariant

#include <cstdio>
#include <cmath>
#include "EMData.h"
#include "List.h"
#include "util.h"

#define N2	16

int main(int argc, char *argv[]) {
int i,j,n,n2;
float *mx[N2],a[N2],c[N2];
EMData *d,*d2;
int nx,ny;
float r;
FILE *out;
char s[150];

appinit(argc,argv);

if (argc!=3) { printf("footprint.c\nUsage:  <input file> <output file>\n"); exit(0); }

if (strlen(argv[1])>119) 
	printf("WARNING: Path over 120 char will cause problems later.\n");

n=fileCount(argv[1]);
if (n<1) { printf("No data\n"); exit(0); }

d=new EMData;
d->readImage(argv[1],0);
nx=d->xSize();
ny=d->ySize();

if (nx!=ny) { printf("nx!=ny\n"); exit(1); }

// memory for all footprints
for (i=0; i<N2; i++) { c[i]=0; mx[i]=(float*)malloc(sizeof(float)*n); }

printf("Calculate footprints\n");
// calculate image dot products
for (i=0; i<n; i++) {
	if (d->readImage(argv[1],i)) printf("Image read failed at %d\n",i);
	d->normalize();
	d2=d->doFFT();
	d2->calcRadDist(16,1.0,1.0,a);
	
	for (j=0; j<N2; j++) {
		mx[j][i]=a[j];
		c[j]+=mx[j][i];
	}
	if (i%50==0) { printf(" %d done\r",i); fflush(stdout); }
}
printf("%d done\n",n);
for (i=0; i<N2; i++) c[i]/=(float)n;	// this is the center of the N2 dim object

// translate each to 'center' and normalize
for (i=0; i<n; i++) {
	r=0;
	for (j=0; j<N2; j++) { mx[j][i]-=c[j]; r+=fabs(mx[j][i]); }
	r=sqrt(r);
	for (j=0; j<N2; j++) { 
		mx[j][i]/=r; 
//		printf("%3.0f",floor(mx[j][i]*99.999)); 
	}
//	printf("\n");
}

// write footprint file
// 120 byte string (filespec)
// 4 byte integer (n)
// 4 byte integer (n2 elements/image)
// n2 arrays of n, 4 byte floats
out=fopen(argv[2],"wb");
strncpy(s,argv[1],119);
s[119]=0;
fwrite(s,120,1,out);
fwrite(&n,4,1,out);
n2=N2;
fwrite(&n2,4,1,out);
for (i=0; i<16; i++) fwrite(mx[i],sizeof(float)*n,1,out);
fclose(out);

// similarity matrix
/*
for (i=0; i<n; i++) {
	for (j=0; j<n; j++) {
		r=0;
		for (k=0; k<16; k++) r+=mx[k][j]*mx[k][i];
		printf("%3.0f",floor(r*49.999)+50.0);
	}
	printf("\n");
}
*/

return 0;
}
