// fpweight.C  Copyright 1998 Steve Ludtke
// Copyright 1998 Baylor College of Medicine
// This program assigns weights to the various footprint axes. This is not a 
// reparamatrization, but simply a weighting factor to focus on the parameters
// which are most closely correlated with alt/az

//N fpweight
//F This program assigns weights to the various axes in a footprint file
//T LT
//1
//P <footprint file>	Input footprint file to be weighted
//P [tfp]	Writes a file which can be used to visualize the space
//U fpweight proj.fp
//D This program attempts to determine the degree of correlation between each
//E axis (dimension) in the footprint file and the euler angles. It produces
//E weights suitable for use by 'classes' or 'classesrefs'.
//D This program is experimental and not for general use.

#include <cstdio>
#include <cmath>
#include "EMData.h"
#include "List.h"
#include "util.h"

int main(int argc, char *argv[]) {
FILE *in;
float *eul[3],**dat;
int nparm,nim,tfp=0;
char s[120];
int i,j,k,n;
float cur,cur2;

appinit(argc,argv);

if (argc<2) { 
	printf("Usage: fpweight <grid proj.fp> [tfp]\n"); 
	exit(0);
}

for (i=2; i<argc; i++) {	
	if (strncmp(argv[i],"tfp",3)==0) tfp=1;
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

in=fopen(argv[1],"rb");
if (!in) { printf("No file.\n"); exit(1); }

fread(s,120,1,in);
fread(&nim,sizeof(int),1,in);
fread(&nparm,sizeof(int),1,in);

dat=(float **)malloc(nparm*sizeof(float *));
for (i=0; i<nparm; i++) {
	dat[i]=(float *)malloc(nim*sizeof(float));
	fread(dat[i],nim*sizeof(float),1,in);
}

for (i=0; i<3; i++) {
	eul[i]=(float *)malloc(nim*sizeof(float));
	if (fread(eul[i],nim*sizeof(float),1,in)!=1) { printf("No eulers in footprint file\n"); exit(0); }
}
fclose(in);

// TFP write for projection space visualization
if (tfp) {
	in=fopen("tfp","wb");
	for (i=0; i<nim; i++) {
		fprintf(in,"%f\t%f\t",eul[1][i],eul[2][i]);
		for (j=0; j<nparm; j++) fprintf(in,"%f\t",dat[j][i]);
		fprintf(in,"\n");
	}
	fclose(in);
}

// curvature
n=(int)floor(sqrt((float)nim));

in=fopen("fpweights","wb");

for (k=0; k<nparm; k++) {
	printf("%d. ",k);

	// curvature for azimuth (or alt?)
	cur=0;
	for (i=0; i<n-2; i++) {
		for (j=0; j<n; j++) {
			cur+=fabs((dat[k][i+j*n]+dat[k][i+2+j*n])/(2.0*dat[k][i+1+j*n])-1.0);
		}
	}
	printf("%f\t",(float)(n*(n-2))/cur);

	// for altitude
	cur2=0;
	for (i=0; i<n; i++) {
		for (j=0; j<n-2; j++) {
			cur2+=fabs((dat[k][i+j*n]+dat[k][i+j*n+2*n])/(2.0*dat[k][i+j*n+n])-1.0);
		}
	}
	printf("%f\t",(float)(n*(n-2))/cur2);

	// average weight based on both
	printf("%f\n",(float)(n*(n-2))/(cur+cur2));

	fprintf(in,"%d\t%f\n",k,(n*(n-2))/(cur+cur2));
}
fclose(in);

return 0;
}
