
//N helixbatchboxer
//F This program is used for offline helixboxing of micrographs
//T LM
//
//1
//P [input=<filespec>]	Image to be boxed	(MRC format only!!!)
//P [output=<filespec>]	Output file for boxed images
//P [length=<length>] length
//P [width=<width>] width
//P [dbbox=<filespec>]	Input box database
//P [nointerp] No interpolation
//P [sep] Output boxed out particle in seperate files
//U helixbatchboxer dbbox=jj0441.box input=jj0441.mrc output=0441.hed
//D This program does offline boxing of particles using a box database or an automatic
//E boxing routine. Please note that it will work only on MRC images.


#include <cstdio>
#include <cmath>
#include "EMData.h"
#include "util.h"
#include <iostream>
#include <cstdlib>

EMData * getPtcl(EMData * src, int x0, int y0, int x1, int y1, int r, int length=0)
{
    float x2=x1-x0;
    float y2=y1-y0;
    float ln=hypot(x2,y2);
    float px= y2/ln;
    float py=-x2/ln;
	x2/=ln;
	y2/=ln;
	ln=floor(ln);
	if (length==0) length=(int)ln;
	EMData *sv = new EMData;
	sv->setSize(r*2,length);
	sv->getData();
	float x,y,v;
	float p1, p2, p3, p4;
	int xx, yy;
	float mean=src->Mean();
	for (y=0; y<length; y+=1.0) {
		for (x=-r; x<r; x+=1.0) {
			float v=src->valueAtInterp(x0+x*px+y*x2,y0+x*py+y*y2);
/*			xx=(int)floor(x0+x*px+y*x2);
			yy=(int)floor(y0+x*py+y*y2);
			p1=src->valueAtSafe(xx,yy);
			p2=src->valueAtSafe(xx+1,yy);
			p3=src->valueAtSafe(xx+1,yy+1);
			p4=src->valueAtSafe(xx,yy+1);
			if ( p1==0.0 || p2==0.0 || p3==0.0 || p4==0.0)
				v=mean;
			else
				v=bilin(p1,p2,p3,p4,x0+x*px+y*x2-xx,y0+x*py+y*y2-yy);*/
			sv->setValueAt((int)x+r,(int)y,0,v);
		}
	}
	sv->doneData();

	return sv;
	
}

EMData * getPtclNoInterp(EMData * src, int x0, int y0, int x1, int y1, int r)
{
	return 0;
}

int main(int argc,char *argv[]) {
	char * boxed, * input, * dbin;
	int length=0,width=0, i;
	char t[121];
	bool nointerp=false,sep=false;
	int count=0;
	
	appinit(argc,argv);
	if (argc<3) appusage(argv[0]);
	for (i=1; i<argc; i++) {	
		if (strncmp(argv[i],"input=",6)==0) input=argv[i]+6;
		else if (strncmp(argv[i],"output=",7)==0) boxed=argv[i]+7; 
		else if (strncmp(argv[i],"length=",7)==0) length=atoi(argv[i]+7); 
		else if (strncmp(argv[i],"width=",6)==0) width=atoi(argv[i]+6); 
		else if (strncmp(argv[i],"dbbox=",6)==0) dbin=argv[i]+6;
		else if (strncmp(argv[i],"nointerp",8)==0) nointerp=true;
		else if (strncmp(argv[i],"sep",3)==0) sep=true;
		
		else { 
			printf("Unknown option %s\n",argv[i]);
			exit(1); 	
		}
	}
	if (!input) error(ERR_EXIT,"Cannot output boxed particles without an input image");
	EMData image;
	image.readImage(input,0);
	FILE *in=fopen((const char *)dbin,"r");
	while (fgets(t,120,in)) {
	    int a0,a1,a2,a3,a4=0,a5,a6;
	    int x0, y0, x1, y1, r;
	    float x2, y2, px, py;
	    sscanf(t," %d %d %d %d %d",&a0,&a1,&a2,&a3,&a4);
	    if (a4==-1) {
			fgets(t,120,in);
			sscanf(t," %d %d %d %d %d",&a2,&a3,&a4,&a5,&a6);
			if (a6!=-2) continue;
			x0=a0+a4/2;
			y0=a1+a4/2;
			x1=a2+a4/2;
			y1=a3+a4/2;
			if (width==0) r=a4/2;
			else r=width/2;
			if (length>0) {
				float xc = (x0 + x1) / 2.0;
				float yc = (y0 + y1) / 2.0;
				float x2=x1-x0;
			    float y2=y1-y0;
			    float ln=hypot(x2,y2);
				x2/=ln;
				y2/=ln;
				int half=length/2;
				x1=(int) floor(xc+half*x2);
				y1=(int) floor(yc+half*y2);
				x0=(int) floor(xc-half*x2);
				y0=(int) floor(yc-half*y2);
			}
			EMData * sv;
			if (nointerp)
				sv=getPtclNoInterp(&image, x0, y0, x1, y1, r);
			else
				sv=getPtcl(&image, x0, y0, x1, y1, r, length);
				
			if (!sep)
				sv->writeImage(boxed, -1);
			else {
				sprintf(t,"%d.%s", count, boxed);
				sprintf(t+strlen(t)-4,"%s", ".hed");
				sv->writeImage(t,-1);
				count++;
			}

			delete sv;	
	    }	
	}
	fclose(in);

	return 0;
}
