// by Wen Jiang, June 2000
// use OpenInventor library, so it is only working on SGI at this moment
//
//N ivFit
//F This program fits the matched paires of helices returned by COSEC
//T X
//1
//P <input to-be-transformed IV file>	The input Open Inventor format file for the helices to be transformed to match the <ref IV>
//P <input ref IV file>	The input Open Inventor format file containing the ref helices 
//P <out IV file>	The output Open Inventor format file with transformed helices matching the ref helices
//P	[matched=<n1,n2,n3,..>]	set the corresponding helices in <ref IV file> that match the helices in <input to-be-transformed IV file>, -1 for non-match
//P	[step=<angle>]	step size in degree for the rotation fit, default to 5
//P	[int]	output the intermediate results with only the matched helices into file: ivFitProbe.iv and ivFitRef.iv
//P	[verbose]	will output some intermediate information
//U	ivFit probe.iv ref.iv probe-out.iv step=4 matched=2,6,-1,3,8,5 verbose
//D	This program uses Open Inventor library, so it is only working on SGI at this moment.
//D	This program works correctly only for the special Open Inventor file containing only cylinders (helices).
//E	The matched option parameters come from the COSEC search result output.
//D	Visit web page <a href="http://ncmi.bcm.tmc.edu/~wjiang/hhfh">Helixhunter and Foldhunter</a> for 
//E	more information.


#include <iostream>
#include "Cylinder.h"

void	main(int argc, char **argv)
{

	if (argc < 4) {
		cerr << argv[0] << ":\tfit the helices defined in the two Open Inventor files and transform the first to match the second\n";
		cerr << "Usage:\t" << argv[0] << " <to-be-fitted iv file> <pdb/ref iv file> <transformed iv file> [matched=<2,3,5,..>] [step=<angular step size>] [int] [verbose]\n";
		exit(1);
	}

	bool verbose=false, matched=false, outputint=false;
	int i, matcharg;
	float step=5;
	for (i = 4; i < argc; i++) {
		if (strncmp(argv[i], "matched=", 8) == 0){
			matched=true;	
			matcharg=i;
		}
		else if (strncmp(argv[i], "step=", 5) == 0){
			step=atof(&argv[i][5]);
			if(step<=0)step=5;
		}
		else if (strncmp(argv[i], "int", 3) == 0)
			outputint = true;
		else if (strncmp(argv[i], "verbose", 7) == 0)
			verbose = true;
		else {
			cerr << argv[0] << ":\tunknown parameter '" << argv[i] << "'\n";
			exit(2);
		}
	}

	step*=PI/180.;
	Cylinders cyls, cylsref;
	if(!cyls.ReadInventor(argv[1])) {
		cerr<<argv[0]<<": cannot read file \""<<argv[1]<<"\"\n";
		exit(3);
	}

	if(!cylsref.ReadInventor(argv[2])) {
		cerr<<argv[0]<<": cannot read file \""<<argv[2]<<"\"\n";
		exit(4);
	}
	Cylinders cyls2=cyls.copy();
	Cylinders cylsref2=cylsref.copy();


	// parse the match list;
	int num=cyls.size();
	int* match=new int(num);
	if(matched){
		int index=8;
		i=0;
		do {
			match[i]=atoi(&argv[matcharg][index])-1;	// COSEC start from 1, we start from 0
			while(argv[matcharg][index]!=','&&argv[matcharg][index]){
				if(argv[matcharg][index])index++;
			}
			//printf("match %2d : %d\tindex=%d %c\n",i,match[i],index,argv[matcharg][index]);
			if(argv[matcharg][index]){
				++index;
				i++;
			}
		} while (i<num && argv[matcharg][index]);
		//printf("i=%d\tnum=%d\n",i,num);
		if(i!=num-1){
			cerr<<argv[0]<<": error reading in \""<<argv[matcharg]<<"\"\n";
			exit(5);
		}
		int* m2=new int(num);
		for(i=0;i<num;i++){
			if(match[i]>=0) m2[i]=i;
			else	m2[i]=-1;	// ignore too	
		}
		cyls=cyls.subset(m2,num);
		cylsref=cylsref.subset(match,num);
	}


	Vect t(0,0,0);

	// this works for identical but just rotated objects
	/*Euler rot(0,0,0), rotref(0,0,0);
	for(i=0;i<cyls.size();i++){
		Euler e=cyls[i]->Axis().euler().inverse();
		Euler eref=cylsref[i]->Axis().euler().inverse();
		cyls.Transform(e,t,t);			
		cylsref.Transform(eref,t,t);			
		rot=rot*e;
		rotref=rotref*eref;
	}
	rot=rot*rotref.inverse();
	*/
	Euler best(0,0,0);
	float mindiff=FLT_MAX;
	float alt0=0, alt1=PI, az0=0, az1=2*PI, phi0=0, phi1=2*PI, dist=0;
	while(step>0.02*PI/180){
		for(float alt=alt0;alt<alt1;alt+=step){
			for(float az=az0;az<az1;az+=step){
				for(float phi=phi0; phi<phi1;phi+=step){
					Euler e(alt,az,phi);
					Cylinders tmp=cyls.copy();
					tmp.Transform(e,t,t);
					float tmpdiff=tmp.OrientationDiff(cylsref);
					if(tmpdiff<mindiff){
						mindiff=tmpdiff;
						best=Euler(alt,az,phi);
						if(verbose)printf("Minimal axis angle diff= %.4f\tat ",tmpdiff*180/PI);
						best.print();
					}
				}
			}
		}
		alt0=best.alt()-step;	
		alt1=best.alt()+step;
		az0=best.az()-step;		
		az1=best.az()+step;
		phi0=best.phi()-step;	
		phi1=best.phi()+step;
		step/=5;
	}

	Cylinders cylstmp=cyls.copy();
	cyls.Transform(best,t,t);

	float totallength=0;
	for(i=0;i<cyls.size();i++){
		float tl=cylsref[i]->Length()+cyls[i]->Length();
		t=t+(cylsref[i]->Center()-cyls[i]->Center())*tl;
		totallength+=tl;
	}
	t=t/totallength;
	
	Vect tc(0,0,0);
	cylstmp.Transform(best,t,tc);
	
	if(outputint){
		cylstmp.WriteInventor("ivFitProbe.iv");
		cylsref.WriteInventor("ivFitRef.iv");
		if(verbose)
			printf("The fitted common cylinders are written to files: \"ivFitProbe.iv\" and \"ivFitRef.iv\"\n");
	}

	cyls2.Transform(best,t,tc);

	cyls2.WriteInventor(argv[3]);

	if(verbose) {
		for(i=0;i<cyls.size();i++){
			float tl=cylsref[i]->Length()+cylstmp[i]->Length();
			dist+=cylsref[i]->CenterDistance(cylstmp[i])*tl;
			totallength+=tl;
		}
		dist/=totallength;
		printf("Average  center   distance= %.4f Angstrom\n",dist);
		printf("Minimal average axis angle= %.4f Degree\n",mindiff*180/PI);
		best.print("Final rotation    needed: ");
		t.print( " Final translation needed: ");
	}
}
