// by Wen Jiang, June 2000
// use OpenInventor library, so it is only working on SGI at this moment
//
//N ivTransform
//F This program transforms the helices in Open Inventor format: rotate, translate, center, mirror
//1
//P <input IV file>	The input Open Inventor format file
//P <output IV file>	The output Open Inventor format file
//P	[rot=<alt,az,phi>|<qx,qy,qz,qa>]	set rotation paramters, 3 numbers (in degree) for Euler angle, 4 numbers for quaternion
//P	[trans=<dx,dy,dz>]	set tranalation parameters in Angstrom
//P	[center=<xc,yc,zc>]	set the center to point(xc,yc,zc), in Angstrom
//P	[mirror]	mirror the helices, e.g. flip the handness
//P	[verbose]	will output some intermediate information
//U	ivTransform in.iv out.iv rot=30,40,50 trans=12,23,34 mirror center=4,6,9 verbose
//U	ivTransform in.iv out.iv rot=0.5,0.4,0.7,270 trans=12,23,34 center=4,6,9 verbose
//D	This program works correctly only for the special Inventor file containing only cylinders (helices).
//E	When rotation is set using quaternion, the vector part does not need to be a unit vector, the angle part
//E	is set in degree


#include <iostream>
#include "Cylinder.h"

void	main(int argc, char **argv)
{

	if (argc < 3) {
		cerr << argv[0] << ":\ttransform a Open Inventor file describing the helices: rotate then translate\n";
		cerr << "Usage:\t" << argv[0] << " <input iv file> <out iv file> [rot=<qx,qy,qz,Q>|<alt,az,phi>] [trans=<dx,dy,dz>] [center=<dx,dy,dz>] [mirror] [verbose]\n";
		exit(1);
	}

	int rotnum;
	float rot1,rot2,rot3,rot4,trans1,trans2,trans3,cen1,cen2,cen3;
	Quater quater;
	Euler euler;
	Vect trans, center;
	
	bool mirror=false, verbose=false, isgood;
	for (int i = 3; i < argc; i++) {
		if (strncmp(argv[i], "rot=", 4) == 0){
			if((rotnum=sscanf(&argv[i][4],"%f,%f,%f,%f",&rot1,&rot2,&rot3,&rot4))==4){
				quater=Quater(rot1,rot2,rot3,rot4);
			} else if (rotnum==3) {
				euler=Euler(rot1*PI/180.,rot2*PI/180.,rot3*PI/180.);
			} else {
				cerr <<argv[0]<<": error in \""<<argv[i]<<"\" for [rot=<qx,qy,qz,Q>|<alt,az,phi>], set to Indentity rotation\n";
			}
		}
		else if(strncmp(argv[i], "trans=", 6) == 0){
			if(sscanf(&argv[i][6],"%f,%f,%f",&trans1,&trans2,&trans3)==3){
				trans=Vect(trans1,trans2,trans3);
			} else {
				cerr <<argv[0]<<": error in \""<<argv[i]<<"\" for [trans=<dx,dy,dz>], set to zero translation\n";
			}
		}
		else if(strncmp(argv[i], "center=", 7) == 0){
			if(sscanf(&argv[i][7],"%f,%f,%f",&cen1,&cen2,&cen3)==3){
				center=Vect(cen1,cen2,cen3);
			} else {
				cerr <<argv[0]<<": error in \""<<argv[i]<<"\" for [center=<dx,dy,dz>], set center to origin\n";
			}
		}
		else if (strncmp(argv[i], "mirror", 6) == 0)
			mirror = true;
		else if (strncmp(argv[i], "verbose", 7) == 0)
			verbose = true;
		else { 
			cerr << argv[0] << ":\tunknown parameter '" << argv[i] << "'\n"; 
			exit(2); 
		}
	}

	Cylinders cyls;
	if (!strcmp(&argv[1][strlen(argv[1])-4],".wrl") || !strcmp(&argv[1][strlen(argv[1])-3],".iv") )
		isgood=cyls.ReadInventor(argv[1]);
	else 
		isgood=cyls.ReadListfile(argv[1]);

	if(isgood) {
		if(rotnum==4)	cyls.Transform(quater,trans,center);
		else 			cyls.Transform(euler,trans,center);
		
		if(mirror)	cyls.mirror();
		
		cyls.WriteInventor(argv[2]);
		if(verbose)
			cout <<argv[0]<<": "<<cyls.size() <<" helices have been tranformed and wirtten into Open Inventor file \""<<argv[2]<<"\"\n";
	} 
}


