// multirefine.C Copyright 1999 Baylor College of Medicine
// 1999 Steve Ludtke
// This is a modified version of refine.C which refines several models
// simultaneously from a single set of raw data

//N multirefine
//F This performs refinement on a set of 3D models simultaneously
//T H
//1
//P <total iter>	As in refine
//P <# models>	The number of 3D models being refined
//P [sym=<sym0>,<sym1>,etc.]
//P []	see refine for other options
//D <I>NOTE: This program is still experimental</i>
//D This program operates almost identically to <a href=refine.html>refine</a>. The difference
//E is, this program will simultaneously refine several models from a single set of raw
//E images. That is, each particle will be grouped with one of the N models. In this way,
//E a heterogeneous data set can be separated into multiple models simultaneously. This can also
//E be used in some cases to filter out partially denatured proteins, etc. The options for this 
//E program are almost the same as those for refine, with the addition of the 2nd parameter
//E which specifies the number of models being refined.
//D Rather than putting a single threed.0a.mrc file in the run directory, a subdirectory should
//E be made for each starting model, called '1', '2', etc. Each directory should then contain
//E a different threed.0a.mrc file.
//D You may specify a (different) enforced symmetry for each of the models to be refined
//E this can help in a situation where the exact symmetry is unknown. It is a poor idea to
//E refine one model with some symmetry and another model with no (C1) symmetry at all. However
//E 2 different symmetries, like c8 and c9 would be fine. That is, the completely asymmetric model
//E is a special case which may cause odd behavior in a multiple refinement unless ALL models
//E have no symmetry (and in certain cases even then).


#include <cstdio>
#include <cstdlib>
//#include <unistd.h>
#include <cmath>
#include <ctime>
#include <cctype>
#include "util.h"
#include "EMData.h"

#define MAXMODELS 20		// arbitrary and poor programming, but here it is

void *cmra(void *fnum);

int main(int argc, char *argv[]) {
int i,j,k,n;
int iter=5,mask=-1,imask=-1,pad=0,classiter=8,classfp=0,goodbad=0,precen=0,nweight=0;
int minproc=0,maxproc=0,lp=0,alito=0,median=0,mode=2,shrink=1,sigfilt=0,can=0;
int noweight=0,multi=1,d3it=0,d3it2=0,amask1=0,amask3=0,phasecls=0,refine=0,usefilt=0;
float ang=7.5,keep=1.2,xfiles=0,appix=0,d3s=0,hard=0,range=90.0,ctfc=0,euler2=0.0,setsflp=0,setsfhp=0;
float amask2=0;
char s[1024],ss[1024],*sym[MAXMODELS],*ctfcw=NULL,*setsffile=NULL;
FILE *log;
time_t ti;

sym[0]=NULL;

appinit(argc,argv);

log=fopen("refine.log","ab");

if (argc<3) appusage(argv[0]);

for (i=3; i<argc; i++) {
	if (strncmp("proc=",argv[i],5)==0) sscanf(&argv[i][5],"%d,%d",&minproc,&maxproc);
	else if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp("3dit=",argv[i],5)==0) d3it=atoi(&argv[i][5]);
	else if (strncmp("3dit2=",argv[i],6)==0) d3it2=atoi(&argv[i][6]);
	else if (strncmp("imask=",argv[i],6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp("mode=",argv[i],5)==0) mode=atoi(&argv[i][5]);
	else if (strncmp("hard=",argv[i],5)==0) hard=atof(&argv[i][5]);
	else if (strncmp("classkeep=",argv[i],10)==0) keep=atof(&argv[i][10]);
	else if (strncmp("classiter=",argv[i],10)==0) classiter=atoi(&argv[i][10]);
	else if (strncmp("pad=",argv[i],4)==0) pad=atoi(&argv[i][4]);
	else if (strncmp("filt3d=",argv[i],7)==0) lp=atoi(&argv[i][7]);
	else if (strncmp("shrink=",argv[i],7)==0) shrink=atoi(&argv[i][7]);
	else if (strncmp("sigfilt",argv[i],7)==0) sigfilt=1;
	else if (strncmp("amask=",argv[i],6)==0) {
		sscanf(argv[i]+6,"%d,%f,%d",&amask1,&amask2,&amask3);
	}
	else if (strncmp("euler2",argv[i],6)==0) {
		if (argv[i][6]!='=') euler2=2.0;
		else euler2=atof(&argv[i][7]);
	}
	else if (strncmp("setsf=",argv[i],6)==0) {
		char s[512];
		s[0]=0;
		sscanf(argv[i]+6,"%f,%f,%s",&setsflp,&setsfhp,s);
		if (strlen(s)) {
			setsffile=(char *)malloc(strlen(s)+1);
			strcpy(setsffile,s);
		}
	}
	else if (strncmp("noweight",argv[i],8)==0) noweight=1;
	else if (strncmp("usefilt",argv[i],7)==0) usefilt=1;
	else if (strncmp("ang=",argv[i],4)==0) ang=atof(&argv[i][4]);
	else if (strncmp("ca3",argv[i],3)==0) can=3;
	else if (strncmp("d3s=",argv[i],4)==0) d3s=atof(&argv[i][4]);
	else if (strncmp("sym=",argv[i],4)==0) {
		if (strchr(argv[i]+4,',')) {
			j=0;
			char *p=(char *)malloc(strlen(argv[i]));
			strcpy(p,argv[i]+4);
			sym[0]=p;
			while (*p!=0) {
				if (*p==',') { j++; sym[j]=p+1; *p=0;}
				p++;
			}
		}
		else {
			for (j=0; j<MAXMODELS; j++) sym[j]=argv[i]+4;
		}
//		for (j=0; j<MAXMODELS; j++) printf("%s\n",sym[j]);
	}
//	else if (strncmp("restart",argv[i],7)==0) restart=1;
	else if (strncmp("precen",argv[i],6)==0) precen=1;
	else if (strncmp("nweight",argv[i],7)==0) nweight=1;
	else if (strncmp("ctfc=",argv[i],5)==0) ctfc=atof(&argv[i][5]);
	else if (strncmp("ctfcw=",argv[i],6)==0) ctfcw=&argv[i][6];
	else if (strncmp("goodbad",argv[i],7)==0) goodbad=1;
	else if (strncmp("classfp=",argv[i],8)==0) classfp=atoi(&argv[i][8]);
	else if (strncmp("median",argv[i],6)==0) median=1;
	else if (strncmp("phasecls",argv[i],8)==0) phasecls=1;
	else if (strncmp("fscls",argv[i],5)==0) phasecls=2;
	else if (strncmp("refine",argv[i],6)==0) refine=1;
	else if (strncmp("xfiles=",argv[i],7)==0) {
		xfiles=atof(&argv[i][7]);
		sscanf(&argv[i][7]," %f , %f , %d",&appix,&xfiles,&alito);
	}
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

int Ref=LOGbegin(argc,argv);
LOG(Ref,"start.hed",LOG_INFILE,NULL);
LOG(Ref,"refine.log",LOG_OUTFILE,NULL);

if (classiter<3) error(ERR_EXIT,"Classiter >=3 for multirefine.");

ti=time(0);
if (log) fprintf(log,"******** multirefine started %s",ctime(&ti));
for (i=0; i<argc; i++) fprintf(log,"%s ",argv[i]);
fprintf(log,"\n");

iter=atoi(argv[1]);
multi=atoi(argv[2]);
if (multi<2) { printf("Multi count must be >1\n"); exit(1); }
if (multi>MAXMODELS) error(ERR_EXIT,"MAXMODELS exceeded\n");

for (i=0; i<multi; i++) {
	sprintf(s,"%d/threed.0a.mrc",i);
	if (access(s,R_OK)) {
		printf("%s does not exist\n",s);
		exit(1);
	}
}


if (minproc<=0) minproc=1;
if (maxproc<=minproc) maxproc=minproc;
if (mask<2) error(ERR_EXIT,"mask= is a required parameter");

if (xfiles) {
	for (i=0; i<multi; i++) {
		sprintf(s,"cd %d; proc3d threed.0a.mrc x.0.mrc clip=%d,%d,%d mask=%d",i,mask*2,mask*2,mask*2,mask);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		run(s);

		if (xfiles>0) {
			sprintf(s,"cd %d; volume x.0.mrc %f set=%f",i,appix,xfiles);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }

			run(s);
		}
	}
}


for (i=1; i<=iter; i++) {
	ti=time(0);
	if (log) { fprintf(log,"Iteration %d beginning at %s",i,ctime(&ti)); fflush(log); }
	
// project3d, classesbymra, classalignall
	sprintf(s,"0/classes.%d.hed",i);
	if (access(s,R_OK)) {
		// project3d
		const char *t = tmpnam2();
		FILE *out = fopen(t,"wb");
		if (!out) { printf("Cannot open temporary file!\n"); exit(1); }

//		int p3dproc=maxproc/multi;
//		if (p3dproc<=0) p3dproc=1;
		int p3dproc=1;		// seems to be some problem with larger numbers...
		for (j=0; j<multi; j++) {
			sprintf(s,"%d/proj.hed",j);
			unlink(s);
			sprintf(s,"%d/proj.img",j);
			unlink(s);
		}
		for (j=0; j<multi*p3dproc; j++) {
			fprintf(out,"project3d %d/threed.%da.mrc out=%d/proj.hed prop=%1.3f mask=%d mode=-4 frac=%d/%d",j/p3dproc,i-1,j/p3dproc,ang,mask,j%p3dproc,p3dproc);
			if (sym[0]) fprintf(out," sym=%s",sym[j/p3dproc]);
			fprintf(out,"\n");
		}

//		for (j=0; j<multi*p3dproc; j++) {
//			fprintf(out,"project3d %d/threed.%da.mrc out=%d/proj.hed prop=%1.3f mask=%d mode=-4 frac=%d/%d",j/p3dproc,i-1,j/p3dproc,ang,mask,j%p3dproc,p3dproc);
//			if (sym[0]) fprintf(out," sym=%s",sym[j/p3dproc]);
//			fprintf(out,"\n");
//		}

		fclose(out);
		sprintf(s,"runpar proc=%d file=%s",maxproc,t);
		if (log) { ti=time(0); fprintf(log,"%s%s\n",ctime(&ti),s); fflush(log); }
		run(s);
		unlink(t);

		for (j=0; j<multi; j++) {
			sprintf(s,"cd %d; proc2d proj.hed cls mraprep",j);
			run(s);
		}

		if (classfp) {
			printf("Classfp not supported in multirefine\n"); exit(1);
/*			if (access("start.fp",R_OK)) { 
				if (log) { ti=time(0); fprintf(log,"footprintr start.hed start.fp   %s",ctime(&ti)); fflush(log); } 
				run("footprintr start.hed start.fp");
			}
			if (log) { ti=time(0); fprintf(log,"footprintr proj.hed proj.fp   %s",ctime(&ti)); fflush(log); } 
			run("footprintr proj.hed proj.fp");
			
			sprintf(s,"msa2 proj.fp start.fp proj.msa.fp start.msa.fp %d 100",classfp);
			if (log) { ti=time(0); fprintf(log,"%s    %s",s,ctime(&ti)); fflush(log); } 
			run(s);

			run("classesrefs start.msa.fp proj.msa.fp");
			if (log) { ti=time(0); fprintf(log,"classesrefs start.msa.fp proj.msa.fp   %s",ctime(&ti)); fflush(log); } */
		}
		else {
			n=fileCount("start.hed");
			if (n>maxproc) {
				if (n/maxproc>100) { n=(n/100); n-=n%maxproc; }
				else n=maxproc;
			}

			const char *t = tmpnam2();
			FILE *out = fopen(t,"wb");
			if (!out) { printf("Cannot open temporary file!\n"); exit(1); }
			
			run("iminfo 0/proj.hed");
			for (j=0; j<n; j++) {
				sprintf(s,"classesbymra2 start.hed proj.hed %d split frac=%d/%d mask=%d imask=%d shrink=%d%s%s%s%s%s",
					multi,j,n,mask,imask,shrink,precen?" precen":"",j==0?"":" quiet",can==3?" ca3":"",sigfilt?" sigfilt":"",(ctfc||ctfcw)?" ctfc":"");
				if (refine) sprintf(s+strlen(s)," refine");
				if (phasecls==1&&ctfcw) sprintf(s+strlen(s)," wtphase");
				else if (phasecls==1) sprintf(s+strlen(s)," phase");
				if (phasecls==2&&ctfcw) sprintf(s+strlen(s)," wtfscmp");
				else if (phasecls==2) sprintf(s+strlen(s)," fscmp");
				if (usefilt) sprintf(s+strlen(s)," usefilt");
				fprintf(out,"%s\n",s);
				if (log && !j) { ti=time(0); fprintf(log,"%s > %s  %s",s,t,ctime(&ti)); fflush(log); }
			}
			fclose(out);
			
			sprintf(s,"runpar proc=%d,%d file=%s",minproc,maxproc,t);
			run(s);
 			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			unlink(t);
		}
		
		for (j=0; j<multi; j++) {
			sprintf(s,"cd %d; tar cvf cls.%d.tar *cls*.lst; classalignall ref1 finalref saverefs keep=%1.2f %d proc=%d,%d mask=%d imask=%d %s%s",
				j, i, keep,classiter,minproc,maxproc,mask,imask,median?"median":"",precen?" precen":"");
			if (amask1>0) sprintf(s+strlen(s)," refmaskali");
			if (ctfc!=0) sprintf(s+strlen(s)," ctfc=%f",ctfc);
			if (ctfcw) sprintf(s+strlen(s)," ctfcw=%s",ctfcw);
			if (refine) sprintf(s+strlen(s)," refine");
			if (nweight) sprintf(s+strlen(s)," nweight");
			if (phasecls==1) sprintf(s+strlen(s)," phase");
			if (phasecls==2) sprintf(s+strlen(s)," fsc");
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			run(s);
		
			sprintf(s,"%d/classes.%d.img",j,i);
			sprintf(ss,"%d/classes.img",j);
			rename(ss,s);
			sprintf(s,"%d/classes.%d.hed",j,i);
			sprintf(ss,"%d/classes.hed",j);
			rename(ss,s);
			if (log) fprintf(log,"mv %d/classes.hed/img to %s/img\n",j,s);
		}
	}

// check for 3d files
	sprintf(s,"%d/threed.%da.mrc",multi-1,i);
	if (!access(s,R_OK)) { 
		if (log) { fprintf(log,"Iteration %d already complete\n",i); fflush(log); }
		continue;
	}

// This reassigns all of the euler angles
	if (euler2>0) {
	
		for (k=0; k<multi; k++) {
			// first, reproject the model with finer spacing
			sprintf(s,"cd %d; project3d threed.%da.mrc prop=%1.3f mask=%d mode=-1",k,i-1,ang/euler2,mask);
			if (sym[0]) sprintf(s+strlen(s)," sym=%s",sym[k]);
			run(s);

			const char *t = tmpnam2();
			FILE *out = fopen(t,"wb");
			if (!out) { printf("Cannot open temporary file!\n"); exit(1); }
			
			for (j=0; j<maxproc; j++) {
				sprintf(s,"cd %d; eulerproj classes.%d.hed proj.hed writeali mask=%d %s %s",k,i,mask,precen?" precen":"",sigfilt?"sigfilt":"");
				fprintf(out,"%s\n",s);
				if (log && !j) { ti=time(0); fprintf(log,"%s > %s  %s",s,t,ctime(&ti)); fflush(log); }
			}
			fclose(out);
			
			sprintf(s,"runpar proc=%d,%d file=%s",minproc,maxproc,t);
			run(s);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			unlink(t);

		}
	}


	for (k=0; k<multi; k++) {
		sprintf(s,"cd %d; make3d classes.%d.hed out=threed.%d.mrc mask=%d",k,i,i,mask);
		if (hard) sprintf(s+strlen(s)," hard=%1.2f",hard);
		if (d3s) sprintf(s+strlen(s)," keep=%1.2f",d3s);
		if (pad) sprintf(s+strlen(s)," pad=%d",pad);
		if (sym[0]) sprintf(s+strlen(s)," sym=%s",sym[k]);
		if (noweight) sprintf(s+strlen(s)," noweight");
		if (mode) sprintf(s+strlen(s)," mode=%d",mode);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		run(s);

// proc3d
		sprintf(s,"cd %d; proc3d threed.%d.mrc threed.%da.mrc norm mask=%d",k,i,i,mask);
		if (lp) sprintf(s+strlen(s)," lp=%d",lp);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		run(s);

//setsf
	if (setsflp) {
		sprintf(s,"cd %d; proc3d threed.%da.mrc threed.%da.mrc setsf=%s apix=%f lp=%f",k,i,i,setsffile?setsffile:ctfcw,appix,setsflp);
		if (setsfhp) sprintf(s+strlen(s)," hp=%f",setsfhp);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		run(s);
		unlink("classes.tmp.hed");
		unlink("classes.tmp.img");
		sprintf(s,"cd %d; proc2d classes.%d.hed classes.tmp.hed filefilt=proc3d.filt apix=%f lp=%f",k,i,appix,setsflp);
		if (setsfhp) sprintf(s+strlen(s)," hp=%f",setsfhp);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		run(s);
	}

//automask
		if (amask1>0 && xfiles) {
			sprintf(s,"cd %d; volume threed.%da.mrc %f set=%f",k,i,appix,xfiles);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			run(s);
		
			sprintf(s,"cd %d; proc3d threed.%da.mrc threed.%da.mrc automask2=%d,%1.3f,%d",k,i,i,amask1,amask2,amask3);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			run(s);
		}
	
	
//xfiles
		if (xfiles) {
			sprintf(s,"cd %d; proc3d threed.%da.mrc x.%d.mrc clip=%d,%d,%d",k,i,i,mask*2,mask*2,mask*2);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			run(s);
	
			if (i>alito) {
				sprintf(s,"cd %d; align3d x.%0d.mrc x.%d.mrc x.%d.mrc",k,alito,i,i);
				if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
				run(s);
			}
			else if (log) fprintf(log,"Alignment skipped\n");
			
			if (xfiles>0) {
				sprintf(s,"cd %d; volume x.%d.mrc %f set=%f",k,i,appix,xfiles);
				if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
				run(s);
			}
		}
		
		if (d3it || d3it2) {
			sprintf(s,"cd %d; clean3d classes.%d.hed threed.%da.mrc proc=%d sym=%s 3dit=%d 3dit2=%d",k,i,i,minproc,sym[k],d3it,d3it2);
			run(s);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		}
	}
	
}

if (log) { ti=time(0); fprintf(log,"%s  Refine complete\n",ctime(&ti)); }
fclose(log);
LOG(Ref,NULL,LOG_END,NULL);

return 0;
}

void *cmra(void *s)
{
char ss[1024];

strcpy(ss,(char *)s);
run(ss);
return NULL;
}
