// proc3d.C 9/97  Steve Ludtke
//N proc3d
//F Misc. 3d image processing
//T LM
//1
//P <input file>	Get info/stats on this file
//2
//P <input file>	Source file
//P <output file>	Destination file
//P [shrink=<div>]	Shrinks the image by an integral scaling factor, median filter
//P [scale=<factor>]	Rescales the image, generally used with clip= 
//P [clip=<x,y,z[,xc,yc,zc]>]	Make the output have this size, no scaling
//P [fftclip=<x,y,z>]	Make the output have this size, rescaling by padding FFT
//P [rot=<alt,az,phi>]	Rotate the volume, angles in degrees
//P [rotspin=<n1,n2,n3,angle>]	Rotate the volume around axis (n1,n2,n3) by angle degrees
//P [trans=<dx,dy,dz>]	Translate the volume, before the rotation
//P [posttrans=<dx,dy,dz>]	Translate the volume, after the rotation
//P [center]	Centers volume using center of mass
//P [acfcen]	Centers volume using cross correlation with mirrored model
//P [apix=<A/pix>]	Specify the Angstr/pixel for S scaling
//P [origin=<x,y,z>]	Set the coordinates for the pixel (0,0,0)
//P [lp=<filt r>]	Gaussian lowpass filter, r in angstroms if apix defined, pixels if undef
//P [hp=<filt r>]	Arctan highpass filter
//P [tlp=<filt r>]	Gaussian/artan mix lowpass
//P [unhp=<filt r>]	Undoes the hp= filter
//P [filefilt=<file>]	Applies a filter defined by an x/y text file
//P [rfilt=<type>[,<val1>[,<val2>[,<val3>]]]]	Apply one of a variety of real-space filters
//P [blfilt=<sigma spa,sigma den,iter,localwidth>]	Apply bilateral filtering
//P [normfilt=<thr,filt>]	This filter attempts to perform a 'local normalization' so low density and high density features will be on a more even playing field in an isosurface display. Thr is an isosurface threshold at which all desired features are visible, filt is a normalization size similar to an lp= value
//P [axialfilt=<filt r>]	This filter applies a correction for cases where reconstructions with many 'side' views have produced an asymmetric resolution. Specify the gaussian resolution in the z direction.
//P [flip]	Mirrors the object across the x-y plane
//P [yflip]	Mirrors the object across the x-z plane
//P [sym=<c?,icos,etc>]	Applies the specified symmetry to the map
//P [mask=<radius>[,<width>]]	Applies a circular mask to the circualr edgemean value, with an optional Gaussian falloff with the specified width
//P [maskzero=<radius>[,<width>]]	Applies a circular mask to 0.0
//P [imask=<radius>[,<width>]]	Applies an circular inner mask to 0.0
//P [automask=<thr1>,<thr2>]	Tries to mask out only interesting density
//P [automask2=<rad>,<thr>,<N>]	Same goal as automask
//P [maskfile=<file>]	Multiplies the input by the specified file using pixel indices(must be same size)
//P [maskfile2=<file>]	Multiplies the input by the specified file using pixel coordinates instead of pixel indices (can be different size) and use the original cube
//P [maskfile3=<file>]	Multiplies the input by the specified file using pixel coordinates instead of pixel indices (can be different size) and use the smaller cube of the two
//P [maskval=<int>]	Value to be used in a multi-valued masks
//P [mask1file=<file>]	This will take a file containing a set of masks and apply the first mask to the image
//P [maskresample=<ref file>]	Resample the input mask file to use the <ref file> apix, origin and size	
//P [maskmerge=<mask file2>]	Merge the second mask file to the first mask	
//P [cut1stquad]	Cut the first quad frin the map
//P [norm]	Normalizes the output, mean->0 sigma->1
//P [normto=<ref>]	Normalizes one volume to another to optimize linear mapping of the densities, ignoring zero values
//P [bin=<thresh>]	use the threshold value to turn the 3D map into a binary map
//P [bin2]	Turn any non-zero values into 1 to create a binary map
//P [mult=<constant>]	Scales the densities in the output
//P [add=<constant>]	Adds a constant to the densities
//P [noise=<level>]	Adds flatband noise at specified absolute level
//P [signoise=<sig mult>]	Same, but level in terms of sigma
//P [setsf=<SF file>]	Sets the structure factor based on a 1D x/y text file, must specify apix
//P [calcsf=<output>]	This will calculate a radial structure factor for the current 3D model and write it to the specified output file, must specify apix.
//P [icos5fTo3f]	Rotate the map from icos 5 fold view to 3 fold view
//P [icos3fTo5f]	Rotate the map from icos 3 fold view to 5 fold view
//P [icos5fTo2f]	Rotate the map from icos 5 fold view to 2 fold view
//P [icos2fTo5f]	Rotate the map from icos 2 fold view to 5 fold view
//P [icos5fhalfmap]	The input is the icos 5f top half map generated by the "tophalf" option
//P [tophalf]	The output only keeps the top half map to save some memory
//P [icos5fhalf2full]	Generate full map from the icos5f half map
//P [vtk]	Output should be in VTK format
//P [df3]	Output in df3 format (POVRAY)
//P [em]	Output should be in EM format.
//P [amira]	Output should be in AMIRA format, not MRC
//P [amiralabel]	Output should be in AMIRA labelField format
//P [amirashort]	Output should be in AMIRA short data type format
//P [amirabyte]	Output should be in AMIRA byte data type format
//P [rawiv]	Output should be in RawIV format, not MRC
//P [xplor]	Output should be in XPLOR format, not MRC
//P [icos]	Output should be in ICOS format, not MRC
//P [spider]	Output should be in Spider format, not MRC. Warning, this is a spider stackfile, and Spider may not be able to read stacks of 3D images. spidersingle is a better choice
//P [spidersingle]	Output should be in single-image Spider format, not MRC
//P [imagic]	Output is an IMAGIC file
//P [mrc8bit]	Output an 8 bit MRC image
//P [mrc16bit]	Output an 16 bit MRC image
//P [pif]	Output to a PIF (Purdue Image Format) file
//P [hdf]	Output to a HDF5 (Hierarchical Data Format) file
//P [png]	Output to a PNG (Portable Network Graphics) file
//P [gwin=<radius>]	Apply a Gaussian radial falloff, r is 1/e width
//P [edgenorm]	Normalize the image based on the mean value at the edge of the box
//3
//P <input 1>	Input 1 to compare/process
//P <input 2>	Input 2 to compare/process
//P [apix=<A/pix>]	Specify the Angstr/pixel for S scaling
//P [fsc=<out>]	Fourier shell correlation
//P [phase=<out>]	mean phase difference (not residual!)
//P [snr=<out>]	Relative signal to noise ratio between 2 images.
//P [rfactor=<out>]	R factor between inputs
//P [rms=<thresh>]	RMS variation above threshold
//P [rdiv=<out>]	Radial plot of input 1/input 2
//P [rdivf=<out>]	Radial plot of input 1/input 2 in Fourier space
//P [diffmap[=<~binary thr>]]	Generates a difference map in "diffmap.mrc"
//P [sum=<output>]	Adds input files
//P [pha1amp2=<out>]	Makes a new image with the phases from image 1 and the amplitudes from image2
//P [anim=<frames>]	Generates linearly interpolated intermediates between two maps
//P [animorph=<frames>,<vecfile>[,<start #>]]	Generates a linear morphing of one map into another based on a vector file
//U proc3d threed.5a.mrc
//U proc3d threed.1.mrc threed.1a.mrc unhp=1 lp=15
//U proc3d threed.1a.mrc threed.2a.mrc apix=4.67 fsc=corr12.dat
//D This program performs a variety of 3d image processing operations. Like all EMAN
//E programs, it will automatically read any supported file format. It can be invoked in
//E one of 3 ways. The first form simply provides statistical information on a single
//E volume data set.
//D The second form will copy an input file to and output file after processing it in
//E various ways. Most of the processing is explained in sufficient detail above, but
//E there are a few notes:
//E <UL>
//E <LI> The scale operation does NOT resize the volume in voxels.
//E Generally you want to specify both scale and clip.
//E <LI> If apix is not specified, all filters are specified in pixels in Fourier space. If apix
//E is specified, all filters are specified in Angstroms. ie - 'apix=4.5 lp=8' would filter
//E the volume using a Gaussian in fourier space with a 1/e width of 1/8 reciprocal angstroms.
//E <LI> If apix is specified, fsx, phase and rfactor will have their s-axis (1st column) in
//E reciprocal angstroms. Otherwise it will be in Fourier pixels.
//E <LI> The automask options are used to try to eliminate spurious noise outside the model.
//E Automask2 is preferred. To use this, specify 3 numbers. First, the smallest radius from
//E the center of the map that contacts some density of the 'good' part of the map. Second,
//E A threshold density at which all of the undesirable density is disconnected from the
//E desired mass. Finally, a number of 1-voxel 'shells' to include after the correct
//E density has been located (this allows you to use threshold densities higher than the desired
//E isosurface threshold density). The iterative shells will include a 'soft' Gaussian edge after 2 pixels. ie -
//E if you add 8 shells, the density will decay in this region using a 1/2 width of 3 pixels starting at the 3rd
//E pixel. If the number of shells is specified as a negative value, then the mask will have a sharp edge, and
//E any hollow areas inside the mask will be filled in.
//E <LI> To invert the density of a map, use 'mult=-1'.
//E <LI> 'phase=' calculates the mean phase difference between two maps, NOT the true
//E phase residual (which is a very messy thing to calculate).
//E <LI> The symmetries supported by 'sym=' are: Cn, Dn and icos. for C and D symmetries,
//E the n-fold axis must be the z axis.
//E </UL>
//D Realfilter options detailed in proc2d docs. 3D - specific filters:
//E <UL>
//E <li>110: Iterative expansion of a binary mask, val1 = number of layers to expand, val2 = number of layers before starting Gaussian decay
//E <li>111: Iterative expansion of a multilevel mask file (see qsegment). Simultaneoulsy expands all levels and prevents overlap problems.
//E </UL>

#include <cstdio>
#include <cmath>
#include "Vect.h"
#include "EMData.h"
#include "List.h"
#include "util.h"

inline float asub(float x,float y) {
float r;

r=fabs(x-y);
if (r>PI) r=2.0*PI-r;

return r;
}

inline bool range(float x1,float x2,float x3) { if (x1<x2 && x2<x3) return 1; return 0; }

int main(int argc, char *argv[]) {
int i,j,k,l,ii,nx=0,ny=0,nz=0,x,y,z,mask=0,imask=0,norm=0,anim=0,animorph2=0,rfilt=-1,eightbit=0,sixteenbit=0;
int flip=0,yflip=0,center=0,acfcen=0,shrink=1,spider=0,imagic=0,vtk=0,em=0,amira=0,icos=0,xplor=0,pif=0,hdf=0,png=0,rawiv=0,mrc=0;
float scale=1.0,noise=0,signoise=0,lp=0,hp=0.0,unhp=0.0,tlp=0,gwin=0,diffmap=0,amasklev=999.0,amasklev2=999.0;
float dx=0,dy=0,dz=0,dalt=0,daz=0,dphi=0,mult=0,add=0,rms=1.0e10,smask=0,apix=-1.0,fsc0=-.0,fsc1=90.0;
EMData *data,*d2,*dan,*amask=NULL,*amask2=NULL;
int a2r=1,a2x=1;
int fnx=0,fny=0,fnz=0;
float *dat,*dat2,*datan;
float a,r,*rd,*nrd,*nrd2,rft1=0,rft2=0,rft3=0;
float normfiltthr=0,normfiltr=0,axial=0.0;
char *pha=NULL,*rfac=NULL,*rdiv=NULL,*rdivf=NULL,fsp[40],s[80],*normto=NULL,*setsf=NULL;
char *sym=NULL,*sum=NULL,*fsc=NULL,*filefilt=NULL,*mfile=NULL,*m1file=NULL,*snr=NULL,*pha1amp2=NULL,*sfout=NULL;
char *animorph=NULL;
char *mfile2=NULL, *mfile3=NULL, *mfile4=NULL;
char * maskreffile=NULL, *mask2file=NULL;
bool cut1stquad = false;
int maskval = 0;
float maskw=0.0, imaskw=0.0;
FILE *out;

bool keep=false;
int kx0,kx1,ky0,ky1,kz0,kz1;
float binthresh=0;
bool bin=false, bin2=false;
bool icos5fhalf = false, icos5fhalf2full = false;
bool icos5fTo3f = false, icos3fTo5f = false, icos5fTo2f = false, icos2fTo5f = false;

int bliter=0, blwidth=0, xc=-1, yc=-1, zc=-1;
float blsigma1=0, blsigma2=0;
bool setorigin=false, tophalf=false;
float origx=0,origy=0,origz=0;
int amiralabel=0, amirashort=0, amirabyte=0,df3=0;
float pdx=0, pdy=0, pdz=0;

appinit(argc,argv);

if (argc<2) appusage(argv[0]);

for (i=3; i<argc; i++) {
	if (strncmp(argv[i],"clip=",5)==0) {
		if( sscanf(&argv[i][5],"%d,%d,%d,%d,%d,%d",&nx,&ny,&nz,&xc,&yc,&zc) != 6)
			sscanf(&argv[i][5],"%d , %d , %d",&nx,&ny,&nz);
	}
	else if (strncmp(argv[i],"fftclip=",8)==0) {
		sscanf(&argv[i][8],"%d,%d,%d",&fnx,&fny,&fnz);
	}
	else if (strncmp(argv[i],"scale=",6)==0) scale=atof(&argv[i][6]);
	else if (strncmp(argv[i],"apix=",5)==0) apix=atof(&argv[i][5]);
	else if (strncmp(argv[i],"origin=",7)==0) {
		if( sscanf(argv[i]+7,"%f,%f,%f",&origx,&origy,&origz) == 3 ) setorigin=true;
	}
	else if (strncmp(argv[i],"signoise=",9)==0) signoise=atof(&argv[i][9]);
	else if (strncmp(argv[i],"noise=",6)==0) noise=atof(&argv[i][6]);
	else if (strncmp(argv[i],"mask=",5)==0) sscanf(&argv[i][5],"%d,%f", &mask, &maskw);
	else if (strncmp(argv[i],"maskzero=",9)==0) { sscanf(&argv[i][9],"%d,%f", &mask, &maskw); mask=-mask; }
	else if (strncmp(argv[i],"smartmask",9)==0) smask=1.0;
	else if (strncmp(argv[i],"maskfile=",9)==0) mfile=&argv[i][9];
	else if (strncmp(argv[i],"maskfile2=",10)==0) mfile2=&argv[i][10];
	else if (strncmp(argv[i],"maskfile3=",10)==0) mfile3=&argv[i][10];
	else if (strncmp(argv[i],"maskfile4=",10)==0) mfile4=&argv[i][10];
	else if (strncmp(argv[i],"maskval=",8)==0) maskval=atoi(&argv[i][8]);
	else if (strncmp(argv[i],"mask1file=",10)==0) m1file=&argv[i][10];
	else if (strncmp(argv[i],"maskresample=",13)==0) maskreffile=&argv[i][13];
	else if (strncmp(argv[i],"maskmerge=",10)==0) mask2file=&argv[i][13];
	else if (strncmp(argv[i],"cut1stquad",10)==0) cut1stquad=true;
	else if (strncmp(argv[i],"bin2",4)==0) bin2=true;
	else if (strncmp(argv[i],"bin=",4)==0) {binthresh=atof(&argv[i][4]); bin=true;}
	else if (strncmp(argv[i],"flip",4)==0) flip=1;
	else if (strncmp(argv[i],"yflip",5)==0) yflip=1;
	else if (strncmp(argv[i],"mrc8bit",7)==0) eightbit=1;
	else if (strncmp(argv[i],"mrc16bit",8)==0) sixteenbit=1;
	else if (strncmp(argv[i],"mrc",3)==0) mrc=1;
	else if (strncmp(argv[i],"spidersingle",12)==0) spider=2;
	else if (strncmp(argv[i],"spider",6)==0) spider=1;
	else if (strncmp(argv[i],"setsf=",6)==0) setsf=&argv[i][6];
	else if (strncmp(argv[i],"calcsf=",7)==0) sfout=&argv[i][7];
	else if (strncmp(argv[i],"axialfilt=",10)==0) axial=atof(&argv[i][10]);
	else if (strncmp(argv[i],"rfilt=",6)==0) {
		sscanf(argv[i]+6,"%d,%f,%f,%f",&rfilt,&rft1,&rft2,&rft3);
	}
	else if (strncmp(argv[i],"blfilt=",7)==0) {
		sscanf(argv[i]+7,"%f,%f,%d,%d",&blsigma1,&blsigma2,&bliter,&blwidth);
	}
	else if (strncmp(argv[i],"imagic",6)==0) imagic=1;
	else if (strncmp(argv[i],"vtk",3)==0) vtk=1;
	else if (strcmp(argv[i],"amiralabel")==0) { amira=1; amiralabel=1; }
	else if (strcmp(argv[i],"amirashort")==0) { amira=1; amirashort=1; }
	else if (strcmp(argv[i],"amirabyte")==0) { amira=1; amirabyte=1; }
	else if (strcmp(argv[i],"amira")==0) amira=1;
	else if (strncmp(argv[i],"rawiv",5)==0) rawiv=1;
	else if (strcmp(argv[i],"icos")==0) icos=1;
	else if (strncmp(argv[i],"xplor",4)==0) xplor=1;
	else if (strncmp(argv[i],"pif",3)==0) pif=1;
	else if (strncmp(argv[i],"df3",3)==0) df3=1;
	else if (strncmp(argv[i],"em",2)==0) em=1;
	else if (strncmp(argv[i],"hdf",3)==0) hdf=1;
	else if (strncmp(argv[i],"png",3)==0) png=1;
	else if (strcmp(argv[i],"icos5fhalfmap")==0) icos5fhalf=true;
	else if (strcmp(argv[i],"icos5fhalf2full")==0) icos5fhalf2full=true;
	else if (strcmp(argv[i],"icos5fTo3f")==0) icos5fTo3f=true;
	else if (strcmp(argv[i],"icos5fTo2f")==0) icos5fTo2f=true;
	else if (strcmp(argv[i],"icos3fTo5f")==0) icos3fTo5f=true;
	else if (strcmp(argv[i],"icos2fTo5f")==0) icos2fTo5f=true;
	else if (strncmp(argv[i],"tophalf",6)==0) tophalf=true;
	else if (strncmp(argv[i],"center",6)==0) center=1;
	else if (strncmp(argv[i],"acfcen",6)==0) acfcen=1;
	else if (strncmp(argv[i],"diffmap",7)==0) {
		if (argv[i][7]=='=') diffmap=atof(&argv[i][8]);
		else diffmap=9999.0;
	}
	else if (strncmp(argv[i],"sum=",4)==0) {
		sum=&argv[i][4];
	}
	else if (strncmp(argv[i],"sym=",4)==0) {
		sym=&argv[i][4];
		Euler e;
		e.setSym(sym);
		if (e.getMaxSymEl()==0) { printf("Unsupported symmetry\n"); exit(1); }
	}
	else if (strncmp(argv[i],"imask=",6)==0) sscanf(&argv[i][6],"%d,%f", &imask, &imaskw);
	else if (strncmp(argv[i],"anim=",5)==0) anim=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"animorph=",9)==0) {
		animorph=(char *)malloc(256);
		sscanf(argv[i]+9,"%d,%[^,],%d",&anim,animorph,&animorph2);
	}
	else if (strncmp(argv[i],"shrink=",7)==0) shrink=atoi(&argv[i][7]);
	else if (strncmp(argv[i],"trans=",6)==0) {
		sscanf(&argv[i][6]," %f , %f , %f",&dx,&dy,&dz);
	}
	else if (strncmp(argv[i],"posttrans=",10)==0) {
		sscanf(&argv[i][6]," %f , %f , %f",&pdx,&pdy,&pdz);
	}
	else if (strncmp(argv[i],"rotspin=",8)==0) {
		float n1=0, n2=0, n3=1, angle=0;
		sscanf(&argv[i][8]," %f , %f , %f, %f",&n1, &n2, &n3, &angle);
		if(angle) {
			Euler e(n1,n2,n3,angle*PI/180.,Euler::SPIN);
			dalt = e.alt() * 180./PI;
			daz = e.az() * 180./PI;
			dphi = e.phi() * 180./PI;
			printf("spin %g,%g,%g,%g => euler %g,%g,%g\n",n1,n2,n3,angle,dalt,daz,dphi);
		}
	}
	else if (strncmp(argv[i],"rot=",4)==0) {
		sscanf(&argv[i][4]," %f , %f , %f",&dalt,&daz,&dphi);
	}
	else if (strncmp(argv[i],"normfilt=",9)==0) {
		sscanf(&argv[i][9],"%f,%f",&normfiltthr,&normfiltr);
		if (normfiltthr<=0) error(ERR_EXIT,"normfilt only works on positive densities with positive thresholds");
	}
	else if (strncmp(argv[i],"norm",4)==0) norm=1;
	else if (strncmp(argv[i],"edgenorm",8)==0) norm=2;
	else if (strncmp(argv[i],"normto=",7)==0) normto=&argv[i][7];
	else if (strncmp(argv[i],"rms=",4)==0) rms=atof(&argv[i][4]);
	else if (strncmp(argv[i],"mult=",5)==0) mult=atof(&argv[i][5]);
	else if (strncmp(argv[i],"add=",4)==0) add=atof(&argv[i][4]);
	else if (strncmp(argv[i],"pha1amp2=",9)==0) pha1amp2=argv[i]+9;
	else if (strncmp(argv[i],"automask2=",10)==0) {
		amask2=new EMData;
		sscanf(&argv[i][10],"%d,%f,%d",&a2r,&amasklev,&a2x);
	}
	else if (strncmp(argv[i],"automask=",9)==0) {
		amask=new EMData;
		sscanf(&argv[i][9],"%f,%f",&amasklev,&amasklev2);
	}
	else if (strncmp(argv[i],"lp=",3)==0) lp=atof(&argv[i][3]);
	else if (strncmp(argv[i],"tlp=",4)==0) tlp=atof(&argv[i][4]);
	else if (strncmp(argv[i],"hp=",3)==0) hp=atof(&argv[i][3]);
	else if (strncmp(argv[i],"unhp=",5)==0) unhp=atof(&argv[i][5]);
	else if (strncmp(argv[i],"phase=",6)==0) pha=&argv[i][6];
	else if (strncmp(argv[i],"snr=",4)==0) snr=&argv[i][4];
	else if (strncmp(argv[i],"fsc=",4)==0) {
		fsc=(char *)malloc(strlen(argv[i])+1);
		sscanf(&argv[i][4],"%[^,],%f,%f",fsc,&fsc0,&fsc1);
	}
	else if (strncmp(argv[i],"filefilt=",9)==0) filefilt=&argv[i][9];
	else if (strncmp(argv[i],"rdiv=",5)==0) rdiv=&argv[i][5];
	else if (strncmp(argv[i],"rdivf=",6)==0) rdivf=&argv[i][6];
	else if (strncmp(argv[i],"rfactor=",8)==0) rfac=&argv[i][8];
	else if (strncmp(argv[i],"gwin=",5)==0) gwin=atof(&argv[i][5]);
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

//if (pif) error(ERR_EXIT,"Sorry, PIF writing not yet supported");

data=new EMData;
data->readImage(argv[1],-1);
if(apix>0)data->setPixel(apix);
if(setorigin)data->setXYZOrigin(origx,origy,origz);
if(icos5fhalf) data->set_map_type(ICOS5F_HALF);

if (fsc1-fsc0<5) {
	printf("Minimimum angular range for Fourier shell correlation is 5 degrees\n");
	exit(1);
}

int Ref=LOGbegin(argc,argv);

if (norm==2) data->edgeNormalize();

if (animorph) {
	nx=data->xSize();
	ny=data->ySize();
	nz=data->zSize();

	float *coord;
	int nv=0;
	char s[256];
	float a;
	FILE *vecfile=fopen(animorph,"r");
	if (!vecfile) { printf("Cannot read animorph vectors\n"); exit(1); }
	while (fgets(s,255,vecfile) && sscanf(s,"%f %f %f %f %f %f",&a,&a,&a,&a,&a,&a)==6) nv++;
	
	coord=(float*) malloc(nv*6*sizeof(float));
	rewind(vecfile);
	
	float fa,fb,fc,fd,fe,ff;
	for (i=0; i<nv*6; i+=6) {
		fgets(s,255,vecfile);
		// read (x1,y1,z1) (x2,y2,z2) -> (xc,yc,zc) (dx,dy,dz)
		sscanf(s," %f %f %f %f %f %f",&fa,&fb,&fc,&fd,&fe,&ff);
		coord[i]  =(data->xSize()/2+fa+fd)/2.0;
		coord[i+1]=(data->ySize()/2+fb+fe)/2.0;
		coord[i+2]=(data->zSize()/2+fc+ff)/2.0;
		coord[i+3]=-(fd-fa);
		coord[i+4]=-(fe-fb);
		coord[i+5]=-(ff-fc);
	}
	
	d2=new EMData;
	d2->readImage(argv[2],-1);
	d2->setPixel(data->Pixel());
	
	dan=new EMData;
	dan->setSize(nx,ny,nz);

	for (i=0; i<anim; i++) {
		r=(float)i/(float)(anim-1);
		int x,y,z;
		// ok, this is the magic morphing formula
		for (x=0; x<nx; x++) {
			for (y=0; y<ny; y++) {
				for (z=0; z<nz; z++) {
					float dx=0,dy=0,dz=0,ws=0,w;
					for (j=0; j<nv*6; j+=6) {
						w=1.0/hypot3(coord[j]-x,coord[j+1]-y,coord[j+2]-z);		// 1/r^2 weighting
						dx+=coord[j+3]*w;
						dy+=coord[j+4]*w;
						dz+=coord[j+5]*w;
						ws+=w;
					}
					dx/=ws;
					dy/=ws;
					dz/=ws;
					// now dx,dy,dz is the local translation vector from the initial map to the final map
					
					fa=data->valueAtInterp(x-dx*r,y-dy*r,z-dz*r);
					fb=  d2->valueAtInterp(x+dx*(1.0-r),y+dy*(1.0-r),z+dz*(1.0-r));
					dan->setValueAt(x,y,z,fa*(1.0-r)+fb*r);
				}
			}
		}
		dan->update();
		sprintf(fsp,"morph.%02d.mrc",i+1+animorph2);
		dan->writeMRC(fsp);
		printf("wrote %s\n",fsp);
		LOG(Ref,fsp,LOG_OUTFILE,NULL);
	}

	LOG(Ref,NULL,LOG_END,NULL);
	exit(0);	
}
else if (anim) {
	nx=data->xSize();
	ny=data->ySize();
	nz=data->zSize();

	d2=new EMData;
	d2->readImage(argv[2],-1);
	d2->setPixel(data->Pixel());

	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[2],LOG_INFILE,NULL);

	dan=new EMData;
	dan->setSize(nx,ny,nz);

	dat=data->getDataRO();
	dat2=d2->getDataRO();

	for (i=0; i<anim; i++) {
		r=(float)i/(float)(anim-1);
		datan=dan->getData();
		for (j=0; j<nx*ny*nz; j++) datan[j]=dat[j]*r+dat2[j]*(1.0-r);
		dan->doneData();
		sprintf(fsp,"anim.%02d.mrc",i+1);
		dan->writeMRC(fsp);
		printf("wrote %s\n",fsp);
		LOG(Ref,fsp,LOG_OUTFILE,NULL);
	}

	LOG(Ref,NULL,LOG_END,NULL);
	exit(0);
}

if (diffmap) {
	nx=data->xSize();
	ny=data->ySize();
	nz=data->zSize();

	d2=new EMData;
	d2->readImage(argv[2],-1);
	d2->setPixel(data->Pixel());

	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[2],LOG_INFILE,NULL);
	LOG(Ref,"diffmap.mrc",LOG_OUTFILE,NULL);

	dan=new EMData;
	dan->setSize(nx,ny,nz);

	dat=data->getDataRO();
	dat2=d2->getDataRO();

	datan=dan->getData();

	if (diffmap==9999.0) {
		for (j=0; j<nx*ny*nz; j++) datan[j]=dat[j]-dat2[j];
	}
	else {
//		for (j=0; j<nx*ny*nz; j++) datan[j]=(dat[j]>=diffmap?1.0:0)-(dat2[j]>=diffmap?1.0:0);
		for (j=0; j<nx*ny*nz; j++) datan[j]=(1+tanh(4.0*dat[j]/diffmap-4.0))-(1+tanh(4.0*dat2[j]/diffmap-4.0));
	}
	dan->doneData();
	dan->writeMRC("diffmap.mrc");
	
	LOG(Ref,NULL,LOG_END,NULL);
	exit(0);
}

// combines phases from image 1 with amplitudes from image 2 
if (pha1amp2) {
	d2=new EMData;
	d2->readImage(argv[2],-1);
	d2->setPixel(data->Pixel());
	
	EMData *df = data->doFFT();
	EMData *df2 = d2->doFFT();
	df->ri2ap();
	df2->ri2ap();
	
	float *dt = df->getData();
	float *dt2 = df2->getData();
	
	for (i=1; i<df->xSize()*df->ySize()*df->zSize(); i+=2) {
		dt2[i]=dt[i];
	}
	df->doneData();
	df2->doneData();
	df2->ap2ri();
	
	d2=df2->doIFT();
	d2->writeMRC(pha1amp2);
	exit(0);
}

if (sum) {
	nx=data->xSize();
	ny=data->ySize();
	nz=data->zSize();

	d2=new EMData;
	d2->readImage(argv[2],-1);
	d2->setPixel(data->Pixel());

	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[2],LOG_INFILE,NULL);
	LOG(Ref,sum,LOG_OUTFILE,NULL);
	
	dan=new EMData;
	dan->setSize(nx,ny,nz);
	
	dat=data->getDataRO();
	dat2=d2->getDataRO();

	datan=dan->getData();

	for (j=0; j<nx*ny*nz; j++) datan[j]=dat[j]+dat2[j];

	dan->doneData();
	dan->writeMRC(sum);
	
	LOG(Ref,NULL,LOG_END,NULL);
	exit(0);
}

if (rdiv) {
	nx=data->xSize();
	ny=data->ySize();
	nz=data->zSize();


	d2=new EMData;
	d2->readImage(argv[2],-1);
	d2->setPixel(data->Pixel());
	float sig=d2->Sigma()/2.0;
	dat=data->getDataRO();
	dat2=d2->getDataRO();

	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[2],LOG_INFILE,NULL);

	rd=(float *)malloc(ny/2*sizeof(float));
	nrd=(float *)malloc(ny/2*sizeof(float));

	memset(rd,0,ny/2*sizeof(float));
	memset(nrd,0,ny/2*sizeof(float));
	for (k=0,y=0; k<nz; k++) {
		for (j=0; j<ny; j++) {
			for (i=0; i<nx; i++,y++) {
				r=sqrt(SQR(i-nx/2)+SQR(j-ny/2)+SQR(k-nz/2));
				x=floor(r+.5)-1;
//				if (x<ny/2 && x>=0) { rd[x]+=fabs(dat[y]); nrd[x]+=fabs(dat2[y]); } 
				if (x<ny/2 && x>=0 && fabs(dat2[y])>sig) { rd[x]+=(dat[y]); nrd[x]+=(dat2[y]); } 
			}
		}
	}
	out=fopen(rdiv,"wb");
	for (i=0; i<ny/2; i++) fprintf(out,"%d\t%f\n",i,rd[i]/nrd[i]);
//	for (i=0; i<ny/2; i++) fprintf(out,"%d\t%f\n",i,rd[i]);
	fclose(out);

	LOG(Ref,rdiv,LOG_OUTFILE,NULL);
	LOG(Ref,NULL,LOG_END,NULL);

	exit(0);
}

if (rdivf) {
	nx=data->xSize();
	ny=data->ySize();
	nz=data->zSize();


	d2=new EMData;
	d2->readImage(argv[2],-1);
	d2->setPixel(data->Pixel());
	float sig=d2->Sigma()/2.0;

	d2=d2->doFFT();	// not safe. Only ok because we're exiting soon
	d2->ri2ap();
	data=data->doFFT();
	data->ri2ap();
	
	dat=data->getDataRO();
	dat2=d2->getDataRO();

	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[2],LOG_INFILE,NULL);

	rd=(float *)malloc(ny/2*sizeof(float));
	nrd=(float *)malloc(ny/2*sizeof(float));

	memset(rd,0,ny/2*sizeof(float));
	memset(nrd,0,ny/2*sizeof(float));
	for (k=0,y=0; k<nz; k++) {
		for (j=0; j<ny; j++) {
			for (i=0; i<nx/2+1; i++,y+=2) {
				r=sqrt(SQR(i)+SQR(j-ny/2)+SQR(k-nz/2));
				x=floor(r+.5)-1;
//				if (x<ny/2 && x>=0) { rd[x]+=fabs(dat[y]); nrd[x]+=fabs(dat2[y]); } 
//				if (x<ny/2 && x>=0 && fabs(dat2[y])>sig) { rd[x]+=(dat[y]/dat2[y]); nrd[x]++; } 
				if (x<ny/2 && x>=0) { rd[x]+=(dat[y]/dat2[y]); nrd[x]++; } 
			}
		}
	}
	out=fopen(rdivf,"wb");
	if (!out) { printf("Cannot open '%s'\n",rdivf); exit(1); }
	for (i=0; i<ny/2; i++) fprintf(out,"%d\t%f\n",i,rd[i]/nrd[i]);
//	for (i=0; i<ny/2; i++) fprintf(out,"%d\t%f\n",i,rd[i]);
	fclose(out);

	LOG(Ref,rdivf,LOG_OUTFILE,NULL);
	LOG(Ref,NULL,LOG_END,NULL);

	exit(0);
}

if (rms!=1.0e10) {
	float v=0,c=0;

	nx=data->xSize();
	ny=data->ySize();
	nz=data->zSize();

	d2=new EMData;
	d2->readImage(argv[2],-1);
	d2->setPixel(data->Pixel());
	dat=data->getDataRO();
	dat2=d2->getDataRO();

	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[2],LOG_INFILE,NULL);
	
	for (k=y=0; k<nz; k++) {
		for (j=0; j<ny; j++) {
			for (i=0; i<nx; i++,y++) {
				if (dat[y]>rms ||dat2[y]>rms) { v+=pow((float)dat[y]-dat2[y],(float)2.0); c+=1.0; }
			}
		}
	}
	if (c==0) { printf("RMS: Nothing above threshold!!!\n"); exit(0); }
	v=sqrt(v/c);
	printf("RMS=%f\n",v);
	sprintf(s,"RMS=%f\n",v);
	data->doneData();
	d2->doneData();
	if (!pha && !rfac) { LOG(Ref,NULL,LOG_END,s); exit(0); }
}

// calculate amplitude weighted phase residual
if (pha || rfac || fsc || snr ) {
	nz=data->zSize();

	d2=new EMData;
	d2->readImage(argv[2],-1);
	d2->setPixel(data->Pixel());
//	d2=d2->clip(-nz/2,-nz/2,-nz/2,nz*2,nz*2,nz*2);
	d2=d2->doFFT();
	d2->toCorner();
	d2->ri2ap();

	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[2],LOG_INFILE,NULL);
	
//	data=data->clip(-nz/2,-nz/2,-nz/2,nz*2,nz*2,nz*2);
	data=data->doFFT();
	data->toCorner();
	data->ri2ap();

	nx=data->xSize();
	ny=data->ySize();
	nz=data->zSize();
	
	dat=data->getDataRO();
	dat2=d2->getDataRO();

	rd=(float *)malloc(ny/2*sizeof(float));
	nrd=(float *)malloc(ny/2*sizeof(float));
	nrd2=(float *)malloc(ny/2*sizeof(float));

	if (pha) {
		memset(rd,0,ny/2*sizeof(float));
		memset(nrd,0,ny/2*sizeof(float));
		for (k=0,y=1; k<nz; k++) {
			for (j=0; j<ny; j++) {
				for (i=0; i<nx; i+=2,y+=2) {
					r=sqrt(SQR(i/2)+SQR(j-ny/2)+SQR(k-nz/2));

					x=floor(r+.5)-1;
					if (x<ny/2 && x>=0) { rd[x]+=dat[y-1]*dat2[y-1]*asub(dat[y],dat2[y]); nrd[x]+=dat[y-1]*dat2[y-1]; } 
				}
			}
		}
		out=fopen(pha,"wb");
		if (!out) { printf("Cannot open output\n"); exit(1); }
		if (apix>0) {
			for (i=0; i<ny/2; i++) if (nrd[i]>0) fprintf(out,"%g\t%f\n",(i+1)/(apix*y),rd[i]/nrd[i]/PI*180.0);
		}
		else {
			for (i=0; i<ny/2; i++) if (nrd[i]>0) fprintf(out,"%d\t%f\n",i+1,rd[i]/nrd[i]/PI*180.0);
		}
		LOG(Ref,pha,LOG_OUTFILE,"phase residual");
		fclose(out);
	}

	
	if (rfac) {
		memset(rd,0,ny/2*sizeof(float));
		memset(nrd,0,ny/2*sizeof(float));
		for (k=y=0; k<nz; k++) {
			for (j=0; j<ny; j++) {
				for (i=0; i<nx; i+=2,y+=2) {
					r=sqrt(SQR(i/2)+SQR(j-ny/2)+SQR(k-nz/2));

					x=floor(r+.5)-1;
					if (x<ny/2 && x>=0) { rd[x]+=fabs(dat[y]-dat2[y]); nrd[x]+=fabs(dat[y]); } 
				}
			}
		}
		out=fopen(rfac,"wb");
		if (!out) { printf("Cannot open output\n"); exit(1); }
		if (apix>0) {
			for (i=0; i<ny/2; i++) if (nrd[i]>0) fprintf(out,"%g\t%f\n",(i+1)/(apix*y),rd[i]/nrd[i]);
		}
		else {
			for (i=0; i<ny/2; i++) if (nrd[i]>0) fprintf(out,"%d\t%f\n",i+1,rd[i]/nrd[i]);
		}
		fclose(out);
		LOG(Ref,rfac,LOG_OUTFILE,"R-factor");
	}

	if (fsc) {
		data->doneData();
		d2->doneData();
		data->ap2ri();
		d2->ap2ri();
		dat=data->getDataRO();
		dat2=d2->getDataRO();

		if (fsc0==0.0 && fsc1==90.0) fsc0=fsc1=0;
		fsc0=fsc0*PI/180.0;
		fsc1=fsc1*PI/180.0;

		memset(rd,0,ny/2*sizeof(float));
		memset(nrd,0,ny/2*sizeof(float));
		memset(nrd2,0,ny/2*sizeof(float));
		for (k=0,y=0; k<nz; k++) {
			for (j=0; j<ny; j++) {
				for (i=0; i<nx; i+=2,y+=2) {
					r=sqrt(SQR(i/2)+SQR(j-ny/2)+SQR(k-nz/2));
					if (fsc0 || fsc1) {
						a=fabs(asin((k-nz/2)/r));
						if (!goodf(&a) || a<fsc0 || a>fsc1) continue;
					}

					x=floor(r+.5)-1;
					if (x<ny/2 && x>=0) { 
						rd[x]+=dat[y]*dat2[y]+dat[y+1]*dat2[y+1]; 
						nrd[x]+=dat[y]*dat[y]+dat[y+1]*dat[y+1]; 
						nrd2[x]+=dat2[y]*dat2[y]+dat2[y+1]*dat2[y+1];
					}
				}
			}
		}
		out=fopen(fsc,"wb");
		if (!out) { printf("Cannot open output\n"); exit(1); }
		if (apix>0) {
			for (i=0; i<ny/2; i++) if (nrd[i]>0) fprintf(out,"%g\t%f\n",(i+1)/(apix*ny),rd[i]/(sqrt(nrd[i])*sqrt(nrd2[i])));
		}
		else {
			for (i=0; i<ny/2; i++) if (nrd[i]>0) fprintf(out,"%d\t%f\n",i+1,rd[i]/(sqrt(nrd[i])*sqrt(nrd2[i])));
		}
		fclose(out);
		LOG(Ref,fsc,LOG_OUTFILE,"PSC");
	}

	if (snr) {
		data->doneData();
		d2->doneData();
		data->ap2ri();
		d2->ap2ri();
		dat=data->getDataRO();
		dat2=d2->getDataRO();

		memset(rd,0,ny/2*sizeof(float));
		memset(nrd,0,ny/2*sizeof(float));
		memset(nrd2,0,ny/2*sizeof(float));
		for (k=0,y=0; k<nz; k++) {
			for (j=0; j<ny; j++) {
				for (i=0; i<nx; i+=2,y+=2) {
					r=sqrt(SQR(i/2)+SQR(j-ny/2)+SQR(k-nz/2));
					if (fsc0 || fsc1) {
						a=fabs(asin((k-nz/2)/r));
						if (!goodf(&a) || a<fsc0 || a>fsc1) continue;
					}

					x=floor(r+.5)-1;
					if (x<ny/2 && x>=0) { 
						rd[x]+=(dat[y]*dat[y]+dat[y+1]*dat[y+1]+dat2[y]*dat2[y]+dat2[y+1]*dat2[y+1])/
							hypot2(dat[y]-dat2[y],dat[y+1]-dat2[y+1]) -1; 
						nrd[x]+=1.0; 
					}
				}
			}
		}
		out=fopen(snr,"wb");
		if (!out) { printf("Cannot open output\n"); exit(1); }
		if (apix>0) {
			for (i=0; i<ny/2; i++) if (nrd[i]>0) fprintf(out,"%g\t%f\n",(i+1)/(apix*y),rd[i]/nrd[i]);
		}
		else {
			for (i=0; i<ny/2; i++) if (nrd[i]>0) fprintf(out,"%d\t%f\n",i+1,rd[i]/nrd[i]);
		}
		fclose(out);
		LOG(Ref,fsc,LOG_OUTFILE,"PSC");
	}

	LOG(Ref,NULL,LOG_END,NULL);
	exit(0);
}


if (nx==0 || ny==0 || nz==0) {
	nx=data->xSize();
	ny=data->ySize();
	nz=data->zSize();
}


x=data->xSize();
y=data->ySize();
z=data->zSize();

sprintf(s,"%dx%dx%d",x,y,z);
LOG(Ref,argv[1],LOG_INFILE,s);

printf("Original image : %dx%dx%d Mean=%1.3g Sigma=%1.3g Min=%1.3g Max=%1.3g\n",x,y,z,data->Mean(),data->Sigma(),data->Min(),data->Max());
printf("      Sampling : %g A/pixel\tOrigin = %g,%g,%g\n",data->Pixel(), data->getXorigin(), data->getYorigin(), data->getZorigin());
if (argc==2) { LOG(Ref,NULL,LOG_END,NULL); exit(0); }

sprintf(s,"%dx%dx%d",nx,ny,nz);
LOG(Ref,argv[2],LOG_OUTFILE,NULL);

if (sfout) {
	float *curve = (float *)malloc(y*4*sizeof(float));
	EMData *dataf=data->doFFT();
	data->gimmeFFT();
	dataf->calcRadDist(y,0,.5,curve);
	save_data(0,1.0/(apix*2.0*y),curve,y,sfout);
}

if (setsf) {
	XYData SF;
	if (SF.readFile(setsf)) error(ERR_EXIT,"Cannot read structure factor file");
	if (apix<=0) error(ERR_EXIT,"Must specify apix with setsf");
	if (x!=y || y!=z) error(ERR_EXIT,"setsf works only on even cubic images");

	float *curve = (float *)malloc(x*4*sizeof(float));
	EMData *dataf=data->doFFT();
	data->gimmeFFT();
	dataf->calcRadDist(x,0,.5,curve);
	save_data(0,1.0/(apix*2.0*x),curve,x,"proc3d.raddist");

	double av=0;
	for (i=0; i<x; i++) {
		if (curve[i]>0) curve[i]=sqrt(SF.yatx(i/(apix*2.0*x))/curve[i]);
		else curve[i]=0;
		av+=curve[i];
	}
	av/=(double)x;
	for (i=0; i<x; i++) curve[i]/=av;
	float lv,lv2=curve[0];
	for (i=1; i<x-1; i++) {
		lv=curve[i];
		curve[i]=.25*lv2+.5*curve[i]+.25*curve[i+1];	// mild smoothing filter
		lv2=lv;
	}
	save_data(0,1.0/(apix*2.0*x),curve,x,"proc3d.filt");

	dataf->applyRadFn(x,0,.5,curve);
	delete data;
	data=dataf->doIFT();
}

if(!( xc>=0 && yc>=0 && zc>=0 && xc<x && yc<y && zc<z)) xc=x/2, yc=y/2, zc=z/2; 
if (scale>1.0) data=data->clip(xc-nx/2,yc-ny/2,zc-nz/2,nx,ny,nz);
if (scale!=1.0) {
	data->setTAlign(0,0,0);
	data->setRAlign(0,0,0);
	data->rotateAndTranslate(scale);
}
if (scale<1.0 || (scale==1.0 &&(x!=nx || y!=ny ||z!=nz))) data=data->clip(xc-nx/2,yc-ny/2,zc-nz/2,nx,ny,nz);

if (shrink>1) {
	data->medianShrink(shrink);
	nx=data->xSize();
	ny=data->ySize();
	nz=data->zSize();
}

if (signoise) noise=signoise*data->Sigma();

if (noise) {
	dat=data->getData();
	for (i=0; i<nx*ny*nz; i++,dat++) *dat+=grand(0,noise);
	data->doneData();
}

if (apix>0 && lp) { lp=y*apix/lp; printf("LP filter radius=%1.1f\n",lp); }
if (apix>0 && hp) { hp=y*apix/hp; printf("HP filter radius=%1.1f\n",hp); }
if (apix>0 && axial) { axial=y*apix/axial; printf("Axial filter width=%1.1f\n",axial); }

if (hp && lp==0) lp=1000.0;
if (lp && hp==0) hp=-10.0;


if (lp || hp) data->filter(hp,lp,1);
if (tlp) {
	if (apix>0) tlp=y*apix/tlp;
	data->filter(-10,tlp,2);	// tanh lowpass
}
if (unhp) {
	if (apix>0) unhp=y*apix/unhp;
	data->filter(unhp,1000.0,4);
}
//if (unhp) data->filter(unhp,1000.0,99);

if (axial>0) {
	EMData *f=data->doFFT();
/*	data->gimmeFFT();
	f->ri2ap();
	float *d=f->getData();
	i=0;
	for (int z=-f->zSize()/2; z<f->zSize()/2; z++) {
		for (int y=-f->ySize()/2; y<f->ySize()/2; y++) {
			for (int x=0; x<f->xSize(); x+=2,i+=2) {
				if (x==0 && y==0 && z==0) continue;
				d[i]*=axial+(1.0-axial)*hypot(x,y)/sqrt(hypot3(x,y,z));
			}
		}
	}
	f->ap2ri();*/

	i=0;
	float *d=f->getData();
	for (int z=-f->zSize()/2; z<f->zSize()/2; z++) {
		for (int y=-f->ySize()/2; y<f->ySize()/2; y++) {
			for (int x=0; x<f->xSize()/2; x++,i+=2) {
				float r=sqrt(hypot3((float)x,(float)y,(float)z));
				d[i]  *=exp((r*r-z*z)/(axial*axial));
				d[i+1]*=exp((r*r-z*z)/(axial*axial));
//				d[i]*=exp((r-fabs(z))/axial);
//				d[i+1]*=exp((r-fabs(z))/axial);
			}
		}
	}


	f->doneData();
	data=f->doIFT();
	f->gimmeFFT();
	delete f;
}
if (normfiltthr) {
	if (apix>0) { normfiltr=y*apix/normfiltr; printf("Norm filter radius=%1.1f\n",normfiltr); }

	EMData *blur = data->copy(0,0);			// This is the blurred model itself
	EMData *maskblur = data->copy(0,0);		// This will give us an integral of the number of voxels included in the average

	maskblur->realFilter(2,normfiltthr);	// make a binary mask
	maskblur->filter(-10.0,normfiltr,1);	// gaussian filter identical to blur
	maskblur->realFilter(0,.001);			// any areas with too little information are set to 0

	blur->realFilter(0,normfiltthr);		// anything below threshold -> 0
	blur->filter(-10.0,normfiltr,1);		// blur as above

	maskblur->mult(blur,1);	// divides
	data->mult(maskblur);	// apply the normalization
	maskblur->writeMRC("norm.mrc");

	delete maskblur;
	delete blur;
}

if (flip) data->vFlip();
if (yflip) {	// mirror along y axis: x->x, y->-y, z=z
	nx = data->xSize();
	ny = data->ySize();
	nz = data->zSize();
	dat=data->getData();
	int dj = ny%2?-1:0;
	for (k=0; k<nz; k++) {
		for (j=ny/2; j<ny; j++) {
			int j2 = ny - j + dj;
			for (i=0; i<nx; i++) {
				float tmp = dat[k*nx*ny+j*nx+i];
				dat[k*nx*ny+j*nx+i] = dat[k*nx*ny+j2*nx+i];
				dat[k*nx*ny+j2*nx+i] = tmp;
			}
		}
	}	
	data->doneData();
}

if (smask && mask) {
	float r,sma=0,smn=0;

	dat=data->getDataRO();
	for (k=0; k<nz; k++) {
		for (j=0; j<ny; j++) {
			for (i=0; i<nx; i++,dat++) {
				r=sqrt(SQR(i-nx/2)+SQR(j-ny/2)+SQR(k-nz/2));
				if (r>mask-1.5 && r<mask-.5) 
					{ sma+=*dat; smn+=1.0; }
			}
		}
	}
	smask=sma/smn;
	printf("Smartmask density=%g\n",smask); 
	data->doneData();
	
	dat=data->getData();
	for (k=0; k<nz; k++) {
		for (j=0; j<ny; j++) {
			for (i=0; i<nx; i++,dat++) {
				r=sqrt(SQR(i-nx/2)+SQR(j-ny/2)+SQR(k-nz/2));
				if (r>mask-.5 ) *dat=0; else *dat-=smask;
			}
		}
	}
	data->doneData();	
}
else if (mask>0 && maskw==0) data->applyMask(mask,6);
else if (mask>0 && maskw>0) {
	float r;
	EMData *tmp=data->copyHead();
	tmp->applyMask(mask-1,8);
	float em=tmp->dot(data);
	delete tmp;

	dat=data->getData();

	for (k=0; k<nz; k++) {
		for (j=0; j<ny; j++) {
			for (i=0; i<nx; i++,dat++) {
				r=sqrt(SQR(i-nx/2)+SQR(j-ny/2)+SQR(k-nz/2));
				if(r>mask) *dat = em+(*dat-em)*exp(-SQR((r-mask)/maskw));
			}
		}
	}
	data->doneData();
}
else if (mask<0 && maskw==0) data->applyMask(-mask,4);
else if (mask<0 && maskw>0) {
	float r;

	dat=data->getData();
	for (k=0; k<nz; k++) {
		for (j=0; j<ny; j++) {
			for (i=0; i<nx; i++,dat++) {
				r=sqrt(SQR(i-nx/2)+SQR(j-ny/2)+SQR(k-nz/2));
				if(r>-mask) *dat *= exp(-SQR((r+mask)/maskw));
			}
		}
	}
	data->doneData();
}

if (imask && imaskw==0) data->applyMask(imask,5);
else if (imask && imaskw>0) {
	float r;
	
	dat=data->getData();
	for (k=0; k<nz; k++) {
		for (j=0; j<ny; j++) {
			for (i=0; i<nx; i++,dat++) {
				r=sqrt(SQR(i-nx/2)+SQR(j-ny/2)+SQR(k-nz/2));
				if(r<imask) *dat *= exp(-SQR((r-imask)/imaskw));
			}
		}
	}
	data->doneData();
}


if (gwin) data->applyMask((int)floor(gwin+.5),3);

if (sym) {
	Euler e,*a;
	EMData *d,*d2;
	int ns=0;

	d=data->copy();
	d->setParent(NULL);
	d2=d->copy();

	printf("Applying %s symmetry\n",sym);
	a=d->getEuler();
	a->setAngle(0,0,0);
	a->setSym(sym);
	data->zero();
	for (e=a->FirstSym(); e.valid(); e=a->NextSym(),ns++) {
		printf("Angle %f,%f,%f\n",e.alt()*180/PI,e.az()*180/PI,e.phi()*180/PI);
		d2->setRAlign(e.alt(),e.az(),e.phi());
		d2->rotateAndTranslate();
		data->add(d2);
	}
	data->multConst(1.0/ns);
/*	if (tolower(sym[0])=='d') {
		d2->setRAlign(PI,0,0);
		d2->rotateAndTranslate();
		data->add(d2);
		data->multConst(.5);
	}*/
	delete d;
	delete d2;
}

// This filters an image using a filter specified as a 2 column text file with s and the filter amp
if (filefilt) {
	float *xd,*yd,f,sf;
	FILE *in = fopen(filefilt,"rb");
	if (!in) { printf("No file\n"); exit(1); }
	int i;
	
	for (i=0; fscanf(in," %f %f",&f,&f)==2; i++) ;
	rewind(in);
	
	xd=(float *)malloc(i*sizeof(float));
	yd=(float *)malloc(i*sizeof(float));

	if (apix<0) { printf("Please specify apix\n"); exit(1); }
	sf=apix*y;		// this is pixels/s, ie  1/(1/(apix*2 * y/2))
	for (i=0; fscanf(in," %f %f",xd+i,yd+i)==2; i++) xd[i]*=sf;
	if (xd[2]-xd[1] != xd[1]-xd[0]) printf("Warning, x spacing appears nonuniform %g!=%g\n",xd[2]-xd[1],xd[1]-xd[0]);
	
	d2=data->doFFT();
	data->gimmeFFT();
	d2->applyRadFn(i,xd[0],xd[1]-xd[0],yd,1);
	data=d2->doIFT();
	d2->gimmeFFT();
}

if (norm==1) data->normalize();
else if (norm==2) data->edgeNormalize();

if (mult) data->multConst(mult);

if (add) data->addConst(add);

if (center) {
	data->cmCenter(1);
	printf("Center %1.2f %1.2f %1.2f\n",data->Dx(),data->Dy(),data->Dz());
	data->rotateAndTranslate();
}

if (acfcen) {
	data->transAlign3d(NULL,0,1);
	printf("Center %1.2f %1.2f %1.2f\n",data->Dx(),data->Dy(),data->Dz());
	data->rotateAndTranslate();
}

if (mfile) {
	EMData msk;
	msk.readImage(mfile,-1);
	if (data->xSize()!=msk.xSize() || data->ySize()!=msk.ySize() || data->zSize()!=msk.zSize()) {
		printf("Mask size different than image (%d,%d,%d) vs (%d,%d,%d)\n",data->xSize(),data->ySize(),data->zSize(),msk.xSize(),msk.ySize(),msk.zSize());
		exit(1);
	}
	if(maskval>0) msk.realFilter(14,maskval-0.5,maskval+0.5);	// produces a mask give maskval region only
	data->mult(&msk);
}

if (mfile4) {
        EMData msk;
        msk.readImage(mfile4,-1);
        if (data->xSize()!=msk.xSize() || data->ySize()!=msk.ySize() || data->zSize()!=msk.zSize()) {
                printf("Mask size different than image (%d,%d,%d) vs (%d,%d,%d)\n",data->xSize(),data->ySize(),data->zSize(),msk.xSize(),msk.ySize(),msk.zSize());
                exit(1);
        }
        if(maskval>0) msk.realFilter(14,maskval-0.5,maskval+0.5);       // produces a mask give maskval region only
        data->mult_sqrt(&msk);
}



if (mfile2) {
	EMData *msk = new EMData;;
	msk->readImage(mfile2,-1);
	if(maskval>0) msk->realFilter(14,maskval-0.5,maskval+0.5);	// produces a mask give maskval region only
	
	int xm=msk->xSize(), ym=msk->ySize(), zm=msk->zSize();
	float apix = data->Pixel(), apixm = msk->Pixel();
	float xo = data->getXorigin(), yo = data->getYorigin(), zo = data->getZorigin();
	float xom = msk->getXorigin(), yom = msk->getYorigin(), zom = msk->getZorigin();
	float *dp = data->getData(), *dpm=msk->getDataRO();
	
	for(int k=0; k< z; k++){
		float zc = zo + k*apix;
		if(zc<=zom || zc>=zom+zm*apixm) {	// if this plane is outside of the mask cube, set all plane to zero
			memset(&(dp[k*x*y]),0, sizeof(float)*x*y);
			continue;
		}
		int km = (zc-zom)/apixm;
		//printf("km=%d\n",km);
		for(int j=0; j<y; j++){
			float yc = yo + j*apix;
			if(yc<=yom || yc>=yom+ym*apixm) {	// if this row is outside of the mask cube, set all row to zero
				memset(&(dp[k*x*y+j*x]),0, sizeof(float)*x);
				continue;
			}
			int jm = (yc-yom)/apixm;
			//printf("jm=%d\n",jm);
			for(int i=0; i<x; i++){
				float xc = xo + i*apix;
				if(xc<=xom || xc>=xom+xm*apixm) {
					dp[k*x*y+j*x+i]=0.0;
					continue;
				}
				int im = (xc-xom)/apixm;
				//printf("im=%d\n",im);
				if(dpm[km*xm*ym+jm*xm+im] <= 0)	dp[k*x*y+j*x+i]=0.0;
			}
		}
	}
	data->doneData();
	data->update();
	msk->doneData();	
	delete msk;
}

if (mfile3) {
	EMData *msk = new EMData;;
	msk->readImage(mfile3,-1);
	if(maskval>0) msk->realFilter(14,maskval-0.5,maskval+0.5);	// produces a mask give maskval region only
	
	int xm=msk->xSize(), ym=msk->ySize(), zm=msk->zSize();
	float apix = data->Pixel(), apixm = msk->Pixel();
	float xo = data->getXorigin(), yo = data->getYorigin(), zo = data->getZorigin();
	float xom = msk->getXorigin(), yom = msk->getYorigin(), zom = msk->getZorigin();
	float *dpm=msk->getDataRO();
	
	// find the smallest enclosing cube for the mask
	float xmin=xm, xmax=0, ymin=ym, ymax=0,  zmin=zm, zmax=0;
	for(int k=0; k<zm; k++) {
		int indexz = k*ym*xm;
		for(int j=0; j<ym; j++) {
			int indexy = indexz+j*xm;
			for(int i=0; i<xm; i++) {
				int index = indexy + i;
				if (dpm[index]>0) {
					if( i > xmax ) xmax = i;
					else if( i < xmin ) xmin = i;					
					if( j > ymax ) ymax = j;
					else if( j < ymin ) ymin = j;
					if( k > zmax ) zmax = k;
					else if( k < zmin ) zmin = k;
				}
			}
		}
	}
	// now convert to coordinates in Angstrom from pixels
	xmin = xmin * apixm + xom;	xmax = xmax * apixm + xom;
	ymin = ymin * apixm + yom;	ymax = ymax * apixm + yom;
	zmin = zmin * apixm + zom;	zmax = zmax * apixm + zom;
	
	// now compare these to the map to find the smallest common cube
	if(xmin < xo) xmin = xo;
	if(xmax > xo + x * apix) xmax = xo + x * apix;
	if(ymin < yo) ymin = yo;
	if(ymax > yo + y * apix) ymax = yo + y * apix;
	if(zmin < zo) zmin = zo;
	if(zmax > zo + z * apix) zmax = zo + z * apix;
	
	// now allocate a cube to represent this cube
	int imin = int((xmin-xo)/apix), in = int((xmax-xmin)/apix)+1;  
	int jmin = int((ymin-yo)/apix), jn = int((ymax-ymin)/apix)+1;  
	int kmin = int((zmin-zo)/apix), kn = int((zmax-zmin)/apix)+1; 	
	xmin = xo + apix * imin;
	ymin = yo + apix * jmin;
	zmin = zo + apix * kmin;

	EMData* masked = data->clip(imin,jmin,kmin,in,jn,kn);
	masked->setPixel(apix);
	masked->setXYZOrigin(xmin,ymin,zmin);
	float *dp = masked->getData();
					
	for(int k=0; k<kn; k++){
		float zc = zmin + k*apix;
		if(zc<=zom || zc>=zom+zm*apixm) {	// if this plane is outside of the mask cube, set all plane to zero
			memset(&(dp[k*in*jn]),0, sizeof(float)*in*jn);
			continue;
		}
		int km = (zc-zom)/apixm;
		//printf("km=%d\n",km);
		for(int j=0; j<jn; j++){
			float yc = ymin + j*apix;
			if(yc<=yom || yc>=yom+ym*apixm) {	// if this row is outside of the mask cube, set all row to zero
				memset(&(dp[k*in*jn+j*in]),0, sizeof(float)*in);
				continue;
			}
			int jm = (yc-yom)/apixm;
			//printf("jm=%d\n",jm);
			for(int i=0; i<in; i++){
				float xc = xmin + i*apix;
				if(xc<=xom || xc>=xom+xm*apixm) {
					dp[k*in*jn+j*in+i]=0.0;
					continue;
				}
				int im = (xc-xom)/apixm;
				//printf("im=%d\n",im);
				if(dpm[km*xm*ym+jm*xm+im] <= 0)	dp[k*in*jn+j*in+i]=0.0;
			}
		}
	}
	msk->doneData();	
	delete msk;
	masked->doneData();
	masked->update();
	delete data;
	data = masked;
	x = data->xSize();
	y = data->ySize();
	z = data->zSize();
}

if (maskreffile) {
	EMData *mskref = new EMData;;
	mskref->readImage(maskreffile,-1,1);	// read header only

	int nx=data->xSize(), ny=data->ySize(), nz=data->zSize();
	int xm=mskref->xSize(), ym=mskref->ySize(), zm=mskref->zSize();
	float apix = data->Pixel(), apixm = mskref->Pixel();
	float xo = data->getXorigin(), yo = data->getYorigin(), zo = data->getZorigin();
	float xom = mskref->getXorigin(), yom = mskref->getYorigin(), zom = mskref->getZorigin();
	
	EMData* datanew = new EMData();
	datanew->setSize(xm, ym, zm);
	datanew->zero();
	datanew->setXYZOrigin(xom, yom, zom);
	datanew->setPixel(apixm);
	
	if(apix>apixm) {
		for(int k=0; k<zm; k++) {
			int k2 = int( ((zom + k * apixm) - zo)/apix  + 0.5);
			if(k2<0 || k2>=nz) continue;
			for(int j=0; j<ym; j++) {
				int j2 = int( ((yom + j * apixm) - yo)/apix  + 0.5);
				if(j2<0 || j2>=ny) continue;
				for(int i=0; i<xm; i++) {
					int i2 = int( ((xom + i * apixm) - xo)/apix  + 0.5);
					if(i2<0 || i2>=nx) continue;
					float val = data->valueAt(i2, j2, k2);
					if (val) datanew->setValueAt(i, j, k, val);
				}
			}
		}
	}
	else  {
		for(int k=0; k<nz; k++) {
			int k2 = int( ((zo + k * apix) - zom)/apixm  + 0.5);
			if(k2<0 || k2>=zm) continue;
			for(int j=0; j<ny; j++) {
				int j2 = int( ((yo + j * apix) - yom)/apixm  + 0.5);
				if(j2<0 || j2>=ym) continue;
				for(int i=0; i<nx; i++) {
					int i2 = int( ((xo + i * apix) - xom)/apixm  + 0.5);
					if(i2<0 || i2>=xm) continue;
					float val = data->valueAt(i, j, k);
					if (val) datanew->setValueAt(i2, j2, k2, val);
				}
			}
		}
	}
	
	delete data;
	data = datanew;
	x = data->xSize();
	y = data->ySize();
	z = data->zSize();
}
// merge masks
if (mask2file) {
	EMData *mask2 = new EMData;;
	mask2->readImage(mask2file,-1);
	data->addAsMask(mask2);
	delete mask2;
}
if(cut1stquad) {
	float *dp = data->getData();
	for(int k=0; k<z; k++){
		for(int j=y/2; j<y; j++){
			for(int i=x/2; i<x; i++){
				dp[k*x*y+j*x+i]=0.0F;
			}
		}
	}
	data->doneData();
}

if (m1file) {
	EMData msk;
	msk.readImage(m1file,-1);
	if (data->xSize()!=msk.xSize() || data->ySize()!=msk.ySize() || data->zSize()!=msk.zSize()) {
		printf("Mask size different than image (%d,%d,%d) vs (%d,%d,%d)\n",data->xSize(),data->ySize(),data->zSize(),msk.xSize(),msk.ySize(),msk.zSize());
		exit(1);
	}
	msk.realFilter(14,.5,1.5);
	data->mult(&msk);
}

if (amask) {
	float sig,mean,t;
	int k0,k1;
	
	amask->setSize(x,y,z);
	amask->zero();
	sig=data->Sigma();
	mean=data->Mean();
	dat=data->getData();
	dat2=amask->getData();

	// start by finding strong data	
	if (amasklev!=999.0) t=amasklev;
	else t=mean+sig*2.5;
	for (k=l=0; k<z; k++) {
		for (j=0; j<y; j++) {
			for (i=0; i<x; i++,l++) {
				if (dat[l]>t) dat2[l]=1.0;
			}
		}
	}
	
	// now iteratively find anything significant nearby
	ii=1;
	k0=0;
	if (amasklev2!=999.0) t=amasklev2;
	else t=mean+sig*.5;
	while (ii) {
		ii=0;
		for (k=1; k<z-1; k++) {
			for (j=1; j<y-1; j++) {
				l=j*x+k*x*y+1;
				for (i=1; i<x-1; i++,l++) {
					if (dat[l]<t || dat2[l]) continue;
					if (dat2[l-1] || dat2[l+1] || dat2[l-x] || dat2[l+x] ||
						dat2[l-x*y] || dat2[l+x*y]) {
						dat2[l]=1.0;
						ii=1;
					}
				}
			}
		}
		k0++;
		printf(" Building mask %d\r",k0); fflush(stdout);
	}
	putchar('\n');
	
	// Now add a 1 pixel 'buffer' around this object
	for (k=1; k<z-1; k++) {
		for (j=1; j<y-1; j++) {
			l=j*x+k*x*y+1;
			for (i=1; i<x-1; i++,l++) {
				if (dat2[l-1]==1.0 || dat2[l+1]==1.0 || dat2[l-x]==1.0 
					|| dat2[l+x]==1.0 || dat2[l-x*y]==1.0 || dat2[l+x*y]==1.0) 
						dat2[l]=2.0;
			}
		}
	}

	for (i=0; i<x*y*z; i++) if (dat2[i]==2.0) dat2[i]=1.0;

	// finally fill in surrounded areas in x/y/z
	for (i=0; i<x; i++) {
		for (j=0; j<y; j++) {
		
			for (k=0; k<z; k++) {
				l=i+j*x+k*x*y;
				if (dat2[l]) break;
			}
			if (k==z) continue;
			k0=k;
			
			for (k=z-1; k>=0; k--) {
				l=i+j*x+k*x*y;
				if (dat2[l]) break;
			}
			k1=k;
			
			for (k=k0+1; k<k1; k++) dat2[i+j*x+k*x*y]=1.0;
		}
	}
	
	for (i=0; i<x; i++) {
		for (j=0; j<z; j++) {
		
			for (k=0; k<y; k++) {
				if (dat2[i+k*x+j*x*y]) break;
			}
			if (k==y) continue;
			k0=k;
			
			for (k=y-1; k>=0; k--) {
				if (dat2[i+k*x+j*x*y]) break;
			}
			k1=k;
			
			for (k=k0+1; k<k1; k++) dat2[i+k*x+j*x*y]=1.0;
		}
	}

	for (i=0; i<y; i++) {
		for (j=0; j<z; j++) {
		
			for (k=0; k<x; k++) {
				if (dat2[k+i*x+j*x*y]) break;
			}
			if (k==x) continue;
			k0=k;
			
			for (k=x-1; k>=0; k--) {
				if (dat2[k+i*x+j*x*y]) break;
			}
			k1=k;
			
			for (k=k0+1; k<k1; k++) dat2[k+i*x+j*x*y]=1.0;
		}
	}
	
	data->doneData();
	amask->doneData();
	
	
	data->mult(amask);
	amask->writeMRC("mask.mrc");
}

if (amask2) {
	float sig,mean,t;
	int k0,k1;

	amask2->setSize(x,y,z);
	amask2->zero();
	sig=data->Sigma();
	mean=data->Mean();
	dat=data->getData();
	dat2=amask2->getData();

	// start with an initial sphere
	for (k=-z/2,l=0; k<z/2; k++) {
		for (j=-y/2; j<y/2; j++) {
			for (i=-x/2; i<x/2; i++,l++) {
				if (abs(k)>a2r || abs(j)>a2r || abs(i)>a2r) continue;
				if (sqrt((float)(k*k+j*j+i*i))>a2r || dat[l]<amasklev) continue;
				dat2[l]=1.0;
			}
		}
	}

	// now iteratively find anything significant nearby
	ii=1;
	k0=0;
	while (ii) {
		ii=0;
		for (k=1; k<z-1; k++) {
			for (j=1; j<y-1; j++) {
				l=j*x+k*x*y+1;
				for (i=1; i<x-1; i++,l++) {
					if (dat[l]<amasklev || dat2[l]) continue;
					if (dat2[l-1] || dat2[l+1] || dat2[l-x] || dat2[l+x] ||
						dat2[l-x*y] || dat2[l+x*y]) {
						dat2[l]=1.0;
						ii=1;
					}
				}
			}
		}
		k0++;
		printf(" Building mask %d\r",k0); fflush(stdout);
	}
	putchar('\n');




	// Now add an 'a2x' pixel 'buffer' around this object
/*	for (ii=0; ii<abs(a2x); ii++) {
	for (k=1; k<z-1; k++) {
		for (j=1; j<y-1; j++) {
			l=j*x+k*x*y+1;
			for (i=1; i<x-1; i++,l++) {
				if (range(0,dat2[l-1],ii+1.5) || range(0,dat2[l+1],ii+1.5) || range(0,dat2[l-x],ii+1.5)
					|| range(0,dat2[l+x],ii+1.5) || range(0,dat2[l-x*y],ii+1.5) || range(0,dat2[l+x*y],ii+1.5))
						dat2[l]=ii+2.0;
			}
		}
	}
	}

	for (i=0; i<x*y*z; i++) if (dat2[i]>0) dat2[i]=1.0;
*/
	amask2->doneData();

	// Now add an 'a2x' pixel soft 'buffer' around this object
	//amask2->realFilter(110,abs(a2x),abs(a2x)>2.0?2.0:0.0);	// eman 1.8 style

	amask2->realFilter(110,abs(a2x),abs(a2x)/4+1);		// now the pre-gaussian mask is proportional to the number of shells

	dat2=amask2->getData();


	if (a2x<0) {
	// finally fill in surrounded areas in x/y/z
	for (i=0; i<x; i++) {
		for (j=0; j<y; j++) {

			for (k=0; k<z; k++) {
				l=i+j*x+k*x*y;
				if (dat2[l]) break;
			}
			if (k==z) continue;
			k0=k;

			for (k=z-1; k>=0; k--) {
				l=i+j*x+k*x*y;
				if (dat2[l]) break;
			}
			k1=k;

			for (k=k0+1; k<k1; k++) dat2[i+j*x+k*x*y]=1.0;
		}
	}

	for (i=0; i<x; i++) {
		for (j=0; j<z; j++) {

			for (k=0; k<y; k++) {
				if (dat2[i+k*x+j*x*y]) break;
			}
			if (k==y) continue;
			k0=k;

			for (k=y-1; k>=0; k--) {
				if (dat2[i+k*x+j*x*y]) break;
			}
			k1=k;

			for (k=k0+1; k<k1; k++) dat2[i+k*x+j*x*y]=1.0;
		}
	}

	for (i=0; i<y; i++) {
		for (j=0; j<z; j++) {

			for (k=0; k<x; k++) {
				if (dat2[k+i*x+j*x*y]) break;
			}
			if (k==x) continue;
			k0=k;

			for (k=x-1; k>=0; k--) {
				if (dat2[k+i*x+j*x*y]) break;
			}
			k1=k;

			for (k=k0+1; k<k1; k++) dat2[k+i*x+j*x*y]=1.0;
		}
	}
	}

	data->doneData();
	amask2->doneData();

	EMData *norm=NULL;
	EMData *norm2=NULL;
	norm=amask2->copy(0,0);
	norm->realFilter(6);	// binarize the mask
	norm2=norm->copy(0,0);
	norm->addMaskShell(1);
	norm->subtract(norm2);
	delete norm2;
	norm->addMaskShell(2);
	data->maskNormalize(norm,0);
	delete norm;

	data->mult(amask2);
	amask2->writeMRC("mask.mrc");
}

if (rfilt>=0) data->realFilter(rfilt,rft1,rft2,rft3);

// lump all translations into a single one
if(pdx || pdy || pdz) {
	Vect posttrans(pdx,pdy,pdz);
	Euler e(dalt*PI/180.0,daz*PI/180.0,dphi*PI/180.0);
	Euler einv = e.inverse();
	posttrans.rotate(einv);
	dx += posttrans.X();
	dy += posttrans.Y();
	dz += posttrans.Z();
}

if (dx || dy || dz || dalt || daz || dphi) {
	data->setRAlign(dalt*PI/180.0,daz*PI/180.0,dphi*PI/180.0);
	data->setTAlign(dx,dy,dz);
	data->rotateAndTranslate();
}

if (normto) {
	float mm,bb;
	EMData t;
	t.readMRC(normto);
	data->normalizeTo(&t,1,1,&mm,&bb);
	printf("NormalizeTo: mx+b -> m=%1.4f b=%1.4f\n",mm,bb);
}

if (bliter >0 && blwidth>0) {	// bilateral filtering
	data->BilateralFilter(blsigma1, blsigma2, bliter, blwidth);
}

if(fnx>=data->xSize()&&fny>=data->ySize()&&fnz>=data->zSize()){
	EMData* fft=data->doFFT();
	data->gimmeFFT();
	delete data;
	EMData* padfft=fft->clip(0,0,0,fnx+2,fny,fnz);
	delete fft;
	data=padfft->doIFT();
	padfft->gimmeFFT();
	delete padfft;
}
if(bin){
		float *dat=data->getData();
		for(i=0;i<data->xSize()*data->ySize()*data->zSize();i++){
			if (dat[i]<binthresh)dat[i]=0.0;
			else dat[i]=1.0;
		}
		data->doneData();		
}
if(bin2){
		float *dat=data->getData();
		for(i=0;i<data->xSize()*data->ySize()*data->zSize();i++){
			if (dat[i]) dat[i]=1.0;
		}
		data->doneData();		
}
if(icos5fTo3f) {
	data->setRAlign(79.1853*PI/180.0,341.996*PI/180.0,60.00*PI/180.0);
	data->setTAlign(0,0,0);
	data->rotateAndTranslate();
}
if(icos3fTo5f) {
	data->setRAlign(79.1853*PI/180.0,120*PI/180.0,198.004*PI/180.0);
	data->setTAlign(0,0,0);
	data->rotateAndTranslate();
}
if(icos5fTo2f) {
	data->setRAlign(58.282523*PI/180.0,270.0*PI/180.0,0.00*PI/180.0);
	data->setTAlign(0,0,0);
	data->rotateAndTranslate();
}
if(icos2fTo5f) {
	data->setRAlign(58.282523*PI/180.0,180.0*PI/180.0,270.00*PI/180.0);
	data->setTAlign(0,0,0);
	data->rotateAndTranslate();
}
if(icos5fhalf2full) {
	int nx = data->xSize(), ny=data->ySize(), nz=data->zSize();
	int mx = nx, my = ny, mz = 2*nz;	
	if(nx==ny && 2*nz-1 == nx) {	// the map is the odd sized map from the original Fourier Bessel reconstruction
		mx = nx + 1; 
		my = ny + 1;
		mz = 2 *nz;
	}
	EMData * out = new EMData;
	out->setSize(mx, my, mz);
	out->zero();

	out->setPixel(data->Pixel());
	out->setXYZOrigin(data->getXorigin()-(mx-nx)*data->Pixel(), data->getYorigin()-(my-ny)*data->Pixel(), data->getZorigin() - (mz-nz) * data->Pixel());
	
	for (int k = 1; k < nz; k++)		// central plane is special
		for (int j = 0; j < ny; j++)
			for (int i = 0; i < nx; i++) {
				out->setValueAt(i,j,k+mz/2,data->valueAt(i,j,k));
				out->setValueAt(mx-i,j,mz/2-k,data->valueAt(i,j,k));
			}

	for (int j = 0; j < ny; j++)  	  // central plane
		for (int i = 0; i < nx; i++) {
			out->setValueAt(i,j,mz/2,data->valueAt(i,j,0));
		}	

	delete data;
	data = out;
}
if(tophalf) {
	EMData* half=new EMData;
	half->setSize(nx,ny,nz/2);
	float* hdatafp=half->getData();
	float* datafp=data->getDataRO();
	memcpy(hdatafp,&datafp[nz/2*nx*ny], sizeof(float)*nx*ny*nz/2);
	data->doneData();
	half->doneData();
	half->setPixel(data->Pixel());
	half->setXYZOrigin(data->getXorigin(),data->getYorigin(),data->getZorigin()+data->Pixel()*nz/2);
	half->update();
	delete data;
	data=half;
}

printf("   Final image : %dx%dx%d Mean=%1.3g Sigma=%1.3g\n",data->xSize(),data->ySize(),data->zSize(),data->Mean(),data->Sigma());
printf("      Sampling : %g A/pixel\tOrigin = %g,%g,%g\n",data->Pixel(), data->getXorigin(), data->getYorigin(), data->getZorigin());

if (spider==1) data->writeSPIDER(argv[2],-1);
else if (spider==2) data->writeSingleSPIDER(argv[2],-1);
else if (imagic) data->writeIMAGIC3D(argv[2],-1);
else if (vtk) data->writeVTK(argv[2]);
else if (df3) data->writeDF3(argv[2]);
else if (amira) {
	char type[16] = "float";
	if (amirashort) strcpy(type, "short");
	if (amirabyte) strcpy(type, "byte");
	data->writeAMIRA(argv[2], type, amiralabel);
}
else if (rawiv) data->writeRawIV(argv[2]);
else if (icos) data->writeICOS(argv[2]);
else if (xplor) data->writeXPLOR(argv[2]);
else if (pif) data->writePIF(argv[2]);
else if (em) data->writeEM(argv[2]); 
else if (hdf) data->writeHDF(argv[2],-1);
else if (png) data->writePNG(argv[2]);
else if (eightbit) data->writeMRC(argv[2],MODE_char);
else if (sixteenbit) data->writeMRC(argv[2], MODE_short);
else if (mrc) data->writeMRC(argv[2]);
else { 
	const char* filename = argv[2];
	int flen = strlen(filename);
	if (flen > 4 && strcasecmp(&filename[flen - 4], ".img") == 0) data->writeIMAGIC3D(argv[2],-1);
	else if (flen > 4 && strcasecmp(&filename[flen - 4], ".hed") == 0) data->writeIMAGIC3D(argv[2],-1);
	else data->writeImage(argv[2],-1);
}

LOG(Ref,NULL,LOG_END,NULL);

return 0;
}
