// raw2mrc.C Copyright Baylor College of Medicine
// 11/5/01 Steve Ludtke
//N raw2mrc
//F This program will import raw binary data and write a MRC file
//T LA
//1
//P <input>	Raw input file
//P <output>	MRC output file
//P size=<x>,<y>[,<z>]	Image size, scans in x first
//P type=<datatype>	Type of data: byte, short, long or float
//P [skip=<bytes>]	Used to skip a header if present
//P [swap]	If file was created on machine with opposite byte-ordering
//P [byteout]	Normally the MRC file will be floating point, this will write a 16-bit file.
//U raw2mrc tst.dat tst.mrc size=1024,1024 type=short
//D Pretty self explanatory.

#include "config.h"
#include <cstdlib>
#ifdef HAVE_MALLOC_H
#include <cmalloc>
#endif
#include <cstring>
#include <cmath>
#include "List.h"
#include "portable_fileio.h"
#include "EMData.h"

enum { TYPE_BYTE, TYPE_SHORT, TYPE_LONG, TYPE_FLOAT };

int main(int argc, char** argv)
{
int i,j;
int xsize=1,ysize=1,zsize=1,swap=0,byteout=0;
EMAN_off_t skip=0;
int type=-1;

appinit(argc,argv);

if (argc<5) appusage(argv[0]);

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"size=",5)==0) sscanf(argv[i]+5,"%d,%d,%d",&xsize,&ysize,&zsize);
	else if (strncmp(argv[i],"type=",5)==0) {
		if (strcmp(argv[i]+5,"byte")==0) type=TYPE_BYTE;
		else if (strcmp(argv[i]+5,"short")==0) type=TYPE_SHORT;
		else if (strcmp(argv[i]+5,"long")==0) type=TYPE_LONG;
		else if (strcmp(argv[i]+5,"float")==0) type=TYPE_FLOAT;
		else error(ERR_EXIT,"Unknown type");
	}
	else if (strncmp(argv[i],"skip=",5)==0) skip=atoi(argv[i]+5);
	else if (strncmp(argv[i],"swap",4)==0) swap=1;
	else if (strncmp(argv[i],"byteout",8)==0) byteout=1;
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}
if (xsize==1 && ysize==1) error(ERR_EXIT,"Must specify size");
if (type<0) error(ERR_EXIT,"Must specify type");

FILE *in = fopen(argv[1],"rb");
if (!in) error(ERR_EXIT,"Cannot open input file");

EMData img;
img.setSize(xsize,ysize,zsize);
float *data = img.getData();
if (skip) portable_fseek(in,skip,SEEK_SET);

switch (type) {
case TYPE_BYTE: {
	char *datab = (char *)data;
	if (fread(datab,xsize*ysize,zsize,in)!=zsize) error(ERR_ERROR,"Data read incomplete");
	for (i=xsize*ysize*zsize-1; i>=0; i--) data[i]=(float)datab[i];
	break; }
case TYPE_SHORT: {
	short *datas = (short *)data;
	if (fread(datas,xsize*ysize,zsize*2,in)!=zsize*2) error(ERR_ERROR,"Data read incomplete");
	if (swap) swapshort((char *)datas,xsize*ysize*zsize);
	for (i=xsize*ysize*zsize-1; i>=0; i--) data[i]=(float)datas[i];
	break; }
case TYPE_LONG: {
	int	*datai = (int *)data;
	if (fread(datai,xsize*ysize,zsize*4,in)!=zsize*4) error(ERR_ERROR,"Data read incomplete");
	if (swap) swapint((char *)datai,xsize*ysize*zsize);
	for (i=xsize*ysize*zsize-1; i>=0; i--) data[i]=(float)datai[i];
	break; }
case TYPE_FLOAT: {
	if (fread(data,xsize*ysize,zsize*4,in)!=zsize*4) error(ERR_ERROR,"Data read incomplete");
	if (swap) swapint((char *)data,xsize*ysize*zsize);
	break; }
}
img.doneData();

if (byteout) img.writeMRC(argv[2],MODE_char);
else img.writeMRC(argv[2]);

return 0;
}

