// refine.C 1997  Steve Ludtke
// Copyright 1997 Baylor College of Medicine 

//N refine
//F This is the main refinement loop in EMAN. It has a tremendous number of options.
//T H
//1
//P <total iter>	Total number of iterations to be run in the current directory
//P mask=<rad>	Applies an outside mask at several stages of refinement
//P [proc=<maxproc>]	Number of proceesors to use for parallel processing
//P [hard=<maxpr>]	Hard limit for <a href=make3d.html>make3d</a>
//P [simple]	Use 'make3dsimple inorm'
//P [ctfc=<res in A>]	Performs CTF correction filtered to the specified resolution (see <a href=classalignall.html>classalignall</a>)
//P [ctfcw=<SF file>]	Performs CTF correction with Wiener filtration using a structure factor specified in a file
//P [phaseopt]	Experimental modification to class-averaging
//P [setsf=<lowpass res>[,<highpass res>[,<SF file>]]	Applies a final filter to set the radial structure factor to match the ctfcw structure factor multiplied by a gaussian filter. Filter is specified in A.
//P [median]	Specify this when CTF correction is NOT being performed
//P [nweight]	Used when start.hed contains averages, to properly weight during further averaging. This will not work in conjunction with median or ctfc/w.
//P [sym=<c2,c4,etc>]	Imposes symmetry on the model, omit this option for no/unknown symmetry
//P [ang=<dang>]	Angular step for projections (in degrees, see 'prop' in <a href=project3d.html>project3d</a>)
//P [anglist=<file>]	Use Euler angles included in this file for projection
//P [maxshift=<rad>]	Maximum translation during image alignment
//P [pad=<size>]	Pads the model during Fourier reconstruction (see <a href=make3d.html>make3d</a>)
//P [classkeep=<sigma coef>]	Classkeep value for <a href=classalignall.html>classalignall</a>
//P [classiter=<iter>]	Iterations for <a href=classalignall.html>classalignall</a>
//P [refmaskali]	This passes the refmaskali option to classalignall if the amask option is being used, otherwise refmask is used when amask is specified.
//P [filt3d=<lp rad>]	Apply a lowpass filter to the model after each iteration, this occurs before 3dit
//P [sep=<n>]	This option is passed to <a href=classesbymra.html>classesbymra</a>
//P [xfiles=<a/pix>,<mass in kd>,<ali to>]	Used to generate 3d models that are easy to evaluate
//P [3dit=<n it>]	After constructing a model with Fourier reconstruction, this runs n iterations of real-space model 'cleansing' This option isdeprecated, suggest not using it
//P [3dit2=<n it>]	This is real-space refinement with a special noise elimination feature enabled. This option is deprecated, suggest not using it
//P [speed=<1-5>]	Speed parameter in clean3d (3dit,3dit2). Convergence rate, 1 dfl
//P [sigfilt]	Applies a 1-sigma real-space filter to images at various points
//P [euler2=<oversmp>]	Reassigns Euler angles to class averages with an optionally reduced angular increment. ie - 2 will use ang/2 (see <a href=classesrefs.html>classesrefs</a>
//P [tree=<2,3>]	Faster classifications for large numbers of projections (see <a href=classesbymra.html>classesbymra</a>. Limited parallel support, see text.
//P [imask=<rad>]	Applies an inside mask in several stages of refinement
//P [amask=<r>,<thr>,<n iter>]	This applies an automask between steps in a reconstruction. see automask2 in <a href=proc3d.html>proc3d</a>. REQUIRES xfiles to be used.
//P [planarmask=<thickness>]	Masks the object along the z-axis with a total thickness specified in pixels.
//P [goodbad]	Saves good and bad class averages from 3D reconstruction. Overwrites each new iteration.
//P [slow]	This will use a very slow, exhaustive search, algorithm for 2D alignments. VERY SLOW
//P [refine]	This will do subpixel alignment of the particle translations for classification and averaging. May have a significant impact at higher resolutions.
//P [shrink=<n>]	Experimental, shrinks images at several points for faster runs
//P [projbatches=<n>]	Experimental, split the projection into n batches to minimize total memory requirement
//P [ca3]	Experimental. Uses classalign3 instead of classalign2
//P [classfp=<msa#>]	Experimental, classification using rotational/translational invariants
//P [perturb]	Experimental option to produce a more uniform distribution of points in 3D Fourier space.
//P [usefilt]	Experimental
//P [rfp]	Experimental, classification using rotational footprints
//P [precen]	Indicates that raw particles have been accurately precentered (experimental)
//P [phasecls]	Uses weighted mean phase error for classification (experimental)
//P [fscls]	Uses FSC for classification (experimental)
//P [axialfilt=<res>]	See proc3d, 0 provides maximal effect.
//P [msa=<proj>]	Experimental
//P [mra2]	Experimental
//P [3dr[=<range>]]	Experimental
//P [d3s=<sigmult>]	Experimental
//P [collapse=<ang>]	Experimental
//P [dfilt]	Experimental
//P [frm2d[=<rad>,<samp>]]	This will classify using Fast Rotational Matching 2D. rad is the maximum radius of particles; samp is the number of angular sampling points at each radius circle
//P [classalign2log]	Experimental. Calls classalign2 with classalign2log option to print out particle that were used in the class average.
//P [continue]	If a job is killed in the middle of an iteration, this will allow it to pick up where it left off. Otherwise the last iteration will start from the beginning.
//P [norm3d]	Pass the 'norm' option in to the <a href=make3d.html>'make3d'</a> command.
//U refine 5 ang=3 mask=28 pad=96 hard=25 classkeep=.3 classiter=8 proc=12 sym=d7 ctfc=12 sep=2 xfiles=4.2,1000,40 
//D Note: Despite the experimental tags, the 'phasecls' and 'fscls' almost always produce better results
//E than the default linear variance classifier. Note that these options are mutually exclusive.
//D This is the main refinement command in EMAN. You should have your starting
//E images in 'start.img' and your initial 3D model in threed.0a.mrc. The
//E 'refine' command will then refine this model iteratively. Note that the
//E 'total number of iterations' specifies the
//E number of iterations the program will finish with. If 5 iterations have
//E been run, then 'refine 8 ...' is run, the program will pick up after 5
//E iterations and run only 3 more cycles. If you want to start over again
//E from the first iteration, you must remove all of the 'threed.?.mrc'
//E files (except for 0a) and all of the 'classes.?.mrc' files.
//D Most of the options are simply passed through to other programs. Typical options
//E for this program can be determined by running <a href=eman.html>eman</a> and following the reconstruction
//E procedure it provides.
//D The tree option may not work in certian parallel configurations, such as grid configurations
//E where the filesystem isn't cross-mounted.
//D 'frm2d' better used in combination with maxshift, which indicates the 
//E center of mass uncertainty. 


#include <cstdio>
#include <cstdlib>
//#include <unistd.h>
#include <cmath>
#include <ctime>
#include <cctype>
//#include <sys/wait.h>
#include <vector>
#include "util.h"
#include "EMData.h"

struct ClassesByMRCLog { int iter, job_index, job_num; };

void *cmra(void *fnum);
int runpar_run(const char* cmd, int minproc=1,  int maxproc=1, int usefs=1);	// use runpar to run the command
		
int main(int argc, char *argv[]) {
int i,j,n,dfilt=0,simple=0,contin=0,phaseopt=0,nweight=0;
int iter=5,restart=0,mask=-1,imask=-1,pad=0,classiter=8,d3r=0,classfp=0,goodbad=0,precen=0,maxshift=-1;
int minproc=0,maxproc=0,lp=0,alito=0,median=0,mode=2,msa=0,shrink=1,mra2=0,can=2,sigfilt=0,perturb=0,phasecls=0;
int noweight=0,ntk=1,tree=0,d3it=0,d3it2=0,amask1=0,amask3=0,pmask=0,rfp=0,usefilt=0,slow=0,rma=0,refine=0,cls=0;
int projbatches=0, projmode = -6, icoshalf = 0;
int classalign2log=0;
int frm2d=0,ccc=0,size=-1;		//ycong
float ang=7.5,keep=1.2,xfiles=0,appix=0,d3s=0,hard=0,range=90.0,ctfc=0,euler2=0.0,highpass=0.,amask2=0,setsflp=0,setsfhp=0;
float axial=0;
float collapse=0,speed=1.0;
float rmaxH=-1.0f;		//ycong
char s[1024],*sym=NULL,*ctfcw=NULL,*setsffile=NULL,*anglistfile=NULL;
FILE *log;
time_t ti;
int norm3d = 0;

appinit(argc,argv);

log=fopen("refine.log","ab");

if (argc<2) appusage(argv[0]);

int Ref=LOGbegin(argc,argv);
LOG(Ref,"start.hed",LOG_INFILE,NULL);
LOG(Ref,"refine.log",LOG_OUTFILE,NULL);

ti=time(0);
if (log) fprintf(log,"******** refine started %s",ctime(&ti));
for (i=0; i<argc; i++) fprintf(log,"%s ",argv[i]);
fprintf(log,"\n");

iter=atoi(argv[1]);

for (i=2; i<argc; i++) {
	if (strncmp("proc=",argv[i],5)==0) sscanf(&argv[i][5],"%d,%d",&minproc,&maxproc);
	else if (strncmp("tree=",argv[i],5)==0) tree=atoi(&argv[i][5]);
	else if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp("3dit=",argv[i],5)==0) d3it=atoi(&argv[i][5]);
	else if (strncmp("3dit2=",argv[i],6)==0) d3it2=atoi(&argv[i][6]);
	else if (strncmp("hpBetRds=",argv[i],9)==0) highpass=atof(&argv[i][9]);
	else if (strncmp("imask=",argv[i],6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp("planemask=",argv[i],10)==0) pmask=atoi(&argv[i][10]);
	else if (strncmp("speed=",argv[i],6)==0) speed=atof(&argv[i][6]);
	else if (strncmp("mode=",argv[i],5)==0) mode=atoi(&argv[i][5]);
	else if (strncmp("slow",argv[i],4)==0) slow=1;
	else if (strncmp("dfilt2",argv[i],6)==0) dfilt=2;
	else if (strncmp("dfilt",argv[i],5)==0) dfilt=1;
	else if (strncmp("simple",argv[i],6)==0) simple=1;
	else if (strncmp("hard=",argv[i],5)==0) hard=atof(&argv[i][5]);
	else if (strncmp("classkeep=",argv[i],10)==0) keep=atof(&argv[i][10]);
	else if (strncmp("classiter=",argv[i],10)==0) classiter=atoi(&argv[i][10]);
	else if (strncmp("refmaskali",argv[i],10)==0) rma=1;
	else if (strncmp("phasecls",argv[i],8)==0) phasecls=1;
	else if (strncmp("phaseopt",argv[i],8)==0) phaseopt=1;
	else if (strncmp("fscls",argv[i],5)==0) phasecls=2;
	else if (strncmp("pad=",argv[i],4)==0) pad=atoi(&argv[i][4]);
	else if (strncmp("filt3d=",argv[i],7)==0) lp=atoi(&argv[i][7]);
	else if (strncmp("shrink=",argv[i],7)==0) shrink=atoi(&argv[i][7]);
	else if (strncmp("projbatches=",argv[i],12)==0) projbatches=atoi(&argv[i][12]);
	else if (strncmp("sigfilt",argv[i],7)==0) sigfilt=1;
	else if (strncmp("axialfilt=",argv[i],10)==0) axial=atof(&argv[i][10]);
	else if (strncmp("setsf=",argv[i],6)==0) {
		char s[512];
		s[0]=0;
		sscanf(argv[i]+6,"%f,%f,%s",&setsflp,&setsfhp,s);
		if (strlen(s)) {
			setsffile=(char *)malloc(strlen(s)+1);
			strcpy(setsffile,s);
		}
	}
	else if (strncmp("euler2",argv[i],6)==0) {
		if (argv[i][6]!='=') euler2=2.0;
		else euler2=atof(&argv[i][7]);
	}
	else if (strncmp("amask=",argv[i],6)==0) {
		sscanf(argv[i]+6,"%d,%f,%d",&amask1,&amask2,&amask3);
	}
	else if (strncmp("maxshift=",argv[i],9)==0) maxshift=atoi(&argv[i][9]);
	else if (strncmp("noweight",argv[i],8)==0) noweight=1;
	else if (strncmp("perturb",argv[i],7)==0) perturb=1;
	else if (strncmp("refine",argv[i],6)==0) refine=1;
	else if (strncmp("sep=",argv[i],4)==0) ntk=atoi(&argv[i][4]);
	else if (strncmp("ang=",argv[i],4)==0) ang=atof(&argv[i][4]);
	else if (strncmp("anglist=",argv[i],8)==0) anglistfile=&argv[i][8];
	else if (strncmp("msa=",argv[i],4)==0) msa=atoi(&argv[i][4]);
	else if (strncmp("mra2",argv[i],4)==0) mra2=1;
	else if (strncmp("ca3",argv[i],3)==0) can=3;
	else if (strncmp("rfp",argv[i],3)==0) rfp=1;
	else if (strncmp("d3s=",argv[i],4)==0) d3s=atof(&argv[i][4]);
	else if (strncmp("3dr",argv[i],3)==0) {
		d3r=1;
		if (argv[i][3]=='=') range=atof(&argv[i][4]);
	}
	else if (strncmp("sym=",argv[i],4)==0) sym=&argv[i][4];
//	else if (strncmp("restart",argv[i],7)==0) restart=1;
	else if (strncmp("precen",argv[i],6)==0) precen=1;
	else if (strncmp("ctfc=",argv[i],5)==0) ctfc=atof(&argv[i][5]);
	else if (strncmp("ctfcw=",argv[i],6)==0) {
		ctfcw=&argv[i][6];
		if (access(ctfcw,R_OK)) error(ERR_EXIT,"Structure factor file (ctfcw) not found!");
	}
	else if (strncmp("goodbad",argv[i],7)==0) goodbad=1;
	else if (strncmp("classfp=",argv[i],8)==0) classfp=atoi(&argv[i][8]);
	else if (strncmp("median",argv[i],6)==0) median=1;
	else if (strncmp("nweight",argv[i],7)==0) nweight=1;
	else if (strncmp("usefilt",argv[i],7)==0) usefilt=1;
	else if (strncmp("collapse=",argv[i],9)==0) collapse=atof(&argv[i][9])*PI/180.0;
	else if (strncmp("xfiles=",argv[i],7)==0) {
		xfiles=atof(&argv[i][7]);
		sscanf(&argv[i][7]," %f , %f , %d",&appix,&xfiles,&alito);
	}
	else if (strncmp("dot",argv[i],3)==0) error(ERR_EXIT,"The 'dot' option is obsolete. Please omit it.");
	else if (strncmp("frm2d",argv[i],5)==0) {		//ycong
		frm2d=1;
		if (argv[i][5] == '=') sscanf(argv[i]+6,"%f,%d",&rmaxH,&size);
	}
	else if (strncmp("ccc",argv[i],3)==0) ccc=1;		//ycong
	else if (strncmp("continue",argv[i],8)==0) contin=1;
	else if (strncmp("classalign2log",argv[i],14)==0) classalign2log=1;
	else if (strncmp("norm3d", argv[i], 6)==0) norm3d=1;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

if (mask<2) error(ERR_EXIT,"mask= is a required parameter");
if (minproc<=0) minproc=1;
if (maxproc<=minproc) maxproc=minproc;
if (setsflp && ((!ctfcw && !setsffile) || !appix)) error(ERR_EXIT,"setsf= functions only in conjunction with ctfcw= and xfiles=");

if (xfiles) {
	if( access("x.0.mrc",R_OK) ){	// test if the file already exists
		sprintf(s,"proc3d threed.0a.mrc x.0.mrc clip=%d,%d,%d mask=%d",mask*2,mask*2,mask*2,mask);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		runpar_run(s,minproc,maxproc,0);

		sprintf(s,"volume x.0.mrc %f set=%f",appix,xfiles);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		runpar_run(s,minproc,maxproc,0);
	}
}

// read in the classesbymra.log file to find out which job has been done
vector< ClassesByMRCLog * > finished;
if (!contin) unlink("classesbymra.log");
FILE* cfp = fopen("classesbymra.log","rb");
if(cfp){
	int tmp_iter, tmp_job_index, tmp_job_num;
	while(fscanf(cfp,"Cycle %d\tjob %d/%d is done\n", &tmp_iter, &tmp_job_index, &tmp_job_num) == 3) {
		ClassesByMRCLog* tmp_entry = new ClassesByMRCLog;
		tmp_entry->iter 		= tmp_iter;
		tmp_entry->job_index 	= tmp_job_index;
		tmp_entry->job_num 		= tmp_job_num;
		finished.push_back(tmp_entry);
	}
	fclose(cfp);
}
if (sym && !strcasecmp(sym,"icos")){	// to detect if the half icos map is used. sometimes half map is used to get around memory limitation
	EMData* map = new EMData();
	map->readImage("threed.0a.mrc",-1,1);
	if(map->xSize()==map->ySize() && map->xSize()==2*map->zSize()) icoshalf=1;
	delete map;
}

for (i=1; i<=iter; i++) {
	ti=time(0);
	if (log) { fprintf(log,"Iteration %d beginning at %s",i,ctime(&ti)); fflush(log); }

// project3d, classesbymra, classalignall
	sprintf(s,"classes.%d.hed",i);
	if (restart || access(s,R_OK)) {

		FILE *plog = fopen("particle.log","ab");
		fprintf(plog,"S\t%d\t%d\n",i,time(0));
		fclose(plog);

		// project3d

#ifdef NOMULTIP3D

		sprintf(s,"project3d threed.%da.mrc mask=%d mode=%d",i-1,mask,projmode);
		if (sym) sprintf(s+strlen(s)," sym=%s",sym);
		if (sym && !strcasecmp(sym,"icos") && icoshalf) sprintf(s+strlen(s)," icos5fhalfmap");
		if (perturb) sprintf(s+strlen(s)," perturb");
		if (anglistfile) sprintf(s+strlen(s)," list=%s",anglistfile);
		else sprintf(s+strlen(s)," prop=%g",ang);
		sprintf(s+strlen(s)," logit=%0d",i);
		runpar_run(s,minproc,minproc,0);
#else
		{
		const char *t = tmpnam2();
		FILE *out = fopen(t,"wb");
		if (!out) { printf("Cannot open temporary file!\n"); exit(1); }
		int p3dp=minproc<24?minproc:24;
		for (j=0; j<p3dp; j++) {
			sprintf(s,"project3d threed.%da.mrc mask=%d mode=%d frac=%d/%d",i-1,mask,projmode,j,p3dp);
			if (sym) sprintf(s+strlen(s)," sym=%s",sym);
			if (sym && !strcasecmp(sym,"icos") && icoshalf) sprintf(s+strlen(s)," icos5fhalfmap");
			if (perturb) sprintf(s+strlen(s)," perturb");
			if (anglistfile) sprintf(s+strlen(s)," list=%s",anglistfile);
			else sprintf(s+strlen(s)," prop=%g",ang);
			sprintf(s+strlen(s)," logit=%0d",i);
			fprintf(out,"%s\n",s);
			if (log && !j) { ti=time(0); fprintf(log,"%s > %s  %s",s,t,ctime(&ti)); fflush(log); }
		}
		fclose(out);

		unlink("proj.hed");
		unlink("proj.img");
		sprintf(s,"runpar proc=%d file=%s",p3dp,t);
		run(s);
#ifndef DEBUG
		unlink(t);
#endif
		}
#endif

		if (highpass){
		  system("mv proj.img projtmp.img; mv proj.hed projtmp.hed");
		  sprintf(s,"proc2d projtmp.img proj.img hp=%f mask=%d; /bin/rm projtmp.???",highpass,mask);
		  system(s);
		 }



		if (classfp) {
			if (access("start.fp",R_OK)) {
				if (log) { ti=time(0); fprintf(log,"footprintr start.hed start.fp   %s",ctime(&ti)); fflush(log); }
				runpar_run("footprintr start.hed start.fp", minproc, maxproc, 0);
			}
			if (log) { ti=time(0); fprintf(log,"footprintr proj.hed proj.fp   %s",ctime(&ti)); fflush(log); }
			runpar_run("footprintr proj.hed proj.fp", minproc, maxproc, 0);

//			run("msa2 proj.fp start.fp proj.msa.fp start.msa.fp %d 100",classfp);
			sprintf(s,"msa2 proj.fp start.fp proj.msa.fp start.msa.fp %d 100",classfp);
			if (log) { ti=time(0); fprintf(log,"%s    %s",s,ctime(&ti)); fflush(log); }
			runpar_run(s, minproc, maxproc, 0);
//			if (log) { ti=time(0); fprintf(log,"msa2 proj.fp start.fp proj.msa.fp start.msa.fp 6 100  %s",ctime(&ti)); fflush(log); }

			runpar_run("classesrefs start.msa.fp proj.msa.fp", minproc, maxproc, 0);
			if (log) { ti=time(0); fprintf(log,"classesrefs start.msa.fp proj.msa.fp   %s",ctime(&ti)); fflush(log); } 			
		}
		else {
			n=fileCount("start.hed");
			if (n>maxproc) {
				if (n/maxproc>50) { n=(n/50); n-=n%maxproc; }
				else n=maxproc;
			}
			if (n==0) n=maxproc;

			if (tree) {
				sprintf(s,"projtree proj.hed proj.tree level=%d",tree);
				if (sym) sprintf(s+strlen(s)," sym=%s",sym);
				runpar_run(s, minproc, maxproc, 0);
			}
			
			if ( access("cls0000.lst", R_OK) || !contin) runpar_run("proc2d proj.hed cls mraprep", minproc, maxproc, 0);	//prepares references in cls files

			const char *t = tmpnam2();
			FILE *out = fopen(t,"wb");
			if (!out) { printf("Cannot open temporary file!\n"); exit(1); }

			for (j=0; j<n; j++) {
				
				if(finished.size()){
					bool done = 0;
					for(int entry=0; entry<finished.size(); entry++){
						if(i==finished[entry]->iter && j==finished[entry]->job_index && n==finished[entry]->job_num) {
							done =1;
							break;
						}
					}
					if (done) continue;
				}
				
				if (dfilt==3) {
					sprintf(s,"classesbymradfilt.py start.hed proj.hed %s frac=%d/%d mask=%d logit=%d",refine?"refine":"",j,n,mask,i);
					if (usefilt) sprintf(s+strlen(s)," usefilt");
				} else {
					sprintf(s,"classesbymra  start.hed proj.hed split %s %sfrac=%d/%d mask=%d imask=%d logit=%d shrink=%d%s%s%s%s",refine?"refine":"",
						rfp?"rfp ":"",j,n,mask,imask,i,shrink,precen?" precen":"",j==0?"":" quiet",can==3?" ca3":"",sigfilt?"sigfilt":"");
					if (ctfc) sprintf(s+strlen(s)," ctfc");
					if (ctfcw) sprintf(s+strlen(s)," ctfc=%s",ctfcw);
					if (ntk) sprintf(s+strlen(s)," sep=%d",ntk);
					if (usefilt) sprintf(s+strlen(s)," usefilt");
					if (dfilt==1) sprintf(s+strlen(s)," dfilt");
					if (dfilt==2) sprintf(s+strlen(s)," dfilt2");
					if (phasecls==1&&ctfcw) sprintf(s+strlen(s)," wtphase");
					else if (phasecls==1) sprintf(s+strlen(s)," phase");
					if (phasecls==2&&ctfcw) sprintf(s+strlen(s)," wtfscmp");
					else if (phasecls==2) sprintf(s+strlen(s)," fscmp");
					if (maxshift>=0) sprintf(s+strlen(s)," maxshift=%d",maxshift);
					if (tree) strcat(s," tree1=proj.tree");
					if (frm2d && rmaxH>0.0 && size>0) sprintf(s+strlen(s)," frm2d=%f,%d",rmaxH,size);
					else if (frm2d && rmaxH>0.0) sprintf(s+strlen(s)," frm2d=%f",rmaxH);
					else if (frm2d) sprintf(s+strlen(s)," frm2d");
					if (ccc) sprintf(s+strlen(s)," ccc");		//ycong
					if (projbatches) sprintf(s+strlen(s)," projbatches=%d",projbatches);
					if (slow) strcat(s," slow");
					else if (mra2) s[12]='2';
				}
				fprintf(out,"%s\n",s);
				if (log && !j) { ti=time(0); fprintf(log,"%s > %s  %s",s,t,ctime(&ti)); fflush(log); }
			}
			fclose(out);

			sprintf(s,"runpar proc=%d,%d file=%s",minproc,maxproc,t);
			run(s);
#ifndef DEBUG
			unlink(t);
#endif
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }

			out = fopen(t,"wb");
			if (!out) { printf("Cannot open temporary file!\n"); exit(1); }

			if (tree) {
				int fn;
				FILE *tree=fopen("proj.tree","rb");
				if (!tree) { printf("Tree file vanished !\n"); exit(1); }

				while (fgets(s,256,tree)) {
					if (sscanf(s," %d",&fn)!=1) continue;			
					sprintf(s,"classesbymra  tcls%04d.lst proj.hed %ssplit mask=%d imask=%d logit=%d shrink=%d%s%s%s%s%s",
						fn,rfp?"rfp ":"",mask,imask,i,shrink,precen?" precen":"",
						j==0?"":" quiet",can==3?" ca3":"",sigfilt?" sigfilt":"",(ctfc||ctfcw)?" ctfc":"");
					strcat(s," tree2=proj.tree");
					if (slow) strcat(s," slow");
					if (dfilt==1) strcat(s," dfilt");
					if (dfilt==2) sprintf(s+strlen(s)," dfilt2");
					if (refine) strcat(s," refine");
					if (phasecls==1&&ctfcw) sprintf(s+strlen(s)," wtphase");
					else if (phasecls==1) sprintf(s+strlen(s)," phase");
					if (phasecls==2&&ctfcw) sprintf(s+strlen(s)," wtfscmp");
					else if (phasecls==2) sprintf(s+strlen(s)," fscmp");
					if (maxshift>=0) sprintf(s+strlen(s)," maxshift=%d",maxshift);
					if (usefilt) strcat(s," usefilt");
					if (frm2d && rmaxH>0.0 && size>0) sprintf(s+strlen(s)," frm2d=%f,%d",rmaxH,size);
					else if (frm2d && rmaxH>0.0) sprintf(s+strlen(s)," frm2d=%f",rmaxH);
					else if (frm2d) sprintf(s+strlen(s)," frm2d");
					if (ccc) sprintf(s+strlen(s)," ccc");
					fprintf(out,"%s\n",s);
					if (log && !j) { ti=time(0); fprintf(log,"%s > %s  %s",s,t,ctime(&ti)); fflush(log); }
				}
				fclose(tree);
				fclose(out);
			
				sprintf(s,"runpar proc=%d,%d file=%s",minproc,maxproc,t);
				run(s);
	 			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
#ifndef DEBUG
				unlink(t);
#endif
			}
		}
		
		// save the cls*lst files so that we have a copy for record instead of relying on clsregen from particle.log file
		sprintf(s,"tar cvf cls.%d.tar *cls*.lst",i);
		run(s);
		sprintf(s,"classalignall ref1 finalref saverefs keep=%1.2f %d proc=%d,%d mask=%d imask=%d logit=%d %s%s%s",
			keep,classiter,minproc,minproc,mask,imask,i,median?"median":"",precen?" precen":"",amask1>0?(rma?" refmaskali":" refmask"):"");
		if (ctfc!=0) sprintf(s+strlen(s)," ctfc=%f",ctfc);
		if (ctfcw) sprintf(s+strlen(s)," ctfcw=%s",ctfcw);
		if (refine) sprintf(s+strlen(s)," refine");
		if (dfilt==1) sprintf(s+strlen(s)," dfilt");
		if (dfilt==2) sprintf(s+strlen(s)," dfilt2");
		if (phaseopt) sprintf(s+strlen(s)," phaseopt");
		if (nweight) sprintf(s+strlen(s)," nweight");
		if (phasecls==1) sprintf(s+strlen(s)," phase");
		if (phasecls==2) sprintf(s+strlen(s)," fsc");
		if (maxshift>=0) sprintf(s+strlen(s)," maxshift=%d",maxshift);
		if (slow) strcat(s," slow");
		if (frm2d && rmaxH>0.0 && size>0) sprintf(s+strlen(s)," frm2d=%f,%d",rmaxH,size);
		else if (frm2d && rmaxH>0.0) sprintf(s+strlen(s)," frm2d=%f",rmaxH);
		else if (frm2d) sprintf(s+strlen(s)," frm2d");
		if (ccc) sprintf(s+strlen(s)," ccc");
		if (classalign2log) strcat(s," classalign2log");
//		if (usefilt) sprintf(s+strlen(s)," usefilt");
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		run(s);

		sprintf(s,"classes.%d.img",i);
		rename("classes.img",s);
		sprintf(s,"classes.%d.hed",i);
		rename("classes.hed",s);
		if (log) fprintf(log,"mv classes.hed/img to %s/img\n",s);
	}

// make3d
	if (!restart) {
		sprintf(s,"threed.%da.mrc",i);
		if (!access(s,R_OK)) { 
			if (log) { fprintf(log,"Iteration %d already complete\n",i); fflush(log); }
			continue;
		}
	}

// This reassigns all of the euler angles
	if (euler2>0) {
	
		// first, reproject the model with finer spacing
#ifdef NOMULTIP3D

		sprintf(s,"project3d threed.%da.mrc out=proj.e2.hed prop=%1.3f mask=%d mode=%d",i-1,ang/euler2,mask,projmode);
		if (sym) sprintf(s+strlen(s)," sym=%s",sym);
		if (sym && !strcasecmp(sym,"icos") && icoshalf) sprintf(s+strlen(s)," icos5fhalfmap");
		if (perturb) sprintf(s+strlen(s)," perturb");
//sprintf(s+strlen(s)," logit=%0d",i);
		runpar_run(s,minproc,minproc,0);
#else
		{
		const char *t = tmpnam2();
		FILE *out = fopen(t,"wb");
		if (!out) { printf("Cannot open temporary file!\n"); exit(1); }
		for (j=0; j<minproc; j++) {
			sprintf(s,"project3d threed.%da.mrc out=proj.e2.hed prop=%1.3f mask=%d mode=%d frac=%d/%d",i-1,ang/euler2,mask,projmode,j,minproc);
			if (sym) sprintf(s+strlen(s)," sym=%s",sym);
			if (sym && !strcasecmp(sym,"icos") && icoshalf) sprintf(s+strlen(s)," icos5fhalfmap");
			if (perturb) sprintf(s+strlen(s)," perturb");
			sprintf(s+strlen(s)," logit=%0d",i);
			fprintf(out,"%s\n",s);
			if (log && !j) { ti=time(0); fprintf(log,"%s > %s  %s",s,t,ctime(&ti)); fflush(log); }
		}
		fclose(out);

		unlink("proj.e2.hed");
		unlink("proj.e2.img");
		sprintf(s,"runpar proc=%d file=%s",minproc,t);
		run(s);
#ifndef DEBUG
		unlink(t);
#endif
		}
#endif

//		sprintf(s,"project3d threed.%da.mrc prop=%1.3f mask=%d mode=-4",i-1,ang/euler2,mask);
//		if (sym) sprintf(s+strlen(s)," sym=%s",sym);
//		run(s);

		const char *t = tmpnam2();
		FILE *out = fopen(t,"wb");
		if (!out) { printf("Cannot open temporary file!\n"); exit(1); }
		
		for (j=0; j<maxproc; j++) {
			sprintf(s,"eulerproj classes.%d.hed proj.e2.hed writeali frac=%d/%d mask=%d %s %s",i,j,maxproc,mask,precen?" precen":"",sigfilt?"sigfilt":"");
			if (phasecls==1) sprintf(s+strlen(s)," phase");
			if (phasecls==2) sprintf(s+strlen(s)," fsc");
			fprintf(out,"%s\n",s);
			if (log && !j) { ti=time(0); fprintf(log,"%s > %s  %s",s,t,ctime(&ti)); fflush(log); }
		}
		fclose(out);

		sprintf(s,"runpar proc=%d,%d file=%s",minproc,maxproc,t);
		run(s);
#ifndef DEBUG
		unlink(t);
#endif
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
	}

	// This will shift altitudes > collapse to 90 degrees and expand the space
	// from 45-collapse to 90
	if (collapse>0) {
		sprintf(s,"classes.%d.hed",i);
		int n=fileCount(s);
		for (int l=0; l<n; l++) {
			EMData tmp;
			tmp.readImage(s,l);
			float alt=tmp.alt();
			if (alt>collapse) alt=PI/2.0;
			else if (alt>PI/4.0) alt=PI/4.0+((alt-PI/4.0)*((PI/4.0)/(collapse-PI/4.0)));
			tmp.setRAlign(alt,tmp.az(),tmp.phi());
			tmp.writeIMAGIC(s,l);
		}
	}

	if (msa) {
		sprintf(s,"cloud classes.%d.hed classes.msa.hed %d 1",i,msa);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		runpar_run(s,minproc,maxproc,0);
	}


/*	if (strncmp(sym,"icos",4)==0) {
		sprintf(s,"make3dicos classes.%d.%shed out=threed.%d.mrc",i,msa?"msa.":"",i);
		if (minproc>1) sprintf(s+strlen(s)," proc=%d",minproc);
	}
	else */
	{
		if (d3r) sprintf(s,"make3dr classes.%d.%shed out=threed.%d.mrc mask=%d range=%f %s",i,msa?"msa.":"",i,mask,range,goodbad?"goodbad":"");
		else if (simple) sprintf(s,"make3dsimple classes.%d.%shed out=threed.%d.mrc mask=%d inorm norm",i,msa?"msa.":"",i,mask);
		else {
			sprintf(s,"make3d classes.%d.%shed out=threed.%d.mrc mask=%d",i,msa?"msa.":"",i,mask);
			if (goodbad) sprintf(s+strlen(s)," goodbad");
			if (hard) sprintf(s+strlen(s)," hard=%1.2f",hard);
			if (d3s) sprintf(s+strlen(s)," keep=%1.2f",d3s);
			if (noweight) sprintf(s+strlen(s)," noweight");
			sprintf(s+strlen(s)," logit=%0d",i);
			if (norm3d) sprintf(s+strlen(s)," norm");
			if (appix>0) sprintf(s+strlen(s)," apix=%f",appix);
		}
		if (sym) sprintf(s+strlen(s)," sym=%s",sym);
		if (mode) sprintf(s+strlen(s)," mode=%d",mode);
		if (pad) sprintf(s+strlen(s)," pad=%d",pad);
		if (sym && !strcasecmp(sym,"icos")) sprintf(s+strlen(s)," lowmem");
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
	}

	runpar_run(s,minproc,maxproc,0);
	
// proc3d
	sprintf(s,"proc3d threed.%d.mrc threed.%da.mrc norm mask=%d",i,i,mask);
	if (lp) sprintf(s+strlen(s)," lp=%d",lp);
	if (axial>0) sprintf(s+strlen(s)," axialfilt=%f apix=%f",axial,appix);
	if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
	runpar_run(s,minproc,maxproc,0);

//setsf
	if (setsflp) {
		sprintf(s,"proc3d threed.%da.mrc threed.%da.mrc setsf=%s apix=%f lp=%f",i,i,setsffile?setsffile:ctfcw,appix,setsflp);
		if (setsfhp) sprintf(s+strlen(s)," hp=%f",setsfhp);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		runpar_run(s,minproc,maxproc,0);
	}


// make3diter if selected
	if (d3it || d3it2) {
		if (setsflp) {
			unlink("classes.tmp.hed");
			unlink("classes.tmp.img");
			sprintf(s,"proc2d classes.%d.hed classes.tmp.hed filefilt=proc3d.filt apix=%f lp=%f",i,appix,setsflp);
			if (setsfhp) sprintf(s+strlen(s)," hp=%f",setsfhp);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			runpar_run(s,minproc,maxproc,0);
			sprintf(s,"clean3d classes.tmp.hed threed.%da.mrc proc=%d 3dit=%d 3dit2=%d",i,minproc,d3it,d3it2);
			if (sym) sprintf(s+strlen(s)," sym=%s",sym);
			if (speed!=1.0) sprintf(s+strlen(s)," speed=%f",speed);
		}
		else {
			sprintf(s,"clean3d classes.%d.hed threed.%da.mrc proc=%d 3dit=%d 3dit2=%d",i,i,minproc,d3it,d3it2);
			if (sym) sprintf(s+strlen(s)," sym=%s",sym);
			if (speed!=1.0) sprintf(s+strlen(s)," speed=%f",speed);
		}

		run(s);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }

/*		char *t = tmpnam(NULL);
		char *t2 = tmpnam(NULL);
		FILE *out = fopen(t,"wb");
		FILE *out2 = fopen(t2,"wb");
		if (!out) { printf("Cannot open temporary file!\n"); exit(1); }

		for (j=0; j<minproc; j++) {
			fprintf(out,"make3diter threed.%da.mrc classes.%d.hed threed.%da.mrc tozero frac=%d/%d\n",i,i,i,j,minproc);
			fprintf(out2,"make3diter threed.%da.mrc classes.%d.hed threed.%da.mrc frac=%d/%d\n",i,i,i,j,minproc);
			if (log && !j) { ti=time(0); fprintf(log,"%s > %s  %s",s,t,ctime(&ti)); fflush(log); }
		}
		fclose(out);
		fclose(out2);

		for (j=0; j<d3it2; j++) {
			sprintf(s,"runpar proc=%d file=%s",minproc,t);
			run(s);

			if (sym) {
				sprintf(s,"proc3d threed.%da.mrc threed.%da.mrc sym=%s",i,i,sym);
				runpar_run(s,minproc,maxproc,0);
			}
		}

		for (j=0; j<d3it; j++) {
			sprintf(s,"runpar proc=%d file=%s",minproc,t2);
			run(s);

			if (sym) {
				sprintf(s,"proc3d threed.%da.mrc threed.%da.mrc sym=%s",i,i,sym);
				runpar_run(s,minproc,maxproc,0);
			}
		}
		sprintf(s,"make3diter threed.%da.mrc classes.%d.hed threed.%da.mrc testonly",i,i,i);
		runpar_run(s,minproc,maxproc,0);

		unlink(t);
		unlink(t2);*/
	}

	system("rm cls????.lst");


//automask
	if (amask1>0 && xfiles) {
		sprintf(s,"volume threed.%da.mrc %f set=%f",i,appix,xfiles);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		runpar_run(s,minproc,maxproc,0);

		sprintf(s,"proc3d threed.%da.mrc threed.%da.mrc automask2=%d,%1.3f,%d",i,i,amask1,amask2,amask3);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		runpar_run(s,minproc,maxproc,0);
	}

// planar mask
	if (pmask>0) {
		sprintf(s,"proc3d threed.%da.mrc threed.%da.mrc rfilt=105,0,0,%d",i,i,pmask);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		runpar_run(s,minproc,maxproc,0);
	}

// x-files
	if (xfiles) {
		sprintf(s,"proc3d threed.%da.mrc x.%d.mrc clip=%d,%d,%d",i,i,mask*2,mask*2,mask*2);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		runpar_run(s,minproc,maxproc,0);

		if (i>alito) {
			sprintf(s,"align3d x.%0d.mrc x.%d.mrc x.%d.mrc",alito,i,i);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			runpar_run(s,minproc,maxproc,0);
		}
		else if (log) fprintf(log,"Alignment skipped\n");

		if (xfiles>0) {
			sprintf(s,"volume x.%d.mrc %f set=%f",i,appix,xfiles);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			runpar_run(s,minproc,maxproc,0);
		}
	}

// for icos symmetry, turn it into half map to save some memory 
	if (sym && !strcasecmp(sym,"icos") && icoshalf) {
			sprintf(s,"proc3d threed.%da.mrc threed.%da.mrc tophalf",i,i);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			runpar_run(s,minproc,maxproc,0);
	}
}

if (log) { ti=time(0); fprintf(log,"%s  Refine complete\n",ctime(&ti)); }
fclose(log);
LOG(Ref,NULL,LOG_END,NULL);

return 0;
}

void *cmra(void *s)
{
char ss[1024];

strcpy(ss,(char *)s);
run(ss);
return NULL;
}

int runpar_run(const char* cmd, int minproc,  int maxproc, int usefs) {	// use runpar to run the command
	const char *t = tmpnam2();
	FILE *out = fopen(t,"wb");
	if (!out) { printf("Cannot open temporary file \"%s\"!\n", t); exit(1); }
	fprintf(out,"%s ", cmd);		
	fclose(out);

	char s[256];
	sprintf(s,"runpar proc=%d,%d file=%s %s", minproc, maxproc, t, usefs?"":"nofs");
	int r = run(s);
#ifndef DEBUG
	unlink(t);
#endif
	return r;
}
