// seteulers.C	4/7/02		Steven Ludtke

//N seteulers
//F Will assign Eulers from a text file or modify existing assignments (IMAGIC only)
//T M
//1
//P <images>	File to assign to or modify
//P [dump=<listfile>]	Dumps the (processed) Eulers to a textfile. If used, Eulers are NOT written to <images>
//P [spiderout]	Used with 'dump' to produce Spider style Eulers (phi,theta,gamma)
//P [imagicout]	Used with 'dump' to produce Imagic style Eulers (alpha,beta,gamma)
//P [mrcout]	Used with 'dump' to produce MRC style Eulers (theta,phi,omega)
//P [sgiout]	Used with 'dump' to produce IRIX Explorer angles (Q,n1,n2,n3)
//P	[fromfile=<listfile>]	Listfile contains 'N alt az phi' in degrees
//P [collapse=<angle>]	If alt>angle -> 90, alt<angle expanded to reach 90
//D This program is used to work with Euler angles present in the image header.
//E It can be used to set angles from a text file, dump existing angles to a text file,
//E or modify the angles in one of several specific ways.
//D 'collapse' allows compensation for the problem that sometimes the equatorial Eulers (alt=90)
//E get pushed up towards the pole (alt=0). This will linearly 'collapse' the angles back towards the equator.
#include <cstdio>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include <cmath>

struct EUL { float alt,az,phi; };

int main(int argc, char *argv[] ) {
int i,j,N;
float collapse=-1.0;
char *fromfile=NULL,*dump=NULL;
int dumptype=0;

if (argc<2) appusage(argv[0]);

appinit(argc,argv);

for (i=2; i<argc; i++) {	
	if (strncmp(argv[i],"fromfile=",9)==0) fromfile=argv[i]+9;
	else if (strncmp(argv[i],"collapse=",9)==0) collapse=atof(argv[i]+9);
	else if (strncmp(argv[i],"spiderout",9)==0) dumptype=1;
	else if (strncmp(argv[i],"imagicout",9)==0) dumptype=2;
	else if (strncmp(argv[i],"mrcout",6)==0) dumptype=3;
	else if (strncmp(argv[i],"sgiout",6)==0) dumptype=4;
	else if (strncmp(argv[i],"dump=",5)==0) dump=argv[i]+5;
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_REFFILE,NULL);

N=fileCount(argv[1]);

Euler **angle = (Euler **)calloc(N,sizeof(Euler *));

// Read Eulers from a text file
if (fromfile) {
	FILE *in=fopen(fromfile,"rb");
	char s[121];
	s[120]=0;
	float a1,a2,a3;
	while (fgets(s,120,in)) {
		sscanf(s," %d %f %f %f",&i,&a1,&a2,&a3);
		if (i>=0 && i<N) angle[i]=new Euler(a1*PI/180.,a2*PI/180.,a3*PI/180.);
	}
	fclose(in);
}

// Fill in any missing Eulers from the input file
for (i=0; i<N; i++) {
	if (!angle[i]) {
		EMData d;
		d.readImage(argv[1],i,1);
		angle[i]=new Euler(d.alt(),d.az(),d.phi());
	}
}

if (collapse>0) {
	collapse*=PI/180.;
	for (i=0; i<N; i++) {
		if (angle[i]->alt()>collapse) angle[i]->setAngle(PI/2.0,angle[i]->az(),angle[i]->phi());
		else angle[i]->setAngle(angle[i]->alt()*(PI/2.)/collapse,angle[i]->az(),angle[i]->phi());
	}
}

// Write to a text file if requested
if (dump) {
	FILE *out=fopen(dump,"wb");
	if (!out) error(ERR_EXIT,"Cannot write to dump= file");
	for (i=0; i<N; i++) {
		if (dumptype==0) fprintf(out,"%d\t%1.3f\t%1.3f\t%1.3f\n",i,angle[i]->alt()*180./PI,angle[i]->az()*180./PI,angle[i]->phi()*180./PI);
		else if (dumptype==1) fprintf(out,"%d\t%1.3f\t%1.3f\t%1.3f\n",i,angle[i]->phiSpider()*180./PI,angle[i]->thetaSpider()*180./PI,angle[i]->gammaSpider()*180./PI);
		else if (dumptype==2) fprintf(out,"%d\t%1.3f\t%1.3f\t%1.3f\n",i,angle[i]->alpha()*180./PI,angle[i]->beta()*180./PI,angle[i]->gamma()*180./PI);
		else if (dumptype==3) fprintf(out,"%d\t%1.3f\t%1.3f\t%1.3f\n",i,angle[i]->thetaMRC()*180./PI,angle[i]->phiMRC()*180./PI,angle[i]->omegaMRC()*180./PI);
		else if (dumptype==4) fprintf(out,"%d\t%1.3f\t%1.3f\t%1.3f\t%1.3f\n",i,angle[i]->Qsgi(),angle[i]->n1sgi(),angle[i]->n2sgi(),angle[i]->n3sgi());
	}
	fclose(out);
	LOG(Ref,dump,LOG_OUTFILE,NULL);
	LOG(Ref,NULL,LOG_END,NULL);
	exit(0);
}

// Otherwise write back to argv[1]
for (i=0; i<N; i++) {
	EMData d;
	d.readImage(argv[1],i,1);
	d.setRAlign(angle[i]->alt(),angle[i]->az(),angle[i]->phi());
	d.writeIMAGIC(argv[1],i);
}

LOG(Ref,argv[1],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);
} 


