// sheethunter.c Copyright 1999 Steve Ludtke
// 1/31/99
// This program will search a hi-res (~5A) map for beta sheets
// very very experimental

#include <cstdio>
#include <cmath>
#include "EMData.h"
#include "List.h"
#include "util.h"

float DA=(5.0*PI/180.0);
#define SHTR	(7.0/2.0)
//#define SHTW	(4.0/2.0)

int main(int argc, char *argv[]) {
EMData *model,*cyl,*out,*ccf,*best,*balt,*baz;
int nz,i,j,k,l,ps,ps2;
int ii,jj,kk,nn=0;
float alt,az,dlaz,apix,x,y,z,s,bm,bs;
float *d,*d2,*dalt,*daz,*dt,*dt2,*dt3;
float *a[4],dl,da;
char *intfile=NULL;

//appinit(argc,argv);

if (argc<4) { 
	printf("sheethunter - the search for beta sheets\nUsage:\nsheethunter <input> <output> <a/pix> [int=<file>] [da=<angle>]\n"); 
	exit(0); 
}

for (i=4; i<argc; i++) {	
	if (strncmp(argv[i],"int=",4)==0) intfile=&argv[i][4];
	else if (strncmp(argv[i],"da=",3)==0) DA=atof(&argv[i][3])*PI/180.0;
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}


int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

apix=atof(argv[3]);
if (apix<=0) { printf("A/pix must be >=0\n"); exit(1); }

model=new EMData;
if (model->readImage(argv[1],-1)) { printf("Error reading image\n"); exit(1); }
model->normalize();
nz=model->zSize();
if (nz!=model->xSize() || nz!=model->ySize()) {
	printf("The model MUST be in a square box (nx=ny=nz)\n");
	exit(1);
}

out=new EMData;
out->setSize(nz,nz,nz);
out->zero();

best=new EMData;
best->setSize(nz,nz,nz);
best->zero();

balt=new EMData;
balt->setSize(nz,nz,nz);
balt->zero();

baz=new EMData;
baz->setSize(nz,nz,nz);
baz->zero();

cyl=new EMData;
cyl->setSize(nz,nz,nz);

ps=(int)floor(3*SHTR/apix);
//ps2=(int)floor(SHTW/apix);

//model->toCorner();

for (alt=0; alt<PI+DA/2.0; alt+=DA) {
	i=floor(sin(alt)*PI/DA);
	dlaz=PI/(float)(i+1);
	for (az=0; az<PI-dlaz/2; az+=dlaz) {
		// generate cylinder for ccf
		cyl->zero();
		d=cyl->getData();
		for (k=-ps; k<=ps; k++) {
			for (j=-ps; j<=ps; j++) {
				for (i=-ps; i<=ps; i++) {
					l=(i+nz/2)+(j+nz/2)*nz+(k+nz/2)*nz*nz;
					x= i*cos(az)         +j*sin(az);
					y=-i*sin(az)*cos(alt)+j*cos(alt)*cos(az)+k*sin(alt);
					z= i*sin(alt)*sin(az)-j*sin(alt)*cos(az)+k*cos(alt);
					dl=z*apix;
//					d[l]=exp(-SQR(dl))*cos(dl*PI/2.0);
					d[l]=exp(-SQR(dl/6))*cos(dl*PI/3.4); // sheet - sheet dist ~7A
					dl=hypot(x*apix/SHTR,y*apix/SHTR);
					if (dl>1.0) d[l]*=exp(-SQR(dl-1.0)/.25);
				}
			}
		}
		cyl->doneData();
//		cyl->toCorner();

		
		
		// now do a CCF
		ccf=model->calcCCF(cyl,1);
		ccf->multConst(1.0e9);
//		if (ccf->Max()<fabs(ccf->Min())) { ccf->multConst(-1.0); printf("Inv  "); }
//		else printf("     ");

//		char t[80];
//		sprintf(t,"ccf.%1.0f.%1.0f.mrc",alt*180/PI,az*180/PI);
//		cyl->writeMRC(t);

		// Now keep track of the best cylinder at each voxel
		d=ccf->getDataRO();
		d2=best->getData();
		dalt=balt->getData();
		daz=baz->getData();

		for (k=0; k<nz; k++) {
			for (j=0; j<nz; j++) {
				for (i=0; i<nz; i++) {					
					if (*d>*d2) {
						*d2=*d;
						*dalt=alt;
						*daz=az;
					}
					d++;
					d2++;
					dalt++;
					daz++;
				}
			}
		}
		ccf->doneData();
		best->doneData();
		balt->doneData();
		baz->doneData();

//		char t[80];
//		sprintf(t,"ccf.%d.mrc",nn++);
//		best->writeMRC(t);

		printf("%1.1f,%1.1f\tccf max=%g sig=%g\n",alt*180.0/PI,az*180.0/PI,ccf->Max(),ccf->Sigma());
//		printf(" %f\t%f\r",alt*180.0/PI,az*180.0/PI); fflush(stdout);
		delete ccf;
	}
}

printf("ccf max=%g sig=%g\n",best->Max(),best->Sigma());
bm=best->Max();
bs=best->Sigma();

if (intfile) {
	best->writeMRC(intfile);
	LOG(Ref,intfile,LOG_OUTFILE,"intermediate");
}

// Now make a map out of the best cylinders
a[0]=out->getData();
a[1]=best->getDataRO();
a[2]=balt->getDataRO();
a[3]=baz->getDataRO();
ps=(int)floor(2.0*SHTR/apix);
for (s=bm-bs*4.0; s<=bm+bs/4.0; s+=bs/2.0) {
//for (s=bm-bs/2.0; s<=bm+bs/4.0; s+=bs/2.0) {
d=a[0];
d2=a[1];
dalt=a[2];
daz=a[3];
for (k=0; k<nz; k++) {
	for (j=0; j<nz; j++) {
		for (i=0; i<nz; i++) {
			// if we have a big peak, draw a line in the output file
			if (*d2>s) {
				dt3=NULL;
				for (dl=1.0; dl<=ps; dl+=1.0) {
					for (da=0; da<2.0*PI; da+=1.0/dl) {
						x=dl*cos(da);
						y=dl*sin(da);

						ii=x*cos(*daz)-y*sin(*daz)*cos(*dalt);
						jj=x*sin(*daz)+y*cos(*dalt)*cos(*daz);
						kk=y*sin(*dalt);
						if (i+ii<0||i+ii>=nz || j+jj<0||j+jj>=nz || k+kk<0||k+kk>=nz) continue;
						dt2=d+(ii)+(jj)*nz+(kk)*nz*nz;
//						if (dt2==dt3) continue;
//						dt3=dt2;
						(*dt2)=s;
					}
				}
			}
			d++;
			d2++;
			dalt++;
			daz++;
		}
	}
}
}
out->doneData();
best->doneData();
balt->doneData();
baz->doneData();

out->writeMRC(argv[2]);
LOG(Ref,argv[2],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);

return 0;
}
