// symeval.C  Copyright 1990  Steve Ludtke
// 1/23/99  based on starticos.c
// This will try to assess possible rotational symmetries present in a
// set of raw images

//N symeval
//F Experimental - not documented yet
//T X
//P <centered particles>	Particles to assess
//P [max=<max sym>]	Highest c symmetry to consider
//P [mask=<rad>]	Mask radius in pixels
//P [imask=<rad>]	Inner mask radius in pixels
//P [sigfilt]	Apply a real-space sigma filter to the images
//P [conly]	Skip the evaluation phase
#include <cstdio>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include <cmath>

//#define NITER 8

int main(int argc, char *argv[]) {
EMData *d,*d1,*msk;
int nx,ny,imask=0,mask=0,ccl=0,sigfilt=0,maxsym=10,conly=0;
Euler *a,e;
float phi[4],f,norm,u,v,val[181];
int nk,n,i,j,k;
char s[1024];
char *file;

appinit(argc,argv);

if (argc<2) { printf("symeval <centered particles> [max=<max sym>] [mask=<rad>] [imask=<rad>] [sigfilt] [conly]\n"); exit(0); }

n=fileCount(argv[1]);
if (n<50) { printf("Input file too small (n<50)\n"); exit(1); }
nk=n/20;


for (i=2; i<argc; i++) {	
	if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"imask=",6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp(argv[i],"max=",4)==0) maxsym=atoi(&argv[i][4]);
	else if (strncmp(argv[i],"sigfilt",7)==0) sigfilt=1;
	else if (strncmp(argv[i],"conly",5)==0) conly=1;
	else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
}


int Ref=LOGbegin(argc,argv);

system("/bin/rm -f cls*hed cls*img");

// for each symmetry axis (5,3,2 fold), find the best representative
// particles and store them for alignment
LOG(Ref,argv[1],LOG_INFILE,NULL);

if (!conly) {

for (i=2; i<=maxsym; i++) {
	printf("Check sym %d\n",i);
	sprintf(s,"symbest %s cls%04d.hed sym=c%0d",argv[1],i-2,i);
	if (mask) sprintf(s+strlen(s)," mask=%0d",mask);
	if (imask) sprintf(s+strlen(s)," imask=%0d",imask);
	system(s);

	sprintf(s,"cls%04d.hed",i-2);
	LOG(Ref,s,LOG_OUTFILE,NULL);
}

// Now generate classums for each of the symmetric axes
sprintf(s,"classalignall median dot keep=1.2 12 precen");
if (imask) sprintf(s+strlen(s)," imask=%0d",imask);
if (mask) sprintf(s+strlen(s)," mask=%0d",mask);
system(s);
}

d1=new EMData;
d1->readImage(argv[1],0);

msk=d1->copy();
msk->setParent(NULL);
msk->one();
if (mask) msk->applyMask(mask,4);
if (imask) msk->applyMask(imask,5);
norm=msk->dot(msk);


// Now we read in the classums from above and check their symmetry
for (i=2; i<=maxsym; i++) {
	d1->readImage("classes.hed",i-2);
	if (sigfilt) d1->realFilter(1,d1->Sigma());
	d1->radialSubtract();
	d=d1->copy();

	v=1e10;
	for (float f=2.0*PI/(float)i; f<(2.0-1.0/(float)i)*PI; f+=2.0*PI/(float)i) {
		d->setRAlign(f,0,0);
		d->rotateAndTranslate();
		d->mult(msk);
//		v+=d->dot(d1);
		if (d->dot(d1)<v) v=d->dot(d1);
	}
	delete d;
	
//	v/=(float)(i-1);
	v/=d1->xSize()*d1->ySize();
	
	
	printf("%3d. %5.3f ",i,v);
	for (u=0; u<v; u+=.025) putchar('*');
	putchar('\n');
} 

LOG(Ref,NULL,LOG_END,NULL);

return 0;
}
