// tiltseries.C  Copyright 2001 Baylor College of Medicine
// 1/29/01
// This program will take a tilt series of the same particle and
// align the individual images and set Euler angles

//N tiltseries
//F Aligns a set of images from a tilt series and assigns Eulers
//T LT
//1
//P <input file>	Images to be processed
//P <output file>	Processed output
//P <dang>	Angular step between images
//P [indccl]	Writes individual CCL images to ccl.hed
//P [fixang=<ang>]	If the common-line orientation is known, specify it here
//P [mask=<pixel radius>]	Mask type 4 (step cut off to zero), defaults to xSize()/3
//P [skip=<image0[,imageN]>]	The image index in the input file to skip for alignment.
//P [maxshift=<pixels]	Maximum shift in translational alignment, defaults to xSize()/4
//P [vertical]	Align images so that tilt axis is vertical.
//U tiltseries particle.hed particle.tilt.hed 10
//D This program is still somewhat experimental
//D This proram is used for analysis of tilt-series, ie - tomography.
//E This program will take a tilt series of a single particle, mutually align them,
//E determine the tilt axis, and write the aligned images to a new file with Eulers.

#include <cstdio>
#include <cmath>
#include "EMData.h"
#include "List.h"
#include "util.h"

int main(int argc, char *argv[]) {
int n ,i, indccl=0;
float dang;
float fixang=999.0;
char* skippointer="";
int* skip=0;
int mask=0;
int maxshift=0;
float vertical=PI/2.0;


appinit(argc,argv);


// Print usage
if (argc<4) appusage(argv[0]);


// Get options and parameters
for (i=4; i<argc; i++) {	
	if (strncmp(argv[i],"indccl",6)==0) { 
		indccl=1;
		unlink("ccl.hed");
		unlink("ccl.img");
	}
	else if (strncmp(argv[i],"fixang=",7)==0) {
		fixang=atof(argv[i]+7);
	}
	else if (strncmp(argv[i],"skip=",5)==0){
		skippointer=argv[i]+5;
	}
	else if (strncmp(argv[i],"mask=",5)==0){
		mask=atol(argv[i]+5);
	}
	else if (strncmp(argv[i],"maxshift=",9)==0){
		maxshift=atol(argv[i]+9);
	}
	else if (strncmp(argv[i],"vertical",8)==0){
		vertical=0;
	}
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}
dang=atof(argv[3]);
if (dang==0||fabs(dang)>70.0) {
	dang=10.0;
	error(ERR_WARN,"Illegal angle. Set to 10 degrees.\n");
}


// Read input file
List *data1;
data1=readImages(argv[1],-1,-1);
if (data1->count()==0) {
	error(ERR_EXIT,"No input data");
}


// Get the list of images to skip
if (skippointer) {
	n = data1->count();
	skip = (int*) malloc(sizeof(int)*n);
	for (i=0; i<n; i++) {
		skip[i]=0;
	}
	while (skippointer) {
		i=sscanf(skippointer, "%i", &n);
		if (i==EOF || i<1) {
			break;
		}
		char *comma_index = strchr(skippointer+1, ',');
		if (comma_index) {
			skippointer = comma_index;
		}
		else {
			skippointer = 0;
		}
		//skippointer=(strchr(skippointer+1, ',')?index(skippointer+1, ',')+1:0);
		skip[n]=1;
	}
}


// Add circular edge mean
printf("Aligning particles\n");
for (i=0; i<data1->count(); i++) {
	EMData *ob1=(EMData *)data1->objectAt(i);
	ob1->addConst(-ob1->cirMean());
}


// Get untilted image
EMData *ob1,*ob2,*ob3;
int cen=data1->count()/2;


// Find a non-skippped reference in the negative tilt
i = cen;
while (skip && skip[i]) {
	i--;
}
ob1=(EMData *) data1->objectAt(i);
i--;

// maxshift
if (! maxshift) {
	maxshift = ob1->xSize()/4;
}

// mask
if (! mask) {
	mask = ob1->xSize()/3;
}


// Align the negative tilts
for (; i>=0; i--) {
	if (skip && !skip[i]) {
		ob2=(EMData *)data1->objectAt(i);
	}
	else if (skip) {
		continue;
	}
	else {
		ob2=(EMData *)data1->objectAt(i);
	}
	ob3=ob2->copy(0,0);
	ob3->applyMask(mask,4);
	ob3->transAlign(ob1,0,1,maxshift);
	ob2->setTAlign(ob3->Dx(),ob3->Dy());
	delete ob3;
	ob2->fastTranslate();
	ob1=(EMData *)data1->objectAt(i);
}


// Find a non-skippped reference in the positive tilt
i = cen;
while (skip && skip[i]) {
	i++;
}
ob1=(EMData *) data1->objectAt(i);
i++;


// Align the positive tilts
for (; i<data1->count(); i++) {
	if (skip && !skip[i]) {
		ob2=(EMData *)data1->objectAt(i);
	}
	else if (skip) {
		continue;
	}
	else {
		ob2=(EMData *)data1->objectAt(i);
	}
	ob3=ob2->copy(0,0);
	ob3->applyMask(mask,4);
	ob3->writeIMAGIC("a.hed",-1);
	ob1->writeIMAGIC("a.hed",-1);
	
	ob3->transAlign(ob1,0,1,maxshift);
	ob2->setTAlign(ob3->Dx(),ob3->Dy());
	delete ob3;
	ob2->fastTranslate();

	ob2->writeIMAGIC("a.hed",-1);
	ob1=(EMData *)data1->objectAt(i);
}


EMData common,*sum = NULL;

if (fixang==999.0) {
	printf("Calculating common lines\n");
	int j;
	for (i=0; i<data1->count()-1; i++) {
		printf("%d/%d     \r",i,data1->count()-1);
		fflush(stdout);
		for (j=i+1; j<data1->count(); j++) {
			EMData *ob1,*ob2;
			ob1=(EMData *)data1->objectAt(i);
			ob2=(EMData *)data1->objectAt(j);

			common.commonLines(ob1,ob2,0,180);
			if (indccl) {
				common.writeIMAGIC("ccl.hed",-1);
			}
			if (!sum) {
				sum=common.copy(0,0);
			}
			else {
				sum->add(&common);	
			}
		}
	}
	sum->multConst(1.0/(data1->count()));
	sum->writeIMAGIC("ccl.hed",-1);

	FILE *out=fopen("cclsum.txt","w");
	for (i=0; i<360; i++)
		fprintf(out,"%d\t%g\n",i,sum->valueAt(i,i));
	fclose(out);

	float b=0;
	int bi=0;
	for (i=0; i<360; i++) {
		if (sum->valueAt(i,i)>b) {
			b=sum->valueAt(i,i);
			bi=i;
		}
	}
	fixang=(float)bi;
}
printf("Common line at %0.1f degrees\n",fixang);


// Do actual rotation and translation of the images
for (i=0; i<data1->count(); i++) {
	EMData *ob1;
	ob1=(EMData *)data1->objectAt(i);
	//ob1->setRAlign(PI/2.0-(float)fixang*PI/180.0);
	ob1->setRAlign(vertical-(float)fixang*PI/180.0);
	ob1->rotateAndTranslate();
	ob1->setRAlign((float)(i-data1->count()/2)*dang*PI/180.0,0,0);
	ob1->writeIMAGIC(argv[2],-1);
}

free(skip);

return 0;
}

