// volume.C  1998 Steve Ludtke

//N volume
//F This program calculates the volume/mass of a 3D map
//T LT
//1
//P <input file>	3D map to work with
//P <A/pix>	Angstroms/voxel in the map
//P [thr=<thresh>]	Calculate the volume/mass at a given threshold
//P [calc=<mass>]	Determine the threshold for a given mass
//P [set=<mass>]	Rescale the map densities so a threshold of 1 gives the specified mass
//P [plot]	Output a x/y file containing a plot of threshold vs mass
//U volume threed.1a.mrc 4.667 set=2500
//D This routine calculates volumes enclosed by isosurfaces. The algorithm used is
//E approximate. It doesn't calculate subvoxel volumes. It is fairly accurate as 
//E long as a substantial fraction of the voxels are 'inside' the volume.
//D For convenience, mass is specified rather than volume. Volume/mass
//E conversions assume a density of 1.35 g/ml (0.81 Da/A^3). 
//D Please note that an earlier version of the docs cited 1.35 Da/A^3. This 
//E was an error in the documentation only. The program was using the correct value.

#include <cstdio>
#include <cmath>
#include "EMData.h"
#include "List.h"
#include "util.h"

int main(int argc, char *argv[]) {
FILE *out;
EMData *data;
float res,thr=-999.0,set=0,step;
int i,j,n,plot=0,calc=0,Ref;
float count=0;
float *d;
char s[60];

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

for (i=3; i<argc; i++) {
	if (strncmp(argv[i],"thr=",4)==0) thr=atof(&argv[i][4]);
	else if (strncmp(argv[i],"set=",4)==0) set=atof(&argv[i][4]);
	else if (strncmp(argv[i],"calc=",5)==0) { calc=1; set=atof(&argv[i][5]); }
	else if (strncmp(argv[i],"plot",4)==0) plot=1;
	else printf("Unknown option %s\n",argv[i]);
}

if (set!=0 && plot) { printf("set and plot cannot be used together\n"); exit (1); }

data=new EMData;
data->readImage(argv[1],-1);
Ref=LOGbegin(argc,argv);
sprintf(s,"%dx%dx%d",data->xSize(),data->ySize(),data->zSize());
LOG(Ref,argv[1],LOG_INFILE,s);

res=atof(argv[2]);
if (thr==-999.0) thr=data->Sigma()+data->Mean();
float max=data->Max(), min=data->Min();

printf("Min=%f Max=%f Mean=%f Sigma=%f  Threshold=%f\n\n",data->Min(),
	data->Max(),data->Mean(),data->Sigma(),thr);
	
n=data->xSize()*data->ySize()*data->zSize();

d=data->getDataRO();
if (set!=0) {
	step=data->Sigma()/2.0;
	printf("Threshold\tMass\n");

	count=0;
	for (i=0; i<n; i++) {
		if (d[i]>=thr) count++;
	}
	printf("%f\t%f\n",thr,count*res*res*res*.81/1000.0);


	for (j=0; j<4; j++) {
		while (thr<max && count*res*res*res*.81/1000.0>set) {
			thr+=step;
			count=0;
			for (i=0; i<n; i++) {
				if (d[i]>=thr) count++;
			}
		}
		printf("%f\t%f\n",thr,count*res*res*res*.81/1000.0);
	
		step/=4.0;
	
		while (thr>min && count*res*res*res*.81/1000.0<set) {
			thr-=step;
			count=0;
			for (i=0; i<n; i++) {
				if (d[i]>=thr) count++;
			}
		}
		printf("%f\t%f\n",thr,count*res*res*res*.81/1000.0);

		step/=4.0;
	}
	data->doneData();
	
	// calc -> calculate only!
	if (!calc) {
		if(thr)data->multConst(1.0/thr);
		else data->add(1.0);
		data->writeMRC(argv[1]);
		printf("\nImage has been modified so threshold=1 is %1.2f kDa\n",count*res*res*res*.81/1000.0);
		sprintf(s,"vol @ 1 = %1.2f kDa",count*res*res*res*.81/1000.0);
		LOG(Ref,argv[1],LOG_OUTFILE,s);
		LOG(Ref,NULL,LOG_END,NULL);
	}
	else {
		sprintf(s,"thr=%f vol=%1.2f kDa",thr,count*res*res*res*.81/1000.0);
		LOG(Ref,NULL,LOG_END,s);
	}
}
else if (plot) {
	out=fopen("volume.dat","wb");

	for (thr=data->Mean()+data->Sigma()/4.0; thr<data->Mean()+data->Sigma()*5.0; thr+=data->Sigma()/10.0) {
		count=0;
		for (i=0; i<n; i++) {
			if (d[i]>=thr) count++;
		}

		fprintf(out,"%f\t%f\n",thr,count*res*res*res*.81/1000.0);
	}
	fclose(out);
	data->doneData();
	LOG(Ref,"volume.dat",LOG_OUTFILE,NULL);
	LOG(Ref,NULL,LOG_END,NULL);
}
else {
	for (i=0; i<n; i++) {
		if (d[i]>=thr) count++;
	}
	printf("%1.0f/%d  %f A^3  ~%1.2f kilodaltons (@1.35 g/ml)\n",count,n,count*res*res*res,
		count*res*res*res*.81/1000.0);
	data->doneData();
	sprintf(s,"thr=%f ~%1.2f kDa",thr,count*res*res*res*.81/1000.0);
	LOG(Ref,NULL,LOG_END,s);	
}

return 0;
}
