// Control.cpp  Copyright 1998 Steve Ludtke
// This is the core object for the 'ctfit' package
// It primarily handles user interface operations. Most of the 
// real work is handed off to CtfData

#include "Control.h"
#include <qfiledlg.h>
#include <qapp.h>
#include <qfont.h>
#include <qmsgbox.h> 
#include <qprogressdialog.h>
#include <qmessagebox.h>
#include <cmath>

#define Inherited ControlData

Control::Control(QWidget* parent,const char* name):Inherited( parent, name )
{
QPopupMenu *m1;

initMetaObject();

imgwin.clear();

// FILE MENU
m1=new QPopupMenu(NULL,"File");
menu->insertItem("File",m1,100);
m1->insertItem("Open",this,SLOT(Mopen()),ALT+Key_O);
m1->insertSeparator();
m1->insertItem("Exit",this,SLOT(Mquit()),ALT+Key_X);
menu->insertSeparator();

// HELP MENU
m1=new QPopupMenu(NULL,"Help");
menu->insertItem("Help",m1);
m1->insertItem("About v3",this,SLOT(Mabout()),0);

contrast->setMin(0);
contrast->setMax(1.0);
brite->setMin(-1.0);
brite->setMax(1.0);

QObject::connect(image,SIGNAL(valueChanged(int)),imaged,SLOT(display(int)));
QObject::connect(image,SIGNAL(valueChanged(int)),this,SLOT(setImage(int)));
QObject::connect(contrast,SIGNAL(newValue(float)),this,SLOT(setCont(float)));
QObject::connect(brite,SIGNAL(newValue(float)),this,SLOT(setBrite(float)));
QObject::connect(xw,SIGNAL(valueChanged(int)),this,SLOT(setSize(int)));
QObject::connect(yw,SIGNAL(valueChanged(int)),this,SLOT(setSize(int)));

imgwin.append(new Image);
imgwin.at(0)->resize(100,100);
imgwin.at(0)->show();

image->setRange(0,0);
//printf("Df font: %s %d\n",QFont::defaultFont().family(),QFont::defaultFont().pointSize());
}


Control::~Control()
{
//delete plotwin;
//delete imgwin;
//printf("Memory saved\n");
}

void Control::setSize(int val) { 

}

void Control::setCont(float val) { 
Image *im;

for (im=imgwin.first(); im!=NULL; im=imgwin.next()) {
	im->setCont(val);
}
}

void Control::setBrite(float val) { 
Image *im;

for (im=imgwin.first(); im!=NULL; im=imgwin.next()) {
	im->setBrite(val);
}

}

void Control::setImage(int val) {
imgwin.at(0)->setData(dlist.at(val));
}

void Control::readFiles(char **argv, int argc) {
int i;
EMData *dat;

for (i=0; i<argc; i++) {
	dat=new EMData;
	dat->readImage(argv[i],-1);
	dat->normalize();
	dlist.append(dat);
}
image->setRange(0,dlist.count()-1);

}

void Control::Mopen() { 
QStrList files(QFileDialog::getOpenFileNames ());
char *str;
EMData *emd;

if (files.count()==0) return;
files.setAutoDelete(TRUE); 

for (str=files.first(); str!=NULL; str=files.next()) {
	emd=new EMData;
	emd->readImage(str,0);
	emd->normalize();
	dlist.append(emd);
}

image->setRange(0,dlist.count()-1);
}

void Control::Mabout() { 
	QMessageBox::about(this,"About v3",
 "v3 Copyright 1998 Steve Ludtke\n\n\
 v3 is a simple program for\n\
 viewing 2d images.");
}

void Control::Mquit() { exit(0); }

void Control::warn(char *msg) {
	QMessageBox::warning(this,"v3 Warning",msg,"Ok");
	return;
}
