// Control.cpp  Copyright 1998 Steve Ludtke
// This is the core object for the 'ctfit' package
// It primarily handles user interface operations. Most of the 
// real work is handed off to CtfData

#include "Control.h"
#include <qfont.h>
#include <qprogressdialog.h>
#include <cmath>

#ifdef UBUNTU
	#include <qfiledialog.h>
	#include <qapplication.h>
	#include <qmessagebox.h>
#else	//UBUNTU
	#include <qfiledlg.h>
	#include <qapp.h>
	#include <qmsgbox.h> 
#endif	//UBUNTU

#define Inherited ControlData

Control::Control(QWidget* parent,const char* name):Inherited( parent, name )
{
QPopupMenu *m1;


imgwin.clear();

// FILE MENU
m1=new QPopupMenu(NULL,"File");
menu->insertItem("File",m1,100);
m1->insertItem("Open",this,SLOT(Mopen()),ALT+Key_O);
m1->insertSeparator();
m1->insertItem("Exit",this,SLOT(Mquit()),ALT+Key_X);
menu->insertSeparator();

// HELP MENU
m1=new QPopupMenu(NULL,"Help");
menu->insertItem("Help",m1);
m1->insertItem("About v4",this,SLOT(Mabout()),0);

salt->setMin(-90.0);
salt->setMax(90.0);
saz->setMin(0);
saz->setMax(360.0);
sphi->setMin(0);
sphi->setMax(360.0);
mode->setCurrentItem(6);

QObject::connect(mode,SIGNAL(activated(int)),this,SLOT(setMode(int)));

QObject::connect(salt,SIGNAL(newValue(float)),this,SLOT(setAng(float)));
QObject::connect(saz,SIGNAL(newValue(float)),this,SLOT(setAng(float)));
QObject::connect(sphi,SIGNAL(newValue(float)),this,SLOT(setAng(float)));
QObject::connect(contrast,SIGNAL(newValue(float)),this,SLOT(setCont(float)));

//printf("Df font: %s %d\n",QFont::defaultFont().family(),QFont::defaultFont().pointSize());
}


Control::~Control()
{
//delete plotwin;
//delete imgwin;
//printf("Memory saved\n");
}

void Control::Mopen() { 
EMData *dat;
Image2 *im;

QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

dat=new EMData;
dat->readImage(s.data(),-1);
//dat->toCorner();
//dat->doFFT();
//dat->toCorner();
im=new Image2;
im->setData(dat);
im->show();
imgwin.append(im);

}

void Control::setMode(int val) { 
Image2 *im;

for (im=imgwin.first(); im!=NULL; im=imgwin.next()) {
	im->setMode(val);
}
}

void Control::setCont(float val) { 
Image2 *im;

for (im=imgwin.first(); im!=NULL; im=imgwin.next()) {
	im->setCont(val);
}
}

void Control::setAng(float a) { 
Image2 *im;

for (im=imgwin.first(); im!=NULL; im=imgwin.next()) {
	im->setAngle(sphi->Value(),salt->Value(),saz->Value());
}

}

void Control::Mabout() { 
	QMessageBox::about(this,"About v4",
 "v4\nCopyright 1998 Steve Ludtke\n\n\
 v4 is a program for visualizing projections\n\
 of 3d models. Multiple models can ve viewed\n\
 and rotated simultaneously.");
}

void Control::Mquit() { exit(0); }

void Control::warn(char *msg) {
	QMessageBox::warning(this,"CTFIT Warning",msg,"Ok");
	return;
}
