#include <cstdlib>
#include <cstdio>
#include <cmath>
#include <cstring>
#include <glit/glit.h>
#include <glit/Control.h>
#include <glit/Pane.h>
#include <glit/Window.h>
#include <glit/Variable.h>
#include "V4view.h"

//#define DEBUG
#define RAD2DEG	57.295779

V4view::V4view()
{
int i;

datar=datai=lproj=NULL;
newang=0;
alt=az=phi=0;
stereo=0;
Variable::getVar("alt")->tellMe(this);
Variable::getVar("az")->tellMe(this);
Variable::getVar("phi")->tellMe(this);
Variable::getVar("min")->tellMe(this);
Variable::getVar("max")->tellMe(this);
Variable::getVar("Q")->tellMe(this);
}

V4view::~V4view()
{
int i;

if (lproj) delete lproj;
}

void V4view::setRect(float x0,float y0, float x1,float y1)
{
Control::setRect(x0,y0,x1,y1);

}

void V4view::display(int active)
{
EMData *f,*r,*t;
float *dat,sca;
int i,x,y;
char s[80];

if (!datar || !datai) return;

if (alt!=Variable::getVarVal("alt") ||
	az!=Variable::getVarVal("az") ||
	phi!=Variable::getVarVal("phi") ||
	min!=Variable::getVarVal("min") ||
	max!=Variable::getVarVal("max") ||
	Q!=(int)floor(Variable::getVarVal("Q"))
	) newang=1;

if (!newang && lproj) {
	sca=Size().x/lproj->xSize();
	if (Size().y/lproj->ySize()<sca) sca=Size().y/lproj->ySize();
	
	dat=lproj->getData();
	glPixelZoom(sca,sca);
	glRasterPos2i(0,0);
	glDrawPixels(lproj->xSize(),lproj->ySize(),GL_LUMINANCE,GL_FLOAT,dat);
	lproj->doneData();
	return;
}

if (lproj) delete lproj;

alt=Variable::getVarVal("alt");
az=Variable::getVarVal("az");
phi=Variable::getVarVal("phi");
min=Variable::getVarVal("min");
max=Variable::getVarVal("max");
Q=(int)floor(Variable::getVarVal("Q"));
//printf("q=%d\n",Q);

if (stereo) {
	// these are for stereo views
	alt2=alt;
	az2=az;
	phi2=phi;
	roteulertb(&alt2,&az2,&phi2,4.0,0);
	

}

if (Q<0) r=datar->project3d(alt/RAD2DEG,az/RAD2DEG,phi/RAD2DEG,Q);
else {
	t=datai->project3d(alt/RAD2DEG,az/RAD2DEG,phi/RAD2DEG,Q);
	r=t->clip((t->xSize()-datar->xSize())/2,(t->ySize()-datar->ySize())/2,
		datar->xSize(),datar->ySize());
	delete t;
//	r=t;
}
r->applyMask((int)ceil(.45*r->ySize()),0);

dat=r->getData();
x=r->xSize();
y=r->ySize();
for (i=0; i<x*y; i++) {
	dat[i]=(dat[i]-min)/(max-min);
	if (dat[i]>1.0) dat[i]=1.0;
	if (dat[i]<0) dat[i]=0;
}

sca=Size().x/r->xSize();
if (Size().y/r->ySize()<sca) sca=Size().y/r->ySize();

glPixelZoom(sca,sca);
glRasterPos2i(0,0);
glDrawPixels(r->xSize(),r->ySize(),GL_LUMINANCE,GL_FLOAT,dat);

r->doneData();
newang=0;
lproj=r;

//sprintf(s,"%1.1f %1.1f %1.1f\n",alt*RAD2DEG,az*RAD2DEG,phi*RAD2DEG);
//getPane()->getWindow()->setTitle(s);
}

void V4view::mouseClick(int but,int state, int dbl, float x, float y)
{
int b;

//printf("Button: %d %d %d %d %f %f\n",but,state,b,action[b],x,y);

// mouse down
if (!state && !dbl) {
	mx=x;
	my=y;
	return;
}

// mouse up
if (state) {
}

}

void V4view::mouseDrag(int but,float x,float y)
{
alt=Variable::getVarVal("alt")/RAD2DEG;
az=Variable::getVarVal("az")/RAD2DEG;
phi=Variable::getVarVal("phi")/RAD2DEG;

roteulertb(&alt,&az,&phi,x-mx,y-my);
//alt+=(y-my)/2.0;
//az+=(x-mx);
alt*=RAD2DEG;
az*=RAD2DEG;
phi*=RAD2DEG;
//if (alt>180.0) { alt-=180.0; phi+=180.0; }
//if (alt<0.0) { alt+=180.0; phi+=180.0; }
alt-=360.0*floor(alt/360.0);
if (alt>=180.0) alt-=180.0;
az-=360.0*floor(az/360.0);
phi-=360.0*floor(phi/360.0);
mx=x;
my=y;

Variable::setVar("alt",alt);
Variable::setVar("az",az);
Variable::setVar("phi",phi);


newang=1;
setChanged(1);
}

void V4view::mouseMove(float x,float y)
{

}

void V4view::mouseEnter(int e)
{

}

void V4view::mouseLeave(int e)
{

}

int V4view::keyStroke(int key, int buttons, float x, float y)
{
if (key==9) return 9;		// allow tabs to change control selection

return 0;
}

int V4view::keySpecial(int key, int buttons, float x, float y)
{

return key;
}

void V4view::winResize(float w, float h)
{
setRect(0,0,w,h);
}
