#include <cmath>
#include <glit/glit.h>
#include <glit/Window.h>
#include <glit/ValSlider.h>
#include <glit/Variable.h>
#include <glit/Button.h>
#include "EMData.h"
#include "util.h"
#include "V4view.h"


Button *qual[6];

void qualcb(Control *con,void *d1,int d2)
{
int i;
long t;

for (i=0; i<6; i++) {
	if (i!=d2) qual[i]->setIntValue(0);
	else qual[i]->setIntValue(1);
}

//Variable::setVar("Q",d2==5?-1:d2+1);
Variable::setVar("Q",d2+1);

return;
}

main(int argc, char *argv[]) {
Window *window,*w2;
ControlPane *pane,*p2;
ValSlider *vmi,*vma,*valt,*vaz,*vphi;
V4view *view;
EMData *real,*imag,*f,*r;
float min,ma,x;
long t;
int i;

if (argc!=2) { printf("v4\nUsage: v4 <input file>\n"); exit(0); }

real=new EMData;
if (real->readImage(argv[1],-1)) error("Cannot read input",ERR_EXIT);
if (real->zSize()<2) error("Sorry, volume data required!\n",ERR_EXIT);

printf("Preparing transform.\n");
if (real->isComplex()) {
// this does an extra fft, no mechanism to avoid right now
	imag=real;
	real=imag->doIFT();
	imag->gimmeFFT();
}
else {
	real->normalize();
	real->applyMask((int)ceil(.45*real->ySize()),2);
	f=real->clip(-real->xSize()/4,-real->ySize()/4,-real->zSize()/4,
		(real->xSize()+real->xSize()/2)&0xfffe,
		(real->ySize()+real->ySize()/2)&0xfffe,
		(real->zSize()+real->zSize()/2)&0xfffe);
	f->toCorner();
printf("%d %d %d\n",f->xSize(),f->ySize(),f->zSize());
	imag=f->doFFT();
	f->gimmeFFT();
	delete f;
}
imag->setup4Slice();
real->toCorner();

glitInit(1,argv);

window=new Window;
window->setTitle("Volume");
window->setSize(400,400);

pane=new ControlPane;
pane->setSize(400,400);
pane->setFillWin(1);
window->addPane(pane);

view=new V4view();
view->setRect(0,0,300,300);
view->setData(real,imag);

pane->addControl(view);

// get good min/max
r=imag->project3d(0,0,0,3);
r->applyMask((int)ceil(.45*r->ySize()),0);
min=r->Min();
ma=r->Max();
delete r;
//printf("%f %f\n",min,ma);

r=imag->project3d(PI/2,PI/2,0,3);
//r=real->project3d(0,0,0,-1);
r->applyMask((int)ceil(.45*r->ySize()),0);
if (min>r->Min()) min=r->Min();
if (ma<r->Max()) ma=r->Max();
delete r;
//printf("%f %f\n",min,ma);

// Benchmark
/*
t=time(0);
for (x=0; x<PI; x+=PI/60) {
f=imag->fftSlice(PI/4.0,x,0,3);
r=f->doIFT();
f->gimmeFFT();
r->toCorner();
r->applyMask(-1,10);
r->applyMask((int)ceil(.45*r->ySize()),2);
delete r;
delete f;
}
printf("%f\n",(float)(60.0/time(0)-t));
*/

x=(ma-min)/2.0;
ma+=x;
min-=x;

// Sliders for limit control and angle setting
w2=new Window;
w2->setTitle("Control");
w2->setSize(400,140);

p2=new ControlPane;
p2->setSize(400,120);
w2->addPane(p2);

vmi=new ValSlider();
vmi->setRect(35,90,390,108);
vmi->setXLimits(min,ma);
vmi->setLabel("Min:");
vmi->setLabelPos(GLIT_LAB_LEFT);
vmi->connectVariable("min");
vmi->setLabelSize(16.0);
Variable::setVar("min",min+x);
p2->addControl(vmi);

vma=new ValSlider();
vma->setRect(35,70,390,88);
vma->setXLimits(min,ma);
vma->setLabel("Max:");
vma->setLabelPos(GLIT_LAB_LEFT);
vma->connectVariable("max");
vma->setLabelSize(16.0);
Variable::setVar("max",ma-x);
p2->addControl(vma);

valt=new ValSlider();
valt->setRect(35,50,390,68);
valt->setXLimits(-180.0,180.0);
valt->setLabel("Alt:");
valt->setLabelPos(GLIT_LAB_LEFT);
valt->connectVariable("alt");
valt->setLabelSize(16.0);
p2->addControl(valt);

vaz=new ValSlider();
vaz->setRect(35,30,390,48);
vaz->setXLimits(0,360.0);
vaz->setLabel("Az:");
vaz->setLabelPos(GLIT_LAB_LEFT);
vaz->connectVariable("az");
vaz->setLabelSize(16.0);
p2->addControl(vaz);

vphi=new ValSlider();
vphi->setRect(35,10,390,28);
vphi->setXLimits(0,360.0);
vphi->setLabel("Phi:");
vphi->setLabelPos(GLIT_LAB_LEFT);
vphi->connectVariable("phi");
vphi->setLabelSize(16.0);
p2->addControl(vphi);

for (i=0; i<6; i++) {
	qual[i]=new Button;
	qual[i]->setRect(55+i*32,114,55+i*32+12,126);
	qual[i]->setLabelPos(GLIT_LAB_LEFT);
	qual[i]->setType(GLIT_BUT_TOGGLE,GLIT_BUT_DOT);
	qual[i]->setLabelSize(16.0);
	qual[i]->setCallback(qualcb,NULL,i);
	p2->addControl(qual[i]);
}
qual[0]->setLabel("Qual: 1");
qual[1]->setLabel("2");
qual[2]->setLabel("3");
qual[3]->setLabel("4");
qual[4]->setLabel("5");
qual[5]->setLabel("6");

qual[2]->setIntValue(1);
Variable::setVar("Q",3);


glutMainLoop();

}
